// ICUConverter.h
//
// (C) Copyright IBM Corp. 2013  All Rights Reserved.

#ifndef ICUCONVERTER_INCLUDED
#define ICUCONVERTER_INCLUDED

#include <unicode/ucnv_err.h>
#include <unicode/umachine.h>

#include "nzstd.h"

class UConverter;

/**
 * Wrapper class for ICU functionality that allows converting strings based on a specific CSSID to UTF-8 or CESU-8.
 *
 * Converting a string based on a specific CCSID to UTF-8 or CESU-8 using ICU consists of two steps:
 *   - convert string from source CCSID to UTF-16LE
 *   - convert string from UTF-16LE to UTF-8 or CESU-8
 *
 * @author Jens Mueller <jens.mueller@de.ibm.com>
 */
class ICUConverter {
public:
    enum CODED_CHARACTER_SET_IDENTIFIER {
        CCSID_ASCII = 367,
        CCSID_CESU8 = 9400,
        CCSID_UTF8  = 1208,
        CCSID_UTF16 = 1200
    };

    ICUConverter(int32_t codepage, bool convertToCESU8 = false);
    virtual ~ICUConverter();

    int32 convert(uint8 const* sourceBuffer, size_t const sourceBufferLength, uint8* targetBuffer, size_t const targetBufferLength);
    int32 convertReplaceIdeographicSpaces(uint8 const* sourceBuffer, size_t const sourceBufferLength, uint8* targetBuffer,
            size_t const targetBufferLength);

    int32 convert(uint8 const* sourceBuffer, size_t const sourceBufferLength, uint8* targetBuffer, size_t const targetBufferLength,
            size_t const maximumLength, bool& isTruncated);

    int32 convertReplaceIdeographicSpaces(uint8 const* sourceBuffer, size_t const sourceBufferLength, uint8* targetBuffer,
            size_t const targetBufferLength, size_t const maximumLength, bool& isTruncated);

private:
    /// conversion buffer
    UChar* mConversionBuffer;
    /// conversion buffer length
    uint16 mConversionBufferLength;
    /// source ICU converter
    UConverter* mConverterSource;
    /// target ICU converter
    UConverter* mConverterTarget;
};

#endif /* ICUCONVERTER_INCLUDED */
