#ifndef TXIDCOMMON_INCLUDED
#define TXIDCOMMON_INCLUDED
/// @file TXIDCommon.h
/// @brief Common transaction id definitions
///
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2008, 2011  All Rights Reserved.
//////////////////////////////////////////////////////////////////////
///
/// Description:
///
/// This file contains common definitions related to transaction ids
/// that are consumable by C and C++.
///
/// dd - 2008-12-24
//////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
# include "nzstd.h"
#else
# include "c.h"
#endif

#include "stdint.h"

////////////////////
/// TXidValue
///
/// The TXid value is the 64-bit value part of a transaction id.
/// It can be used and directly compared by C code without requiring
/// the TXid class.
////////////////////
typedef uint64   TXidValue;

////////////////////
/// Well-known values of TXids
////////////////////
#ifdef WIN32
#  define K_TXID_MAX  (0xffffffffffffi64)
#  define K_TXID_MIN  (0x400i64)
#  define K_TXID_ZERO (0x0i64)
#  define K_TXID_ONE  (0x1i64)
#else
#  define K_TXID_MAX  (int64(0xffffffffffffLL))
#  define K_TXID_MIN  (INT64_C(0x400))
#  define K_TXID_ZERO (INT64_C(0x0))
#  define K_TXID_ONE  (INT64_C(0x1))
#endif

#define TXID_MAX (K_TXID_MAX)
#define TXID_MIN (K_TXID_MIN)
#define TXid_NULL (K_TXID_ZERO) // no transaction / not deleted
#define TXid_VOID (K_TXID_ONE)  // mark aborted

#endif // TXIDCOMMON_INCLUDED
