//========================================================================
///\file: TruncateWMMsg.h Truncate Water Mark Message class
//
// (C) Copyright IBM Corp. 2012  All Rights Reserved.
//========================================================================

#ifndef TRUNCATEWMMSG_INCLUDE
#define TRUNCATEWMMSG_INCLUDE

#include "nztypes.h"

//!FIX-pm: This will need to be moved to a more logical place.
//          Also, rename tdjPack48 and tdjUnpack48 to be more generic.
// Pack significant bits (44) of PageId into 6 bytes.
struct uint48 {
    uint16 hi;
    uint16 md;
    uint16 lo;
};

inline uint48 tdjPack48 (const uint64& val) {
    uint48 bits;
    bits.hi = (uint16)(val >> 32);
    bits.md = (uint16)(val >> 16);
    bits.lo = (uint16)(val);
    return bits;
}
inline uint64 tdjUnpack48 (const uint48& bits) {
    return (uint64) bits.hi << 32
        |  (uint64) bits.md << 16
        |  (uint64) bits.lo;
}

/// \brief Truncate Water Mark Message class
class TruncateWMMsg
{
public:
    TruncateWMMsg() {};

    void init( uint64 wm );

    static size_t TruncateWMMsgSize();

    void addWaterMark( table_id_t id, int dsId, uint64 watermark );

    void addWaterMark( int dsId, uint64 watermark );

    uint64 getWaterMark( int dsIndex ) {
        return tdjUnpack48(extents[dsIndex]);
    }

    table_id_t getTableId() const {
        return tableId;
    }

private:
    table_id_t tableId;
    uint48 extents[0];
};

#endif // TRUNCATEWMMSG_INCLUDE

