#ifndef AUTOLOCK_INCLUDED
#define AUTOLOCK_INCLUDED

#include "nzpthread.h"
#include "jobintr.h"

class auto_lock
{
    pthread_mutex_t *m_lock;

public:

    auto_lock(pthread_mutex_t *lock);
    ~auto_lock();
};

inline
auto_lock::auto_lock(pthread_mutex_t *lock)
    :
    m_lock(lock)
{
    NZ_ASSERT(!jobintr_immed_sigint_enabled());
    nzpthread_mutex_lock(lock);
}

inline
auto_lock::~auto_lock()
{
    nzpthread_mutex_unlock(m_lock);
}

class auto_lock_trk
{
    pthread_mutex_t *m_lock;
    const char *m_name;
    int m_id;
    bool m_locked;

public:

    auto_lock_trk(pthread_mutex_t *lock, const char *name, int id = 0);
    ~auto_lock_trk();
    void Unlock();
    void Lock();
};

inline
auto_lock_trk::auto_lock_trk(pthread_mutex_t *lock, const char *name, int id)
    :
    m_lock(lock),
    m_name(name),
    m_id(id),
    m_locked(true)
{
    NZ_ASSERT(!jobintr_immed_sigint_enabled());
    nzpthread_mutex_lock_trk(lock, name, id);
}

inline
auto_lock_trk::~auto_lock_trk()
{
    if (m_locked)
       nzpthread_mutex_unlock_trk(m_lock, m_name, m_id);
}

inline void
auto_lock_trk::Unlock()
{
    NZ_VERIFY(m_locked);
    nzpthread_mutex_unlock_trk(m_lock, m_name, m_id);
    m_locked = false;
}

inline void
auto_lock_trk::Lock()
{
   NZ_VERIFY(!m_locked);
   nzpthread_mutex_lock_trk(m_lock, m_name, m_id);
    m_locked = true;
}
#endif
