/*
 * Copyright IBM Corporation 2012, 2013.
 */

#include <stdarg.h>
#include <alloca.h>
#include "memInterface.h"

#define PC_MIN_SEGMENT_SIZE 1000
#define PC_MAX_SEGMENT_SIZE 9980000  // DBOS stack size - 20K

class CodeSegment {
    friend class CodeFragment;

public:

    void *operator new(size_t);
    void operator delete(void *);

    CodeSegment(int sz);
    ~CodeSegment();

    size_t append(const char *s, size_t sz);

private:
    static mem::AllocContext memCtx;

    /** this segment's capacity */
    int  capacity;

    /** length of data in this segment. */
    int  len;

    /* data belonging to this segment */
    char *segDataP;

    /* Pointer to the next segment in the chain. */
    CodeSegment *next;
};

class CodeFragment {
private:
	CodeSegment *head; 	    // the leading segment
	CodeSegment *tail;		// the last segment
	char *strP;           // the last materialized version of this object

public:
	static bool enable_extend_max_segment_size;
	static int extend_max_segment_size;

	/* Constructors */
    CodeFragment();
    CodeFragment(int len);
    ~CodeFragment();

    /* Member functions */
    char *coalesce();
    void clear();
    bool isempty();
    void sprint_f(const char *fmt, ...);
    void sprint_f_end(const char *fmt, ...);
    void str_cat(CodeFragment *frag);
    void str_cat(const char *src);
    void str_cpy(const char *src);
    void str_cpy(CodeFragment *frag);


protected:
    void sprint_fsub(const char *fmt, va_list ap);

private:
    void copyFrom(const char *s, size_t sz);
};
