#ifndef DPM_COMMON_H
#define DPM_COMMON_H

//======================================================================
///\file dpm_common.h
//
// (C) Copyright IBM Corp. 2008, 2011  All Rights Reserved.
//======================================================================

// NEED A BETTER SOLUTIOM
class CStorageMgr;              // [TEMP] Forward
struct TScanList;               // [TEMP] Forward

// DPMStack is always the top of the data partition manager "layer cake".
class DPMStack;

#ifdef TNS
#error "dpm_common.h is not to be included for TNS builds. Please include the pm handle header files explicitly"
#endif

#include "nztypes.h"
#include "idioms.h"
#include "pmScanListDefs.h"

// Types referenced in public DPM APIs; in time some may move to other headers
//
typedef unsigned DevNo;         ///< SPU-local device number
typedef unsigned DsId;          ///< System-wide data-slice ID

// Declare the distinct storage IDs using forward class declarations.
// This avoids including the storagerefs.h header but fails to define
// the inherentance hierarchy.  Code needing access to that type
// promotion hierarchy must include storagerefs.h explicitly.
//
class StorageId;
class ExtentId;
class PageId;

typedef pm::RawSid RawSid;
typedef uint64 RawEid;
typedef uint64 RawPid;

typedef class DSHand_* DSHand;          ///< Defined data slice handle (pseudo-pointer)
typedef class ADSHand_* ADSHand;        ///< Active data slice handle (pseudo-pointer)
typedef class TCHand_* TCHand;          ///< Table connection handle (pseudo-pointer)


namespace dpm
{
    typedef pm::SLState SLState;

#define SCAN_INIT_LIST \
    _ROW_( siAll,          pm::slsiAll,         false, _(DONE)|_(REDO)|_(NEW), NRD ) _SEP_ \
    _ROW_( siDoneRedoNew,  pm::slsiDoneRedoNew, true,  _(DONE)|_(REDO)|_(NEW), NRD ) _SEP_ \
    _ROW_( siRedoNew,      pm::slsiRedoNew,     true,          _(REDO)|_(NEW), NR  ) _SEP_ \
    _ROW_( siDoneRedo,     pm::slsiDoneRedo,    true,  _(DONE)|_(REDO)       , RD  ) _SEP_ \
    _ROW_( siNew,          pm::slsiNew,         false,                 _(NEW), N   )

enum ScanInit
    /// A ScanInit value conveys to TCHand::scanListInit the desired page
    /// states and their relative ordering.
{
#define _ROW_( ENUM, VALUE, ORDERED, FILTER, FILTER_STRING ) ENUM = VALUE
#define _SEP_ ,
    SCAN_INIT_LIST
#undef _ROW_
#undef _SEP_
};


struct TCStats {
    unsigned    totalExtents;           ///< Total extents independent of ownership
    unsigned    createdPages;           ///< Total unique page references doled out

    // Connected table's statistics as would be seen by a readonly connection
    unsigned    committedExtents;       ///< At least one non-empty committed page
    unsigned    committedPagesNew;      ///< New pages in committed extents
    unsigned    committedPagesRedo;     ///< Redo pages in committed extents
    unsigned    committedPagesDone;     ///< Done pages in committed extents

    // Connected table's statistics as seen by the current connection
    unsigned    scannableExtents;       ///< No owner or non-empty scannable page(s)
    unsigned    scannablePagesNew;      ///< New pages within scannable extents
    unsigned    scannablePagesRedo;     ///< Redo pages within scannable extents
    unsigned    scannablePagesDone;     ///< Done pages within scannable extents

    // Extents owned by this connection with no committed pages
    unsigned    extentsAllocatedEmpty;  ///< Newly allocated extents, empty pending state
    unsigned    extentsAllocatedNew;    ///< Newly allocated extents, type = NEW
    unsigned    extentsAllocatedRedo;   ///< Newly allocated extents, type = REDO
    unsigned    extentsAllocatedDone;   ///< Newly allocated extents, type = DONE

    unsigned    extentsReleasable;      ///< Extents releasble on commit
    unsigned    pagesAcquired;          ///< Newly acquired pages
    unsigned    pagesEmptied;           ///< Newly emptied pages

    unsigned extentsAllocatedAll() const {
        return extentsAllocatedEmpty + extentsAllocatedNew +
            extentsAllocatedRedo + extentsAllocatedDone;
    }
};


/// The type of all defined pages in an extent.
enum ExtentType
{ EXTENT_EMPTY  = 0
, EXTENT_DONE   = 1
, EXTENT_REDO   = 2
, EXTENT_NEW    = 3
};

/// Pseudo-transaction ids for special users of table connections.
/// WARNING: These pseudo-transcations should only be used under some
/// form of mutex guaranteeing that two activities will not unwittingly
/// share a connection.  Failure to observe this restriction will
/// inevitably lead either to a resource leak or one of the activites
/// terminating a connection while another is still trying to use it.
enum
    { TXid_SHARED_APPEND   = 0x81
    , TXid_ZONEMAP_READ    = 0x83
    , TXid_NZSQA           = 0x85
    , TXid_CLIQA           = 0x87
    , TXid_RACK_EXP        = 0x89
    , TXid_ZONEMAP_UPDATE  = 0x8A
    , TXid_RECLAIM_EXTENTS = 0x91
    };

}; // dpm

#endif // DPM_COMMON_H
