//------------------------------------------------------------------------------
// String.h
//
// A std::string that uses the rc::Allocator (via rc::StlAlloc) for memory.
// mgmt - allowing std::string support on DIAB / Nucleus.
//
//      Copyright 2003 Netezza Corporation
//
// Changes:
//
// 06-24-03  dick       Added binarySize ().
// 06-02-03  dick       Initial version.

#ifndef ET_STRING_INCLUDED
#define ET_STRING_INCLUDED

#if WIN32
#pragma warning(disable: 4786)
#endif

#include "rcStdString.h"

#if FOR_POSTGRES

extern "C" {
#include "postgres.h"
#include "utils/palloc.h"
}

namespace et {

// STL Allocator using postgres allocator
template <typename T> struct ETStlAlloc {
    public:
        typedef size_t          size_type;
        typedef ptrdiff_t       difference_type;
        typedef T*              pointer;
        typedef const T*        const_pointer;
        typedef T&              reference;
        typedef const T&        const_reference;
        typedef T               value_type;

        // Convert an allocator<T> to allocator<U>
        template <typename U>
        struct rebind
        {
            typedef ETStlAlloc<U> other;
        };

        // 'structors
        ETStlAlloc() {};
        explicit ETStlAlloc(const ETStlAlloc& src) {};
        ~ETStlAlloc() {};
        template <typename U>
        ETStlAlloc(const ETStlAlloc<U>& src) {};

        // Accessors
        pointer address(reference x) const
        { return &x; };

        const_pointer address(const_reference x) const
        { return &x; };

        size_type max_size () const
        { return size_t (-1) / sizeof (T); }; // Return how many Ts can fit into all of memory

        // Allocators
        void construct(pointer p, const T& val)
        { ::new(p) T(val); };

        void destroy(pointer p)
        { p->~T(); };

        pointer allocate(size_t n, std::allocator<void>::const_pointer hint = 0)
        { return static_cast<pointer>(palloc(n * sizeof (T))); };

        void deallocate (std::allocator<void>::pointer p, size_t n)
        { pfree(p); };

    }; // template struct ETStlAlloc

// Decide if this allocator can deallocate from that allocator
//
// In the new memory allocator, we can deallocate anything from anywhere.
// This implementation may be too restrictive, but I don't think it matters.
template <typename T>
bool operator==(const ETStlAlloc<T>& a1, const ETStlAlloc<T>& a2)
{ return true; }

template <typename T>
bool operator!=(const ETStlAlloc<T>& a1, const ETStlAlloc<T>& a2)
{ return false; }

// And the one and only use of the postgres stl allocator:
typedef std::basic_string<char, std::char_traits<char>, ETStlAlloc<char> > ETString;

} // end namespace et

#elif FOR_DBOS || FOR_CLIENT

#include "memStlAlloc.h"

namespace et {

typedef std::basic_string<char,
                          std::char_traits<char>,
                          mem::stlAlloc<char, mt_ldr_string> > ETString;

} // end namespace et

#else

namespace et {
typedef std::string ETString;
}

#define RC_STRING_IS_STD_STRING 1

#endif

class BinStream;
BinStream& operator<<(BinStream& stream, const et::ETString& s);
BinStream& operator>>(BinStream& stream, et::ETString& s);

#endif /* ET_STRING_INCLUDED */

