#ifndef FCOMMCHANNEL_INCLUDED
#define FCOMMCHANNEL_INCLUDED

#include "nzstd.h"
#include "fcommconsts.h"


// channel types 
enum EChannelType {
    eNoChannel=0,           // un-initialized
    eCmdBcast=1,            // broadcast command channel
    eDataBcast=2,           // broadcast data channel
    eHost2Spu=3,            // host2spu distributes (unicast)
    eSpu2Host=4,            // spu2host return sets (unicast)
    eSpu2Spu=5,             // spu2spu distributes
    eResp=6,                // response channel
    eMirroring=7,           // mirroring traffic 
    eRegen=8,               // regen traffic
    eUnicast2Spu=9,         // general purpose host2spu (rowid delivery...)
    eSpu2HostSpuDistCtrl=10,  // control channel for spu2spu control messages
    eMiniSpu2Host=11,       // response channel used for planstats etc.
    eAsyncSpu2Host=12,      // channel for rowIds,sequences and async errors
    eRespAggr=13,           // spu  channel used for response aggregation 
    eSpu2HostRsrcUsg = 14,  // channel for sending spu resource usage feedback to host.
    eNewChannel=15          // token used to search all sites where adding
                            // a new channel needs to be supported
};

// channel allocated by acquireChannel
struct TChannel {
    uint32 chUID; // unique channel identifier used for discarding obsolete 
                  // channel messages 
    uint16 chIdx; // index into channel array
    EChannelType type; // type of channel

    bool operator == (const TChannel& that) const;
    bool operator != (const TChannel& that) const;

    bool i_isInited () const {
        return(chUID != 0);
    }
    void i_unInit() {
        chUID = 0;
    }

    bool isPermanent() const {
        return ((type == eCmdBcast) || 
                (type == eResp) ||
                (type == eUnicast2Spu) ||
                (type == eAsyncSpu2Host) ||
                (type == eSpu2HostRsrcUsg) ||
                (type == eMirroring)   ||
                (type == eRespAggr)   ||
                (type == eSpu2HostSpuDistCtrl));
    }

    bool i_needSync() const {
       // returns true if the specified transient channel needs synchronization 
#if FOR_SPU
        return (type == eSpu2Spu);
#else
        return ((type == eHost2Spu) || (type == eDataBcast));
#endif
    }

    static const char *ChannelTypeStr(EChannelType Type);
    const char *ChannelTypeStr() { return ChannelTypeStr(type); }

};

// Comparason operator for channels
inline bool TChannel::operator == (const TChannel& that) const
{
    return chUID == that.chUID && chIdx == that.chIdx;
}
inline bool TChannel::operator != (const TChannel& that) const
{
    return chUID != that.chUID || chIdx != that.chIdx;
}

// Message bcast by DBOS to open/close channels on SPUs.
struct dbos_open_close_channel_t {    // sent with SPU_OPEN_CHANNEL
    TChannel channel;   // channel as opened on DBOS
    int32 downloadItn;    // id of associated interaction
    int32 numPackets;     // (open) number of send-side packet buffers
    int32 sizeReassembly; // (open) maximum size of reassembly buffers
    int32 numReassembly;  // (oepn) number of reassembly buffers
    nzbool aborted;     // 1 if aborted
    int planId;         //corresponding planid
};

// used to pass around arguments by COSInteraction
struct channel_open_args_t {
    EChannelType chType;
    unsigned int numPackets;
    unsigned int sizeReassembly;
    unsigned int numReassembly;
};

#endif // FCOMMCHANNEL_INCLUDED
