#ifndef FCOMMCONSTS_INCLUDED
#define FCOMMCONSTS_INCLUDED

// packet types
#define FCOMM_DATA                  1       // unicast data
#define FCOMM_RAN                   2       // unicast rcv Avail Notify
#define FCOMM_EOS_RSVP              3       // unicast end-of-stream message requiring response
#define FCOMM_EOS_RESP_OBSOLETE     4       // unicast end-of-stream response, no used anymore.
#define FCOMM_EOS_RESP_CHANNEL      5       // unicast end-of-stream response send using ctrlPktPool buffer 
#define FCOMM_LDR_CHNG_NOTIFY       6       // ldr change notification
#define FCOMM_LDR_CHNG_NOTIFY_RESP  7       // response to ldr change notification
#define FCOMM_NEW_LDR_XON           8       // xon new funnel ldr
#define FCOMM_SAW_REQ               9       // saw request for spu2spu
#define FCOMM_SAN                   10      // snd ahead avail for spu2spu
#define FCOMM_BACK3                 11      // third level (final) ack to host
#define FCOMM_BCAST_RAN3            12      // broadcast rcv Avail Notify
#define FCOMM_SAW_REQ2              13      // saw request for ldr to dest
#define FCOMM_SAN2                  14      // snd ahead avail for spu2spu


// all unicast acks go here. Be careful to update FCOMM_UNICAST_ACK_LAST
#define FCOMM_UNICAST_ACK_FIRST     50
#define FCOMM_ACK                   50  // unicast ack
#define FCOMM_FNL_ACK               51  // funneling ack
#define FCOMM_SAW_REQ_ACK           52  // Ack for SAW Req message
#define FCOMM_SAW_REQ2_ACK          53  // Ack for SAW Req2 message
#define FCOMM_SAN_ACK               54  //  snd ahead avail for spu2spu
#define FCOMM_SAN2_ACK              55  //  snd ahead avail for spu2spu
#define FCOMM_UNICAST_ACK_LAST      55
#define FCOMM_NACK                  56 // Request for rtx
#define IS_UNICAST_ACK(type) (((type) >= FCOMM_UNICAST_ACK_FIRST) && ((type) <= FCOMM_UNICAST_ACK_LAST))


// all pkt types to be handled by broadcast processing must lie between
// FCOMM_BCAST_FIRST and FCOMM_BCAST_LAST
// all broadcast data like pkts must be between FCOMM_BCAST_DATA_FIRST and
// FCOMM_BCAST_DATA_LAST

#define FCOMM_BCAST_FIRST	    	100
#define FCOMM_BCAST_DATA_FIRST      100
#define FCOMM_BCAST_DATA            100
#define FCOMM_BCAST_EOS_RSVP        101    // eos broadcast


#define FCOMM_BACK1                 102     // first level ack (to grp leader)
#define FCOMM_BACK2                 103     // second level ack (to rack leader)
#define FCOMM_BCAST_RAN1            104     // broadcast rcv Avail Notify
#define FCOMM_BCAST_RAN2            105     // broadcast rcv Avail Notify
#define FCOMM_BCAST_LAST	    	105
#define FCOMM_BCAST_DATA_LAST       101

#define IS_BCAST_DATA(type) ( ((type) >= FCOMM_BCAST_DATA_FIRST) && ((type) <= FCOMM_BCAST_DATA_LAST) )
#define NEEDS_BCAST_HANDLING(type) ( ((type) >= FCOMM_BCAST_FIRST) && ((type) <= FCOMM_BCAST_LAST) )


#define FCOMM_NODE_BIAS 10000 // nodeId's in APIs are biased by this amount
                              // to prevent API callers from erroneouls calling
                              // the API with datasliceIds rather than nodeIds

#define MAX_DATASLICES_PER_SPU 80 // fixme-subchannel this should be 1 for mustang

#endif // FCOMMCONSTS_INCLUDED
