#ifndef GENPGINC_INCLUDED
#define GENPGINC_INCLUDED


/******************************************
 *  Duplicates some structs from postgres headers to avoid
 *  dependencies on postgres headers in generated code and UDXs.
 *
 *  Adds additional c++ capabilities (operators) to some of those structs.
 */


typedef struct TNumericData
{
         TNumericDigit digit[4];
}
TNumericData;


#if __GNUC__
// Control GCC structure mapper to use only four byte alignment,
// esp for PowerPc time. __DCC__ relies upon -Xmember-max-align=4.
# pragma pack(4)
#endif
struct TimeTzADT { // from pg/include/utils/date.h
    int64   time;
    int32   zone;
    inline TimeTzADT(void) {}
    inline TimeTzADT(int64 tm) { time =tm; zone = 0; }
    inline TimeTzADT(int64 tm, int32 zn) { time = tm; zone = zn; }
    inline const TimeTzADT& operator=(int val)  {
        time=0; zone=val; return *this;
    }
};

// convert timetz to GMT for comparisons [see GMT_TIMETZ() in .../pg/.../date.h]
#define GMT_TIMETZ(ttz) (((ttz).time+(((ttz).zone < 0 ? ((ttz).zone + 86400) : (ttz).zone)*1000000LL))%86400000000LL) // now microsecs


struct Interval { // from pg/include/utils/timestamp.h.
	int64		time;
	int32		month;
    inline explicit Interval(void) { time = 0; month = 0; }
    inline explicit Interval(int64 tm, int32 mn) { time = tm; month = mn; }
    inline explicit Interval(int64 tm) { time = tm; month = 0; }
    inline explicit Interval(int32 mn) { time = 0; month = mn; }

    // !FIX-jpb this is fragile.  Can we remove?  [ consider: Interval <- (timetamp-timestamp) ]
    inline const Interval& operator=(int64 val) {
        time=val; month=0; return *this;
    };
};
#if __GNUC__
# pragma pack()
#endif

inline static Interval operator +  (Interval const& lhs, Interval const& rhs) {
    return Interval( nz_add8(lhs.time, rhs.time), nz_add4(lhs.month, rhs.month) );
}

inline static Interval operator -  (Interval const& lhs, Interval const& rhs) {
    return Interval( lhs.time - rhs.time, lhs.month - rhs.month );
}

inline static void operator += (Interval& lhs, Interval const& rval) {
    lhs = lhs + rval;
}

inline static Interval operator *  (Interval const& lhs, int32 const& rhs) {
    return Interval( lhs.time * rhs, lhs.month * rhs );
}

inline static Interval operator *  (int32 const& lhs, Interval const& rhs) {
    return Interval( lhs * rhs.time, lhs * rhs.month );
}

inline static Interval operator /  (Interval const& lhs, int32 const& rhs) {
    if ( rhs == 0 ) throwError( NZ_ERROR_OVERFLOW, "Divide by 0",0);
    int32 mon = lhs.month / rhs;
    int64 time = (lhs.time + ((int64) ((lhs.month - (rhs*mon))) * ((int64) 30 * 86400000000LL)))/rhs;

    return Interval( time, mon );
}


#define SPAN(intr) (int64)((intr).time + (intr).month * (30* 86400000000LL))
#define GMT_TIMETZ(ttz) (((ttz).time+(((ttz).zone < 0 ? ((ttz).zone + 86400) : (ttz).zone)*1000000LL))%86400000000LL) // now microsecs


#endif // GENPGINC_INCLUDED
