#ifndef GROOMCGEN_INCLUDED
#define GROOMCGEN_INCLUDED

#include "nodecommon.h"

class CTable;
class CGenCode;
class TStreamInfo;

/** Groom nodes interface for code generation. */
class IGroomCGen {
public:

    //! Called from CPlan::deriveSnippetProjection
    virtual const TableShape* DeriveSnippetProjection(const TableShape* src) = 0;

    //! Called from CGenCode::chooseNextLoopStartingNode
    virtual bool IsNextLoopStartingNode() = 0;

    //! Called from CGenCode::putStructDefs
    virtual void EmitNeedNode(CGenCode*  genc) = 0;

    //Called from CGenCode::genStreamNodes
    // This method creates stream structure and emits it if needed
    virtual TStreamInfo* EmitStreamInfo(CGenCode* genc, int nodeIx, TStreamInfo *flowInfo) = 0;

    // Called from CGenCode::doEmitLoopHdrStuff
    // Base implementation emits nothing
    virtual void EmitLoopHdrStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) = 0;

    // Called from CGenCode::doEmitLoopHdr
    virtual void EmitLoopHdr(CGenCode* genc, TStreamInfo* sP) = 0;

    // Called from CGenCode::emitLoopBodyStuff
    // Base implementation emits nothing
    virtual void EmitLoopBodyStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) = 0;

    // Called from CGenCode::emitLoopFtr
    // Base implementation emits nothing
    virtual void EmitLoopFtr(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) = 0;


    // THE METHODS BELOW ARE NOT DIRECTLY REALTED TO CODE GENERATION
    // BUT SINCE THEY ALSO RUN ON DBOS WE PUT THEM HERE FOR NOW

    //! Must match the same enum in CPlan::spuSnippetSchedCosts (xplan.cpp)
    enum eDest {
        outToFabric = 1,
        outToDisk = 2
    };

    //! called by void CPlan::DbosNewPlan ()
    virtual void InitOnNewPlan() = 0;

    virtual void SpuSnippetSchedCost(
        unsigned& numChannels,  // incremental
        bool& sorter,  // default: false; optionally assigned true; never assign false
        bool& spuDist, // default: false; optionally assigned true; never assign false
        int& readPasses,     // incremental
        int& writePasses,    // incremental
        int& outBlocks,      // default: set in ScanNode; optionally assigned
        int& dest,           // default: outToFabric; optionally assigned
        int& scanBlocks,     // assigned in ScanNode, used by some other nodes
        int& newMem,         // default 0, reset per node; optionally assigned
        int& newInnerRead,   // default 0, reset per node; optionally assigned
        int& newInnerWrite,  // default 0, reset per node; optionally assigned
        bool& looper         // default false; optionally assinged true
        ) = 0;
};

// We need dynamic cast on Groom nodes.
// Using this class as a base we avoid adding virtual methods to CExecTableOp
class GroomNodeBase: public SnippetNode {
public:
    virtual IGroomCGen* iGroomCGen() { return dynamic_cast<IGroomCGen*>(this); }
    virtual ~GroomNodeBase() { /* nothing to do here */ }
};

// Case list of all nodes derived drom VBaseNode
// It is used in switch statement as follows "case CASE_GROOM_NODES:"
// So we must omit the first case and last colon
#define CASE_GROOM_NODES ReorgPrepNode: case ReorgStatsNode: \
  case ReorgMergeNode: case ReclaimNode

#endif  // GROOMCGEN_INCLUDED
