//------------------------------------------------------------------------------
// hcEnums.h
//
// Generic enumerations that are applicable system wide on SPU / SFI and Host.
//
// (C) Copyright IBM Corp. 2005, 2011  All Rights Reserved.
//

#ifndef HCENUMS_INCLUDED
#define HCENUMS_INCLUDED

//
// Please make it a point to ALWAYS add an enum at the end. Already assigned values cannot change.
// This is because there could be policy code which expects these to be constant across NPS version
// numbers. Hence always add at the end.
//

#ifdef __cplusplus
namespace hc {
#endif

enum {
    ET_UNKNOWN,
    ET_RC_ERR,
    ET_RC_INTNL_ERR,
    ET_CERROR,
    ET_STRING
};


enum {
    HC_MF_DEFAULT = 0
};

enum SpuErrorMsgSeverity {    // Severity of the error. Failure / Failure Imminent / Failure Possible / NA.
    SPU_ERROR_CLASS_INVALID = -1,
    SPU_ERROR_CLASS_NONE = 0,
    SPU_ERROR_CLASS_FAILURE = 1,
    SPU_ERROR_CLASS_FAILURE_IMMINENT = 2,
    SPU_ERROR_CLASS_FAILURE_POSSIBLE = 3,
    SPU_ERROR_CLASS_FAILURE_UNKNOWN = 4,

    SPU_ERROR_CLASS_END,
    SPU_ERROR_CLASS_START = SPU_ERROR_CLASS_INVALID,
    SPU_ERROR_CLASS_COUNT = SPU_ERROR_CLASS_END - SPU_ERROR_CLASS_START
};

enum SpuErrorCode {
    SPU_ERROR_INVALID = -1,
    SPU_ERROR_NONE = 0,
    SPU_ERROR_UNKNOWN = 1,

    SPU_ERROR_ECC_UNCORRECTABLE = 11,
    SPU_ERROR_ECC_CORRECTABLE = 12,

    SPU_ERROR_SMART_TEC_REACHED = 21,

    SPU_ERROR_RECLAIM_EXPANSION = 31,

    SPU_ERROR_DISK_UNSUP_FEATURE = 101,
// obsolete    SPU_ERROR_DISK_JOB_TIMED_OUT = 102,
    SPU_ERROR_DISK_TIMED_OUT = 103,
// obsolete    SPU_ERROR_DISK_FPGA_TIMED_OUT = 104,
// obsolete    SPU_ERROR_DISK_RD_DMA_TIMED_OUT = 105,
// obsolete    SPU_ERROR_DISK_WR_DMA_TIMED_OUT = 106,
// obsolete    SPU_ERROR_DISK_RD_DMA_ERROR = 107,
// obsolete    SPU_ERROR_DISK_WR_DMA_ERROR = 108,
    SPU_ERROR_DISK_CMD_ABORTED = 109,
    SPU_ERROR_DISK_IF_CRC_ERROR = 110,
    SPU_ERROR_DISK_INVALID_LBA = 111,
    SPU_ERROR_DISK_UNCORR_DATA_ERROR = 112,
    SPU_ERROR_DISK_ADDR_MARK_NOT_FOUND = 113,
    SPU_ERROR_DISK_BUFFER_TOO_SMALL = 114,
    SPU_ERROR_DISK_BUFFER_TOO_LARGE = 115,
    SPU_ERROR_DISK_FPGA_ERROR = 116,
    SPU_ERROR_DISK_CONTROLLER_ERROR = 117,
    SPU_ERROR_DISK_FPGA_HUNG = 118,
    SPU_ERROR_DISK_UNEXPECTED_ERROR = 119,
    SPU_ERROR_DISK_FPGA_PARITY_ERROR = 120,
    SPU_ERROR_DISK_SLEEPING = 121,
    SPU_ERROR_DISK_NO_FPGA = 122,
// obsolete    SPU_ERROR_DISK_DEFER_POOL_FULL = 123,
// obsolete    SPU_ERROR_DISK_HAS_NEEDS_DATA = 124,
    SPU_ERROR_DISK_RTOS_CALL_FAILED = 125,
    SPU_ERROR_DISK_FPGA_FLOW_THRU_ERR = 126,
    SPU_ERROR_DISK_BAD_COMMAND = 127,
    SPU_ERROR_DISK_IS_BUSY = 128,
    SPU_ERROR_DISK_NOT_READY = 129,
    SPU_ERROR_DISK_DRIVER_NOT_STARTED = 130,
    SPU_ERROR_DISK_RESET_FAILURE = 131,
    SPU_ERROR_DISK_SMART_THRESHOLD_EXCEEDED = 132,
    SPU_ERROR_DISK_BLOCK_VERIFY_ERROR = 134,
    SPU_ERROR_DISK_BLOCK_MEM_VERIFY_ERROR = 135,
    SPU_ERROR_DISK_SATA_RX_ERROR = 136,
    SPU_ERROR_DISK_SATA_TX_ERROR = 137,
// obsolete    SPU_ERROR_DISK_IDNF_ERROR = 138,
// obsolete    SPU_ERROR_DISK_WRITE_RETRY = 139,
// obsolete    SPU_ERROR_DISK_WRITE_RETRY_COMPLETE = 140,
// obsolete    SPU_ERROR_DISK_READ_RETRY = 141,
// obsolete    SPU_ERROR_DISK_READ_RETRY_COMPLETE = 142,
// obsolete    SPU_ERROR_DISK_MEDIA_CHANGED = 143,
// obsolete    SPU_ERROR_DISK_MEDIA_CHANGE_REQ = 144,
// obsolete    SPU_ERROR_DISK_NO_MEDIA = 145,
    SPU_DISK_COMMAND_QUEUE_FULL = 146,
    SPU_DISK_DMA_INVALID_CHAN = 147,
    SPU_DISK_DMA_ALIGNMENT_ERROR = 148,
    SPU_DISK_DMA_CHAN_BUSY = 149,
    SPU_DISK_DMA_ERROR = 150,
    SPU_DISK_DMA_BURST_COUNT_ERROR = 151,
    SPU_DISK_DMA_BURST_PREFETCH_ERROR = 152,
    SPU_DISK_DMA_BUS_TIMEOUT_ERROR = 153,
    SPU_DISK_DMA_SLAVE_ERROR = 154,
    SPU_DISK_DMA_PARITY_ERROR = 155,

    SPU_ERROR_SCSI_PREDICTIVE_FAILURE_EVENT = 156,

    SPU_ERROR_NO_OBUFS = 157,
    SPU_ERROR_DISK_JOB_CANCELED = 158,
    SPU_ERROR_PWREC_WRITE_ERROR = 159,
    SPU_ERROR_INIT_FILTER_ERROR = 160,
    SPU_ERROR_ISSUE_WORK_ERROR = 161,
    SPU_ERROR_DISK_LABEL_ERROR = 162,
    SPU_ERROR_DATA_SLICE_NEEDS_UPGRADE = 163,

    DEV_UEC_CRITICAL_INPUT = 201,
    DEV_UEC_MACH_CHECK = 202,
    DEV_UEC_MACH_CHECK_INST = 203,
    DEV_UEC_MACH_CHECK_DATA = 204,
    DEV_UEC_MACH_CHECK_TLB = 205, // Please note that ECC is added at the end, since our enums cant change.
    DEV_UEC_DATA_STORAGE = 206,
    DEV_UEC_DATA_STORAGE_READ_ACCESS = 207,
    DEV_UEC_DATA_STORAGE_WRITE_ACCESS = 208,
    DEV_UEC_DATA_STORAGE_CACHE_LOCKING = 209,
    DEV_UEC_DATA_STORAGE_BYTE_ORDERING = 210,
    DEV_UEC_INST_STORAGE = 211,
    DEV_UEC_INST_STORAGE_EXEC_ACCESS = 212,
    DEV_UEC_INST_STORAGE_BYTE_ORDERING = 213,
    DEV_UEC_EXTERNAL_INPUT = 214,
    DEV_UEC_ALIGNMENT = 215,
    DEV_UEC_PROGRAM = 216,
    DEV_UEC_PROGRAM_ILLEGAL_INST = 217,
    DEV_UEC_PROGRAM_PRIVILEGED_INST = 218,
    DEV_UEC_PROGRAM_TRAP_INST = 219,
    DEV_UEC_SYSTEM_CALL = 220,
    DEV_UEC_DECREMENTER = 221,
    DEV_UEC_FIXED_INTERVAL_TIMER = 222,
    DEV_UEC_WATCHDOG_TIMER = 223,
    DEV_UEC_DATA_TLB_MISS = 224,
    DEV_UEC_INST_TLB_MISS = 225,
    DEV_UEC_DEBUG = 226,
    DEV_UEC_DEBUG_TRAP = 227,
    DEV_UEC_DEBUG_INST_ADDR_COMPARE = 228,
    DEV_UEC_DEBUG_DATA_ADDR_COMPARE = 229,
    DEV_UEC_DEBUG_DATA_VALUE_COMPARE = 230,
    DEV_UEC_DEBUG_INST_COMPLETE = 231,
    DEV_UEC_DEBUG_BRANCH_TAKEN = 232,
    DEV_UEC_DEBUG_RETURN = 233,
    DEV_UEC_DEBUG_INTERRUPT = 234,
    DEV_UEC_DEBUG_UNCONDITIONAL = 235,
    DEV_UEC_UNKNOWN_EXCEPTION = 236,
    DEV_UEC_MACH_CHECK_ECC = 237,
    DEV_UEC_L2_PARITY_ERROR = 238,

    DEV_ERROR_WARNING_TEMPERATURE_EXCEEDED = 301,
	DEV_ERROR_CRITICAL_TEMPERATURE_EXCEEDED = 302,
	SYS_ERROR_CRITICAL_TEMPERATURE_EXCEEDED = 303,

    SPU_ERROR_END,
    SPU_ERROR_START = SPU_ERROR_INVALID,
    SPU_ERROR_COUNT = SPU_ERROR_END - SPU_ERROR_START

    // If a new error code is added mapping entries need to exist in hcMsgs.cpp.
    // Also add codes in .cpp files.
};

enum SpuErrorMsgAction {
    SPU_ACTION_INVALID = -1,
    SPU_ACTION_NONE = 0,
    SPU_ACTION_NOTIFY,
    SPU_ACTION_FAILOVER_NOW,
    SPU_ACTION_FAILOVER_SCHEDULED,

    SPU_ACTION_END,
    SPU_ACTION_START = SPU_ACTION_INVALID,
    SPU_ACTION_COUNT = SPU_ACTION_END - SPU_ACTION_START
};

enum {
    MAX_SIM_SPUS = 20,

    MAX_PKT_SIZE = 1400
};

enum {
    VERSION = 100,

    MAGIC = 0x52347865
};

enum {
    FIRST_INTERACT_ID = 100,
    FIRST_SEQUENCE_ID = 100,
    KNOWN_POLL_INTERACTID = 10,
    KNOWN_HBD_INTERACTID = 11,
    KNOWN_POLL_INTERACT_SEQ_ID = -1
};


// Calls are even numbered
// Replies are odd numbered
// Acks are considered replies

enum {
    ACK = 1,
    PAYLOAD = 2,
    ONEWAY_ACK = 3,

    DEBUG_MSG = 10,
    HWOP_MSG = 11,
    REGEN_MSG = 12,         // For unsolicited info from regen SPUs
    CLOCKSYNC_MSG = 13,
    SPU_ECC_MSG = 14,
#if 0 //SWS-75106
    SMTEST_MSG=24,
#endif
    SPU_DISK_SMART_MSG = 16,
    SPU_DISK_FPGA_MSG = 18,
    SPU_DISK_FPGA_MSG_REPLY = 19,
    UEC_MSG = 20,
    UEC_REPLY = 21,
    MICRO_REGEN_MSG = 22,   // For unsolicited micro-regen error messages from SPUs
    RECLAIM_EXPANSION_MSG = 23, // For unsolicited reclaim expansion messages from reclaim
    RECLAIM_RECOVERY_MSG = 25,  // For unsolicited reclaim recovery messages from the reclaimRecoveryMgr
    SPU_DISK_PREDICTIVE_FAILURE_MSG = 27,
    REGEN_FAULT = 29,         // For unsolicited info from regen SPUs
    SPU_DISK_LABEL_EXCEPTION_MSG = 31,  //For unsolicited disk label exception
    KERN_ERROR_MSG = 32,
    DISK_PATH_MSG = 33,
    GEN_INV_MSG = 34,
    HEARTBEAT_MSG = 35,

    // start of two-way messages

    POLL_CALL = 100,
    POLL_REPLY = 101,
    BOOT_CALL = 102,
    BOOT_REPLY = 103,
    INIT_CALL = 104,
    INIT_REPLY = 105,
    SETSTATE_CALL = 106,
    SETSTATE_REPLY = 107,
    GETSTATS_CALL = 108,
    GETSTATS_REPLY = 109,
    HWOP_CALL = 110,
    HWOP_REPLY = 111,
    GET_DBG_CTRLS_CALL = 112,
    GET_DBG_CTRLS_REPLY = 113,
    SET_DBG_CTRLS_CALL = 114,
    SET_DBG_CTRLS_REPLY = 115,
    CORE_REQUEST_CALL = 116,
    CORE_REQUEST_REPLY = 117,
    CORE_COMPLETE_CALL = 118,
    CORE_COMPLETE_REPLY = 119,
    SFI_POLL_CALL = 120,      // NOT used
    SFI_POLL_REPLY = 121,     // reply to POLL_CALL for SFIs
    GET_FAULT_CTRLS_CALL = 122,
    GET_FAULT_CTRLS_REPLY = 123,
    SET_FAULT_CTRLS_CALL = 124,
    SET_FAULT_CTRLS_REPLY = 125,
    POLL_READY_CALL = 126,
    POLL_READY_REPLY = 127,
    SET_DEVICE_MAPPING_CALL = 128,
    SET_DEVICE_MAPPING_REPLY = 129,
    DISK_LABEL_CALL = 130,     // From host to spu.
    DISK_LABEL_REPLY = 131,    // From spu to host.
    KEK_START        = 132,
    KEK_REPLY        = 133,
    KEY_CALL         = 134,    // SED key management 
    KEY_REPLY        = 135,
    AEK_DONE         = 136,
    AEK_DONE_REPLY   = 137,
    CHECK_SED_CALL   = 138,    // To monitor SED enablement.
    CHECK_SED_REPLY  = 139,
    MIRDOM_CALL      = 140,
    MIRDOM_REPLY     = 141,
    REFRESH_ISCSI_INITIATOR_CALL = 142,
    

    MSGS_END,

    MSGS_START = DEBUG_MSG,
    NUM_MSGS = MSGS_END - MSGS_START,

    START_CALL_MSGS = POLL_CALL

};

// Memory Report Types
enum MEM_TBL_REPORT_LEVEL {
    MEM_TBL_REPORT_ERROR   = 0x0,
    MEM_TBL_REPORT_NEW     = 0x1,       // scan (and report) only new blocks
    MEM_TBL_REPORT_ALL     = 0x2,       // scan all blocks
    MEM_TBL_REPORT_TYPES   = 0x4,       // report mem types
    MEM_TBL_REPORT_BLOCKS  = 0x8,       // report each scanned block
    MEM_TBL_REPORT_TOTALS  = 0x10       // report region totals
};

#ifdef __cplusplus
}
#endif

#endif  // HCENUMS_INCLUDED
