/// @file hcHwMgr.h
/// @brief ??
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2001,2011 All rights reserved.
//////////////////////////////////////////////////////////////////////
/// ?? old documnetation was cut'n'paste and did not apply
///
//////////////////////////////////////////////////////////////////////
#ifndef HCHWMGR_INCLUDED
#define HCHWMGR_INCLUDED

#include "nzstd.h"
#include "hcEnums.h"
#include "hcMsgsBase.h"
#include "memInterface.h"
#include "memNewDelete.h"

#if FOR_SPU
# include "hcHwThread.h"
#endif

namespace rc {
 class DbgCtrls;
 class OutDevice;
 class StrVecDevice;
 class Registry;
 class TxtTbl;
 class StrVec;
 class ByteStrm;
 namespace fault {
  class Ctrls;
 }
}

struct NzState;

namespace ac {
 class UdpSocket;
 struct PgmIds;
}

namespace opst {
 class HwMgmtChanTblRec;
}

namespace hc {  // hwcomps

struct DiskPartitionInfo;
struct HwInfo;

class Net;
class SetStateMsg;
class MsgHdr;
class Msg;
class PollCall;
class SetStateCall;
class InitReply;
class HwOpCall;
class GetStatsCall;
class GetDbgCtrlsCall;
class SetDbgCtrlsCall;
class GetFaultCtrlsCall;
class SetFaultCtrlsCall;
class ClockSyncMsg;
class SpuDiskFpgaMsgReply;
class SetDeviceMappingCall;
class PollReadyReply;
class DiskLabelCall;
class KekStart;
class KeyCall;
class MirDomReply;
class MirDomCall;
class RefreshInitiatorsCall;
class CheckSedCall;
struct GenInvMsg;


struct hwmgr_helper_info_t;

#ifdef NORETURN
# undef NORETURN
#endif
#if __GNUC__
# define NORETURN __attribute__((__noreturn__))
#else
# define NORETURN
#endif

//==============================================================================
// Class: HwMgr - an Spu manager object

class HwMgr : public mem::newDelete<mt_spu_mgr>
{
public:
    enum HwExecState {
        Unreachable,
        Discovering,
        Discovered,
        Initializing,
        Running
    };

    enum HwMonParamType {
        ENV_HWMON_1_2V,
        ENV_HWMON_1_55V,
        ENV_HWMON_2_5V,
        ENV_HWMON_3_3V,
        ENV_HWMON_VDD_33VSB,
        ENV_HWMON_5_0V,
        ENV_HWMON_12_0V,

        ENV_HWMON_SPU_PCB_TEMP
    };

    enum HwMonParamSubType {
        ENV_HWMON_INPUT,
        ENV_HWMON_MIN,
        ENV_HWMON_MAX
    };

    enum {
        MAX_SERIALNO_STR_LEN = 32
    };

    hwmgr_helper_info_t *m_helper_info;

private:
    int                 m_hwId;
    unsigned int        m_myIp;
    unsigned int        m_hostIp;
    int                 m_myPort;
    int                 m_hostPort;
    unsigned            m_doReset;
    int                 m_pollReadyCallId;
    unsigned            m_nextInitTime;
    unsigned            m_nextRetryTime;
    time_t              m_nextClockSync;

    HwExecState         m_execState;
    hc::Net*            m_net;
    ac::UdpSocket*      m_sock;

    int                 m_interactId;
    uint32              m_lastPollIntSeq;

    hc::HwInfo*         m_hwInfo;

    rc::OutDevice*      m_stdOutDevice;
    rc::StrVecDevice*   m_strVecDevice;

    opst::HwMgmtChanTblRec* m_hwChTblRec;

    bool                m_dropPoll;
    unsigned            m_nextReplyTime;
    unsigned            m_numUnsolicitedReplies;
    unsigned            m_maxUnsolicitedReplies;
    unsigned int        m_lastPollIpAddr;
    hc::Msg*            m_lastPollHdr;

    int                 m_registryVersion;
    int                 m_lastClockSyncIntId;
    
    bool                m_clockSynced;

protected:

    unsigned int        m_mySpaId;
    unsigned int        m_mySlotId;
    NzState*            m_stateInfo;
    int                 m_pollInterval;
    int                 m_lastPollIntervalNumber;
    bool                m_roleIsFailed;
    int                 m_initCallId;
    char                m_serialNo[MAX_SERIALNO_STR_LEN];

public:

    volatile int m_sysStateVersionProcessed; // last sys state version processed

    HwMgr (int simId,
           unsigned int myIp, int myPort,
           unsigned int hostIp, int hostPort);
    ~HwMgr();

    // ACCESSORS
    const char*  uuidStr() const;

    static void clockSyncStatus(bool *sync_enabled, unsigned int *freq_hz, int64 *offset_us);

    static int64        tickToTime64(const uint64& ticks);
    static int64        time64();

    static bool         envReadVoltage(HwMonParamType type, HwMonParamSubType subtype, float *voltage);
    static bool         envReadDegreesC(HwMonParamType type, HwMonParamSubType subtype, float *degreesC);
    static bool         envReadSpaAndSlot(int* spa, int* slot);

    HwExecState         execState() const;
    hc::Net*            net() const;
    int                 myPort() const;
    int                 hwId() const;
    hc::HwInfo*         hwInfo() const;
    unsigned int        myIp() const;
    unsigned int        mySpaId() const;
    unsigned int        mySlotId() const;
    unsigned int        hostIp() const;
    int                 hostPort() const;
    int                 simId() const;
    bool                roleIsFailed() const;

    bool                dropPoll() const;

    bool                clockSynced() const;

    // MANIPULATORS

    static void         initClockFreq();

    static NORETURN void terminate_app();
    static NORETURN void reboot_spu(bool force);

    void                setExecState(HwExecState state);
    void                handleState(uint64 bootSeq) throw ();
    virtual void        setupIds(int hwId, const char* uuidStr);
    virtual bool        flushStats(int tblType);
    void                initRoleIsFailedFlag(rc::ByteStrm &s);

    void                captureDbgOutput();
    const rc::StrVec&   endCaptureDbgOutput();

    void                enableClockSync(bool enable);
    void                forceClockSync();

    void                initHelperThreadInfo();
    virtual void        hwMgrThreadEntry();
    virtual void        hwMgrHelperThreadEntry();

    void                sendPollReadyCall(uint64 bootSeq);

#if SPU_OS_NUCLEUS
    void                sendSfiPollReadyCall(uint64 bootSeq);
#endif

    void transmitReply(hc::Msg *reply, unsigned int ipAddr,
                       const hc::Msg *call, bool use_heap);

protected:
    void    setupDbgCtrls(const rc::DbgCtrls& ctrls);
    void    setupAutoReply(unsigned int ipAddr, int sysMgrRespondToPort, int sysMgrRespondToCount);
    void    updateAutoReplyTime();
    void    doPollReplyBookkeeping(int intervalNumber, uint32 startTime);

    virtual void    processSetStateMsg (hc::SetStateMsg* msg);
    virtual void    sendPollReply (unsigned int ipAddr, hc::Msg* msg, bool needPayload = true) = 0;

    virtual
    void    handleHwPollCb(unsigned int ipAddr, int port, hc::PollCall* msg);

    virtual
    void    handleSetStateCb(unsigned int ipAddr, int port, hc::SetStateCall* msg) = 0;

    virtual
    void    handleGenInventoryMsgCb(unsigned int ipAddr, int port, hc::GenInvMsg* msg) = 0;

    virtual
    void    handleInitReplyCb(unsigned int ipAddr, int port, hc::InitReply* msg);

    virtual
    void    handlePollReadyReplyCb(unsigned int ipAddr, int port, hc::PollReadyReply* msg);
    
    virtual
    void    handleHwOpCb(unsigned int ipAddr, int port, hc::HwOpCall* msg);

    virtual
    void    handleGetStatsCb(unsigned int ipAddr, int port, hc::GetStatsCall* msg);

    virtual
    void    handleGetDbgCtrlsCb(unsigned int ipAddr, int port,
                                hc::GetDbgCtrlsCall* call);
    virtual
    void    handleSetDbgCtrlsCb(unsigned int ipAddr, int port,
                                hc::SetDbgCtrlsCall* call);

    virtual
    void    handleGetFaultCtrlsCb(unsigned int ipAddr, int port,
                                  hc::GetFaultCtrlsCall* call);
    virtual
    void    handleSetFaultCtrlsCb(unsigned int ipAddr, int port,
                                  hc::SetFaultCtrlsCall* call);
    virtual
    void    handleDiskFpgaMsgReplyCb(unsigned int ipAddr, int port,
                               hc::SpuDiskFpgaMsgReply* msg);
    
    virtual
    void    handleSetDeviceMappingCallCb(unsigned int ipAddr, int port,
                                         hc::SetDeviceMappingCall* msg);

    virtual
    void    handleDiskLabelCallCb(unsigned int ipAddr, int port,
                                         hc::DiskLabelCall* msg);

    virtual
    void    handleKekStartCb (unsigned int ipAddr, int port,
                              hc::KekStart* msg);

    virtual
    void    handleKeyCallCb (unsigned int ipAddr, int port,hc::KeyCall* msg);

    virtual
    void    handleCheckSedCallCb (unsigned int ipAddr, int port,hc::CheckSedCall* msg);

    void    handleRefreshInitiatorsCallCb (unsigned long ipAddr, int port,hc::RefreshInitiatorsCall* msg);

    char         m_uuidStr[HC_MSG_UUID_SIZE];

private:

    void    handlePollRequest(const hc::MsgHdr& hdr, unsigned int ipAddr, int port, const void *buffer, size_t bufSize);
    
    void    handleClockSyncMsg(const hc::MsgHdr& hdr, unsigned int ipAddr, int port, const void *buffer, size_t bufSize);

    static
    void    handleHwPollCCb(void* cbObj, unsigned long ipAddr, int port,
                            hc::Msg* msg);
    static
    void    handleSetStateCCb(void* cbObj, unsigned long ipAddr, int port,
                              hc::Msg* msg);
    static
    void    handleGenInventoryMsgCCb(void* cbObj, unsigned long ipAddr, int port,
                              hc::Msg* msg);
    static
    void    handleInitReplyCCb(void* cbObj, unsigned long ipAddr, int port,
                               hc::Msg* msg);
    static
    void    handlePollReadyReplyCCb(void* cbObj, unsigned long ipAddr, int port,
                                    hc::Msg* msg);
    static
    void    handleGetStatsCCb(void* cbObj, unsigned long ipAddr, int port,
                              hc::Msg* msg);

    static
    void    handleHwOpCCb(void* cbObj, unsigned long ipAddr, int port,
                                hc::Msg* msg);
    static
    void    handleDbgCallCCb(void* cbObj, unsigned long ipAddr, int port,
                                hc::Msg* msg);
    static
    void    handleFaultCallCCb(void* cbObj, unsigned long ipAddr, int port,
                               hc::Msg* msg);
    static
    void    handleClockSyncCCb(void *cbObj, unsigned long ipAddr, int port,
                               hc::Msg *msg);
    void    handleClockSyncCb(const hc::ClockSyncMsg *msg, unsigned int ipAddr,
                              unsigned short port);
    static
    void    handleDiskFpgaMsgReplyCCb(void *cbObj, unsigned long ipAddr, int port,
                               hc::Msg *msg);
    
    static
    void    handleUdpPacketCCb (void* cbObj, void *data, int nRead,
                                    unsigned long ipAddr, int port);
    static
    void    handleSetDeviceMappingCallCCb (void* cbObj, unsigned long ipAddr, int port,
                                           hc::Msg* msg);

    static
    void    handleDiskLabelCallCCb (void* cbObj, unsigned long ipAddr, int port,
                                           hc::Msg* msg);

    static 
    void    handleKekStartCCb (void* cbObj, unsigned long ipAddr, int port,
                                   hc::Msg* msg);

    static 
    void    handleKeyCallCCb (void* cbObj, unsigned long ipAddr, int port,
                              hc::Msg* msg);

    static 
    void    handleCheckSedCallCCb (void* cbObj, unsigned long ipAddr, int port,
                              hc::Msg* msg);

    static  
    void handleRefreshInitiatorsCallCCb (void* cbObj, unsigned long ipAddr, int port,
                              hc::Msg* msg);

    // Thread stuff
    static
    void    threadEntryCCb(unsigned long argc, void* cbObj);

};

// Inline ACCESSORS

inline const char* HwMgr::uuidStr() const
{
    return m_uuidStr;
}

inline HwMgr::HwExecState HwMgr::execState() const
{
    return m_execState;
}

inline hc::Net* HwMgr::net() const
{
    return m_net;
}

inline hc::HwInfo*  HwMgr::hwInfo() const
{
    return m_hwInfo;
}

inline int HwMgr::hwId() const
{
    return m_hwId;
}

inline bool HwMgr::roleIsFailed() const
{
    return m_roleIsFailed;
}

inline unsigned int HwMgr::hostIp() const
{
    return m_hostIp;
}

inline unsigned int HwMgr::myIp() const
{
    return m_myIp;
}

inline unsigned int HwMgr::mySpaId() const
{
    return m_mySpaId;
}

inline unsigned int HwMgr::mySlotId() const
{
    return m_mySlotId;
}

inline int HwMgr::hostPort() const
{
    return m_hostPort;
}

inline int HwMgr::myPort() const
{
    return m_myPort;
}

inline bool HwMgr::dropPoll() const
{
    return m_dropPoll;
}

inline bool HwMgr::clockSynced() const
{
    return m_clockSynced;
}

inline void
HwMgr::setExecState(HwExecState state) 
{
    m_execState = state;
}
    
        
        
        

} // end namespace hc

#endif /* HCHWMGR_INCLUDED */
