///
/// @file hcMsgHdr.h
/// @brief The innards of hc::Msg 
///
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2010, 2011  All Rights Reserved.
//////////////////////////////////////////////////////////////////////
///
/// Description:
///
///  This file defines the  serialized version of hc::Msg used by 
///  the backchannel protocol 
///  This is is used in two places - the hc::Msg framework and the
///  NZ heart beat daemon. 
///
///  DO NOT change anything here without looking at its consequences
///  on the heart beat daemon
///
/// ask - Apr 23, 2010
//////////////////////////////////////////////////////////////////////

#ifndef HCMSGHDR_INCLUDED
#define HCMSGHDR_INCLUDED

// DONOT include any C++ headers or inline methods in this file
// This is used by heartbeat deamon; it mlocks() its address space
// and so it is NOT desirable to have more code/memory utilization
// than absolutely necessary

#if WIN32
typedef int                 int32_t;
typedef unsigned int        uint32_t;
typedef unsigned long long  uint64_t;
#elif HP800 || SUN4S || AIX
#include <inttypes.h>
#else
#include <stdint.h>
#endif

struct hc_msg_hdr {
    int32_t     msg_type;
    int32_t     interact_id;
    int32_t     hw_addr;
    int32_t     block_id;
    int32_t     magic;
    int32_t     version;
    uint32_t    msg_flags;
    uint32_t    interact_seq;
};

#define HC_MSG_UUID_SIZE 40

struct hc_msg_payload_hdr {
    uint32_t    payload_size;
    int32_t     block_count;
    int32_t     rport;
    int32_t     rport_count;
    int32_t     seq_id;
    int32_t     last_seq_id;
    char        uuidstr[HC_MSG_UUID_SIZE];
};

// DO NOT include any C++ headers or inline methods in this file
// This is used by heartbeat deamon; it mlocks() its address space
// and so it is NOT desirable to have more code/memory utilization
// than absolutely necessary

#endif // HCMSGHDR_INCLUDED
