#ifndef INODECGEN_INCLUDED
#define INODECGEN_INCLUDED

#include "nodecommon.h"

class TableShape;
class CGenCode;
class TStreamInfo;

/** Node interface for code generation. */
class INodeCGen {
public:

    virtual ~INodeCGen() { };
    //! Called from CGenCode::chooseNextLoopStartingNode
    virtual bool IsNextLoopStartingNode() = 0;

    //! Called from CGenCode::putStructDefs
    // virtual void EmitNeedNode(CGenCode*  genc) = 0;
    
    //Called from CGenCode::genStreamNodes 
    // This method creates stream structure that will be the resulting stream of this node and emits it if needed
    virtual TStreamInfo* EmitStreamInfo(CGenCode* genc, int nodeIx) = 0;
    
    // Called from CGenCode::doEmitLoopHdrStuff
    // Base implementation emits nothing
    // 1. emit declarations and assignments
    virtual void EmitLoopHdrStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) = 0;

    // Called from CGenCode::doEmitLoopHdr
    // Base implementation emits nothing
    // called after EmitLoopHdrStuff
    virtual void EmitLoopHdr(CGenCode* genc, TStreamInfo* sP) = 0;
    
    // Called from CGenCode::emitLoopBodyStuff
    // Base implementation emits nothing
    // 3. called after EmitLoopHdr
    virtual void EmitLoopBodyStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) = 0;
    
    // Called from CGenCode::emitLoopFtr
    // Base implementation emits nothing
    // 4. called after EmitLoopBodyStuff
    virtual void EmitLoopFtr(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) = 0;


};


class INodeRunT {
public:
    //! Called from CPlan::deriveSnippetProjection, CPlan::initNodesState??
    virtual const TableShape* DeriveSnippetProjection(const TableShape* src) = 0;
    virtual const TableShape* Init(const TableShape* src, EPlanNodeLocus place) = 0;
    //! called by void CPlan::DbosNewPlan ()
   // virtual void InitOnNewPlan() = 0;
};

// We use this class to avoid adding virtual methods to SnippetNode for now
class CGenBase : public SnippetNode {
public:
    virtual INodeCGen* iCGen()  { return m_genNode; }
    virtual void setCGen( INodeCGen *gptr) { m_genNode = gptr; }
    virtual void freeCGen( ) { delete m_genNode; m_genNode = NULL; }
    virtual INodeRunT* iRunT () { return dynamic_cast<INodeRunT*>(this); }

private:
INodeCGen * m_genNode;
};


// Case list of all nodes derived drom VBaseNode 
// It is used in switch statement as follows "case CASE_ICGEN_NODES:"
// So we must omit the first case and last colon 
#define CASE_ICGEN_NODES SortedAggregateNode: case StreamAppendNode\

/*#define CASE_ICGEN_NODES ReorgPrepNode: case ReorgStatsNode: \
  case WindowAggrNode: \
  case ReorgMergeNode\
*/
#endif  // INODECGEN_INCLUDED
