#ifndef JOBBASE_INCLUDED
#define JOBBASE_INCLUDED

#ifndef NZTYPES_INCLUDED
# include "nztypes.h"
#endif

// class JobBase: Common base class for OsJob and SpuJob
//
// In DBOS, all jobs are OsJob instances; on the spu all instances are
// SpuJob instances. Nevertheless, we factor out some of the common features
// here.
//

enum EJobType
{
    JOBTYPE_EXEC_PLAN = 1,
    JOBTYPE_LOAD = 2,
    JOBTYPE_REGEN = 3,
    JOBTYPE_TXCOMMIT = 4,
    JOBTYPE_TXROLLBACK = 5,
    JOBTYPE_SPUDSKREWR = 6
};

class JobBase
{
    // not implemented
    JobBase(const JobBase&);
    JobBase& operator=(const JobBase&);

protected:

    JobBase() { }
    ~JobBase() { }

    int m_jobid;
    int m_jobtype; // JOBTYPE_*
    int m_base_prio; // base (unbiased) job priority
    uint64 m_ctime; // time job created

public:

    int jobid() const throw ();
    int jobtype() const throw ();
    int base_prio() const throw ();
    uint64 job_ctime() const throw ();

    void set_base_prio(int prio) throw ();

};

inline int
JobBase::jobid() const throw ()
{
    return m_jobid;
}

inline int
JobBase::jobtype() const throw ()
{
    return m_jobtype;
}

inline int
JobBase::base_prio() const throw ()
{
    return m_base_prio;
}

inline uint64
JobBase::job_ctime() const throw ()
{
    return m_ctime;
}

inline void
JobBase::set_base_prio(int p) throw ()
{
    m_base_prio = p;
}

#endif
