#ifndef KEYVALUE_H
#define KEYVALUE_H

//========================================================================
///\file: keyvalue.h
//
// (C) Copyright IBM Corp. 2009, 2011  All Rights Reserved.
//========================================================================

#include "nzstd.h"
#include "nztypes.h"


/* Definition in accurev upstream:
enum EKeyRelationalOp {
    KeyNA = -1,
    KeyFALSE,
    KeyTRUE,
    KeyGT,
    KeyGE,
    KeyLT,
    KeyLE,
    KeyEQ,
    KeyNE,
    KeyIN,
    KeyNotIN,
    KeyNULL,
    KeyNotNULL,
    KeyBEGINS,
    KeyNotBEGINS,
    KeyEverything  // magic "everything" value for use in zone maps
};
*/

enum EKeyRelationalOp {
    KeyNA = -1,
    KeyFALSE,
    KeyTRUE,
    KeyGT,
    KeyGE,
    KeyLT,
    KeyLE,
    KeyEQ,
    KeyNE,
    KeyBEGINS,
    KeyNotBEGINS,

    KeyEverything       // magic "everything" value for use in zone maps
};

struct TKeyValue {
    EKeyRelationalOp relationalOp; // relational operator
    Datum value;                   // pointer to value (data type implied by type of key field)
    int valueLength;               // length of keyValue
    int valueType;                 // PG data type of key value
    int16 fieldID;                 // field being compared to (only needed for zone maps)
    bool padEbcdic;                // when padding is necessary use EBCDIC space (0x40)

    unsigned padChar() const { return padEbcdic ? 0x40 : ' '; }
    void dump(unsigned indent);
    void dumpValue(unsigned indent);

    bool KeyValueMatch( TKeyValue const & that ) const;
    bool DatumMatch( Datum const & that, int type, int length ) const;
};

struct TZoneMapExpr {
    TKeyValue zmValue;         // value, op, etc.

    void dump(unsigned indent);
};

enum EZMCombiner {
    zmAND = 0,
    zmOR,
    zmEND
};

struct TZoneMapExprArray {
    EZMCombiner zmCombiner;    // zmAND, zmOR
    unsigned zmExprCount;
    TZoneMapExpr* zmExprP;

    void dump(unsigned indent);
};

struct TZoneMapConj {
    EZMCombiner zmCombiner;    // zmAND, zmOR

    void dump(unsigned indent);
};


enum EZMType {
    zmCONJ = 0,
    zmEXPR,
    zmEXPR_ARRAY,
    zmNONE
};

struct TZoneMapUnion {
    EZMType zmType;            // zmCONJ, zmEXPR to discriminate union
    TZoneMapUnion* zmNextP;    // next sibling (conj or expr)
    TZoneMapUnion* zmChildP;   // first child  (conj or expr)
    union {
        TZoneMapConj zmConj;
        TZoneMapExpr zmExpr;
        TZoneMapExprArray zmArray;
    };

  void dump(unsigned indent, bool recurse);
};

// WARNING: FilterForm exists in vc98/include/qos.h
enum FilterForm {
    FFORM_UNKNOWN = -1,
    FFORM_NONE = 0,
    FFORM_PRODUCT = 1,
    FFORM_SUM = 2,
    FFORM_PRODUCT_OF_SUMS = 3,
    FFORM_SUM_OF_PRODUCTS = 4
};

struct TKeyValuePacket {
    FilterForm form;               // FFORM_PRODUCT or FFORM_SUM, most likely
    TKeyValue keyValue;            // key value
    TKeyValuePacket* pNext;        // pointer to next packet in chain

    void dump(unsigned indent);
    void dumpPacket(unsigned indent, bool first);
};

#endif // KEYVALUE_H
