//******************************************************************************
//    NzSpinFile("spin_load");

    // Copy all object's frequently referenced values into locals;
    // goal is to have NO references to the implicit "this" within
    // the rest of this method
    Parser* copyIn = this;

    // Dynamic state that will need to be copied back on exit
    // (see code following label END_OF_BUFFER at bottom of file).
    bool halted = copyIn->m_halted;

    if (halted == true) {
        return;
    }

    // Various row counters
    uint64 maxErrors = copyIn->m_maxErrors;
    uint rowsToBeProcessed = copyIn->m_rowsToBeProcessed;
    uint rowsToBeSkipped = copyIn->m_rowsToBeSkipped;
    uint rowsConsumed = 0;
    uint rowsRejected = 0;

    // Local copy of the start of the field parse control sequence
    FieldParseControl* pFPC = copyIn->m_FPC;

    // Context for error diagnostics
    unsigned badParseDataType;

    // Space to collect the current row
    union {
        uint64 forceNiceAlignment;
        uint16 sizeWord;
        long   longs[MAX_RECORD_SIZE / sizeof(long)];
        uint8  bytes[MAX_RECORD_SIZE];
    } recordBuffer;

#define pRecordBuffer (recordBuffer.bytes)

    // Even though this argument never gets modified it is referenced
    // in every call to NEXT_BYTE; since we are compiling without
    // support of a frame pointer arguments are moderately distant
    // from the base of the frame; by copy the argument it ends up
    // being addressable using a 1 byte displacement (instead of 4).
    uint8 const* pEnd = pEndArg;

    // These variables are declared outside of the loops so that
    // their lifetimes include the loop's back edge.
    register uint8 const* pText REG(esi);

    std::ostringstream badField;
    std::ostringstream badStr;
    int32 db2zPartitionID = copyIn->m_db2zPartitionID;
    ptrdiff_t inputBufferLength;
    uint16 inputFieldLength;
    uint16 inputFieldOffset;
    uint8 const* pBadField;
    FieldParseControl* pCurrentFPC;
    uint8 const* pInputField;
    uint16 precision;
    uint8* pVarying;
    uint16 recordLength;

    if (copyIn->m_firstExecution == true) {
        // skip \n
        if (*pRemainingText + 1 > pEnd) {
            goto END_OF_BUFFER;
        }

        ++(*pRemainingText);

        copyIn->m_firstExecution = false;
    } else if ((copyIn->m_incompleteRowInBuffer == true) && (pEnd - *pRemainingText == m_previousRemainingTextLength)) {
        // the input buffer does not contain a complete row and no more data were received!
        pCurrentFPC = pFPC;
        pText = *pRemainingText;
        recordLength = pEnd - *pRemainingText;

        setBadFieldData(badField, badStr, pText, pEnd, 0, 0, 0, 0, pBadField);

        goto DB2Z_CORRUPT_ROW;
    }

#ifdef DB2Z_PARSER_TRACING
    inputBufferLength = pEnd - *pRemainingText;

    printf(
        "--------------------------------------------------------------------------------\n" \
        " Conversion\n\n" \
        " Input buffer length: %u B\n" \
        "--------------------------------------------------------------------------------\n", inputBufferLength);

//    for (int i = 0; i < inputBufferLength; ++i) {
//        printf("#%02d: %02X\n", i + 1, (*pRemainingText)[i]);
//    }
#endif

    // Outermost scan loop: deliver successive rows from input
    while (TRUE) {

//******************************************************************************
