//******************************************************************************

// ======== VARCHAR FOR MIXED DATA =============================================
            case DB2N_VARCHAR_MIXED: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARCHAR(<P>) FOR MIXED DATA                   | NVARCHAR(<P>)
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : 1 + [1 ... <P>]             | Number of bytes : 2 + [1 ... 4 * <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion: n/a
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 10, value = U+0041 + U+00AE + U+20AC + U+1D11E | CCSID: 1208 (UTF-8)
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence:                                | Byte sequence:
                * 00 41 C2 AE E2 82 AC F0 9D 84 9E              | 0C 00 41 C2 AE E2 82 AC F0 9D 84 9E
                * ^^ [0x00: value != NULL]                      | ^^^^^ [Length: 0x000C = 12]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * b) Example: <P> = 10, value = NULL
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence: FF                             | Byte sequence:
                *                ^^ [0xFF: value = NULL]        | 02 00
                *                                               | ^^^^^ [Length: 0x0002 = 2]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARCHAR(%u) FOR MIXED DATA | ", pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

                if (pInputField[0] == 0x00) {
                    inputFieldOffset += DB2Z_NULL_HEADER_LENGTH;
                    pInputField += DB2Z_NULL_HEADER_LENGTH;

                    goto VARCHAR_MIXED_CONVERSION;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | NULL\n", inputFieldOffset);
#endif

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t);
                // update pointers
                REG_pField += sizeof(vlen_t);
                pVarying = REG_pField;

                NzSetBit(recordBuffer.bytes, pCurrentFPC->nullIndex);

                goto NEXT_FIELD;
            }
// ======== VARCHAR FOR MIXED DATA NOT NULL ====================================
            case DB2_VARCHAR_MIXED: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARCHAR(<P>) FOR MIXED DATA NOT NULL          | NVARCHAR(<P>) NOT NULL
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : [1 ... <P>]                 | Number of bytes : 2 + [1 ... 4 * <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion: n/a
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 10, value = U+0041 + U+00AE + U+20AC + U+1D11E | CCSID: 1208 (UTF-8)
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence:                                | Byte sequence:
                * 41 C2 AE E2 82 AC F0 9D 84 9E                 | 0C 00 41 C2 AE E2 82 AC F0 9D 84 9E
                *                                               | ^^^^^ [Length: 0x000C = 12]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARCHAR(%u) FOR MIXED DATA NOT NULL | ", pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

            VARCHAR_MIXED_CONVERSION:
                uint16 inputFieldLength;

                // compute length of string in input stream
                // last column?
                if (pCurrentFPC->rrfLastVarColumn == true) {
                    // = record length - header length - inputFieldOffset
                    inputFieldLength = recordLength - DB2Z_HEADER_LENGTH - inputFieldOffset;
                } else {
                    // = offset of next zone - offset (= inputFieldOffset) of this zone
                    inputFieldLength = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                            + pCurrentFPC->rrfOffset + DB2Z_OFFSET_LENGTH))) - inputFieldOffset;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | LENGTH = %u\n", inputFieldOffset, inputFieldLength);
#endif

                // (string length > precision) || (string exceeds input stream)?
                if ((inputFieldLength > pCurrentFPC->extra.precAndScale) || (pInputField + inputFieldLength > REG_pEnd)) {
                    badField.clear();
                    badField.seekp(0);
                    badField << inputFieldLength;

                    pBadField = pInputField;

                    goto DB2Z_INVALID_FIELD_LENGTH_INFERRED;
                }

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                // check if input field could contain more than 16,000 UTF-8 characters
                if (inputFieldLength > MAX_NVARCHAR_SIZE) {
                    // check if input field contains more than 16,000 UTF-8 characters
                    size_t index = 0;
                    size_t numberOfCharacters = 0;

                    while (index < inputFieldLength) {
                        if (numberOfCharacters >= MAX_NVARCHAR_SIZE) {
                            inputFieldLength = index;
                            ++pCurrentFPC->truncations;

                            break;
                        }

                        // advance the string offset from one code point boundary to the next (1-4 bytes)
                        U8_FWD_1_UNSAFE(pInputField, index);

                        ++numberOfCharacters;
                    }
                }

                int utf8Status;

                if (copyIn->validateUTF8(pInputField, inputFieldLength, pBadField, utf8Status) == false) {
                    setBadFieldData(badField, badStr, pInputField, pEnd, 0, pBadField - pInputField, pBadField - pInputField,
                            inputFieldLength - (pBadField - pInputField), pBadField);

                    if (utf8Status == _UTF8_ILLEGAL_INITIAL_BYTE) {
                        goto UTF8_ILLEGAL_INITIAL_BYTE;
                    } else {
                        goto UTF8_BAD_CONTINUATION_BYTE;
                    }
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t) + inputFieldLength;
                // copy string
                memcpy(REG_pField + sizeof(vlen_t), pInputField, inputFieldLength);
                // update pointers
                REG_pField += sizeof(vlen_t) + inputFieldLength;
                pVarying = REG_pField;

                goto NEXT_FIELD;
            }
// ======== VARCHAR FOR MIXED/SBCS DATA (ICU conversion required) ===================
            case DB2N_VARCHAR_MIXED_ICU:
            case DB2N_VARCHAR_SBCS_ICU: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARCHAR(<P>) FOR MIXED/SBCS DATA              | NVARCHAR(<P>)
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : 1 + [1 ... <P>]             | Number of bytes : 2 + [1 ... 4 * <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion: ICU conversion to UTF-8 or CESU-8
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 10, value = U+0041 + U+00AE + U+20AC + U+1D11E | CCSID: 1208 (UTF-8)
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence:                                | Byte sequence:
                * 00 41 C2 AE E2 82 AC F0 9D 84 9E              | 0C 00 41 C2 AE E2 82 AC F0 9D 84 9E
                * ^^ [0x00: value != NULL]                      | ^^^^^ [Length: 0x000C = 12]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * b) Example: <P> = 10, value = NULL
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence: FF                             | Byte sequence:
                *                ^^ [0xFF: value = NULL]        | 02 00
                *                                               | ^^^^^ [Length: 0x0002 = 2]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARCHAR(%u) FOR MIXED/SBCS DATA (ICU conversion required) | ",
                        pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

                if (pInputField[0] == 0x00) {
                    inputFieldOffset += DB2Z_NULL_HEADER_LENGTH;
                    pInputField += DB2Z_NULL_HEADER_LENGTH;

                    goto VARCHAR_MIXED_ICU_CONVERSION;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | NULL\n", inputFieldOffset);
#endif

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t);
                // update pointers
                REG_pField += sizeof(vlen_t);
                pVarying = REG_pField;

                NzSetBit(recordBuffer.bytes, pCurrentFPC->nullIndex);

                goto NEXT_FIELD;
            }
// ======== VARCHAR FOR MIXED/SBCS DATA NOT NULL (ICU conversion required) ==========
            case DB2_VARCHAR_MIXED_ICU:
            case DB2_VARCHAR_SBCS_ICU: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARCHAR(<P>) FOR MIXED/SBCS DATA NOT NULL     | NVARCHAR(<P>) NOT NULL
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : [1 ... <P>]                 | Number of bytes : 2 + [1 ... 4 * <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion: ICU conversion to UTF-8 or CESU-8
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 10, value = U+0041 + U+00AE + U+20AC + U+1D11E | CCSID: 1208 (UTF-8),
                *    CREATE DATABASE [...] WITH [...] NATIONAL CHARACTER SET cesu8;
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence:                                | Byte sequence:
                * 41 C2 AE E2 82 AC F0 9D 84 9E                 | 0E 00 41 C2 AE E2 82 AC ED A0 B4 ED B4 9E
                *                                               | ^^^^^ [Length: 0x000E = 14]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARCHAR(%u) FOR MIXED/SBCS DATA NOT NULL (ICU conversion required) | ",
                        pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

            VARCHAR_MIXED_ICU_CONVERSION:
                uint16 inputFieldLength;

                // compute length of string in input stream
                // last column?
                if (pCurrentFPC->rrfLastVarColumn == true) {
                    // = record length - header length - inputFieldOffset
                    inputFieldLength = recordLength - DB2Z_HEADER_LENGTH - inputFieldOffset;
                } else {
                    // = offset of next zone - offset (= inputFieldOffset) of this zone
                    inputFieldLength = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                            + pCurrentFPC->rrfOffset + DB2Z_OFFSET_LENGTH))) - inputFieldOffset;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | LENGTH = %u\n", inputFieldOffset, inputFieldLength);
#endif

                // (string length > precision) || (string exceeds input stream)?
                if ((inputFieldLength > pCurrentFPC->extra.precAndScale) || (pInputField + inputFieldLength > REG_pEnd)) {
                    badField.clear();
                    badField.seekp(0);
                    badField << inputFieldLength;

                    pBadField = pInputField;

                    goto DB2Z_INVALID_FIELD_LENGTH_INFERRED;
                }

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                ICUConverter* converter = (*copyIn->m_converters)[pCurrentFPC->extra2.ccsid];

                NZ_VERIFY(converter != NULL);

                bool isTruncated;
                ptrdiff_t targetBufferLength = &recordBuffer.bytes[MAX_RECORD_SIZE - 1] - REG_pField + sizeof(vlen_t);
                int32 outputLength = converter->convertReplaceIdeographicSpaces(pInputField, inputFieldLength,
                        REG_pField + sizeof(vlen_t), targetBufferLength, MAX_NVARCHAR_SIZE, isTruncated);

                if (isTruncated == true) {
                    ++pCurrentFPC->truncations;
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t) + outputLength;
                // update pointers
                REG_pField += sizeof(vlen_t) + outputLength;
                pVarying = REG_pField;

                goto NEXT_FIELD;
            }
// ======== VARCHAR FOR SBCS DATA ==============================================
            case DB2N_VARCHAR_SBCS: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARCHAR(<P>) FOR SBCS DATA                    | VARCHAR(<P>)
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : 1 + [1 ... <P>]             | Number of bytes : 2 + [1 ... <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion: n/a
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 5, value = "IBM" | CCSID: 367 (ASCII)
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence: 00 49 42 4D                    | Byte sequence:
                *                ^^ [0x00: value != NULL]       | 05 00 49 42 4D
                *                                               | ^^^^^ [Length: 0x0005 = 5]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * b) Example: <P> = 5, value = NULL
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence: FF                             | Byte sequence:
                *                ^^ [0xFF: value = NULL]        | 02 00
                *                                               | ^^^^^ [Length: 0x0002 = 2]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARCHAR(%u) FOR SBCS DATA | ", pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

                if (pInputField[0] == 0x00) {
                    inputFieldOffset += DB2Z_NULL_HEADER_LENGTH;
                    pInputField += DB2Z_NULL_HEADER_LENGTH;

                    goto VARCHAR_SBCS_CONVERSION;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | NULL\n", inputFieldOffset);
#endif

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t);
                // update pointers
                REG_pField += sizeof(vlen_t);
                pVarying = REG_pField;

                NzSetBit(recordBuffer.bytes, pCurrentFPC->nullIndex);

                goto NEXT_FIELD;
            }
// ======== VARCHAR FOR SBCS DATA NOT NULL =====================================
            case DB2_VARCHAR_SBCS: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARCHAR(<P>) FOR SBCS DATA NOT NULL           | VARCHAR(<P>) NOT NULL
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : [1 ... <P>]                 | Number of bytes : 2 + [1 ... <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion: n/a
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 5, value = "IBM" | CCSID: 367 (ASCII)
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence: 49 42 4D                       | Byte sequence:
                *                                               | 05 00 49 42 4D
                *                                               | ^^^^^ [Length: 0x0005 = 5]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARCHAR(%u) FOR SBCS DATA NOT NULL | ", pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

            VARCHAR_SBCS_CONVERSION:
                uint16 inputFieldLength;

                // compute length of string in input stream
                // last column?
                if (pCurrentFPC->rrfLastVarColumn == true) {
                    // = record length - header length - inputFieldOffset
                    inputFieldLength = recordLength - DB2Z_HEADER_LENGTH - inputFieldOffset;
                } else {
                    // = offset of next zone - offset (= inputFieldOffset) of this zone
                    inputFieldLength = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                            + pCurrentFPC->rrfOffset + DB2Z_OFFSET_LENGTH))) - inputFieldOffset;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | LENGTH = %u\n", inputFieldOffset, inputFieldLength);
#endif

                // (string length > precision) || (string exceeds input stream)?
                if ((inputFieldLength > pCurrentFPC->extra.precAndScale) || (pInputField + inputFieldLength > REG_pEnd)) {
                    badField.clear();
                    badField.seekp(0);
                    badField << inputFieldLength;

                    pBadField = pInputField;

                    goto DB2Z_INVALID_FIELD_LENGTH_INFERRED;
                }

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t) + inputFieldLength;
                // copy string
                memcpy(REG_pField + sizeof(vlen_t), pInputField, inputFieldLength);
                // update pointers
                REG_pField += sizeof(vlen_t) + inputFieldLength;
                pVarying = REG_pField;

                goto NEXT_FIELD;
            }
// ======== VARGRAPHIC =========================================================
            case DB2N_VARGRAPHIC: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARGRAPHIC(<P>)                               | NVARCHAR(<P>)
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : 1 + 2 * [1 ... <P>]         | Number of bytes : 2 + [<P> ... 4 * <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion:
                * - ICU conversion
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 5, value = U+0041 + U+00AE + U+20AC + U+1D11E | CCSID: 1200 (UTF-16)
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence:                                | Byte sequence:
                * 00 00 41 00 AE 20 AC D8 34 DD 1E              | 0C 00 41 C2 AE E2 82 AC F0 9D 84 9E
                * ^^ [0x00: value != NULL]                      | ^^^^^ [Length: 0x000C = 12]
                *                                               |
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * b) Example: <P> = 5, value = NULL
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence: FF                             | Byte sequence:
                *                ^^ [0xFF: value = NULL]        | 02 00
                *                                               | ^^^^^ [Length: 0x0002 = 2]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARGRAPHIC(%u) | ", pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

                if (pInputField[0] == 0x00) {
                    inputFieldOffset += DB2Z_NULL_HEADER_LENGTH;
                    pInputField += DB2Z_NULL_HEADER_LENGTH;

                    goto VARGRAPHIC_CONVERSION;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | NULL\n", inputFieldOffset);
#endif

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t);
                // update pointers
                REG_pField += sizeof(vlen_t);
                pVarying = REG_pField;

                NzSetBit(recordBuffer.bytes, pCurrentFPC->nullIndex);

                goto NEXT_FIELD;
            }
// ======== VARGRAPHIC NOT NULL ================================================
            case DB2_VARGRAPHIC: {
                /*
                * ****************************************************************************************
                * 1. Type Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS                                  | NPS
                * ==============================================+=========================================
                * VARGRAPHIC(<P>) NOT NULL                      | NVARCHAR(<P>) NOT NULL
                *
                * ****************************************************************************************
                * 2. Format Mapping
                * ****************************************************************************************
                *
                * DB2 for z/OS internal format (RRF)            | NPS internal format
                * ==============================================+=========================================
                * Length type     : varying-length              | Length type     : varying-length
                * Number of bytes : 2 * [1 ... <P>]             | Number of bytes : 2 + [<P> ... 4 * <P>]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * Conversion:
                * - ICU conversion
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                * a) Example: <P> = 5, value = U+0041 + U+00AE + U+20AC + U+1D11E | CCSID: 1200 (UTF-16)
                * ----------------------------------------------+-----------------------------------------
                * Byte sequence:                                | Byte sequence:
                * 00 41 00 AE 20 AC D8 34 DD 1E                 | 0C 00 41 C2 AE E2 82 AC F0 9D 84 9E
                *                                               | ^^^^^ [Length: 0x000C = 12]
                * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                */
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : VARGRAPHIC(%u) NOT NULL | ", pCurrentFPC->extra.precAndScale);
#endif

                // get string offset in input stream
                inputFieldOffset = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                        + pCurrentFPC->rrfOffset)));

                pInputField = *pRemainingText + DB2Z_HEADER_LENGTH + inputFieldOffset;

            VARGRAPHIC_CONVERSION:
                uint16 inputFieldLength;

                // compute length of string in input stream
                // last column?
                if (pCurrentFPC->rrfLastVarColumn == true) {
                    // = record length - header length - inputFieldOffset
                    inputFieldLength = recordLength - DB2Z_HEADER_LENGTH - inputFieldOffset;
                } else {
                    // = offset of next zone - offset (= inputFieldOffset) of this zone
                    inputFieldLength = bswap_16(*(reinterpret_cast<uint16 const*>(*pRemainingText + DB2Z_HEADER_LENGTH
                            + pCurrentFPC->rrfOffset + DB2Z_OFFSET_LENGTH))) - inputFieldOffset;
                }

#ifdef DB2Z_PARSER_TRACING
                printf("OFFSET = %u | LENGTH = %u\n", inputFieldOffset, inputFieldLength);
#endif

                // (string length > precision) || (string exceeds input stream)?
                if ((inputFieldLength > (pCurrentFPC->extra.precAndScale * 2)) || (pInputField + inputFieldLength > REG_pEnd)) {
                    badField.clear();
                    badField.seekp(0);
                    badField << inputFieldLength;

                    pBadField = pInputField;

                    goto DB2Z_INVALID_FIELD_LENGTH_INFERRED;
                }

                REG_pField = pVarying;

                // align length to even address
                if (reinterpret_cast<size_t>(REG_pField) & 1) {
                    *REG_pField++ = 0;
                }

                ICUConverter* converter = (*copyIn->m_converters)[pCurrentFPC->extra2.ccsid];

                NZ_VERIFY(converter != NULL);

                bool isTruncated;
                int32 outputLength;
                ptrdiff_t targetBufferLength = &recordBuffer.bytes[MAX_RECORD_SIZE - 1] - REG_pField + sizeof(vlen_t);

                if (pCurrentFPC->extra2.ccsid == ICUConverter::CCSID_UTF16) {
                    outputLength = converter->convert(pInputField, inputFieldLength, REG_pField + sizeof(vlen_t), targetBufferLength,
                            MAX_NVARCHAR_SIZE, isTruncated);
                } else {
                    outputLength = converter->convertReplaceIdeographicSpaces(pInputField, inputFieldLength, REG_pField + sizeof(vlen_t),
                            targetBufferLength, MAX_NVARCHAR_SIZE, isTruncated);
                }

                if (isTruncated == true) {
                    ++pCurrentFPC->truncations;
                }

                // set length
                *(reinterpret_cast<vlen_t*>(REG_pField)) = sizeof(vlen_t) + outputLength;
                // update pointers
                REG_pField += sizeof(vlen_t) + outputLength;
                pVarying = REG_pField;

                goto NEXT_FIELD;
            }
// ======== SKIPPED COLUMN (VARYING LENGTH) ====================================
            case DB2_VARSKIPPED: {
#ifdef DB2Z_PARSER_TRACING
                printf("Processing                : SKIPPED VARYING-LENGTH COLUMN\n");
#endif

                goto NEXT_FIELD;
            }

//******************************************************************************
