//******************************************************************************

#define MAX_GRAPHEME_COUNT 256
#define BLOB_BUFFER_EXTEND_INCREMENT    (1<<24)

// Quoting semantics
#if   QUOTED_VALUES == 0

#undef  NO_QUOTES
#define NO_QUOTES      1
#undef  ALLOW_QUOTES
#define ALLOW_QUOTES   0
#undef  REQUIRE_QUOTES
#define REQUIRE_QUOTES 0

#elif QUOTED_VALUES == 1

#undef  NO_QUOTES
#define NO_QUOTES      0
#undef  ALLOW_QUOTES
#define ALLOW_QUOTES   1
#undef  REQUIRE_QUOTES
#define REQUIRE_QUOTES 0

#else

#undef  NO_QUOTES
#define NO_QUOTES      0
#undef  ALLOW_QUOTES
#define ALLOW_QUOTES   0
#undef  REQUIRE_QUOTES
#define REQUIRE_QUOTES 1

#endif // Quoting semantics

#define MIN_INT1 (int8)     (~0x7f)
#define MIN_INT2 (int16)    (~0x7fff)

// <CR> handling
#if CR_IN_STRING

#undef  CR_IS_EOR
#define CR_IS_EOR 0
#undef  CH_IS_CR_EOR
#define CH_IS_CR_EOR 0

#else

#undef  CR_IS_EOR
#define CR_IS_EOR 1
#undef  CH_IS_CR_EOR
#define CH_IS_CR_EOR (ch == '\r')

#endif // <CR> handling

// executable equivalent of encodedToDecimal[]
// (saves a register in crucial location in PT_NUMERIC16)

#undef  ENC2DEC
#define ENC2DEC(VAR) \
do {\
    if ( VAR == -7 )\
        VAR += 7;\
    else if ( VAR >= 16 ) {\
        if ( VAR >= 32 ) {\
            if ( VAR >= 48 ) {\
                if ( VAR >= 64 ) {\
                    VAR -= 7;\
                    }\
                VAR -= 7;\
                }\
            VAR -= 7;\
            }\
        VAR -= 7;\
        }\
    }\
while ( FALSE )

union x87_packed_bcd {
    uint  words[3];
    uint8 bytes[10];
};


#undef  COPY_TO_MEMORY
#define COPY_TO_MEMORY(REGISTER,MEMORY) \
asm(   "movl	%1,%0"\
   : "=m" (MEMORY)\
   :  "r" (REGISTER)\
   )

#undef  CMOVE_MAX
#define CMOVE_MAX(NEW,MAX) \
asm(   "cmpl	%[new],%[max]\n"\
   "	cmoval	%[new],%[max]"\
   : [max] "+r" (MAX)\
   : [new]  "g" (NEW)\
   : "cc"\
   )


#undef  CMOVE_NEG
#define CMOVE_NEG(FLAGS,VAL) \
asm(   "testb	%[bit],%[flags]\n"\
    "	cmovnel	%[neg],%[val]"\
   : [val]    "+r" (VAL)\
   : [neg]     "g" (-VAL)\
   , [flags] "qQm" (FLAGS)\
   , [bit]     "i" (FPF_NEGATE)\
   : "cc"\
   )

#undef  CMOVE_NEG2
#define CMOVE_NEG2(FLAGS,VALHI,VALLO) \
do {\
    uint tmpHi, tmpLo;\
    asm(   "xor	%[tmpLo],%[tmpLo]\n"\
       "	xor	%[tmpHi],%[tmpHi]\n"\
       "	subl	%[valLo],%[tmpLo]\n"\
       "	sbbl	%[valHi],%[tmpHi]\n"\
       "	testb	%[bit],%[flags]\n"\
       "	cmovnel %[tmpLo],%[valLo]\n"\
       "	cmovnel %[tmpHi],%[valHi]"\
       : [valHi]  "+r" (VALHI)\
       , [valLo]  "+r" (VALLO)\
       , [tmpHi] "=&r" (tmpHi)\
       , [tmpLo] "=&r" (tmpLo)\
       : [flags] "qQm" (FLAGS)\
       , [bit]     "i" (FPF_NEGATE)\
       : "cc"\
       );\
    }\
while (FALSE)


// Treatment of <NUL>
#if IGNORE_ZERO
#undef  NEXT_CHAR
#define NEXT_CHAR(LIMIT) \
do {\
    ch = *pText;\
    if ( pText >= (LIMIT) ) goto END_OF_BUFFER;\
    pText++;\
    }\
while( ch == '\0' )
#undef  NUCL_RESCAN_NEXT_CHAR
#define NUCL_RESCAN_NEXT_CHAR \
do {\
    ch = *pText;\
    pText++;\
    }\
while( ch == '*' || ch == '\0' )
#else
#undef  NEXT_CHAR
#define NEXT_CHAR(LIMIT) \
do {\
    ch = *pText;\
    if ( pText >= (LIMIT) ) goto END_OF_BUFFER;\
    pText++;\
    }\
while( FALSE )
#undef  NUCL_RESCAN_NEXT_CHAR
#define NUCL_RESCAN_NEXT_CHAR \
do {\
    ch = *pText;\
    pText++;\
    }\
while( ch == '*' )
#endif // Treatment of <NUL>

#include "unicodeProperties.h"

#if DISABLE_NFC_CHECK
#define CHECK_UNICODE_PROPS { Props = PROPS_STABLE; }
#else
#define CHECK_UNICODE_PROPS \
                              {      if (codePoint > MAX_ASSIGNED_UNICODE_CODEPOINT) {\
                                        badUnicodeCodePoint = codePoint;\
                                        goto UNASSIGNED_UNICODE_CODEPOINT;\
                                    }\
                                    Props =  unicodeProperties[codePoint];\
                                    if (CSmapP->nchar && (Props == SROGT) ) Props = PROPS_STABLE; }
#endif

#undef  NEXT_NCHAR
#define NEXT_NCHAR(LIMIT,LENGTH_ACCUMULATOR,CONTEXT,TRAIL)\
                        do {\
                            enum {\
                                mYes, mNo, mSPrime\
                            } MaybeEncountered = mNo;\
                            uint8 CurWeight = NO_WEIGHT;\
                            bool firstPass = TRUE;\
                            unsigned int Utf32GraphemeStack[MAX_GRAPHEME_COUNT];\
                            unsigned int* pUtf32GraphemeEnd = Utf32GraphemeStack;\
                            pUtf8GraphemeStart = pText - 1;\
                            while (TRUE) {   /* Loop exits at the end of a grapheme after stable character found */\
                                unsigned int codePoint = ch;\
                                unsigned int charByteSize = 1;\
                                uint8 Props = PROPS_STABLE;\
                                pUtf8GraphemeEnd = pText - 1;\
                                /* The stopper need not be valid UTF8. Cannot use IF_STOPPER_GOTO */\
                                /* because it advances pText for a double quote. (SWS42635) Only need check for */\
                                /* the delimiter here. */\
                                if (ch == REG_valueDelim) goto STOPPER_##CONTEXT;\
                                if ((ch & 0x80) != 0)   /* Decode multiple UTF8 bytes into a codepoint */\
                                    {\
                                    unsigned int ContByteLo = 0x80;\
                                    unsigned int ContByteHi = 0xBF;\
                                    unsigned int nContBytes = 1;\
                                    charByteSize = 2;\
                                    codePoint &= 0x1F;\
                                    if (ch <= 0xC1)\
                                        goto UTF8_ILLEGAL_INITIAL_BYTE;\
                                    if (ch <= 0xDF)\
                                        goto DECODE_CONTINUATION_BYTES_##CONTEXT;\
                                    nContBytes = 2;\
                                    charByteSize = 3;\
                                    codePoint &= 0xF;\
                                    if (ch == 0xE0) {\
                                        ContByteLo = 0xA0;\
                                        goto DECODE_CONTINUATION_BYTES_##CONTEXT;\
                                    }\
                                    if (ch == 0xED) {\
                                        ContByteHi =  CSmapP->nchar ?  0xBF  : 0x9F ;\
                                        goto DECODE_CONTINUATION_BYTES_##CONTEXT;\
                                    }\
                                    if (ch <= 0xEF)\
                                        goto DECODE_CONTINUATION_BYTES_##CONTEXT;\
                                    if (CSmapP->nchar) goto CESU8_ILLEGAL_INITIAL_BYTE;\
                                    nContBytes = 3;\
                                    charByteSize = 4;\
                                    if (ch == 0xF0) {\
                                        ContByteLo = 0x90;\
                                        goto DECODE_CONTINUATION_BYTES_##CONTEXT;\
                                    }\
                                    if (ch < 0xF4)\
                                        goto DECODE_CONTINUATION_BYTES_##CONTEXT;\
                                    if (ch == 0xF4) {\
                                        ContByteHi = 0x8F;\
                                        goto DECODE_CONTINUATION_BYTES_##CONTEXT;\
                                    }\
                                    goto UTF8_ILLEGAL_INITIAL_BYTE;\
                                    do {\
                                        ContByteLo = 0x80;\
                                        ContByteHi = 0xBF;\
                                        DECODE_CONTINUATION_BYTES_##CONTEXT:\
                                        TRAIL ;/* SWS 41355 */\
                                        ch = *pText++;\
                                        codePoint <<= 6;\
                                        if (ch < ContByteLo || ch > ContByteHi)\
                                            goto UTF8_BAD_CONTINUATION_BYTE;\
                                        codePoint |= (ch & 0x3F);\
                                    }\
                                    while (--nContBytes);\
                                    CHECK_UNICODE_PROPS; \
                                } /* End of the non 7 bit ASCII clause*/\
                                STOPPER_##CONTEXT:\
                                if ((pUtf32GraphemeEnd - Utf32GraphemeStack) >= MAX_GRAPHEME_COUNT) {\
                                    goto EXCEED;\
                                }\
                                *pUtf32GraphemeEnd++ = codePoint;\
                                if ((Props & 0x80) == 0x80 ||\
                                    (firstPass) ||\
                                    !declExtra) { /* Stable or Stable' or the first character in the grapheme or filled the record */\
                                    /* If we are at stable character, but have encountered a previous Maybe then */\
                                    /* let the ICU routine decide. If here on firstPass then MaybeEncountered will be No */\
                                    if ((MaybeEncountered == mYes) &&\
                                        !is_NFC(Utf32GraphemeStack, pUtf32GraphemeEnd - 1)) {\
                                        goto BAD_SEQUENCE;\
                                    }\
                                    if (firstPass) {    /* Continue if this is the first stable char in the grapheme */\
                                        firstPass = FALSE;\
                                    } else {\
                                    /* This stable character is processed twice. Once to mark the end of the previous grapheme */\
                                    /* and then at the start of the next. At some point we may want to eliminate this if it demonstrates */\
                                    /* a real performance loss. No need to check for the stopper or record size here because we decrease pText*/\
                                        IF_STOPPER_GOTO(REG_valueDelim,LIMIT,G_FINISH_##CONTEXT);\
                                        if (!declExtra) goto FINISH_##CONTEXT;\
                                        pText=pUtf8GraphemeEnd;/* Ended the grapheme and it is NFC */\ 
                                        NEXT_CHAR(LIMIT);\
                                        break;/* out of the while(true) loop */\
                                    }\
                                    if ((Props & 0xC0) == 0xC0) {  /* Stable' */\
                                        MaybeEncountered = mSPrime; /* Never count any Maybe in this grapheme */\
                                    }\
                                }/* End of Stable Props (Grapheme end) or first pass */\
                                if ((MaybeEncountered != mSPrime) && \
                                    (Props & 0xC0) == 0x40) {\
                                    MaybeEncountered = mYes;\
                                }\
                                Props &= 0x3F;\
                                if ((Props < CurWeight) && Props !=BASE_WEIGHT) {\
                                    badUnicodeCodePoint = codePoint;\
                                    goto DIAGNOSE_BAD_WEIGHT;\
                                }\
                                CurWeight = Props;\
                                LENGTH_ACCUMULATOR+=charByteSize;\
                                NEXT_CHAR(LIMIT);\
                                declExtra--; /* Exit on zero after seeing if the final grapheme is NFC' */\
                            }/*End of While(TRUE) break out of here at the end of a grapheme*/\
                            /*LENGTH_ACCUMULATOR += pUtf8GraphemeEnd - pUtf8GraphemeStart;*/\
                            pUtf8GraphemeStart = pUtf8GraphemeEnd;\
                        } while (FALSE); /*In a do  while false structure just for the macro semantics*/

#undef  IF_RESCAN_END_GOTO
#define IF_RESCAN_END_GOTO(RESCAN_LIMIT,LABEL) \
do {\
    if (pText == RESCAN_LIMIT) goto LABEL;\
    }\
while ( FALSE )

#undef  CHECK_INF_CH
#define CHECK_INF_CH(LITERAL1, LITERAL2, FAILURE_TARGET) \
do {\
    if (ch != LITERAL1 && ch != LITERAL2) goto FAILURE_TARGET;\
    NEXT_CHAR(pEnd);\
}\
while ( FALSE )

#undef  SKIP_SPACES
#define SKIP_SPACES \
do {\
    if ( ch == ' ' ) {\
        do {\
            NEXT_CHAR(pEnd);\
        }\
        while ( ch == ' ' );\
    }\
}\
while ( FALSE )


#undef  COND_CH
#define COND_CH(LITERAL, FAILURE_TARGET) \
do {\
    if (ch != LITERAL) goto FAILURE_TARGET;\
    NEXT_CHAR(pEnd);\
}\
while ( FALSE )


#undef  SKIP_SPACES
#define SKIP_SPACES \
do {\
    if ( ch == ' ' ) {\
        do {\
            NEXT_CHAR(pEnd);\
        }\
        while ( ch == ' ' );\
    }\
}\
while ( FALSE )


#undef  SKIP_SPACES_ELSE
#define SKIP_SPACES_ELSE(NO_SPACES_TARGET) \
do {\
    if ( ch != ' ' )\
        goto NO_SPACES_TARGET;\
    do {\
        NEXT_CHAR(pEnd);\
       }\
    while ( ch == ' ' );\
   }\
while(FALSE)

#undef SKIP_SPACES_ELSE_TIMESTAMP
#define SKIP_SPACES_ELSE_TIMESTAMP(NO_SPACES_TARGET) \
do {\
    if ( ch == dateTimeDelim ){\
        if ( ch != ' ' ){\
            NEXT_CHAR(pEnd);\
            goto RETURN_TO_SPACES_TIME;\
        }\
        else{\
	    do {\
		NEXT_CHAR(pEnd);\
	    }\
	    while ( ch == ' ' );\
	    goto RETURN_TO_SPACES_TIME;\
	}\
    }\
    else\
	goto NO_SPACES_TARGET;\
}\
while(FALSE)

#undef  REQUIRE
#define REQUIRE(CH,DELIM,ERROR,END) \
do {\
    if ( CH != (DELIM) ) goto ERROR;\
    NEXT_CHAR(END);\
    }\
while (FALSE)

// Simplest test involvs no Quote Doubling
#undef  IF_STOPPER_GOTO_NO_QD
#define IF_STOPPER_GOTO_NO_QD(CH,DELIM,LABEL)\
do {\
    if ( CH == DELIM ) goto LABEL;\
    if ( CH < ' ' ) {\
        uint tmp = 0;\
        uint scc = stopperCtrlChars;\
        asm (\
               "bt	%2,%1\n"\
            "	rcll	$1,%0"\
            : "+r" (tmp)\
            :  "m" (scc), "r" (CH)\
            : "cc"\
            );\
        if ( tmp ) goto LABEL;\
        }\
    }\
while (FALSE)


#if !QUOTED_VALUES

   // If no quote processing then no quote doubling either
#  undef  IF_STOPPER_GOTO
#  define IF_STOPPER_GOTO(DELIM,LIMIT,LABEL) IF_STOPPER_GOTO_NO_QD(ch,DELIM,LABEL)

#else

   // Macro to guard doubled quotes check
#  if REQUIRE_QUOTES
#    undef  IF_NOT_QUOTED_GOTO
#    define IF_NOT_QUOTED_GOTO(LABEL)
#  else
#    undef  IF_NOT_QUOTED_GOTO
#    define IF_NOT_QUOTED_GOTO(LABEL)\
        if ( ch != quoteChar ) goto LABEL
#  endif

   // Doubled quotes look-ahead:
   //   if 2nd quote is present then advance character position and break
   //   if no 2nd quote goto LABEL with character position unchanged
#  if IGNORE_ZERO
#    undef  IF_DOUBLED_QUOTE_BREAK_ELSE_GOTO
#    define IF_DOUBLED_QUOTE_BREAK_ELSE_GOTO(QUOTE_FLAVOR,LIMIT,LABEL)\
        pTextSave = pText;\
        NEXT_CHAR(LIMIT);\
        if ( ch == QUOTE_FLAVOR ) break;\
        pText = pTextSave;\
        ch = pText[-1];\
        goto LABEL
#  else
#    undef  IF_DOUBLED_QUOTE_BREAK_ELSE_GOTO
#    define IF_DOUBLED_QUOTE_BREAK_ELSE_GOTO(QUOTE_FLAVOR,LIMIT,LABEL)\
        if ( pText >= (LIMIT) ) goto END_OF_BUFFER;\
        if ( *pText != ch ) goto LABEL;\
        pText += 1;\
        break
#  endif


#   undef  IF_STOPPER_GOTO
#   define IF_STOPPER_GOTO(DELIM,LIMIT,LABEL)\
do {\
    if ( ch == DELIM ) {\
        IF_NOT_QUOTED_GOTO(LABEL);\
        IF_DOUBLED_QUOTE_BREAK_ELSE_GOTO(DELIM,LIMIT,LABEL);\
        }\
    if ( ch < ' ' ) {\
        uint tmp = 0;\
        uint scc = stopperCtrlChars;\
        asm (\
               "bt	%2,%1\n"\
            "	rcll	$1,%0"\
            : "+r" (tmp)\
            :  "m" (scc), "r" (ch)\
            : "cc"\
            );\
        if ( tmp ) goto LABEL;\
        }\
    }\
while (FALSE)

#endif

#if NULL_CASE_FOLD
#define FOLD_CASE_NULL(ch) ch |= 0x20202020
#else
#define FOLD_CASE_NULL(ch)
#endif


// In string context:
//  - leading spaces are significant
//  - terminator must be EndOfField or EndOfRecord
#if ALLOW_QUOTES

#undef  NULL_STRING_CHECK
#define NULL_STRING_CHECK \
do {\
    if ( pText != pStartOfField && valueDelim != quoteChar ) {\
        pText = const_cast<uint8*>(pStartOfField);\
        ch = ' ';\
        }\
    uint ch_0_3 = *((uint*)(((char*)pText)-1+0));\
    FOLD_CASE_NULL(ch_0_3);\
    ch_0_3 &= nullMask_0_3;\
    if ( ch_0_3 == nullText_0_3 ) {\
        uint nxtCh = pText[nullAdvance];\
        IF_STOPPER_GOTO_NO_QD(nxtCh,valueDelim,SKIP_NULL_TOKEN_SET_NULL_BIT);\
        }\
    }\
while (FALSE)

#else

#undef  NULL_STRING_CHECK
#define NULL_STRING_CHECK \
do {\
    uint ch_0_3 = *((uint*)(((char*)pText)-1+0));\
    FOLD_CASE_NULL(ch_0_3);\
    ch_0_3 &= nullMask_0_3;\
    if ( ch_0_3 == nullText_0_3 ) {\
        uint nxtCh = pText[nullAdvance];\
        IF_STOPPER_GOTO_NO_QD(nxtCh,valueDelim,SKIP_NULL_TOKEN_SET_NULL_BIT);\
        }\
    }\
while (FALSE)

#endif

// In token context:
//  - both leading and trailing space are ignored
#undef  NULL_TOKEN_CHECK
#define NULL_TOKEN_CHECK \
do {\
    if ( ch == ' ' ) {\
        do {\
            NEXT_CHAR(REG_pEnd);\
            }\
        while ( ch == ' ' );\
        }\
    uint ch_0_3 = *((uint*)(((char*)pText)-1+0));\
    FOLD_CASE_NULL(ch_0_3);\
    ch_0_3 &= nullMask_0_3;\
    if ( ch_0_3 == nullText_0_3 ) {\
        uint nxtChLower = pText[nullAdvance] | 0x20;\
        if ( nxtChLower < 'a' || nxtChLower > 'z' ) goto SKIP_NULL_TOKEN_SET_NULL_BIT;\
        }\
    }\
while(FALSE)

#undef  LOWER_ALPHA_FIRST
#define LOWER_ALPHA_FIRST(WHERE, ERROR) \
WHERE = ch | 0x20;\
if ( WHERE < 'a' || WHERE > 'z' ) goto ERROR

#undef  LOWER_ALPHA_NEXT
#define LOWER_ALPHA_NEXT(WHERE, ERROR) \
do {\
    uint chLower = ch | 0x20;\
    if ( chLower < 'a' || chLower > 'z' ) goto ERROR;\
    WHERE <<= 8;\
    WHERE += chLower;\
    }\
while (FALSE)

// Perform month match using CMOVE (no change in control flow)
#undef  MATCH_MONTH
#define MATCH_MONTH(m1,m2,m3) \
asm (  "addl	$1,%1\n" \
	"	cmpl	%3,%2\n" \
	"	cmovzl	%1,%0" \
    	: "+r" (monthReg)\
    	, "+r" (monthCounter)\
    	:  "r" (ch3)\
    	,  "i" ((((m1<<8)+m2)<<8)+m3)\
    	: "cc"\
    )


// Match three-letter month leaving result in 'month'
#undef  MATCH_MMM_MONTH
#define MATCH_MMM_MONTH \
do {\
	uint ch3;\
	LOWER_ALPHA_FIRST(ch3, EMPTY_DATE);\
	NEXT_CHAR(pEnd);\
	LOWER_ALPHA_NEXT(ch3, UNKNOWN_MONTH);\
	NEXT_CHAR(pEnd);\
	LOWER_ALPHA_NEXT(ch3, UNKNOWN_MONTH);\
	uint monthCounter = 0;\
	uint monthReg = 0;\
	MATCH_MONTH('j','a','n');\
	MATCH_MONTH('f','e','b');\
	MATCH_MONTH('m','a','r');\
	MATCH_MONTH('a','p','r');\
	MATCH_MONTH('m','a','y');\
	MATCH_MONTH('j','u','n');\
	MATCH_MONTH('j','u','l');\
	MATCH_MONTH('a','u','g');\
	MATCH_MONTH('s','e','p');\
	MATCH_MONTH('o','c','t');\
	MATCH_MONTH('n','o','v');\
	MATCH_MONTH('d','e','c');\
	month = monthReg;\
	if ( monthReg == 0 ) goto UNKNOWN_MONTH;\
	NEXT_CHAR(pEnd);\
    }\
while (FALSE)


// COLLECT_SMALL_INT leaves ch biased by -'0'
#undef  COLLECT_SMALL_INT
#define COLLECT_SMALL_INT(VAL, LIMIT, OVERFLOW) \
do {\
    ch -= '0';\
    if (ch <= 9) {\
        do {\
            VAL *= 10;\
            VAL += ch;\
            if (VAL > LIMIT) goto OVERFLOW;\
            NEXT_CHAR(pEnd);\
            ch -= '0';\
        }\
        while (ch <= 9);\
    }\
}\
while (FALSE)

// COLLECT_UNSIGNED_INT leaves ch biased by -'0'
#undef  COLLECT_UNSIGNED_INT
#define COLLECT_UNSIGNED_INT(VAL, OVERFLOW) \
do {\
    ch -= '0';\
    if (ch <= 9) {\
        uint ovf = 0;\
        uint hi;\
        do {\
            asm (  "movl	$10,%[hi]\n"\
                "	mull	%[hi]\n"\
                "	addl	%[ch],%[val]\n"\
                "	adcl	%[hi],%[ovf]"\
                : [val] "+a" (VAL)\
                , [hi] "=&d" (hi)\
                , [ch]  "+g" (ch)\
                , [ovf] "+g" (ovf)\
                :\
                : "cc"\
                );\
            NEXT_CHAR(pEnd);\
            ch -= '0';\
        }\
        while (ch <= 9);\
        if (ovf) goto OVERFLOW;\
    }\
}\
while (FALSE)

// COLLECT_SIGNED_INT leaves ch biased by -'0'
#undef  COLLECT_SIGNED_INT
#define COLLECT_SIGNED_INT(VAL, OVERFLOW, SIGN_BIT_SET) \
do {\
    ch -= '0';\
    if (ch <= 9) {\
        uint ovf = 0;\
        uint hi; \
        do {\
            asm (  "movl	$10,%[hi]\n"\
                "	mull	%[hi]\n"\
                "	addl	%[ch],%[val]\n"\
                "	adcl	%[hi],%[ovf]"\
                : [val] "+a" (VAL)\
                , [hi] "=&d" (hi)\
                , [ch]	"+g" (ch)\
                , [ovf] "+g" (ovf)\
                :\
                : "cc"\
                );\
            NEXT_CHAR(pEnd);\
            ch -= '0';\
        }\
        while (ch <= 9);\
        if (ovf) goto OVERFLOW;\
        if ( ((int) VAL) < 0 ) goto SIGN_BIT_SET;\
    }\
}\
while (FALSE)

// COLLECT_SMALL_INT8 leaves ch biased by -'0'
#undef  COLLECT_SMALL_INT8
#define COLLECT_SMALL_INT8(VHI, VLO, LIMIT, OVERFLOW) \
do {\
    ch -= '0';\
    if (ch <= 9) {\
        uint64 v64 = VLO;\
        do {\
            v64 *= 10;\
            v64 += ch;\
            if (v64 > (uint64)LIMIT) goto OVERFLOW;\
            NEXT_CHAR(pEnd);\
            ch -= '0';\
        }\
        while (ch <= 9);\
        VLO = (unsigned)v64;\
        VHI = (unsigned)(v64 >> 32);\
    }\
}\
while (FALSE)

// COLLECT_SIGNED_INT8 leaves ch biased by -'0'
#undef  COLLECT_SIGNED_INT8
#define COLLECT_SIGNED_INT8(VHI, VLO, OVERFLOW, SIGN_BIT_SET) \
do {\
    ch -= '0';\
    if (ch <= 9) {\
        uint ovf = 0;\
        uint hi;\
        do {\
            asm (  "imull	$10,%[vhi],%[hi]\n"\
                "	movl	%[hi],%[vhi]\n"\
                "	adcl	$0,%[ovf]\n"\
                "	movl	$10,%[hi]\n"\
                "	mull	%[hi]\n"\
                "	addl	%[ch],%[vlo]\n"\
                "	adcl	%[hi],%[vhi]\n"\
                "	adcl	$0,%[ovf]"\
                : [vhi] "+g" (VHI)\
                , [vlo] "+a" (VLO)\
                , [hi] "=&d" (hi)\
                , [ch]  "+g" (ch)\
                , [ovf] "+g" (ovf)\
                :\
                : "cc"\
                );\
            NEXT_CHAR(pEnd);\
            ch -= '0';\
        }\
    while (ch <= 9);\
    if (ovf) goto OVERFLOW;\
    if ( ((int) VHI) < 0 ) goto SIGN_BIT_SET;\
    }\
}\
while (FALSE)


#if 0
#undef  REG
#define REG(name) asm(#name)
#else
#undef  REG
#define REG(name)
#endif

#undef  ENCACHE_REG
#define ENCACHE_REG(var) REG_##var = var

#ifdef INSPECT_GENERATE_CODE
static int begin_case;
#undef  BEGIN_CASE
#define BEGIN_CASE(ID) begin_case = ID
#else
#undef  BEGIN_CASE
#define BEGIN_CASE(ID)
#endif // INSPECT_GENERATE_CODE

using ldr::Parser;

//******************************************************************************
