//------------------------------------------------------------------------------
// ldrStrCodes.h
//
// loader string code definitions. Actual strings are defined in an
// externally loaded file (ldrStrings.<locale dir>.txt).
//
// (C) Copyright IBM Corp. 2001, 2011  All Rights Reserved.
//
// Changes:
//
// 09-13-02  tho        added code if binary data is found in record.
// 02-28-02  tho        use -quotedValue instead of -ignoreQuotes.
// 01-09-02  tho        Added string code for NULL value length msg.
// 10-28-01  dick       Renamed severity, strCode enums to be global symbols.
// 10-16-01  dick       Initial shell.

#ifndef LDRSTRCODES_INCLUDED
#define LDRSTRCODES_INCLUDED

// String codes are global enums (rather than class scoped) for convenience -
// because they are heavily used.

enum LdrStrCode {                       // Ranges are defined in rcStrCodeDefs.h
    LDR_SC_DELIM_OUT_OF_RANGE        = 1000,
    LDR_SC_NEED_TABLE_NAME           = 1001,
    LDR_SC_NEED_DATAFILE_OR_CTRLFILE = 1002,
    LDR_SC_EXCEED_NULL_VAL_LENGTH    = 1003,
    LDR_SC_STATS_CTRL_C_ABORT        = 1004,
    LDR_SC_SESSIONS_DISABLED         = 1005,
    LDR_SC_CONTROL_LOAD_INFO_EMPTY   = 1006,
    LDR_SC_FILE_NOT_FOUND            = 1007,
    LDR_SC_CONFLICT_DATAFILE_AND_CTRLFILE = 1008,
    LDR_SC_INTERVAL_NOT_SUPPORTED    = 1009,
    LDR_SC_ILLEGAL_DELIM             = 1010,
    LDR_SC_UPDATEROWCOUNT_ERR        = 1011,
    LDR_SC_NO_BLOB_SUPPORT_IN_NZLOAD = 1012,
    LDR_SC_REQUIREQUOTES_WITHOUT_QUOTEDVALUE = 1013,
    LDR_SC_UNKNOWN_OPTION_VALUE      = 1014,
    LDR_SC_DATAFIELD_EXCEEDED_COLUMN_COUNT = 1015,
    LDR_SC_SESSION_ERROR_COUNT       = 1016,
    LDR_SC_SESSION_SUCCESS           = 1017,

    LDR_SC_GEN_STATS_SQL_CMD         = 1018,
    LDR_SC_SESSION_STATUS_COUNT      = 1019,
    LDR_SC_ERR_THINTABLE_EXIST       = 1020,
    LDR_SC_ERR_THINTABLELIST         = 1021,
    LDR_SC_PARSE_ERR                 = 1022,
    LDR_SC_CTL_MISSING_VAL           = 1023,
    LDR_SC_MAX_RECORD_LENGTH         = 1024,
    LDR_SC_UPDATE_STATS_STATUS       = 1025,
    LDR_SC_MAX_ESCAPE_CHAR_LENGTH    = 1026,
    LDR_SC_LOAD_FAILED               = 1027,               // general load failed status
    LDR_SC_CONVERSION_FROM_LATIN9_NOT_SUPPORTED = 1028,
    LDR_SC_CONVERSION_FROM_UTF8_NOT_SUPPORTED = 1029,
    LDR_SC_USER_CANCEL               = 1030,
    LDR_SC_MANUAL_GEN_STATS          = 1031,
    LDR_SC_LOG_AND_BAD_MUST_BE_DIFFERENT = 1032,

// parse - datatype errors:
    LDR_SC_NOT_NFC                   = 1033,
    LDR_SC_EXCEED_COMBINING          = 1034,
    LDR_SC_UNSUPPORTED_PRECOMPOSED_CODEPOINT = 1035,
    LDR_SC_UTF8_ILLEGAL_INITIAL_BYTE = 1036,
    LDR_SC_UNASSIGNED_UNICODE_CODEPOINT = 1037,
    LDR_SC_UNICODE_BOM               = 1038,
    LDR_SC_SURROGATE_HALF_ONLY_IN_UTF16 = 1039,
    LDR_SC_UTF8_BAD_CONTINUATION_BYTE = 1040,
    LDR_SC_INVALID_CHARACTER         = 1041,
    LDR_SC_TRUNC_TIME_OUT_OF_RANGE   = 1042,
    LDR_SC_ILLEGAL_EMPTY_FIELD       = 1043,
    LDR_SC_INCOMPLETE_NO_FILL_RECORD = 1044,
    LDR_SC_CANNOT_FILL_FIELD         = 1045,
    LDR_SC_NULL_TOKEN_BUT_NOT_NULLABLE = 1046,
    LDR_SC_INT1_OUT_OF_RANGE         = 1047,
    LDR_SC_INT2_OUT_OF_RANGE         = 1048,
    LDR_SC_INT4_OUT_OF_RANGE         = 1049,
    LDR_SC_INT8_OUT_OF_RANGE         = 1050,
    LDR_SC_EXPONENT_OUT_OF_RANGE     = 1051,
    LDR_SC_FP_FORMAT_OVERFLOW        = 1052,
    LDR_SC_FP_FORMAT_UNDERFLOW       = 1053,
    LDR_SC_TOO_MANY_DIGITS_AFTER_DP  = 1054,
    LDR_SC_TOO_MANY_DIGITS_BEFORE_DP = 1055,
    LDR_SC_TEXT_FIELD_TOO_LONG       = 1056,
    LDR_SC_UNIDENTIFIED_FIELD_VALUE  = 1057,
    LDR_SC_EXPECTED_TIMESTAMP_TIME   = 1058,
    LDR_SC_BAD_AM_PM                 = 1059,
    LDR_SC_EXPECTED_CLOSING_QUOTE    = 1060,
    LDR_SC_EXPECTED_DATE_DELIMITER   = 1061,
    LDR_SC_EXPECTED_DAY_DIGIT        = 1062,
    LDR_SC_EXPECTED_DIGIT            = 1063,
    LDR_SC_EXPECTED_DIGIT_OR_DECIMAL_POINT = 1064,
    LDR_SC_EXPECTED_FIELD_DELIMITER     = 1065,
    LDR_SC_EXPECTED_END_OF_RECORD    = 1066,
    LDR_SC_EXPECTED_HOUR_DIGIT       = 1067,
    LDR_SC_EXPECTED_MINUTE_DIGIT     = 1068,
    LDR_SC_EXPECTED_MONTH_DIGIT      = 1069,
    LDR_SC_EXPECTED_OPENING_QUOTE    = 1070,
    LDR_SC_EXPECTED_OPENING_QUOTE_OR_NULL_TOKEN = 1071,
    LDR_SC_EXPECTED_SECOND_DIGIT     = 1072,
    LDR_SC_EXPECTED_TIME_DELIMITER   = 1073,
    LDR_SC_EXPECTED_TZ_HOUR_DIGIT    = 1074,
    LDR_SC_EXPECTED_TZ_MINUTE_DIGIT  = 1075,
    LDR_SC_EXPECTED_TZ_SIGN          = 1076,
    LDR_SC_EXPECTED_YEAR_DIGIT       = 1077,
    LDR_SC_EXPECTED_COMMA_OR_DATE_DELIM = 1078,
    LDR_SC_EXPECTED_SPACE            = 1079,
    LDR_SC_MULTIPLE_DECIMAL_POINTS   = 1080,
    LDR_SC_ZERO_DAY                  = 1081,
    LDR_SC_DAY_TOO_LARGE_FOR_MONTH   = 1082,
    LDR_SC_MONTH_TOO_LARGE           = 1083,
    LDR_SC_ZERO_MONTH                = 1084,
    LDR_SC_UNKNOWN_MONTH             = 1085,
    LDR_SC_CESU8_ILLEGAL_INITIAL_BYTE = 1086,
    LDR_SC_HOURS24_TOO_LARGE         = 1087,
    LDR_SC_HOURS12_TOO_LARGE         = 1088,
    LDR_SC_MINUTES_TOO_LARGE         = 1089,
    LDR_SC_MICROSECONDS_TOO_MANY_DIGITS = 1090,
    LDR_SC_SECONDS_TOO_LARGE         = 1091,
    LDR_SC_TZ_TOO_LARGE              = 1092,
    LDR_SC_TZ_TOO_SMALL              = 1093,
    LDR_SC_TZ_MINUTES_TOO_LARGE      = 1094,
    LDR_SC_EXPECTED_DAY_DIGIT_OR_SPACE = 1095,

    LDR_SC_NULL_VAL_ALPHABET         = 1096,
    LDR_SC_MISSING_Y2BASE            = 1097,
    LDR_SC_BAD_Y2BASE                = 1098,
    LDR_SC_OPTION_DEPRECATED         = 1099,
    // NOTE: Dont add new values here;
    // it will overlap with other component errors
    // Use below range (2). For more, read rcStrCodeDefs.h

    //Loader range #2
    LDR_SC_NOT_FILE                  = 2500,
    LDR_SC_NOT_DIR                   = 2501,

    LDR_SC_ESCCHAR_NULLVAL_MATCH     = 2502,
    LDR_SC_DATAFILE_MISSING          = 2503,
    LDR_SC_NO_RELPATH_OPTION         = 2504,
    LDR_SC_NEW_FORMAT_OLD_BACKEND    = 2505,

    LDR_SC_FORMAT_OPTION_NA          = 2507,
    LDR_SC_FORMAT_OPTION_REQ         = 2508,
    LDR_SC_OPTION_COMBO_NA           = 2509,
    LDR_SC_OPTION_COMBO_REQ          = 2510,
    LDR_SC_MAX_DECIMAL_DELIM_LENGTH  = 2511,
    LDR_SC_NEW_DECIMAL_DELIM_OLD_BACKEND = 2512,

    LDR_SC_ZN_BAD_USE_TYPE           = 2520,
    LDR_SC_ZN_BAD_TYPE               = 2521,
    LDR_SC_ZN_BAD_STYLE              = 2522,
    LDR_SC_ZN_BAD_LENGTH             = 2523,
    LDR_SC_ZN_BAD_ALIGNMENT          = 2524,
    LDR_SC_ZN_BAD_MODULUS            = 2525,
    LDR_SC_ZN_MISSING_TYPE           = 2526,
    LDR_SC_ZN_MISSING_STYLE          = 2527,
    LDR_SC_ZN_MISSING_LENGTH         = 2528,
    LDR_SC_ZN_USETYPE_NULLIF_NS      = 2529,
    LDR_SC_ZN_USETYPE_AROUND_NS      = 2530,
    LDR_SC_ZN_USETYPE_LENREF_NS      = 2531,
    LDR_SC_ZN_USETYPE_VARLEN_NS      = 2532,
    LDR_SC_ZN_LEN_REF_ONLY_INT       = 2533,
    LDR_SC_ZN_MULTIPLE_REF_NS        = 2534,
    LDR_SC_ZN_NO_REF                 = 2535,
    LDR_SC_ZN_LEN_REF_ONLY_INTERNAL  = 2536,
    LDR_SC_ZN_LEN_REF_BAD_SELF       = 2537,
    LDR_SC_ZN_NULLIF_REF_SELF_INTERNAL_NS = 2538,
    LDR_SC_ZN_NULLIF_WITH_INDIC_NS   = 2539,
    LDR_SC_ZN_FIXED_BAD_STYLE        = 2540,
    LDR_SC_ZN_STYLE_TYPE_MISMATCH    = 2541,
    LDR_SC_ZN_FIXED_TYPE_VARLEN_NS   = 2542,
    LDR_SC_ZN_BINARY_ONLY_BYTES      = 2543,
    LDR_SC_ZN_BINARY_BAD_STYLE       = 2544,
    LDR_SC_ZN_BINARY_BAD_LENGTH      = 2545,
    LDR_SC_ZN_BINARY_BAD_ENDIAN      = 2546,
    LDR_SC_ZN_BINARY_BAD_AROUND      = 2547,
    LDR_SC_ZN_TEXT_TOO_BIG_LENGTH    = 2548,
    LDR_SC_ZN_TEXT_BAD_ENDIAN        = 2549,
    LDR_SC_ZN_TEXT_BAD_ALIGN         = 2550,
    LDR_SC_ZN_FIXED_TOO_SMALL_LENGTH = 2551,
    LDR_SC_ZN_FIXED_TOO_LARGE_LENGTH = 2552,
    LDR_SC_ZN_FIXED_BAD_ENDIAN       = 2553,
    LDR_SC_ZN_FIXED_BAD_ALIGN        = 2554,
    LDR_SC_ZN_NULLIF_INT_REQ         = 2555,
    LDR_SC_ZN_NULLIF_STR_REQ         = 2556,
    LDR_SC_ZN_NULLIF_ALWAYS_FALSE    = 2557,
    LDR_SC_ZN_REF_OUTOFBOUND         = 2558,
    LDR_SC_ZN_REF_FORWARD            = 2559,
    LDR_SC_ZN_REF_NON_REF            = 2560,
    LDR_SC_ZN_LEN_INT_OFB            = 2561,
    LDR_SC_ZN_REC_LEN_REF_OFB        = 2562,
    LDR_SC_ZN_REC_LEN_REF_NEGATIVE   = 2563,
    LDR_SC_ZN_REC_LEN_REF_NON_REF    = 2564,
    LDR_SC_ZN_REC_LEN_REF_BAD        = 2565,
    LDR_SC_ZN_REC_LEN_INT_OFB        = 2566,
    LDR_SC_ZN_LEN_REF_NO_OTHER_REF   = 2567,
    LDR_SC_ZN_NULL_REF_NO_OTHER_REF  = 2568,
    LDR_SC_ZN_LEN_REF_NULL_REF_ORDER = 2569,
    LDR_SC_ZN_NULL_INDIC_NO_REF_LEN  = 2570,
    LDR_SC_ZN_NULLIF_STR_TOO_LONG    = 2571,
    LDR_SC_ZN_NULLIF_COLUMN_NOT_NULL = 2572,
    LDR_SC_ZN_REC_LEN_REF_MULTI      = 2573,
    LDR_SC_ZN_DUPLICATE_NAME         = 2574,
    LDR_SC_ZN_REC_LEN_REF_TYPE_BAD   = 2575,
    LDR_SC_ZN_REC_LEN_REF_INT        = 2576,
    LDR_SC_ZN_NULLIF_REF_TYPE_NS     = 2577,
    LDR_SC_ZN_Y2BASE_MISSING         = 2578,
    LDR_SC_BAD_REC_LEN               = 2579,
    LDR_SC_ZN_BAD_VALUE              = 2580,
    LDR_SC_PARTIAL_RECORD            = 2581,
    LDR_SC_PARTIAL_ZN_BAD_REC_LEN    = 2582,
    LDR_SC_INVALID_CTRLCHARS         = 2583,
    LDR_SC_EXPECTED_IGNORE_ZERO      = 2584,
    LDR_SC_INVALID_OUTPUT_DIR        = 2585,
    LDR_SC_NULL_DELIM                = 2586,
    LDR_SC_HOURS25_TOO_LARGE         = 2587,
    LDR_SC_TIME_GREATER_THAN_240000  = 2588,
    LDR_SC_INTERVAL_EXPECTED_NUMBER  = 2589,
    LDR_SC_INTERVAL_OVERFLOW         = 2590,
    LDR_SC_INTERVAL_YEARS_RANGE      = 2591,
    LDR_SC_INTERVAL_MONTHS_RANGE     = 2592,
    LDR_SC_INTERVAL_SUM_YEAR_MONTH_OVERFLOW = 2593,
    LDR_SC_INTERVAL_PG_MAX_MONTHS    = 2594,
    LDR_SC_INTERVAL_DAYS_RANGE       = 2595,
    LDR_SC_INTERVAL_HOURS_RANGE      = 2596,
    LDR_SC_INTERVAL_TIME_OVERFLOW    = 2597,
    LDR_SC_INVALID_QUOTES            = 2598,
    LDR_SC_DATA_FILE_NOT_EXIST       = 2599,
    LDR_SC_INVALID_INFINITY          = 2600,

    //Loader range #3
    LDR_SC_DB2Z_CORRUPT_ROW          = 3200,
    LDR_SC_DB2Z_EXPECTED_BRF_NOT_RRF = 3201,
    LDR_SC_DB2Z_EXPECTED_RRF_NOT_BRF = 3202,
    LDR_SC_DB2Z_ICU_CREATION_FAILED  = 3203,
    LDR_SC_DB2Z_INVALID_BCD          = 3204,
    LDR_SC_DB2Z_INVALID_FIELD_LENGTH = 3205,
    LDR_SC_DB2Z_INVALID_FIELD_LENGTH_INFERRED = 3206,
    LDR_SC_DB2Z_INVALID_REAL_DOUBLE  = 3207,
    LDR_SC_DB2Z_MISSING_CCSID        = 3208,
    LDR_SC_DB2Z_UNSUPPORTED_REAL     = 3209,
    LDR_SC_DB2Z_ZERO_RECORD_LENGTH   = 3210,
    LDR_SC_DB2Z_ZERO_YEAR            = 3211,
    LDR_SC_DB2Z_ZN_LEN_REF           = 3212,
    LDR_SC_EXPECTED_CONDITION        = 3213,
    
    LDR_SC_LAST_CODE
  };

#endif /* LDRSTRCODES_INCLUDED */
