#ifndef LOCKTRACKER_INCLUDED
#define LOCKTRACKER_INCLUDED

#define LKTRKR_DECLARE_NAME(name) extern const char *const name
#define LKTRKR_DECLARE(name) extern const char *const name = #name

#if FOR_SPU
# define LKTRKR_SETWAITING(a, b)  LockTracker::setWaitingLock((a), (b))
# define LKTRKR_CLRWAITING(a, b)  LockTracker::clearWaitingLock((a), (b))
# define LKTRKR_ADDLOCK(a, b)     LockTracker::addLock((a), (b))
# define LKTRKR_REMOVELOCK(a, b)  LockTracker::removeLock((a), (b))
#else
# define LKTRKR_SETWAITING(a, b)  {}
# define LKTRKR_CLRWAITING(a, b)  {}
# define LKTRKR_ADDLOCK(a, b)     {}
# define LKTRKR_REMOVELOCK(a, b)  {}
#endif

//#define ENABLE_LOCK_TIMING 1

struct LockTracker
{
    enum
    {
        MAX_LOCKS = 128
    };

    bool        m_waiting;
    const char *m_waitingForType;
    int         m_waitingForId;

    unsigned int m_holdCount;
    bool        m_holding[MAX_LOCKS];
    const char *m_holdingType[MAX_LOCKS];
    int         m_holdingId[MAX_LOCKS];

#if ENABLE_LOCK_TIMING
    unsigned long long m_cur_wait_ts;
    unsigned long long m_wait_ts[MAX_LOCKS];
    unsigned long long m_acquire_ts[MAX_LOCKS];
#endif

    LockTracker();

    static void setWaitingLock(const char *type, int id);
    static void clearWaitingLock(const char *type, int id);
    static void addLock(const char *type, int id);
    static void removeLock(const char *type, int id);

    static bool holding (const char *type, int id);
    static bool holdingType(const char *type);
    static bool holdingAny();

    static void dump();
};

#endif
