#ifndef MARSHALL_INCLUDED
#define MARSHALL_INCLUDED

#include "nzstd.h"
#include "nztypes.h" // for record_t
#include "TXID.h"
#include "TXMsgs.h"

class CTable;
struct dbos_create_table_t;
struct dbos_drop_table_t;
struct dbos_trunc_table_t;
struct dbos_reclaim_extents_t;
struct dbos_resize_table_t;
struct TCreateTempTableMsg;
struct field_t;
struct TNumericData;
struct TimeTzADT;
struct Interval;
struct dbos_create_sequence_t;
struct dbos_create_sequences_t;
struct dbos_drop_sequence_t;
struct dbos_create_library_t;
struct dbos_msg_large_info_t;
struct dbos_drop_library_t;
struct SeqCachePayload;
struct dbos_open_close_channel_t;
struct TChannel;
struct dbos_info_mls_t;
struct TCreateThinTableMsg;

bool spuHasIntelArch();
bool spuIsLittleEndian();

#if __i386__ || __x86_64__
void NzCvtCheck(); // Verifies size of structs (for spu marshalling)
#define NzCvtEndian(x) (x)
#define NzCvtEndian16(x) (x)

inline static void NzCvtToHost(void *a) {}
inline static void NzCvtFromHost(void *a) {}
inline static void NzCvtFromHost(void *a, void *b) {}
inline static void NzCvtFromHost(void *a, void *b, void *c) {}

int16* NzIntelEndianSwap(
    int16* dst );
uint16* NzIntelEndianSwap(
    uint16* dst );
int32* NzIntelEndianSwap(
    int32* dst );
int64* NzIntelEndianSwap(
    int64* dst );
int64* NzIntelEndianSwap2(
    int64* dst );
TNumericData* NzIntelEndianSwap(
    TNumericData* dst );
Interval* NzIntelEndianSwap(
    Interval* dst );
TimeTzADT* NzIntelEndianSwap(
    TimeTzADT* dst );
TNumericData* NzIntelEndianSwapN8(
    TNumericData* dst );
void NzIntelEndianSwapSpecials(
    CTable *t, 
    record_t *rec, 
    bool diskblock);
void NzIntelEndianSwap2Args(
    TNumericDigit* dst,
    int digits );
void NzIntelEndianSwap2Args(
    CTable *t,
    record_t *rec,
    bool HOSTtoPPC,
    bool diskrec);

#elif __PPC__

struct CStartTxMsg;
struct CRestartTxMsg;
struct CCompleteTxMsg;
struct dbos_msg_header_t;
struct TInsertRowsMsg;
struct spu_msg_header_t;
struct TMirrorInitMsg;
struct TMemblockInfo;
struct TruncateWMMsg;

#define NzIntelEndianSwap(x)
#define NzIntelEndianSwap2(x)
#define NzIntelEndianSwapN8(x)
#define NzIntelEndianSwap2Args(w,x,y,z)

/*************************************************************************
function: NzCvtFromHost - convert int32
*/
inline static void NzCvtFromHost (register int *dst)
{
    register unsigned char *src = (unsigned char*)dst;

    *dst = (src[3] << 24) | (src[2] << 16) | (src[1] << 8) | src[0];
}

void NzCvtCheck(); // Verifies size of structs (for spu marshalling)

int NzCvtEndian( // convert int32
    int val);

uint16 NzCvtEndian16( // convert int16
     uint16 val);

void NzCvtFromHost( // convert int16
    short *dst);

void NzCvtFromHost( // convert uint16
    uint16 *dst);

void NzCvtFromHost( // convert int32
    int *p);

void NzCvtFromHost( // convert uint32
    uint32 *p);

void NzCvtFromHost( // convert numeric data
    TNumericDigit *dst, int digitCount);

void NzCvtFromHost( // convert boolean
    bool *p);

void NzCvtFromHost( // convert boolean
    nzbool *p);

void NzCvtFromHost( // convert int64
    int64 *p);

void NzCvtFromHost( // convert uint64
    uint64 *p);

void NzCvtFromHost( // convert 48-bit number (array of 3 uint16s)
    uint48 *);

void NzCvtFromHost(TXid *p);

void NzCvtFromHost( // convert Interval
    Interval *p);

void NzCvtFromHost( // convert TimeTzADT
    TimeTzADT *p);

void NzCvtFromHost( // convert list of table ids
    dbos_recover_since_msg_t* pMsg);

void NzCvtFromHost( // convert field type
    field_t *p);

void NzCvtFromHost( // convert the create-table message
    dbos_create_table_t *p);

void NzCvtToHost ( // convert the create-table message
    dbos_create_table_t *p);

void NzCvtFromHost( // convert the drop-table message
    dbos_drop_table_t *p);

void NzCvtToHost( // convert the drop-table message
    dbos_drop_table_t *p);

void NzCvtFromHost( // convert the truncate-table message
    dbos_trunc_table_t *p);

void NzCvtFromHost( // convert the reclaim-extents message
    dbos_reclaim_extents_t* p);

void NzCvtToHost (  // convert the reclaim-extents message
    dbos_reclaim_extents_t* p);

void NzCvtFromHost( // convert the create-sequence message
    dbos_create_sequence_t *p);

void NzCvtFromHost( // convert the packed create-sequences message
    dbos_create_sequences_t *p);

void NzCvtToHost( // convert the packed create-sequences message
    dbos_create_sequences_t *p);

void NzCvtFromHost( // convert the drop-sequence message
    dbos_drop_sequence_t *p);

void NzCvtFromHost( // convert the MLS name info message
    dbos_info_mls_t *p);

void NzCvtFromHost( // convert the large msg info
    dbos_msg_large_info_t *p);

void NzCvtFromHost( // convert the create-library message
    dbos_create_library_t *p);

void NzCvtFromHost( // convert the drop-library message
    dbos_drop_library_t *p);

void NzCvtFromHost (SeqCachePayload *p);

void NzCvtFromHost( // convert message header from dbos to spu (or vice versa)
    dbos_msg_header_t *hdr);

void NzCvtFromHost( // convert the open-fcomm-channel message
    dbos_open_close_channel_t *p);

void NzCvtFromHost( // convert Fcomm channel
    TChannel *p);

void NzCvtFromHost( // convert row insertion msg to spu form
    TInsertRowsMsg *p);

void NzCvtFromHost ( // convert fpgaRegs from dbos to spu
    struct fpgaRegs *p);

void NzCvtFromHost ( // convert fpgaSubstitutionBlock from dbos to spu
    struct fpgaSubstitutionBlock *p);

void NzCvtFromHost ( // convert fpgaDataStringRegister from dbos to spu
    struct fpgaDataStringRegister *p);

void NzCvtFromHost ( // convert fpgaAggrInfo from dbos to spu
    struct fpgaAggrInfo *p);

void NzCvtFromHost ( // convert fpgaField from dbos to spu
    struct fpgaField *p);

void NzCvtFromHost ( // convert fpgaPortablePacket from dbos to spu
    struct fpgaPortablePacket *p);

void NzCvtToHost( // convert messages header from spu to host
    spu_msg_header_t *hdr);

void NzCvtToHost( // convert table from spu to host
    CTable *t);

void NzCvtToHost( // convert a record from spu to host
    CTable *t, record_t *rec);

void NzCvtFromHost( // convert a recordstore block from host form to spu
    CTable *t, TMemblockInfo *block);

void NzCvtRecsToHost( // Convert records (but not header) to host format
    CTable *tbl, TMemblockInfo *m);

void NzCvtRecsFromHost( // Convert records (but not header) from host format
    CTable *tbl, TMemblockInfo *m);

void NzCvtFromHost( // convert a record from host to spu
    CTable *t, record_t *rec);

void NzCvtFromHost( // convert the download-table message
    TCreateTempTableMsg *p);


void NzCvtFromHost( // convert a start transaction message from host to spu
    CStartTxMsg *dst);
void NzCvtFromHost(struct CRestartTxMsg *dst);  //restart a transaction message host to spu
void NzCvtFromHost(struct partial_rollback_msg_t* p); // convert rollback of sub transaction

void NzCvtFromHost( TruncateWMMsg *dst );
void NzCvtFromHost( // convert a commit/rollback transaction message from host to spu
    CCompleteTxMsg *dst);



void NzCvtFromHost( // convert a mirror init message from host to spu
    TMirrorInitMsg *dst);

void NzCvtFromHost(TCreateThinTableMsg *p);

#else
#error "marshall.h: unsupported compilation target"
#endif // __i386 || __x86_64__ || __PPC__

#endif // !MARSHALL_INCLUDED
