#ifndef MEMALLOCCONTEXT_INCLUDE
#define MEMALLOCCONTEXT_INCLUDE
/// @file memAllocContext.h
/// @brief Allocation Context interface
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2011 All rights reserved.
//////////////////////////////////////////////////////////////////////
///
/// AllocContext - allocation/deallocation context
///
/// This is a convenience structure that packages up all of the
/// parameters for memory allocation.  That allows the parameters
/// to be generated statically and passed into the allocation routines.
////////////////////

#include "memTypeDefs.h"

namespace mem {

#include "nzpack_x86_on.h"
class AllocContext
{
private:
    memType_t           m_memType   : 16;
    regionName_t        m_region    : REGION_BITS;            // (3 for Debug and 4 for Turbo)
#if NZDEBUG
    bool                POISION     : 1;
#endif
    memScope_t          m_memScope  : 2;
    memAlign_t          m_ioAlign   : 1;
    unsigned            m_MBZ       : 9;

public:
    AllocContext(memType_t          memType      = mt_anonymous,
                 regionName_t       name         = REGION_GLOBAL,
                 memAlign_t         ioAlign      = MEM_NOALIGN,
                 memScope_t         memScope     = MEM_SCOPE_TEMPORARY) :
        m_memType(memType),
        m_region(name),
        m_memScope(memScope),
        m_ioAlign(ioAlign),
        m_MBZ(0)
    {};

    ////////////////////
    /// Construct an allocation context given the pointer to another block.
    /// The memtype, name, scope, and alignment are taken from the other block.
    ////////////////////
    AllocContext(void* p, memType_t mt = mt_unused);

    // -- Accessors --
    void setRegion(regionName_t region)
    { m_region = region; }

    regionName_t getRegion() const
    { return m_region; }

    memType_t getMemType() const
    { return m_memType;}

    memScope_t getMemScope() const
    { return m_memScope; }

    memAlign_t getIoAlign() const
    { return m_ioAlign; }

}; // AllocContext;
#include "nzpack_x86_off.h"

////////////////////
/// k_anonHeap - constant for anonymous heap allocations
///
/// To simplify converting the old nzmalloc interface to the heap allocContext
/// interface we supply this static allocation context.
////////////////////
const AllocContext k_anonHeap(mt_anonymous, mem::REGION_HEAP);

}; // namespace mem

#endif // MEMALLOCCONTEXT_INCLUDE
