#ifndef MEMNUMAPOLICY_INCLUDE
#define MEMNUMAPOLICY_INCLUDE
/// @file memNumaPolicy.h
/// @brief Enumeration of numa policies
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2012 All rights reserved.
//////////////////////////////////////////////////////////////////////
/// Numa policies are enforced in the lock_free_allocator.  We want a
/// common definition of the enumeration.  This is it.
///
/// It can't be in lock_free_allocator.h, since we don't want to expose
/// lock_free_allocator interface to components that don't need it. So we factor
/// it into its own file so it can be included only in the places that need it.
///
/// Note: to change the system default policy modify the DEFAULT_POLICY
/// enumeration.  That way, both nzcfgldr and lock_free_allocator choose
/// the same default value.
//////////////////////////////////////////////////////////////////////

namespace mem {
////////////////////
/// numaPolicy_t - memory numa policy
////////////////////
typedef enum {
    MEM_ONE_HEAP = 0,       // all processes use one heap
    MEM_MULTI_HEAP_FIXED,   // spu job task applies numa policy; other processes use one heap
    MEM_MULTI_HEAP_RANDOM,   // spu job task applies numa policy; other processes get random heap

    MEM_DEFAULT_POLICY = MEM_ONE_HEAP
} numaPolicy_t;

}; // namespace mem

#endif // MEMNUMAPOLICY_INCLUDE
