#ifndef MEMSTRDUP_INCLUDE
#define MEMSTRDUP_INCLUDE
/// @file memStrdup.h
/// @brief Trivial methods to duplicate a C-string into a memory region
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2011 All rights reserved.
//////////////////////////////////////////////////////////////////////
///
/// Description:
///
//////////////////////////////////////////////////////////////////////

#include <string.h>
#include "memAllocContext.h"

namespace mem {

////////////////////
/// Strdup
///
/// Given a C-style NULL-terminated string, create a copy of it in a memory region.
///
/// @param src: the string to copy from
/// @param ctx: the memory context for allocating the dup string
/// @return pointer to the new string; NULL if src is NULL
////////////////////
char* Strdup(const char* src, mem::AllocContext ctx);

////////////////////
/// Strndup
/// Given a C-style NULL-terminated string, create a copy of it into a memory region.
/// @param src: the string to copy from. must be null-terminated.
/// @param maxSize: the maximum size the resulting string may be.
/// @param ctx: allocation context for creating the target string
///
/// Works as follows:
///     VALUE OF MAXSIZE           |  SIZE OF RESULT
///     ---------------------------+-------------------
///     maxSize==0                 |  strlen(source)+1
///     maxSize<(strlen(source)+1) |  maxSize
///     maxSize>(strlen(source)+1) |  strlen(source)+1
///
/// Examples:createShmCString
/// 1) shmStringCopy("abc", 3) ==> [ab\0]
/// 2) shmStringCopy("abc", 5) ==> [abc\0]
///
/// @param mt: the shared memory tag to use
/// @returns:   NULL, if source is NULL, else
///             pointer to R, allocated in the ctx region.
///             MUST BE freed when done.
////////////////////
char* Strndup(const char* src, size_t maxSize, mem::AllocContext ctx);

}; // namespace mem

#endif // MEMSTRDUP_INCLUDE
