#ifndef MEMTYPEDEFS_INCLUDE
#define MEMTYPEDEFS_INCLUDE
/// @file memTypeDefs.h
/// @brief Memory subsystem type definitions
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2011 All rights reserved.
//////////////////////////////////////////////////////////////////////
/// Define (or include) all of the enumerations and type definitions
/// used in the public memory interface.
//////////////////////////////////////////////////////////////////////
#include <stddef.h>             // for size_t
#include <stdint.h>             // for uint16_t
#include "memTypes.h"           // for memType_t
#include "memKnownObjects.h"    // for memKnownObject_t
#include "memTrackTypes.h"      // for memTrack_t
#include "memScope.h"           // for memScope_t
#include "memNumaPolicy.h"      // for numaPolicy_t

namespace mem {

////////////////////
/// regionName_t - Names of regions
////////////////////
typedef enum {
    REGION_HEAP = 0,            ///< heap region - NOTE: not in the valid min/max of real regions
                                ///< It is special-cased and does not use the region memory
                                ///< allocator.
    REGION_MIN,
    REGION_GLOBAL = REGION_MIN, ///< global shared memory
    REGION_PM,                  ///< partition manager shared memory - per data slice
    REGION_MAX,                 ///< must be last for purposes of array allocation and looping,
    REGION_UFILE = REGION_MAX,  ///< unconventional, rarely & dynamically created
    REGION_INSANE,              ///< sanity check limit for all region variants (eval to 4?)

    // This is used in memAllocContext and memHeader definitions which need to be in sync
#if NZDEBUG
    REGION_BITS = 3           ///< bits to allow for all region names - for block header
#else
    REGION_BITS = 4
#endif
} regionName_t;

////////////////////
/// regionType_t - The region type specifies the scope and usage of the region
////////////////////
typedef enum {
    REGION_TYPE_SHARED_SINGLE,  ///< single shared region in the same
                                ///< virtual address range between processes.

    REGION_TYPE_SHARED_OVERLAY  ///< multiple shared regions all using the
                                ///< same virtual address range between processes.
} regionType_t;

////////////////////
/// regionOverlay_t - simple typedef for consistent typing in the code
////////////////////
typedef uint16_t regionOverlay_t;
const regionOverlay_t NULL_OVERLAY = 0xFFFF;

typedef enum {
    MEM_NOALIGN = 0,
    MEM_ALIGN   = 1
} memAlign_t;

}; // namespace mem

#endif // MEMTYPEDEFS_INCLUDE
