#ifndef MESSAGES_INCLUDED
#define MESSAGES_INCLUDED

// (C) Copyright IBM Corp. 2009, 2014  All Rights Reserved.

#ifndef RCDBG_INCLUDED
# include "rcDbg.h"
#endif

#ifndef TXID_INCLUDED
# include "TXID.h"
#endif

#ifndef NZRECORD_INCLUDED
# include "nzrecord.h"
#endif

#ifndef NZLIMITS_INCLUDED
# include "nzlimits.h"
#endif

#ifndef NZHWDEFS_INCLUDED
# include "nzhwdefs.h"
#endif

#ifndef SDLIST_INCLUDED
# include "sdlist.h"
#endif

#ifndef NZGLOBALS_INCLUDED
# include "nzglobals.h"
#endif

#ifndef FCOMMCHANNEL_INCLUDED
# include "fcommchannel.h"
#endif

#ifndef ERROR_INCLUDED
# include "error.h"
#endif

#ifndef NZTYPES_INCLUDED
# include "nztypes.h"
#endif

class CTable;
struct TMemblockInfo;
class CSPUInteraction;

#define ALL_SPUS_LID (0xFFFF)
#define DBOS_NODE_LID (0x0)

#define CMD_RESTARTED_BIT (0x40000000UL)

/*
    Client <-> Dbos <-> SPU Message formats
*/

// from SPU to DBOS
enum msgToDBOS_t {
    SPU_MSG_RESPONSE=100,           // response to interaction object
    SPU_MSG_ERROR_RESPONSE=101,     // !NOTE-bmz needs better definition...
    SPU_MSG_NOAGGR_RESPONSE=102,    // response to interaction object over async2host channel that isn't aggregated into a single response
//  UNUSED=103,
    SPU_MSG_PERF_LOG=104,           // performance log
//  UNUSED=105,
//  UNUSED=106,
    SPU_MSG_RQST_ROWIDS=107,        // spu requesting more rowids
    SPU_MSG_RECORDS_RESPONSE=108,   // spu responding with recordsets
//  UNUSED=109,
//  UNUSED=110,
    SPU_MSG_PLAN_STATS=111,         // stats for final SPU_MSG_RECORDS_RESPONSE
    SPU_MSG_STATUS_DATA=112,        // user data status destimed for client
    SPU_MSG_RQST_SEQUENCE=113,      // spu requesting sequence cache load
    SPU_MSG_TX_LOG=114,             // record of blocks modified in a tx
    SPU_MSG_UTILIZATION = 115,      // spu reporting system utilization
    SPU_MSG_SNIP_RESACCT = 116,     // per-snippet resource accounting
    SPU_MSG_SPU2SPU_DIST_LAST_MSG_SENT=117, // spu2spu dist control msg to host

    SPU_MSG_MUST_BE_LAST
};

// from DBOS to SPU

//When adding new interactions, consider how responses and errors will be sent.
//If this is not a normal broadcast interaction which goes through message aggregation
//you may need to make changes to "usesAsyncChannel" and "usesResponseChannel" in
//spuinteraction.cpp

enum msgToSPU_t {
    SPU_FIRST_FROM_DBOS=200,
    SPU_PING=201,            // send a round trip 'hello'
    QA_WLM_RESET=202,
    NZ_GET_SYS_INSTANCE_ID=203,
    SPU_CREATE_TABLE=204,
    SPU_RECLAIM_EXTENTS=205,
    NZ_EXECUTE_PLAN=206,
//    NZ_TELL_MEMORY=207,       // unused - removed 2011-05-03
    NZ_SET_TRUNCATE_STATS=208,
    NZ_ABORT_ON_ERROR=209,
//    NZ_PERF_LOG_* removed Aug 2006
//    NZ_PERF_LOG_START=210,
//    NZ_PERF_LOG_STOP=211,
//    NZ_PERF_LOG_DUMP=212,
//    NZ_PERF_LOG_TOSS=213,
    SPU_START_TRANSACTION=214,
    SPU_ROLLBACK=215,
    SPU_PRECOMMIT=216,
    SPU_COMMIT=217,
    NZ_CREATE_JOB=218,        // start job
    NZ_RELEASE_JOB=219,       // end job
    NZ_UPDATE_JOB_PRIO=220,   // update job priority
    SPU_DROP_TABLE=221,
    SPU_QA_1_MSG=222,         // Used by QA for white-box testing of SPU-0
    SPU_QA_ALL_MSG=223,       // Used by QA for parallel testing of all SPUs
    UNUSED_224=224,           // unused
    UNUSED_225=225,
    UNUSED_226=226,
    UNUSED_227=227,           // unused
    SPU_CREATE_TEMP_TABLE=228,// create in-memory temp table
    SPU_ACCEPT_BCAST_ROWS=229,  // add rows to specified table
    SPU_ACCEPT_DIST_ROWS=230, // add distributed rows to specified table
    UNUSED_231=231,
    UNUSED_232=232,
    UNUSED_233=233,
    SPU_SET_DBG_CTRLS=234,    // Change the debug output controls
    UNUSED_235=235,
    NZ_NULL_WORK_EVENT=236,   // Dispatch queue clearing message (Dispatch to Dispatch)
    NZ_WORK_FROM_STATE=237,   // run work item state machine
    NZ_EVENT_PLAN_ABORTED=238,// plan aborted, notify event task
    NZ_CLIENT_DISCONNECT=239, // dbos client disconnect
    NZ_EVENT_ITP_ABORTED=240, // interaction aborted, notify event task
    SPU_OPEN_CHANNEL_SYNCH=241, // bcast telling spus to open FlowComm channel and wait for it to complete. Used by nzload only
    SPU_OPEN_CHANNEL=242,     // bcast telling spus to open FlowComm channel
    SPU_CLOSE_CHANNEL=243,    // bcast telling spus to close FlowComm channel
    NO_OP_INTERACTION=244,    // no-op
    QA_BROADCAST=245,         // simple broadcast test
    QA_DISTRIBUTE=246,        // simple multi-spu unicast test
    QA_RESPONSE=247,          // simple large spu response test
    UNUSED_248=248,
    UNUSED_249=249,
    DBOS_RESUBMIT_CLI_REQ=250,// re-activate deferred client request
    NZ_WORK_COMPLETE=251,     // Event Process is Done / Needs 'reschedule'
    NZ_PLAN_COMPLETE=252,     // allow gatekeeper to send next plan
    NZ_SNIPPET_DELAY=253,     // Event process is entering GRA Ceilings delay / idle
    SPU_TRUNCATE_TABLE=254,   // sql truncate table
    SPU_MIGRATE_EXTENTS=255,  // migrate extents from one table to another
    NZ_SESSION_ABORT_TX=256,  // abort transaction for given sessionId
    QA_STREAM_DIST=257,       // streaming multi-spu unicast test
    SPU_SRC_REQUEST=258,      // adds-to/drops-from snippet-result-caches on SPUs
    SPU_CLEAR_SRC=259,        // unused - clears all snippet-result caches on SPUs
    SPU_MORE_ROWIDS=260,      // host delivering range of rowids to use
    UNUSED_261=261,           // unused
    NZ_SET_CLIENT_PRIORITY=262,// set client priority level
    GET_DBG_CTRLS=263,        // request current ctrls setting
    SPU_DESCRIBE_TABLE=264,   // table info msg (used only by SMACK)
    NZ_CANCEL_QUERY_TX=265,   // Async transaction aborter
    SPU_SET_TRANS_READONLY=266,     // read only (or read write)
    NZ_SET_DB_REPLICATED=267,       // set database as replicated or not
    QA_SPU_DISTRIBUTE=268,    // large spu-spu distribute test
    UNUSED_269=269,           // unused
    UNUSED_270=270,           // unused
    UNUSED_271=271,           // unused
    SPU_MSG_TX_LOG_ACK=272,   // ack of SPU_MSG_TX_LOG
    UNUSED_273=273,           // unused
    SPU_ACCEPT_SPU_DIST_ROWS=274, // SPU-to-SPU distribution (don't ask why in 'DBOS')
    SPU_INFO_CTABLE=275,      // send CTable info to SPU
    SPU_LIMIT_REACHED=276,    // Limit reached. Shut Up.
    NZ_GET_STABLE_XID=277,    // return "stable" TXid (for statistics/reclaim)
    NZ_GET_REPL_STABLE_XID=278, // get "replication stable txid" on subordinate node
    SPU_RECOVER_TRANSACTIONS_SINCE=279, // on host restore, undo all later transactions
    NZ_SET_REPL_STABLE_XID=280, // set "replication stable txid" on subordinate node
    NZ_CAPTURE_HIGH_WATER_DEFUNCT=281,// capture high-water row-addrs on SPUs
    SPU_START_ABORT=282,          // first phase of "new" rollback
    SPU_INVALIDATE_ROWIDBATCH=283,// force SPUs to get new rowids
    NZ_FCOMM_QA=284,          // fcomm's cliqa tests
    NZ_BT_ON_ERROR=285,       // cliqa -bt
    SPU_INFO_SEQUENCE=286,    // send CNzSequence info to SPU
    SPU_PREDROP_SEQUENCE=287, // send predrop sequence msg to SPU
    SPU_DROP_SEQUENCE=288,    // send drop sequence msg to SPU
    SPU_SEQ_CACHE=289,        // send sequence cache load to SPU
    SPU_SYNC_DIST=290,        // all spus have crt'd table and opened channel
    SPU_SYNC_DIST_PROCEED=291,// tells spus to proceed with the distribute
    SPU_CLOSE_CHANNEL_HELPER=292, // helper message to complete channel close
    SPU_DOWNLOAD=293,         // interaction that is the owner of spu download channel
    SPU_RESTART_TRANSACTION=294, // restart a transaction that was aborted on the spu
    DBOS_RESTART_DONE=295,    // event->dispatch: done restarting jobs
    SPU_PARTIAL_COMMIT=296,   // tell the spus to write everything modified so far to disk
    SPU_PARTIAL_ROLLBACK=297, // rollback a sub-transaction
    DBOS_PAUSE_LOAD=298,      // tell dbos event to stop waking up the data reader during state change
    NZ_SET_RSG_DEFINITION=299,   // Change the group def (allocation percent, name) for a Resource Sharing Group
    NZ_EVENT_REQUEST_ROLLBACKDATA = 300, // Forward request to collect rollback data to a specific event process
    SPU_GET_EOTS=301,         // request from the host to the SPUs to return the current eot (eof ) for a list of tables
    NZ_GET_INVISILIST=302,    // Return invisibility list (of txids) given a reference txid.
                              // WHY IS THIS (and NZ_GET_STABLE_TXID) in msgToSpu_t?
    NZ_ALTER_TABLE=303,
    SPU_INFO_MLS=304,         // add, change, drop a MLS name/value in the mls table on the SPUs
    SPU_DEFERRED_ITCN=305,    // deferred interaction message

    SPU_INFO_LIBRARY=306,     // send CNzLibrary info to SPU
    SPU_DROP_LIBRARY=307,     // send drop library msg to SPU
    NZ_COPY_VISIBILITY=308,   // copy one transactions visibility to another txn
    SPU_DONETOREDO = 309,     // Convert a table's DONE extents to REDO (metadata)
    NZ_UDX_CLEANUP = 310,     // message to clean up UDX resources.
    SPU_SPU2SPU_DIST_LAST_MSG_SEND_DONE = 311, // all data slices done sending spu2spu dist last msg
    NZ_EVENT_PRECOMMIT_TX = 312, // forwarded from dispatch to event
    NZ_EVENT_COMMIT_TX = 313,    // forwarded from dispatch to event
    SPU_SET_TRANS_REPLICATED = 314, // Replication project
    SPU_SET_TRANS_SERIALIZABLE = 315, // Set transaction to serializable or not
    NZ_SEND_LIBRARIES = 316,  // Instructs DBOS Sys Event to send NzLibraries to the SPUs.
    NZ_FORCE_TX_RETRY = 317,  // Instructs HostTXMgr to retry failed abort or commit now.
    SPU_UNMASK_OOM = 318,     // disable the OOM mask
    NZ_ADD_SCHED_RULE = 319, // Creation of a WLM scheduler rule
    NZ_RENAME_SCHED_RULE = 320, // Renaming a WLM scheduler rule
    NZ_DROP_SCHED_RULE = 321, // Dropping a WLM scheduler rule
    NZ_ENABLE_SCHED_RULE = 322, // Enabling a WLM scheduler rule
    NZ_SET_USER_DEFINITION = 323,
    NZ_SET_DB_DEFINITION = 324,
    NZ_RESERVE_XID = 325,      // reserve a transaction ID without starting a transaction
    NOTIFY_REDR_START = 326,
    NOTIFY_REDR_START_TABLE = 327,
    NOTIFY_REDR_END = 328,
    REDR_TBLID_MAP_OP = 329,
    NOTIFY_REDR_TBL_DONE = 330,
    REDR_TX_ABORT_UPD = 331,
    SPU_SIZE_TXTABLE = 332,
    NZ_EVENT_CLEANUP_TXS = 333,     // Instructs DBOS SYS-EVENT to cleanup transactions.
    NZ_FORCE_EXTENT_RECLAIM = 334,  // Instructs HostTXMgr to reclaim truncated extents through a given txid.
    NZ_GET_LAST_XID = 335,          // Get last txid assigned by HostTXMgr.

    SPU_LAST_FROM_DBOS        // doesn't do anything, just the last type
};

// from SPU to SPU
enum msgFromSPUtoSPU_t {
    SPU_FIRST_TO_SPU=400,
//    SPU_MIRROR_DATA=401,      // called by SPU: send data from primary to mirror
//    SPU_MIRROR_BLOCK=402,     // called by SPU: send a whole block from one SPU to another (for regeneration)
//    SPU_MIRROR_COMMIT=403,    // unused
//    SPU_MIRROR_DATA_ACK=404,  // called by mirror, received on primary
//    SPU_MIRROR_TX_END=405,    // called by primary upon precommit or abort, mirror calls RecordMgr::PreCommit

    SPU_REGEN_BLOCK=406,       // used in regen to send absolute block within partition
    SPU_REGEN_TABLE_BLOCK=407, // used in regen to send absolute block within (core) table

    SPU_REGEN_WRITE_BLOCK=408,     // rf
    SPU_REGEN_TARGET_PROGRESS=409, // rf

//    SPU_MIRROR_CREATE_TABLE=410,
//    SPU_MIRROR_DROP_TABLE=411,
//    SPU_MIRROR_TRUNC_TABLE=412,

//    SPU_MIRROR_FLOWC_RQST=413,  // OBSOLETE with flowcomm
//    SPU_MIRROR_FLOWC_GRANT=414, // Receiver allows sender to send more

    SPU_DIST_FLOWC_RQST=415,    // SPU-SPU distribution sender requests permission to send more
    SPU_DIST_FLOWC_GRANT=416,   // Receiver allows sender to send more

    SPU_QA_DISTRIBUTE=417,      // used in "cliqa -spudistribute"
//    SPU_MIRROR_ZMAP_STATUS=418,

    UNUSED_419=419,

    SPU_REGEN_CONTROL=420,      // used in half-regen to signal (abnormal) end

    SMACK_TABLE_VALIDATE_SPAWN_MIRROR_THREAD=421, // Start mirror thread that blocks from some table
    SMACK_RQST_BLOCK_HEADERS=422,  // Get block headers for some table from mirror
    SMACK_REPLY_BLOCK_HEADERS=423, // Reply to SMACK_RQST_BLOCK_HEADERS

    SPU_MICRO_REGEN_RQST=424,   // Request for recovery of block from mirror
    SPU_MICRO_REGEN_REPLY=425,  // Reply to SPU_MICRO_REGEN_RQST

//    SPU_MIRROR_RESIZE_TABLE=426,// Reset a table's EOF and free any extents on mirror
    SPU_REGEN_BIAS_MAP=427,   // used in regen to send starting block biases
    NZ_VALIDATE_TRANSACTION=428, // msg from postgres to dbos dispatch to validate a transaction
    SPU_AGGR_RESPONSE=429,     // msg sent on aggregation channel to aggr leader
    SPU_AGGR_ERROR_RESPONSE=430, //error message sent on aggr channel to aggr leader
    SPU_AGGR_ERROR_FWD_RESPONSE=431, //error msg sent on aggr channel to mirror, for fwding
    SPU_NOTIFY_ABORT_DATARCV=432, // abort notification sent to datarcv from dispatch (fixme-xs1f4)
    SPU_AGGR_SPU2SPU_DIST_CTRL_RESP=433,  // spu2spu distribute host ctrl msg
    SPU_LAST_TO_SPU
};


/** 
 *  Extra payloads attached to a message
 *  Used as a bit mask to indicate multiple attached payloads
 *  If multiple payloads are attached, they are attached in the order of this enum.
 */
enum payload_t {
    PAYLOAD_NONE = 0,            // No extra payload in the message
    PAYLOAD_RESACCT   = 1 << 0,  // message also contains resacct_msg_t
    PAYLOAD_TABLESIZE = 1 << 1,  // message also contains one or more tablesize_msg_t followed by uint32 count

    PAYLOAD_COUNT  = 2           // Number of defined payloads (PAYLOAD_NONE is not counted)
};

enum OsBufType
{
    OsBufTypeFcommPool = 201,
    OsBufTypeItcn = 202,
    OsBufTypePooledMsg = 203,
    OsBufTypeAllocd = 204,
};

#include "nzpack_x86_on.h"

//WARNING: structure saved in compressed external files.  If it changes,
//         bump EXTTAB_HEADER_VERSION and assign a bug to Scott to write
//         code for backward compatibility.

struct spu_msg_header_t { // Messages from SPU to Host
    int32 type;             // SPU_MSG_RESPONSE...
    int32 id;               // 1-based dsid
    int32 unused01;         // unused
    int32 interactionId;    // assigned by host, relevant for responses only
    int32 payloadType;      // Bitmask of enum values from payload_t
    int32 numRecords;       // number of records in content area (derivable!)
    int32 paramTwo;         // used by intra-DBOS LPARAM2 and merge disk temp file
    int32 size;             // bytecount of content area that follows
    char  unused04[4];      // list link
    nzbool moreToCome;      // true if this is not final msg from spu
    nzbool snippetSuccess;  // true if the snippet was successful
    uint8 linked;           // true if message is linked onto msg queue
    uint8 bufferType;       // values from enum OsBufType
    int64 xId;              // transaction id (for async errors)
    union {
        slink_t link;       // list link
        int64   XalignX;    //  make the rest of the fields network-aligned between 64- and 32- bit sides
    };
};

#define MSG_FORMAT_SPU (1<<0)   // payload in SPU-endian
#define MSG_FROM_SPU   (1<<1)   // sender was an SPU

#define MSG_IS_LAST           (1<<0)   // this is the last message.
#define MSG_LOAD_CONT         (1<<1)   // this is part of a load continuation message
#define MSG_TRACK_NO_DELETES  (1<<2)   // mark extents that have no deleted records

#define IS_LAST_MSG(msg)               ((msg)->msgFlags & MSG_IS_LAST)
#define IS_LOAD_CONT_MSG(msg)          ((msg)->msgFlags & MSG_LOAD_CONT)
#define IS_LAST_OR_LOAD_CONT_MSG(msg)  ((msg)->msgFlags & (MSG_IS_LAST | MSG_LOAD_CONT))
#define IS_TRACK_NO_DELETES_MSG(msg)   ((msg)->msgFlags & MSG_TRACK_NO_DELETES)

// Since the messages are allocated and initialized
// to zero, all messages with have largeMessageInfo set to
// 0 initially, so 0 is a good indicator for normal size
#define MSG_NORMAL_SIZE 0       // Message is not split up
#define MSG_LARGE_SIZE  1       // Message is split up

// In order to setup a message type to support being split when
// it it too large, it must meet the following requirements:
// 1 It is in msgToSPU_T
// 2 It is handled in SpuMessageSwitchDispatch
// 3 It does not have an entry in spuDispatchLoop
// 4 Ideally it will be a broadcast messages
//
// If all of those requirements are met, simply modify
// the macro below.
#define MSG_TYPE_CAN_BE_SPLIT(x) ((x == NZ_EXECUTE_PLAN) \
                              || (x == SPU_INFO_LIBRARY) \
                              || (x == SPU_COMMIT))

//WARNING: structure saved in compressed external files.  If it changes,
//         bump EXTTAB_HEADER_VERSION and assign a bug to Scott to write
//         code for backward compatibility.
struct dbos_msg_header_t {  // Messages from Host to SPU
    int32 size;             // size of payload only (does not include dbos_msg_hdr_t)
    int32 type;             // cmdCode
    int32 interactionId;    // unique id
    uint16 srcId;           // data slice id of the sender
    uint16 spuId;           // data slice id of the destination
    int64 xId;              // transaction id
    int32 moreInfo;         // message specific info (tblId for inserts)
    int32 moreInfo2;        // more message specific info (jobid for inserts)
    nzbool fcommBuffer;     // true if an fcomm managed buffer(only user is regen)
    uint8 largeMessageInfo; // Indicator of a split large message
    uint8 msgFlags;         // bit flag: last mesg, load cont mesg, et al.
    uint8 format;           // MSG_FORMAT_SPU ...
};

// This will be inserted between the end of the dbos_msg_header_t
// and the rest of the payload if largeMessageInfo is equal to
// MSG_LARGE_SIZE
struct dbos_msg_large_info_t {
    int32 totalsize;
    int32 chunksize;
    int32 numPieces;
    int32 currentPiece;
};

struct exec_plan_hdr {
    int64 eph_txid;
    int64 eph_identifying_txid;
    int32 eph_jobid;
    int32 eph_planid;
    int32 eph_snippetnum;
};

struct TDbosLimitReached { // The SPU_LIMIT_REACHED message from host to SPU
    dbos_msg_header_t hdr; // Has the interaction ID, hence no payload
};

// Type of sequence errors
enum SeqErrType {
    SEQ_NO_ERROR_ABORT = 0,
    SEQ_EXHAUSTED_ERROR,
    SEQ_DROPPED_RACE_CONDITION_ERROR
};
// Payload of msg sent from DBOS to an SPU with new seq cache load
struct SeqCachePayload {
    int32    m_seqId;
    int32    m_count;
    int64  m_startValue;
    uint8 m_errCode; // enum values from SeqErrType
    uint8 unused017[3]; // pad to multiple of four bytes
};

struct TSeqCache { // The SPU_SEQ_CACHE message from host to SPU
    dbos_msg_header_t hdr;
    SeqCachePayload   payload;
};

// SPU seqence request args structure
struct TSeqRqst {
    int32 m_seqId;
    int32 m_minCount;
};

struct db_replicated_msg_t {
    nzbool replicated;
    char   dbnames[0]; // one or more database names, ending with an extra NUL
};

struct client_result_msg_t {
    long cmdCode; // defined by sysV mesg routine, not included in msgsz
    CTable *result;
    TMemblockInfo *recBlocks; // in streaming mode, the data is here
    int cliInteractId;
    nzbool moreToCome;
    uint8 unused017[3]; // pad to multiple of four bytes
};

struct TDistRowsRqst { // Row insertion request from client
    TXid xId;       // transaction id
    int tblId;      // table to insert into
    CTable *src;    // table to fetch rows from
    int hackMultiplier; // for debug - causes spu to make more data
};

struct TQaCommRqst { // Messages for simple Comms Broadcast/Distribute/Respond tests
    uint32 size;          // size of message
    uint32 count;         // number of messages to send
    uint32 targetCount;   // number of target SPUs
    uint32 startingID;    // first target SPU's ID
    uint32 flags;         // option bits
    uint32 startingSrcID; // first source SPU's ID
    uint32 endingSrcID;   // last source SPU's ID
};

struct TQaSpuDist { // Used to pass qa-spu-dist from dispatch to event thread
    uint32 size;          // size of message
    uint32 count;         // number of messages to send
    uint32 targetCount;   // number of target SPUs
    uint32 startingID;    // first target SPU's ID
    uint32 flags;
    uint32 startingSrcID; // first source SPU's ID
    uint32 endingSrcID;   // last source SPU's ID
    uint32 numSenders;    // number of spus sending
    uint32 startingSenderID;    //starting spu that sending
    int interactionId;
};

struct TQaFCommRqst {     // Used to pass qa-fcomm-rqst from dispatch to event thread
    uint32 size;          // size of message
    uint32 chUID;         // channel UID
    uint16 chIdx;         // channel idx
    int numPkts;          // number of pkts in sndPktPool
    int numReassemblyBuffers; // number of reassembly buffers
    uint32 count;         // number of messages to send
    uint32 targetCount;   // number of target SPUs
    uint32 startingID;    // first target SPU's ID
    uint32 flags;
    uint32 startingSrcID; // first source SPU's ID
    uint32 endingSrcID;   // last source SPU's ID
    uint32 numSenders;    // number of spus sending
    uint32 startingSenderID;    //starting spu that sending
    int interactionId;
    uint32 dsId;          // dataslice Id
};

// See ExtentReclamationMgr:
struct dbos_reclaim_extents_t {
    TXid    stableXid;      // Stable transaction ID
    nzbool  replicated;     // which ERMLIST
    nzbool  force;          // 'stableXid' overrides actual stable txid
    uint8   unused[2];
    int32   numTables;      // Number of tables requiring extent reclamation
    int32   tableIds[0];    // Table IDs
};

enum DbosCreateTableEntryFlags {
    DCTEF_TEMP_TABLE = 1 << 0,
    DCTEF_REPLICATED = 1 << 1,
};

struct dbos_create_table_entry_t {   // create table message (also used for SMACK)
    int32 id;                  // table id - name retained for compatibility
    int32 numFields;
    int32 fieldsVersion;
    int32 flags;          // DbosCreateTableEntryFlags
    uint8 m_virtualLocus; // spu/host locus (e.g. VIRTUAL_LOCUS_SPU...)
    uint8 m_virtualEnabled;   // pad to multiple of two bytes
    int16 m_virtualMaterializer; // record loading function in virtab.cpp
    uint32 m_virtualRingSize;  // if ring-buffer virtab, its size else 0
    uint32 m_distAlgorithm; // DIST_ALGOR_CRC_HASH=0, DIST_ALGOR_RND_ROBIN...
    int32 m_parentId;          // table parent id; needed to identify mviews
    field_t fields[0];    // var extent on number of fields
};

struct dbos_create_table_t {    // create table message (also used for SMACK)
    TXid xId;       // transaction id (not used for SMACK)
    uint32 num_entries;
    dbos_create_table_entry_t entries[1];
};

struct TCreateTempTableMsg {    // Create & load in-memory table on spu
    TXid xId;       // transaction id
    TXid identifyingXid; // "identifying" transaction id (only relevant for PMT)
    int jobId;    // for Snippet Result Cache / Reconstitute
    int32 tblId;           // table to insert into
    int32 linkId;
    int32 publicPartition; // table's initial partition setting
    uint32 planId;   // planId associated with this temp table.
    int32  snippetNum;  // snippet num associated with this temp table.
    int32  cacheId;      // if nonzero, reconstitute transient from cacheId in Snippet Result Cache
    int32  numDropTbls;  // number of entries in dropTbls[] array
    uint16 numFields;  // number of columns
    nzbool createAndDropOnlyOnPassedSlices; // true if creating on a select # of slices
    int32 numPassedSlices; //number of entries in dataSliceIds[] array
    nzbool allowNulls;
    nzbool sizeWord;
    nzbool hasRowNum;
    nzbool usedInMultipleJoins; //true if used in multiple joins
    nzbool isPrebroadcastTbl;
    nzbool isBroadcastTbl;
    table_id_t dropTbls[0];  // table IDs of tables to be dropped
    // Don't try to refer to members under here directly, since its position depends on arrays above them!
    field_t fields[0];    // var extent on number of fields
    int32 dataSliceIds[0];
};

// SPU_SRC_REQUEST-related stuff
enum SnipResCacheOp {
    SRC_ADD_TRANSIENT_TO_CACHE = 1,
    SRC_REMOVE_TRANSIENT_FROM_CACHE = 2,

    SRC_REQUEST_LIMIT = 1000 // maximum number of operations
};

// sub-message of src_request_msg_t
struct src_operation_t {
    int16 op_type;         // add or drop
    uint8 becomingGhost;   // nonzero if op==SRC_REMOVE_TRANSIENT_FROM_CACHE and retaining CSnipResInfo as ghost
    uint8 unused;          // pad to multiple of 4 bytes
    int32 transient_id;    // table to be added or dropped
};

// zero or more drops + zero or one adds
struct src_request_msg_t {
    TXid identifyingXid;    // "identifying" transaction id (applicable only to PMTs)
    int jobid;              // ID of job requesting SRC ops
    int id;                 // Plan dictionary id of plan on whose behalf this is being done.
    int planid;             // Plan on whose behalf this is being done.
    int snippetid;          // Snippet on whose behalf this is being done.
    int16 num_ops;          // number of operations
    bool isLastSnippet;     // true if last snippet in CPlan (for SrcRequestCb).
    uint8 unused[1];        // pad to multiple of 4 bytes
    src_operation_t ops[0]; // operations
};

struct dbos_drop_table_t {    // drop tables message
    TXid xId;       // transaction id
    TXid prevXid;       // Applicable only when dropping of pre-broadcast tables
                        // as part of transaction restart is involved.
    TXid identifyingXid; // This is only for PMT
    int16 numTables;      // number of tables to drop
    nzbool forRollback; // normally false
    uint8  unused011[1];  // pad to multiple of four bytes
    int32 tableIds[0];    // table ids
};

/// Contains the information necessary to recover all transactions since TX ID.
/// This message is sent by the Host TX Manager to the SPU TX Manager.
struct dbos_recover_since_msg_t {
    TXid    xId;            // base TX ID
    int32   numTables;      // Number of Table IDs
    int32   tableIds[0];    // Table IDs
};

struct dbos_donetoredo_t {        // SPU_DONETOREDO message
    TXid xId;                     // transaction id
    int32 tblId;                  // table id
};

enum {
    ALTER_TABLE_COLSIZE = 1,
    ALTER_TABLE_ZMAP,
    ALTER_TABLE_END_OF_ENTRIES = 0 // Terminates list of entries in alter table entries array
};

// base class for alter table entries
struct alter_table_entry_t {
    int32 etype; // entry type, one of ALTER_TABLE enum above
}; // Might have to adjust alignment if derived entries have alignment requirements > int32


// alter table entry to change size of a column
struct alter_table_colsize_t: alter_table_entry_t {
    int32    coldex;
    int32    newsize;
};

// alter table entry to change zonemapped property of a column
struct alter_table_zmap_t: alter_table_entry_t {
    int32    coldex;
    int32    zmapped;  // non-zero: zonemap this column; 0-do not zonemap; int32 is for alignment
};

// clent->dbos alter table message
struct dbos_alter_table_t {
    int32    tblid;
    // variable length array of alter table entries follows
    alter_table_entry_t* first_entry() { return (alter_table_entry_t*) (this + 1); }
};

struct dbos_trunc_table_t {    // truncate table message
    TXid xId;              // transaction id
    int32 id;              // table id - name retained for compatibility
    nzbool createdByMe;    // was table created in same transaction
    nzbool needsWaterMark; // return truncate water-mark to host
    uint8  unused014[2];   // pad to multiple of four bytes
};

struct dbos_trunc_stats_t {    // truncate stats message
    TXid xId;              // transaction id
    int32 id;              // table id - name retained for compatibility
    int64  numrows;        // numrows for trunc stat data
    int32  numpages;       // numpages for trunc stat data
};

struct dbos_create_sequence_t {    // create sequence message
    int32   id;
    int32   dbId;
    int32   classId;
    int32   dataType;
    int64   minValue;
    int64   maxValue;
    int64   increment;
    int64   cacheSize;
    int32   purpose;       // SEQ_PURPOSE_xxx
    nzbool  cycle;
    nzbool  preserveCache;
    uint8   unused058[2];  // pad to multiple of four bytes (60)
};

struct dbos_create_sequences_t {    // create packed sequences message
    TXid xId;
    uint32 num_entries;
    dbos_create_sequence_t entries[1];
};

struct dbos_create_library_t {    // create library message
    TXid    xId;
    int32   id;
    int32   dbId;
    int32   forStartup;
    uint8   cksum[33];
    uint8   baseFileName[256];
    uint8   unused058[3];  // pad to multiple of four bytes
    int32   fileSize;
    uint8   data[0];
};

struct dbos_drop_library_t {    // drop library message
    TXid xId;       // transaction id
    int32 id;       // library id
    int32   dbId;
    uint8   baseFileName[256];
};

// dbos_redr_host_redun_info_t "op_code" values
enum {
    REDRHI_DOWNLOAD_FULL_INFO = 1,
    REDRHI_WRITE_DELTA_UPDATE,
    REDRHI_APPEND_DELTA_UPDATE,
};

struct dbos_redr_host_redun_info_t {
    nzbool  first_chunk;
    nzbool  last_chunk;
    uint8   file_name[64];
    uint64  file_size;
    uint32  data_len;
    uint8   op_code;
    uint8   unused[3];
    char    data[0];
};

struct dbos_redr_tblid_map_entry_t {
    table_id_t   srcTblId;
    table_id_t   dstTblId;
};

struct dbos_redr_op_tblid_map_t {
    uint16 op_code;
    uint16 num_entries;
    dbos_redr_tblid_map_entry_t entries[1];
};

// dbos_create_sequence_t "purpose" values
enum {
    SEQ_PURPOSE_CATALOG_XFER = 0x0001,
    SEQ_PURPOSE_CREATE       = 0x0002,
    SEQ_PURPOSE_ALTER        = 0x0003
};

struct dbos_drop_sequence_t {    // drop sequence message
    TXid xId;       // transaction id
    int32 id;       // seq id
};

enum {
    DBG_DEST_DBOS   = 0x0001,
    DBG_DEST_SPU    = 0x0002,
    DBG_DEST_ALL    = 0x0003
};

#if !FOR_SPU
struct DbosDbgMsg {         // info sent to setup debug
    rc::DbgCtrls    dbgCtrls;
};
#endif


// dbos_info_mls_t
//
// This message conveys one MLS name.  Each name has a type (level, cohort, category)
// and a value.  For alter, both value and new value are used.
// If the type is cohort, then a compressed bitmap of the cohort closure
// is included after the name in the message.
//
// We overload the structure to propagate identifier case.  When the structure is used
// for IDENTCASE, the value field is used to convey whether unquoted identifiers are
// upcased or downcased.  A value of 1 (true) indicates upcasing.  A value
// of 0 (false) indicates downcasing.

struct dbos_info_mls_t {    // Multi-Level Security message
    TXid    xId;
    int32   purpose;        // one of MLS_PURPOSE_xxx
    int32   mlsType;        // one of K_MLS_NAME from mlsNameTable.h
    int32   value;
    int32   newValue;
    int32   nameLen;        // length of the name
    int32   closureLen;     // length of the closure
    nzbool  delim;          // name is a delimited identifier
    char    name[0];        // name follows
};

// dbos_info_mls_t "purpose" values
enum {
    MLS_PURPOSE_XFER         = 0x0001,
    MLS_PURPOSE_CREATE       = 0x0002,
    MLS_PURPOSE_ALTER        = 0x0003,
    MLS_PURPOSE_DROP         = 0x0004,
    MLS_PURPOSE_IDENTCASE    = 0x0005
};

// !fixme-xs2 - this is hardcoded for the time being to cover both
// mustang and XS2. Need to come up with appropriate defines for XS2
// and mustang.
#define MAX_AGGR_RESPONDERS 240  //fixme -- this needs to be more dynamically calculated

// aggregated responses contain an array of dataslice ids that have responded
// to a command broadcast.  -1 is an empty slot.
// the message type for this message is SPU_MSG_RESPONSE, it flows from aggregation
// leader to the host.

struct responder_array_t {
    int16 numResponders;
    int16 responded[MAX_AGGR_RESPONDERS];
};

struct aggr_response_msg_t {
    struct spu_msg_header_t hdr;
    struct responder_array_t  responderArr;
};

struct tablesize_msg_t {
    uint64     tbl_id;     // table id
    uint64     tbl_rows;   // number of rows
    uint64     tbl_pages;  // table size in 128K pages
    uint64     tbl_bytes;  // table size in bytes
    bool       tbl_hasNullsInJoinCols; // For Not IN/relOp ALL
                                       // true  - join cols have nulls
                                       // false - join cols have no nulls
    bool       tbl_sqbHasRows;         // For Not IN/relOp ALL
                                       // true  - subquery has rows
                                       // false - subquery has no rows
};

struct subch_aggr_spu2spu_dist_ctrl_msg_t { // from spu2dbos
    struct spu_msg_header_t hdr;
    uint8    numResponders;
    TChannel spuDistChannel; 
    uint8 subChAggrSpu2SpuDistLastMsgSentList[0]; // array of num of last msg sent to each DsId for this spuDistChannel.
};

struct spu2spu_dist_done_msg_t {  // from dbos2spu
    TChannel  spuDistChannel;
    uint8 spu2spuDistLastMsgSentDoneList[0]; // last msg sender list for the download ch+subch
};

struct TQaAbortOnError {
   int16 abortOnError;
   bool noCrash;
};

#include "nzpack_x86_off.h"

#endif // MESSAGES_INCLUDED        // don't include multiple times
