#ifndef MIRRORTOPO_INCLUDED
#define MIRRORTOPO_INCLUDED

//========================================================================
///\file: mirrortopo.h
//
// (C) Copyright IBM Corp. 2009, 2011  All Rights Reserved.
//========================================================================

#include "nzstd.h"
#include "storagedefs.h"
#include "error.h"

struct dsid_list_t
{
    uint32   nr;
    int16    dsid[0]; // one entry per data slice

    void clear();
    bool add_dslice(int dslice);
    unsigned int find_dslice(int dslice) const;
    bool contains_all(const dsid_list_t *sub) const;
};

#define DSID_LIST_SIZE(n) (sizeof(dsid_list_t) + (n) * sizeof(int16))
#define AUTO_DSID_LIST(n) (dsid_list_t *) alloca(DSID_LIST_SIZE(n))
#define SINGLE_DSID_LIST(ds_id) ({ \
    dsid_list_t *const _l = AUTO_DSID_LIST(1); \
    _l->nr = 1; \
    _l->dsid[0] = (ds_id); \
    _l; \
})

namespace mt {

extern dsid_list_t *mt_dsids; // list of data slices on this spu
extern unsigned int mt_spu_dataslice_cnt; // number of data slices on this spu
extern int *mt_numa_node; // numa node ids indexed by dslice index

extern void mtInitPreFork();
void initSpuDataSliceList();

inline static unsigned int
spuDataSliceCnt()
{
    return mt_spu_dataslice_cnt;
}

inline static int
firstPrimaryDataSlice()
{
    return mt_dsids->nr > 0 ? mt_dsids->dsid[0] : 0;
}

inline static unsigned int
dsidIndex(int dsid)
{
    for (unsigned int i = 0; i < mt_dsids->nr; i++)
        if (mt_dsids->dsid[i] == dsid)
            return i;
    NZ_VERIFY(false);
}

inline static bool
isLocalDataSlice(int dsid)
{
    for (unsigned int i = 0; i < mt_dsids->nr; i++)
        if (mt_dsids->dsid[i] == dsid)
            return true;
    return false;
}

inline static int
numa_node(unsigned int ds_idx)
{
    return mt_numa_node[ds_idx];
}

extern void numa_node_affinity(cpu_set_t *mask, int numa_node);

}; // namespace mt

////////////////////////////////////////////////////////////////////////

inline void
dsid_list_t::clear()
{
    nr = 0;
}

#endif // MIRRORTOPO_INCLUDED
