#ifndef MYTHREAD_INCLUDED
#define MYTHREAD_INCLUDED

/*
  Netezza - access to "global scope" per-task data
     support for: Nucleus
                  LINUX fork() environment
*/

#ifndef TASKXTNS_INCLUDED
# include "taskxtns.h"
#endif

extern TaskXtns s_xtns;
#define TASKXTNS() (&s_xtns)
#define PLATFORM_HOLME(a) (TASKXTNS()->a)

struct LockTracker;
extern __thread LockTracker *tls_lt;

//
// class MyThread - platform-independent interface to thread-specific nz data
//
class MyThread
{
public:

    static void tls_system_init();  /* One time (for system) primeval startup work */

    //
    // general-purpose thread-local storage
    //
    static int tls_alloc();
    static int tls_free(int index);
    static void tls_set_value(int index, void *value);
    static void *tls_get_value(int index);

    static void tls_reset();

    static void tls_set_plan(CPlan *plan);
    static void tls_set_job(SpuJob *job, int dsidx);

    static CPlan *tls_plan();
    static SpuJob *tls_job();
    static job_dslice_t *tls_job_dslice();

    static void tls_set_thrown_error(const CError& err);
    static void tls_set_thrown_error(void *err, int type);
    static int tls_thrown_error(void **err_ptr);

    // values for setThrownError() / from thrownError()
    enum {FROM_NotThrow = 0,    // Not a throw ( perhaps stdlib.h abort()? )
          FROM_CError = 1,      // throw CError (error...)
          FROM_RcErr = 2,       // throw rc::Err (rcErr...)
          FROM_RcIntnlErr = 3,  // throw rc::IntnlErr (rcAbort...)
          FROM_PPcHw = 4        // hack for hardware detected conditions
    };
};

inline CPlan *
MyThread::tls_plan()
{
    return PLATFORM_HOLME(x_plan);
}

inline SpuJob *
MyThread::tls_job()
{
    return PLATFORM_HOLME(x_job);
}

inline job_dslice_t *
MyThread::tls_job_dslice()
{
    return PLATFORM_HOLME(x_job_dslice);
}

inline void
MyThread::tls_set_thrown_error(const CError& err)
{
    TaskXtns *const xtns = TASKXTNS();
    CError *const pe = &xtns->thrown_cerror;
    *pe = err; // copy
    xtns->thrown_error = pe;
    xtns->thrown_error_type = FROM_CError;
}

inline void
MyThread::tls_set_thrown_error(void *err, int type)
{
    TaskXtns *const xtns = TASKXTNS();
    xtns->thrown_error = err;
    xtns->thrown_error_type = type;
}

inline int
MyThread::tls_thrown_error(void **err)
{
    TaskXtns *const xtns = TASKXTNS();
    *err = xtns->thrown_error;
    return xtns->thrown_error_type;
}

//
// thread-safe one time initialization ala pthread_once
//
typedef int nzthread_once_t;
#define NZTHREAD_ONCE_INIT 0

extern void _nzthread_once_impl(nzthread_once_t *once, void (*what)());

#define nzthread_once(once, what) \
do { \
    if (!*once) \
        _nzthread_once_impl(once, what); \
} while(false)

#endif // !defined( MYTHREAD_INCLUDED )
