/// @file nzRecordFormatAllocator.h
/// @brief Record format allocator class and macros.
///
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2013  All Rights Reserved.
//////////////////////////////////////////////////////////////////////
///
/// Description:
///
/// Record format allocator class and macros.
///
//////////////////////////////////////////////////////////////////////

#ifndef NZRECORDFORMATALLOCATOR_H
#define NZRECORDFORMATALLOCATOR_H

#include <limits.h>
#include <memory>
#include <new>
#include <string>
#include <stddef.h>
namespace recordformat
{

typedef void* (*PALLOC)(size_t elements, size_t elementSize);
typedef void (*PFREE)(void *ptr);

extern PALLOC g_pAlloc;
extern PFREE g_pFree;

template<typename T> class RecordFormatAllocator
{
public:
    typedef size_t    size_type;
    typedef ptrdiff_t difference_type;
    typedef T*        pointer;
    typedef const T*  const_pointer;
    typedef T&        reference;
    typedef const T&  const_reference;
    typedef T         value_type;
    template<typename TOther> struct rebind { typedef RecordFormatAllocator<TOther> other; };
    template<typename TOther> RecordFormatAllocator(const RecordFormatAllocator<TOther> &) { }
    RecordFormatAllocator() { }
    ~RecordFormatAllocator() { }
    void construct(pointer p, const_reference val) { ::new(p) T(val); }
    void destroy(pointer p) { p->~T(); }
    bool operator==(const RecordFormatAllocator<T>&) const { return true; }
    size_type max_size() const { return UINT_MAX / sizeof(T); }
    pointer allocate(size_type n, const void* = NULL)
    {
        pointer p = (pointer) g_pAlloc(n, sizeof(value_type));
        if (p == NULL)
        {
            throw std::bad_alloc();
        }
        return p;
    }

    void deallocate(pointer p, size_type = 0)
    {
        if (p != NULL)
        {
            g_pFree(p);
        }
    }
};

typedef RecordFormatAllocator<int> RecordFormatAllocatorType;
typedef std::basic_string<char, std::char_traits<char>, RecordFormatAllocatorType> rfstring;

} // namespace recordformat

#define RECORD_FORMAT_ALLOCATION(c) \
    public : \
    void* operator new(std::size_t size) throw (std::bad_alloc) \
    { \
        recordformat::RecordFormatAllocatorType::rebind<c>::other a; \
        return a.allocate(1); \
    } \
    \
    void* operator new (std::size_t size, void* ptr) throw() \
    { \
        return ptr; \
    } \
    \
    void operator delete(void* ptr) \
    { \
    	recordformat::RecordFormatAllocatorType::rebind<c>::other a; \
        a.deallocate((c *) ptr, sizeof(c)); \
    }

#endif // NZRECORDFORMATALLOCATOR_H
