/// @file nzRecordFormatException.h
/// @brief Record format exception class and macros.
///
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2013  All Rights Reserved.
//////////////////////////////////////////////////////////////////////
///
/// Description:
///
/// Record format exception class and macros.
///
//////////////////////////////////////////////////////////////////////

#ifndef NZRECORDFORMATEXCEPTION_H
#define NZRECORDFORMATEXCEPTION_H

#if !defined(FOR_DBOS) && !defined(FOR_SPU)

#include "ptsException.h"
#include "ptsTrace.h"

using namespace pts;

#else

#include <exception>
#include <string>

#include "nzRecordFormatAllocator.h"

namespace recordformat
{

class RecordFormatException : public std::exception
{
    RECORD_FORMAT_ALLOCATION(RecordFormatException);
public:
    RecordFormatException(const std::string &errorMessage) : m_message(errorMessage.c_str()) { }
    virtual ~RecordFormatException() throw() { }
    virtual const char* what() const throw() { return m_message.c_str(); }
private:
    rfstring m_message;
};

} // namespace recordformat

#define THROW_VERSION(m) throw recordformat::RecordFormatException(m)
#define THROW_ASSERT(m) throw recordformat::RecordFormatException(m)
#define THROW_NETWORK(m) throw recordformat::RecordFormatException(m)
#define PTS_TRACEALWAYS(m)
#define PTSCallStack
#define callStack(c,m)

#endif

#endif // NZRECORDFORMATEXCEPTION_H
