#ifndef nzclocks_H_INCLUDED
#define nzclocks_H_INCLUDED
// ____________________________________________________________________________
// ____________________________________________________________________________
//
// (C) Copyright IBM Corp. 2001, 2011  All Rights Reserved.
// ____________________________________________________________________________
//
// File:                nzclocks.h
//
// Description:
// ------------
// Contains very global clock definitions for use by C files.
//
// Design Notes:
// -------------
// ____________________________________________________________________________
// ____________________________________________________________________________
//
// Header files
//
#include <time.h>

#include "nzstd.h"
// ____________________________________________________________________________
// ____________________________________________________________________________
//
#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif
// ____________________________________________________________________________
// ____________________________________________________________________________
//
// Public Constants
//
typedef enum ClkStatus
{
    CLK_SUCCESS,
    CLK_FAILURE
} ClkStatus;
// ____________________________________________________________________________
// ____________________________________________________________________________
//
// Public Objects
//
// (none)
// ____________________________________________________________________________
// ____________________________________________________________________________
//
// Public functions (prototyped here only)
//
void      clkPopulateClockSpeeds(void); // In board-specific clocks_xxx.c file.
void      clkInitialize(void);
clock_t   clkClock(void);
void      clkSetTimebaseCounter(uint64 newCounter);
uint64    clkGetTimebaseCounter(void);    // Possibly implemented in asm. file.
uint32    clkGetTimebaseCounterLow(void); // Possibly implemented in asm. file.
void      clkClearTimebaseCounter(void);
uint64    clkTimebaseTicksToSec(uint64 tbTicks);
uint64    clkTimebaseTicksToMsec(uint64 tbTicks);
uint64    clkTimebaseTicksToUsec(uint64 tbTicks);
void      clkTimebaseTicksToHrMinSec(uint64 tbTicks, int *pHours, int *pMinutes, int *pSeconds);
uint64    clkCalcTimeBaseDelta(uint64 tbTicksRef);
uint64    clkCalcTimeBaseDeltaSec(uint64 tbTicksRef);
uint64    clkCalcTimeBaseDeltaMsec(uint64 tbTicksRef);
uint64    clkCalcTimeBaseDeltaUsec(uint64 tbTicksRef);
uint32    clkGetTimebaseTicksPerSec(void);
uint32    clkGetTimebaseTicksPerMsec(void);
uint32    clkGetTimebaseTicksPerUsec(void);
uint32    clkGetHwTicksPerOsTick(void);
void      clkPrintDateTime(void);
ClkStatus clkGetUptime(char *pBuf, size_t bufSize);
ClkStatus clkGetDateTime(char *pBuf, size_t bufSize);
void      clkSetDateTime(time_t secondsFromEpoch);
time_t    clkGetSecondsFromEpoch(void);
// ____________________________________________________________________________
// ____________________________________________________________________________
//
// Public (Global) variables ('clk' prefix)
//
// (none)
// ____________________________________________________________________________
//
#ifdef __cplusplus
}
#endif

#endif // nzclocks_H_INCLUDED
