/*
 *  nzfencedcontrol.h
 *  Netezza Fenced Exception
 * (C) Copyright IBM Corp. 2009, 2014  All Rights Reserved.
 */

#ifndef __NZFENCEDCONTROL_H__
#define __NZFENCEDCONTROL_H__

#include <signal.h>

extern "C" {
#include "nzrep.h"
#include "nzrepcommands.h"
#include "nzsds.h"
}

#include "nzfencedmethods.h"
#include "nzudsfenced.h"
#include "nzsdsfenced.h"
#include "nzfenceddatum.h"
#include "nzrepchildinfo.h"
#include "nzfencedexception.h"
#include "nzrepsignalhelpers.h"
#include <udx.h>

struct NzFencedLibraryInfo
{
    NzFencedLibraryInfo() {
        m_name = NULL;
        m_file = NULL;
        m_handle = NULL;
        m_bAutomatic = false;
    }
    ~NzFencedLibraryInfo() {
        if (m_name)
            delete m_name;
        if (m_file)
            delete m_file;
    }

    bool m_bAutomatic;
    char *m_name;
    char *m_file;
    void *m_handle;
};

class NzBaseFencedAdapter : public NzRepChildExecutionInfo
{
public:
    NzBaseFencedAdapter();
    NzBaseFencedAdapter(int param0, int param1, int blocking, char *tempFile);
    virtual ~NzBaseFencedAdapter();
   
    void setupConnection();

    void receiveData(NzrepData &data, const char* context);
    void sendData(NzrepData &data, const char* context);
    void sendError(const char* strMsg);
    void logMsg(const char* strMsg);

    pid_t getChild() { return m_child; }
    int waitForChildExit(bool shouldThrowOnError, int timeoutSeconds);   
    void installParentSignalHandlers();
    void unInstallParentSignalHandlers();
    void installChildSignalHandlers();
 
    NzRepChildExecutionInfo::UdxType getUdxType() { return UDX_TYPE_UNKNOWN; }
    void *getUdxObject() { return NULL; }

    const char*lookupLibrary(const char* name, bool caseSensitive);

    void setHasError() { m_bHasError = true; }

    virtual int returnChildType() = 0;
    
    bool loggingEnabled() { return m_bLoggingEnabled; }

    void remoteCancel();
    void remoteCleanup();

    void setChildTerminated() { m_bTerminated = true; nzrepSetExit(m_init.handle); }
    bool m_bUserQuery;
    int m_dsId;
    int m_sessionId;
    int64 m_txId;
    int m_hwId;
    int m_locus;
    int m_nds;
    int m_nspus;
    int m_rankId;
    int m_localDataslices;
        
    char* m_userName;


    NzRepAutoBlockSignal *m_pBlockChild;
    NzRepAutoBlockSignal *m_pBlockPipe;
    NzRepAutoBlockSignal *m_pBlockInter;

protected:
    void* instantiateV1(int type);
    void* instantiateV2(int type);
    void waitForChildInit();
    void handleInstanceCreated(int instanceType);


    void addLibrary(const char* name, const char*file, bool automatic);
    char *GetLibraryPath();
    void OpenLibraries();


protected:
    NzrepInitialization m_init;
    pid_t m_child;
    

    void *m_dlHandle;
    bool m_bMetadataSent;


        
    nz::udx::FuncDatum makeFromBuffer(NzUdsFenced &uds, bool &bNull, bool stringArg, int idx);
    nz::udx::FuncDatum makeFromStored(int type, bool &bNull, bool stringArg, int idx);

    void readDatum(NzUdsFenced &sds, nz::udx::FuncDatum &datum , bool &null);
    void readDatumCol(NzUdsFenced &sds, nz::udx::FuncDatum &datum , bool &null);

    void sendDatum(NzUdsFenced &uds, int type, nz::udx::FuncDatum datum, bool null, int dataType);   
    void sendReturnInfo(NzUdsFenced &uds, int type, nz::udx::ReturnValue datum, bool null);

    nz::udx::FuncDatum makeReturnInfo(NzUdsFenced &uds, int idx);
    nz::udx::FuncDatum makeReturnInfoStored(int type, int idx);

    nz::udx::ReturnValue makeReturnValue(NzUdsFenced &uds, nz::udx::FuncDatum datum, bool &null);

    int m_nDatums;
    NzFencedModeDatum **m_pDatums;

    NzFencedLibraryInfo *m_pLibraries;
    int m_nLibraries;
    int m_nAllocedLibraries;
    char *m_path;
    bool m_bHasError;

    bool m_bLoggingEnabled;
    bool m_bTerminated;

    NzSdsFenced m_sdsRead;
    NzSdsFenced m_sdsWrite;

    NzUdsFenced m_udsRead;
    NzUdsFenced m_udsWrite;
    


};

class NzBaseFencedAdapterRemote : public NzBaseFencedAdapter
{
public:
    static NzBaseFencedAdapterRemote *createChildInstance();

    NzBaseFencedAdapterRemote(int param0, int param1, int blocking, char *tempFile) ;


    ~NzBaseFencedAdapterRemote();

    

    virtual void runChild() = 0;

    void startChild();

    
};

class NzUdfFencedAdapterV1NPS : public NzBaseFencedAdapter
{
public:
    NzUdfFencedAdapterV1NPS() {
        m_pUdf = NULL;
    }
    virtual void createInstance(const char* filename, const char* classname, int logMask, int memory);
    virtual int returnChildType() { return NZFENCED_METHOD_UDF_CONSTRUCTOR_V1; }
    NzRepChildExecutionInfo::UdxType getUdxType() { return UDX_TYPE_UDF_V1; }
    void setUdxObject(nz::udx::Udf *pUdf) { m_pUdf = pUdf; }
    void *getUdxObject() { return m_pUdf; }

    nz::udx::ReturnValue remoteEvaluate(nz::udx::FuncCallInfo *pInfo);
    uint64 remoteCalculateSize(nz::udx::FuncCallInfo *pInfo);

protected:
    nz::udx::Udf *m_pUdf;
};


class NzUdfFencedAdapterV1Remote : public NzBaseFencedAdapterRemote
{
public:
    
    NzUdfFencedAdapterV1Remote(int param0, int param1,  int blocking, char *tempFile);     
    virtual ~NzUdfFencedAdapterV1Remote(); 

    virtual void runChild();
    virtual int returnChildType() { return NZFENCED_METHOD_UDF_CONSTRUCTOR_V1; }

    void handleEvaluate(NzrepData &s);
    void handleCalculateSize(NzrepData &s);

protected:
    nz::udx::Udf *m_pUdf;
    nz::udx::FuncCallInfo m_oInfo;
};


class NzUdaFencedAdapterV1NPS : public NzBaseFencedAdapter
{
public:
    NzUdaFencedAdapterV1NPS() { 
        m_bAccumSent = false;
        m_bMergeSent = false;
        m_bFinalSent = false; 
        m_bNeedInit = false; 
        m_pUda = NULL;
    }
    virtual void createInstance(const char* filename, const char* classname, int logMask, int memory);
    virtual int returnChildType() { return NZFENCED_METHOD_UDA_CONSTRUCTOR_V1; }
    NzRepChildExecutionInfo::UdxType getUdxType() { return UDX_TYPE_UDA_V1; }
    void setUdxObject(nz::udx::Uda *pUda) { m_pUda = pUda; }
    void *getUdxObject() { return m_pUda; }

    nz::udx::ReturnValue remoteFinalResult(nz::udx::AggCallInfo *pInfo);
    void remoteAccumulate(nz::udx::AggCallInfo *pInfo);
    void remoteMerge(nz::udx::AggCallInfo *pInfo);
    void remoteInitializeState(nz::udx::AggCallInfo *pInfo);

protected:
    void sendAccumMeta(NzSdsFenced &sds, nz::udx::AggCallInfo *pInfo);
    void sendMergeMeta(NzSdsFenced &sds, nz::udx::AggCallInfo *pInfo);
    void sendFinalMeta(NzSdsFenced &sds, nz::udx::AggCallInfo *pInfo);

    bool m_bAccumSent;
    bool m_bMergeSent;
    bool m_bFinalSent;
    bool m_bNeedInit;
    nz::udx::Uda *m_pUda;
};


class NzUdaFencedAdapterV1Remote : public NzBaseFencedAdapterRemote
{
public:
    
// In this class, we need to be able to store args and two variants
// of state variables.
    static const int ARG_DATUM_START_IDX=0;
    static const int STATE_DATUM_START_IDX=100;
    static const int SECOND_STATE_DATUM_START_IDX=200;
    static const int FINAL_DATUM_START_IDX=300;

    NzUdaFencedAdapterV1Remote(int param0, int param1, int blocking, char *tempFile);     
    virtual ~NzUdaFencedAdapterV1Remote(); 

    virtual void runChild();
    virtual int returnChildType() { return NZFENCED_METHOD_UDA_CONSTRUCTOR_V1; }

    void handleFinalResult(NzrepData &s);
    void handleAccumulate(NzrepData &s);
    void handleMerge(NzrepData &s);
    void handleInitializeState(NzrepData &s);
    
protected:

    void handleAccumMeta(NzSdsFenced &sds);
    void handleFinalMeta(NzSdsFenced &sds);
    void handleMergeMeta(NzSdsFenced &sds);

    nz::udx::Uda *m_pUda;
    nz::udx::AggCallInfo m_oInfo;
    bool m_bAccumSent;
    bool m_bMergeSent;
    bool m_bFinalSent;

    int returnType;

    int  numArgs;
    int  argTypes[NZ_FUNC_MAX_ARGS];
    bool argConsts[NZ_FUNC_MAX_ARGS];

    int numStateVars;
    int stateVarTypes[NZ_FUNC_MAX_ARGS];
};


class NzUdtfFencedAdapterV2NPS : public NzBaseFencedAdapter
{
public:
    NzUdtfFencedAdapterV2NPS() {
        m_pUdtf = NULL;
    }
    virtual void createInstance(const char* filename, const char* classname, int logMask, 
                                int memory, nz::udx_ver2::UdxEnvironment *pEnviron);
    virtual int returnChildType() { return NZFENCED_METHOD_UDTF_CONSTRUCTOR_V2; }
    NzRepChildExecutionInfo::UdxType getUdxType() { return UDX_TYPE_UDTF_V2; }
    void setUdxObject(nz::udx_ver2::Udtf *pUdtf) { m_pUdtf = pUdtf; }
    void *getUdxObject() { return m_pUdtf; }

    nz::udx_ver2::Udtf::DataAvailable remoteNextOutputRow(nz::udx_ver2::TableFuncCallInfo *pInfo);
    nz::udx_ver2::Udtf::DataAvailable remoteNextEoiOutputRow(nz::udx_ver2::TableFuncCallInfo *pInfo);
    nz::udx_ver2::Udtf::DataAvailable remoteNextEopOutputRow(nz::udx_ver2::TableFuncCallInfo *pInfo);
    void remoteNewInputRow(nz::udx_ver2::TableFuncCallInfo *pInfo);
    void remoteNewInputStream(nz::udx_ver2::TableFuncCallInfo *pInfo);
    void remoteNewInputPartition(nz::udx_ver2::TableFuncCallInfo *pInfo);
    void remoteCalculateShape(nz::udx_ver2::TableFuncCallInfo *pInfo, nz::udx_ver2::UdxOutputShaper *shaper);
    bool remoteProvideOptimizerInfo(nz::udx_ver2::TableFuncCallInfo *pInfo, nz::udx_ver2::UdxOptimizerInfo *info);
        

protected:
    nz::udx_ver2::Udtf *m_pUdtf;
    
    void sendMeta(NzSdsFenced &sds, nz::udx_ver2::TableFuncCallInfo *pInfo);
};


class NzUdtfFencedAdapterV2Remote : public NzBaseFencedAdapterRemote
{
public:
    static const int ARG_DATUM_START_IDX=0;
    static const int RETURN_DATUM_START_IDX=100;
    

    NzUdtfFencedAdapterV2Remote(int param0, int param1,  int blocking, char *tempFile);     
    virtual ~NzUdtfFencedAdapterV2Remote(); 

    virtual void runChild();
    virtual int returnChildType() { return NZFENCED_METHOD_UDTF_CONSTRUCTOR_V2; }

    void handleNextOutputRow(NzrepData &s);
    void handleNextEoiOutputRow(NzrepData &s);
    void handleNextEopOutputRow(NzrepData &s);
    void handleNewInputRow(NzrepData &s);
    void handleNewInputStream(NzrepData &s);
    void handleNewInputPartition(NzrepData &s);
    void handleCalculateShape(NzrepData &s);
    void handleProvideOptimizerInfo(NzrepData &s);


protected:
    nz::udx_ver2::Udtf *m_pUdtf;
    nz::udx_ver2::TableFuncCallInfo m_oInfo;

    void handleMeta(NzSdsFenced &sds);
};


class NzUdfFencedAdapterV2NPS : public NzBaseFencedAdapter
{
public:
    NzUdfFencedAdapterV2NPS() {
        m_pUdf = NULL;
    }
    virtual void createInstance(const char* filename, const char* classname, int logMask, int memory, 
                                nz::udx_ver2::UdxEnvironment *pEnviron);   
    virtual int returnChildType() { return NZFENCED_METHOD_UDF_CONSTRUCTOR_V2; }
    NzRepChildExecutionInfo::UdxType getUdxType() { return UDX_TYPE_UDF_V2; }
    void setUdxObject(nz::udx_ver2::Udf *pUdf) { m_pUdf = pUdf; }
    void *getUdxObject() { return m_pUdf; }

    nz::udx::ReturnValue remoteEvaluate(nz::udx_ver2::FuncCallInfo *pInfo);
    uint64 remoteCalculateSize(nz::udx_ver2::FuncCallInfo *pInfo);
    bool remoteProvideOptimizerInfo(nz::udx_ver2::FuncCallInfo *pInfo, nz::udx_ver2::UdxOptimizerInfo *info);

protected:
    nz::udx_ver2::Udf *m_pUdf;
};


class NzUdfFencedAdapterV2Remote : public NzBaseFencedAdapterRemote
{
public:
    static const int ARG_DATUM_START_IDX=0;

    NzUdfFencedAdapterV2Remote(int param0, int param1, int blocking, char *tempFile);     
    virtual ~NzUdfFencedAdapterV2Remote(); 

    virtual void runChild();
    virtual int returnChildType() { return NZFENCED_METHOD_UDF_CONSTRUCTOR_V2; }

    void handleEvaluate(NzrepData &s);
    void handleCalculateSize(NzrepData &s);
    void handleProvideOptimizerInfo(NzrepData &s);

protected:
    nz::udx_ver2::Udf *m_pUdf;
    nz::udx_ver2::FuncCallInfo m_oInfo;
};


class NzUdaFencedAdapterV2NPS : public NzBaseFencedAdapter
{
public:

    

    NzUdaFencedAdapterV2NPS() { 
        m_bAccumSent = false;
        m_bMergeSent = false;
        m_bFinalSent = false; 
        m_bNeedInit = false; 
        m_pUda = NULL;
    }
    virtual void createInstance(const char* filename, const char* classname, int logMask, int memory, 
                                nz::udx_ver2::UdxEnvironment *pEnviron);   
    virtual int returnChildType() { return NZFENCED_METHOD_UDA_CONSTRUCTOR_V2; }
    NzRepChildExecutionInfo::UdxType getUdxType() { return UDX_TYPE_UDA_V2; }
    void setUdxObject(nz::udx_ver2::Uda *pUda) { m_pUda = pUda; }
    void *getUdxObject() { return m_pUda; }

    nz::udx::ReturnValue remoteFinalResult(nz::udx_ver2::AggCallInfo *pInfo);
    void remoteAccumulate(nz::udx_ver2::AggCallInfo *pInfo);
    void remoteMerge(nz::udx_ver2::AggCallInfo *pInfo);
    void remoteInitializeState(nz::udx_ver2::AggCallInfo *pInfo);
    bool remoteProvideOptimizerInfo(nz::udx_ver2::AggCallInfo *pInfo, 
                                    nz::udx_ver2::UdxAggregateOptimizerInfo *info);

protected:
    void sendAccumMeta(NzSdsFenced &sds, nz::udx_ver2::AggCallInfo *pInfo);
    void sendMergeMeta(NzSdsFenced &sds, nz::udx_ver2::AggCallInfo *pInfo);
    void sendFinalMeta(NzSdsFenced &sds, nz::udx_ver2::AggCallInfo *pInfo);

    bool m_bAccumSent;
    bool m_bMergeSent;
    bool m_bFinalSent;
    bool m_bNeedInit;
    nz::udx_ver2::Uda *m_pUda;
};


class NzUdaFencedAdapterV2Remote : public NzBaseFencedAdapterRemote
{
public:
    
// In this class, we need to be able to store args and two variants
// of state variables.
    static const int ARG_DATUM_START_IDX=0;
    static const int STATE_DATUM_START_IDX=100;
    static const int SECOND_STATE_DATUM_START_IDX=200;
    static const int FINAL_DATUM_START_IDX=300;

    NzUdaFencedAdapterV2Remote(int param0, int param1, int blocking, char *tempFile);     
    virtual ~NzUdaFencedAdapterV2Remote(); 

    virtual void runChild();
    virtual int returnChildType() { return NZFENCED_METHOD_UDA_CONSTRUCTOR_V2; }

    void handleFinalResult(NzrepData &s);
    void handleAccumulate(NzrepData &s);
    void handleMerge(NzrepData &s);
    void handleInitializeState(NzrepData &s);
    void handleProvideOptimizerInfo(NzrepData &s);

    
protected:
    void handleAccumMeta(NzSdsFenced &sds);
    void handleFinalMeta(NzSdsFenced &sds);
    void handleMergeMeta(NzSdsFenced &sds);

    nz::udx_ver2::Uda *m_pUda;
    nz::udx_ver2::AggCallInfo m_oInfo;
    bool m_bAccumSent;
    bool m_bMergeSent;
    bool m_bFinalSent;

    int returnType;

    int  numArgs;
    int  argTypes[NZ_FUNC_MAX_ARGS];
    bool argConsts[NZ_FUNC_MAX_ARGS];

    int numStateVars;
    int stateVarTypes[NZ_FUNC_MAX_ARGS];

};


#endif
