/*
 *  nzfencedudxs.h
 *  Netezza Fenced Exception
 * (C) Copyright IBM Corp. 2009, 2014  All Rights Reserved.
 */

#ifndef __NZFENCEDUDXS_H__
#define __NZFENCEDUDXS_H__

#include "nzfencedcontrol.h"
#include <udx.h>


class NzFencedUdfV1 : public nz::udx::Udf
{
public:
    static nz::udx::Udf* instantiate();
    NzFencedUdfV1();
    ~NzFencedUdfV1();
    virtual nz::udx::ReturnValue evaluate();
    uint64 calculateSize() const;

    void runFenced(const char*file, const char*classname, int logMask, int memory);
  
    mutable NzUdfFencedAdapterV1NPS m_oAdapter;
  
};
    
class NzFencedUdaV1 : public nz::udx::Uda
{
public:
    static nz::udx::Uda* instantiate();
    NzFencedUdaV1();
    ~NzFencedUdaV1();

    virtual void initializeState();
    virtual void accumulate();
    virtual void merge();

    virtual nz::udx::ReturnValue finalResult();

    void runFenced(const char*file, const char*classname, int logMask, int memory);

    NzUdaFencedAdapterV1NPS m_oAdapter;

};


class NzFencedUdfV2 : public nz::udx_ver2::Udf
{
public:
    static nz::udx_ver2::Udf* instantiate(nz::udx_ver2::UdxInit *pInit);
    NzFencedUdfV2(nz::udx_ver2::UdxInit *pInit);
    ~NzFencedUdfV2();
    virtual nz::udx_ver2::ReturnValue evaluate();
    uint64 calculateSize() const;
    virtual bool provideOptimizerInfo(nz::udx_ver2::UdxOptimizerInfo *info);

    void runFenced(const char*file, const char*classname);
  
    mutable NzUdfFencedAdapterV2NPS m_oAdapter;
  
};
    
class NzFencedUdaV2 : public nz::udx_ver2::Uda
{
public:
    static nz::udx_ver2::Uda* instantiate(nz::udx_ver2::UdxInit *pInit);
    NzFencedUdaV2(nz::udx_ver2::UdxInit *pInit);
    ~NzFencedUdaV2();

    virtual void initializeState();
    virtual void accumulate();
    virtual void merge();
    virtual bool provideOptimizerInfo(nz::udx_ver2::UdxOptimizerInfo *info);
    virtual nz::udx_ver2::ReturnValue finalResult();

    void runFenced(const char*file, const char*classname);

    NzUdaFencedAdapterV2NPS m_oAdapter;

};

class NzFencedUdtfV2 : public nz::udx_ver2::Udtf
{
public:
    

    NzFencedUdtfV2(nz::udx_ver2::UdxInit *pInit);
    virtual ~NzFencedUdtfV2();

        
    virtual void newInputRow();
    virtual void newInputStream();
    virtual void newInputPartition();
    
    virtual DataAvailable nextOutputRow();
    virtual DataAvailable nextEoiOutputRow();
    virtual DataAvailable nextEopOutputRow();
    static nz::udx_ver2::Udtf* instantiate(nz::udx_ver2::UdxInit *pInit);

    void runFenced(const char*file, const char*classname);

    virtual void calculateShape(nz::udx_ver2::UdxOutputShaper *shaper);
    virtual bool provideOptimizerInfo(nz::udx_ver2::UdxOptimizerInfo *info);

    NzUdtfFencedAdapterV2NPS m_oAdapter;

};



#endif
