/*-----------------------------------------------------------------------------
// nzhwdefs.h
//
// Manifest constants that describe the hardware..
//
// (C) Copyright IBM Corp. 2003, 2014  All Rights Reserved.
*/

#ifndef NZHWDEFS_INCLUDED
#define NZHWDEFS_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

#define DISK_SERIAL_NUM_MAX_LEN   32

/* these are upper bounds. use sc::hpfInfo for actual values at runtime */
enum
{
    HWD_MAX_SPUS_PER_SPA = 14,
    HWD_MAX_SPUS_PER_RACK = 112,
    HWD_MAX_SPAS_PER_RACK = 8
};

/* Hardware Class */
//The hardware class mixes Platform, Family, and possibly even model
//The idea is to create a new class for which we will have a distinct set
//of registry settings
typedef enum
{
    HWC_VSPU = 1,      // Virtual SPU
    HWC_MUSTANG,       // Mustang (Rev 7)
    HWC_MUSTANG_GT,    // MustangGT (Rev 8)
    HWC_XS,            // XS (Rev 10)
    HWC_TT,            // ThinkTanks
    HWC_PSERIES,       // PSeries (TwinFin)
    HWC_SSERIES,       // SSeries (Skimmer)
    HWC_OSERIES,       // OSeries (Orca)
    HWC_DSERIES,       // DSeries (Cruiser)
    HWC_QSERIES,       // QSeries (Striper)
    HWC_RSERIES,       // RSeries 
    HWC_VM,            // VM SPU
    HWC_VSERIES        // VSeries (Mako/V10)
} HwClass;

/* SPU CPU Models */
enum
{
    CPU_PPC855 = 1,
    CPU_PPC860,
    CPU_PPC405,
    CPU_PPC440,
    CPU_PPC460,

    CPU_X86,
    CPU_X86_64
};

/* SPU OS Types */
enum
{
    OSTYPE_NUCLEUS = 1,
    OSTYPE_LINUX
};

inline static int
isIntelArch(int cpuModel)
{
    return cpuModel == CPU_X86 || cpuModel == CPU_X86_64;
}

#ifdef __cplusplus
}
#endif

#endif
