// nzlimits.h
//
// System-wide resource limits, array sizes, etc.
//
// (C) Copyright IBM Corp. 2001, 2011  All Rights Reserved.

#ifndef NZLIMITS_INCLUDED
#define NZLIMITS_INCLUDED

#define STRPATH 1000    // must be same as in error.h

#define MAX_DATASLICES 2000          // max data slices (for largest system to date)

#define MAX_MAXCONNECTIONS 2000      // max value of startup.maxConnections
#define MAX_QUERYHISTTBLSIZE 15000      // max value of startup.queryHistTblSize
#define NZ_INTERNAL_CLIENTS 48       // startupsvr adds this to get Nz->maxClients
#define MAX_MAX_CLIENTS (MAX_MAXCONNECTIONS + NZ_INTERNAL_CLIENTS)

#define MAX_INTERACTIONS 2*MAX_MAX_CLIENTS    // max concurrent interactions; allow for rollbacks

#define MAX_FCOMM_FDS    64000    // max per process file descriptors allowed for fcomm process

// convenience for legacy code
#define MAX_CLIENTS      (Nz->maxClients)

#define MAX_FIXED_CHAR_LENGTH 16 // before fixed chars really become vars

// This is growing out of date; Postgres limit is 1600 user columns;
// Planck (Fermi and Curie) FPGA limit is 2048.
//
#define MAX_TABLE_COLUMNS 4000  // limited by fpga hardware to 4091

// following must be in synch with genex.cpp's NZ_TABLE_MAX_COLUMN_SIZE_AS_STR
#define NZ_TABLE_MAX_COLUMN_SIZE 64000  // arbitrary, could be almost max record size

#define MAX_BYTES_PER_NCHAR 4  // worst-case bytes to represent a UTF-8 character

// FPGA invisibility list size (should be max of all fpga / spu models)
#define MAX_FPGA_VISIBLE_XIDS 64    // was 32 before Dirac/8
// Account for Self TX ID in entry[0] in the FPGA list.
#define MAX_NUM_INVISIBLE_XIDS_FPGA (MAX_FPGA_VISIBLE_XIDS-1)
#define MAX_NUM_INVISIBLE_XIDS_SW (2048-1)

#define MAX_INDEX_FIELDS 4  // plenty
#define MAX_INDEX_PROJECT_FIELDS 16 // keep in sync with nz_thin.h

#define SPU_RECORDGROUP_BUFSIZE 1048500    // little under 1 meg

#define MAX_DIST_KEY_FIELDS 4 // table distribution columns

// define "sizeof"s for types that are stored as struct of double/int32
// we write these in 12 bytes on disk, in commbuf, in record, etc

#define SIZEOF_INTERVAL 12 // sizeof(Interval) is 12 on intel, 16 on ppc
#define SIZEOF_TIMETZADT 12 // ditto TimeTzADT

#define NZ_PAGE_SHIFT 17
#define NZ_PAGE_SIZE (1U << NZ_PAGE_SHIFT) // 128K for recordstore blocks, most buffers...
#define NEARLY_128K 130000 // a nice size

#define NZ_MAX_RECORD_SIZE 65535 // due to 16-bit size word at present
#define NZ_DEFAULT_MAX_SPRING_FIELD_SIZE 512 //default max size for spring field

// Rollback Performance
#define MAX_MVIEWS_PER_TABLE 100 // The maximum number of active materialized views for a single table.
// was 10 in xpdelete.h
#define MAX_GENC_DELETE_PAGES (24) // max number of pages to be modified in one call to DeleteRecords

#define  MAX_REGEN_SPEED_LIMIT 400000  // in Kbps
#define  MIN_REGEN_SPEED_LIMIT 100000  // in Kbps

#define  DEFAULT_DBOS_SHM_SIZE_MB 1540  // DBOS global shared mem in MB
#define  DEFAULT_DEV_SHM_SIZE_MB 78116  // default /dev/shm size in MB

#define MAX_SYSOID 200000

// Any code change in this area of temp table id range impacts the perl scripts
// toporegen.pl and find_orphaned_tables.pl which are part of nzhostrestore.
// Make sure to duplicate the changes there.
#define NZ_TEMP_TABLE_ID_RANGE        500000000U
#define NZ_BASE_DBOS_TEMP_TABLE_ID    0x80000000U
#define NZ_BASE_SPU_TEMP_TABLE_ID     (NZ_BASE_DBOS_TEMP_TABLE_ID + NZ_TEMP_TABLE_ID_RANGE)

#endif // NZLIMITS_INCLUDED
