#ifndef NZMUTEX_INCLUDED
#define NZMUTEX_INCLUDED
/// @file nzmutex.h
/// @brief 
///
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corporation 2000, 2012.  All rights reserved.
//////////////////////////////////////////////////////////////////////
///
/// Description:
///
//////////////////////////////////////////////////////////////////////

#include "nztypes.h"
#include "nzpthread.h"

class CMutex { // basic mutex struct that contains stackable support
    CMutex(const CMutex&); // not implemented
    CMutex& operator=(const CMutex&); // not implemented
public:
    nzmutex_t m_mutex;
    int m_owner;   // process id of current mutex owner
#if LINUX
    pthread_t m_tid; // opaque thread id of current mutex owner; 
                     // ONLY meaningfull when m_owner is not zero
#endif
    int m_nesting; // recursion counter
    void *operator new(size_t size);
    void operator delete(void *p);
    CMutex();
    ~CMutex();
};

class CStackMutex {  // generalized nestable mutex
    CStackMutex(const CMutex&); // not implemented
    CStackMutex& operator=(const CMutex&); // not implemented
public:
    CStackMutex();
    //constructor to disable the mutex
    CStackMutex(CMutex *mutex, const char *type, int id = 0,  bool disabled = false);   // Locks mutex to this thread
    ~CStackMutex();  // Unlocks mutex when object goes out of scope
    void Unlock();  // explicity unlock mutex (only affects this nesting level)
    void Lock();    // explicitly re-lock the mutex, after an unlock
    bool Lock(CMutex *mutex, const char *type, int id = 0, bool trylock = false);
    bool TryLock(CMutex *mutex, const char *type, int id = 0);
    bool isLocked() const { return m_lockedAtThisLevel; };
#if FOR_SPU
    int getID() const { return m_id; };
#endif
protected:
    CMutex *m_mutex;
    bool m_lockedAtThisLevel; // flag which tells whether current routine
            // has an active, though possibly nested, lock
            // This only gets reset by the explicit unlock
#if FOR_SPU
    const char *m_type;
    int m_id;
#endif
};

/*********** LOWER LEVEL MUTEX SUPPORT ********************/

extern void NzInitMutex(nzmutex_t *mutex);

inline static void
NzDestroyMutex(nzmutex_t *mutex)
{
    nzpthread_mutex_destroy(mutex);
}

inline static void
NzSetMutex(nzmutex_t *mutex)
{
    nzpthread_mutex_lock(mutex);
}

inline static void
NzClearMutex(nzmutex_t *mutex)
{
    nzpthread_mutex_unlock(mutex);
}

inline static bool
NzTrySetMutex(nzmutex_t *mutex)
{
    return nzpthread_mutex_trylock(mutex);
}

extern nzmutex_t *NzCreateMutex(); // create a mutex in shared memory
extern void NzFreeMutex(nzmutex_t *);

#endif // NZMUTEX_INCLUDED
