// nzrecord.h
//
// Common field and record definitions.
//
// Specific 'nz' includes files are:
//
//  nzstd.h         - basic C/C++ defs that are product independent (int64)
//  nzlimits.h      - system-wide resource limits, array sizes, etc.
//  nzrecord.h      - basic record and field definitions
//  nzcomm.h        - high-level communication types and messages
//  nzglobals.h     - common memory structure and a few often-used globals
//
// (C) Copyright IBM Corp. 2001, 2011  All Rights Reserved.

#ifndef NZRECORD_INCLUDED
#define NZRECORD_INCLUDED

#ifndef NZSTD_INCLUDED
# include "nzstd.h"
#endif

#ifndef NZTYPES_INCLUDED
# include "nztypes.h"
#endif

#ifndef NZFIELDTYPE_INCLUDED
# include "nzfieldtype.h"
#endif

#if WIN32
# pragma warning(disable: 4200)	// Warning about zero sized arrays
#endif

// This number must not exceed MaxHeapAttributeNumber (=1600, see
// src/pg/include/access/htup.h).  The current definition accomodates
// FPGA design: 1500 is less than 6*256, and leave some room for the
// FPGA-introduced specials.
#define NZ_MAX_USER_COLUMNS 1500

// Define a 'casting' type to hold ptrs - not a real field type
#if PTRS_SAME_SIZE_AS_INTS
#define NzTypePtrCast NzTypeInt
#else
#error Need to adjust NzTypePtrCast
#endif

struct field_t {    // table field definitions (used in table.m_fields[])
    uint16 type;
    uint16 size;        // for fixed types and constant strings
    uint16 trueSize;    // char strings preceeded by count, 32-bit aligned
    uint16 offset;      // recordpos for fixed fields, dex for varying fields
    uint16 physField;   // physical field dex for this logical field
    uint16 logField;    // to get field-desc from physdex m_field[m_field[dex].logField]
    nzbool nullAllowed;
    nzbool fixedSize;
    nzbool springField; // exists to compensate for changes to preceding varying fields
    uint8  flags;
};

// flag definitions for flags member of field_t
#define FieldNone             0 // No flags
#define FieldIsZoneMapped  0x01 // This field is zone-mapped
#define FieldIsSoleDistCol 0x02 // Can apply distribution restriction using this field
#define FieldPadEbcdic     0x04 // Pad this field with EBCDIC space (0x40)

#if !FOR_CLI

typedef uint32 rdex_t;

#endif // !FOR_CLI

struct TUseSpecialFields { // table fields that don't show in the projection list
    unsigned int
        dtidOff   : 1,  // [pageId | bit offset of record's DTID]
        recOrd    : 1,  // [pageId | record's ordinal position on page]
        rowId     : 1,  // sequencing integral field
        txCreated : 1,  // the 'created' transaction id field
        txDeleted : 1;  // the 'deleted' transaction id field
};

#endif // NZRECORD_INCLUDED
