/*
 *  nzrepchildexecutioninfo.h
 *  Netezza Fenced Remote Execution Protocol
 *  signal helers
 * (C) Copyright IBM Corp. 2009, 2011  All Rights Reserved.
 */

#ifndef __NZREPCHILDEXECUTIONINFO_H__
#define __NZREPCHILDEXECUTIONINFO_H__

#include <unistd.h>
#include <signal.h>


// Only the destructor and killChild
// should be implemented, all others should
// be pure virtual.
class NzRepChildExecutionInfo
{
public:
        // Identify the type of UDX or AE function
        enum UdxType {
            UDX_TYPE_UNKNOWN = 0,
            UDX_TYPE_UDF_V1,
            UDX_TYPE_UDF_V2,
            UDX_TYPE_UDA_V1,
            UDX_TYPE_UDA_V2,
            UDX_TYPE_UDTF_V2
        };

	virtual ~NzRepChildExecutionInfo() {}
	
	// Return pid of child
	virtual pid_t getChild() = 0;

        // Return SQL function type
        virtual UdxType getUdxType() = 0;

        // Return ptr to UDX instance
        virtual void *getUdxObject() = 0;

	// Bookkeeping - Lets child know it has been terminated.
	// Likely should call nzrepSetExit.
	virtual void setChildTerminated() = 0;

	// return exit status of child
	// Only wait for timeoutSeconds before returning.
	// When timeout has expired without the child exiting, you
	// must call killChild and return -1. Before returning, you
	// need to remove yourself from the global signal handlers
	virtual int waitForChildExit(bool shouldThrowOnError, int timeoutSeconds) = 0;
	
	void killChild(bool hardKill) {
        pid_t child = getChild();
        if (child <= 1)
            return;
		if (hardKill)
			kill(child, SIGKILL);
		else
			kill(child, SIGTERM);
		
	}
};

#endif
