/*
 *  nzrepsignalcontroller.h
 *  Netezza Fenced Remote Execution Protocol
 *  signal controller
 * (C) Copyright IBM Corp. 2009, 2011  All Rights Reserved.
 */

#ifndef __NZREPSIGNALCONTROLLER_H__
#define __NZREPSIGNALCONTROLLER_H__

#include <unistd.h>
#include <signal.h>

#include "nzrepchildinfo.h"
#include "udx_context.h"

class NzRepSignalController
{
public:
    NzRepSignalController();
    ~NzRepSignalController();

    static pid_t timeoutWaitPid(pid_t pid, int *status, int timeoutMs);

	static NzRepSignalController& getSingleton();
	
	// Should be called by parent, right after fork.
	// Parent should be blocking SIGCHLD, SIGPIPE and SIGINT
	// prior to fork until this is called.
    void addHandler(NzRepChildExecutionInfo *chld);
	
	// Should be called by ~NzRepChildExecutionInfo
    void removeHandler(NzRepChildExecutionInfo *chld);
    
    void terminateAll(bool inHandler);

    
    struct Entry
    {
        Entry() {
            m_pData = NULL;
            m_pNext = NULL;
            m_bActive = true;
            m_nExit = 0;
            m_wlmCookie = -1;
        }
            
        NzRepChildExecutionInfo * m_pData;

        // child resource usage
        struct ChildRsUsage m_cru;
        bool m_bActive;
        int m_nExit;
        struct Entry *m_pNext;
        int m_wlmCookie;
    };


    void updateStatus(Entry *cur);
    Entry * getHandlerClass(pid_t child);
private:
    Entry *m_pHead;
    struct sigaction m_oldchild;
    struct sigaction m_oldpipe;
    struct sigaction m_oldint;
};

#endif
