/*
 *  nzrepsignalhelpers.h
 *  Netezza Fenced Remote Execution Protocol
 *  signal helers
 * (C) Copyright IBM Corp. 2009, 2011  All Rights Reserved.
 */

#ifndef __NZREPSIGNALHELPERS_H__
#define __NZREPSIGNALHELPERS_H__

#include <signal.h>
#include <errno.h>

struct NzRepAutoBlockSignal
{
    sigset_t m_save;
    int m_sig;
    NzRepAutoBlockSignal(int sig)
    {
        sigset_t new_mask; 

        m_sig = sig;
        sigemptyset(&new_mask);
        sigaddset(&new_mask, sig);

        int ret = pthread_sigmask(SIG_BLOCK, &new_mask, &m_save);
        if (ret != 0)
            throwFencedError("Couldn't block signal");

        // If we are in a signal hanlder
        // We may have SIGCHLD or SIGINT temporarilg blocked
        // by the handler. Don't restore blocked mask
        if (sigismember(&m_save, sig)) {
            sigdelset(&m_save, sig);
        }
    }

    ~NzRepAutoBlockSignal()
    {
        int ret = pthread_sigmask(SIG_SETMASK, &m_save, NULL);
        if (ret != 0)
            throwFencedError("Couldn't unblock signal");
    }
};




struct NzRepAutoHandleSignal
{
    struct sigaction m_save;
    int m_sig;

    typedef void (*fenced_handler)(int, siginfo_t*, void*);

    NzRepAutoHandleSignal(int sig, fenced_handler handler)
    {
        m_sig = sig;
        struct sigaction newaction;
    
        memset(&newaction, 0, sizeof(newaction));
        
        newaction.sa_sigaction = handler;
        newaction.sa_flags = SA_RESTART | SA_SIGINFO;
    
        if (sigaction(sig, &newaction, &m_save) == -1)
        {
            throwFencedError("Sigaction failed: %s", strerror(errno));
        }
    }

    ~NzRepAutoHandleSignal()
    {
        if (sigaction(m_sig, &m_save, NULL) == -1)
        {
            throwFencedError("Sigaction failed: %s", strerror(errno));
        }
    }
};


#endif
