#ifndef NZTICKS_INCLUDED
#define NZTICKS_INCLUDED

// Support for the GET_TICKS macros

// WARNING: The basic GET_TICKS is 32-bits and will wrap.

// WARNING: GET_TICKS64 returns incorrect values for the Sparrow/405.
//          The 405 Timebase returns one tick per clock cycle (say 266M/sec)
//          instead of the 16M ticks/sec for the Mercury/855.
//          For perfstats, a new GET_TICKS64_US will return time values
//          with units of 1M ticks/second for all platforms.

#if FOR_SPU
// Nucleus SPU or Linux SPU, or SPU simulator

#include "clocks.h"

#define GET_TICKS()                 clkReturnTimebaseCounterLow()
#define GET_TICKS_US()              (clkTimebaseTicksToUsec(clkReturnTimebaseCounterLow()))
#define GET_TICKS64()               clkReturnTimebaseCounter()
#define GET_TICKS64_US()            (clkTimebaseTicksToUsec(clkReturnTimebaseCounter()))
#define GET_TICK_DELTA(tbRef)       (clkReturnTimebaseCounterLow()-(uint32)(tbRef))
#define GET_TICK_DELTA_US(tbRef)    clkTimebaseTicksToUsec(GET_TICK_DELTA((tbRef)))
#define GET_TICK64_DELTA(tbRef)     clkCalcTimeBaseDelta((tbRef))
#define GET_TICK64_DELTA_US(tbRef)  clkCalcTimeBaseDeltaUsec((tbRef))

#else
// Linux Host

#include <sys/time.h>

inline static uint64
UnixGetTimeInTicks64()
{
    struct timeval tv;
    gettimeofday(&tv,0);
    return tv.tv_sec * 1000000ULL + tv.tv_usec;
}

inline static unsigned
UnixGetTimeInTicks()
{
    return (unsigned) UnixGetTimeInTicks64();
}

#define GET_TICKS()                 UnixGetTimeInTicks()
#define GET_TICKS_US()              UnixGetTimeInTicks()
#define GET_TICKS64()               UnixGetTimeInTicks64()
#define GET_TICKS64_US()            UnixGetTimeInTicks64()
#define GET_TICK_DELTA(tbRef)       (UnixGetTimeInTicks() - (unsigned) (tbRef))
#define GET_TICK_DELTA_US(tbRef)    (UnixGetTimeInTicks() - (unsigned) (tbRef))
#define GET_TICK64_DELTA(tbRef)     (UnixGetTimeInTicks64() - (uint64) (tbRef))
#define GET_TICK64_DELTA_US(tbRef)  (UnixGetTimeInTicks64() - (uint64) (tbRef))

#endif // FOR_SPU

#endif // NZTICKS_INCLUDED
