/* (C) Copyright IBM Corp. 2005, 2011  All Rights Reserved. */

#ifndef NZTIMESTAMP_INCLUDED
#define NZTIMESTAMP_INCLUDED

// 23-Feb-2005: more uniform treatment of 64-bit microsecond "timestamps"

#ifndef NZSTD_INCLUDED
# include "nzstd.h"
#endif

#ifndef NZTYPES_INCLUDED
# include "nztypes.h"
#endif

#ifndef NZTICKS_INCLUDED
# include "nzticks.h"
#endif

#include <stdio.h> //for FILE

#if defined(__DCC__)
//  gettimeofday not in DIAB C Library, in nztimestamp.cpp
CEXTERN int gettimeofday(struct timeval *tv, struct timezone *tz);
#endif // __DCC__

#define FORMAT_TIME64_BUFSIZE (sizeof("YYYY-MM-DD hh:mm:ss.uuuuuu ZONE")+1)
#define FORMAT_TIME64_MICROSEC_POS ((int)sizeof("YYYY-MM-DD hh:mm:ss"))

#define SECS_TO_MICROSECS (1000000LL)
#define MICROSECS_TO_NANOSECS (1000LL)

// format 64-bit timestamp with microseconds
// timestr must be at least FORMAT_TIME64_BUFSIZE bytes
void formattime64 (const time64_t& stamp, char* timestr);

// convert time64 to Postgres 'Timestamp'
Timestamp time64ToTimestamp (const time64_t& stamp);

/////////////////////////////////
/// Record a time64 to file. Creates a buffer, calls formattime64 on it,
/// then dumps to targetFile.
/// @param targetFile the file to record to. Must not be NULL.
///                   ferror(targetFile) must return 0.
/// @param timeStamp the value to record
/////////////////////////////////
void fprintTime64 (FILE* targetFile, const time64_t& timeStamp);

#if FOR_SPU

extern time64_t time64();

#else // FOR_SPU

inline static time64_t time64()
{
    return GET_TICKS64_US();
}

#endif // FOR_SPU

#if FOR_SPU
# ifndef HCHWMGR_INCLUDED
#  include "hcHwMgr.h"
# endif
# define TICK_TO_TIME64(tick) ((int64) hc::HwMgr::tickToTime64(tick))
#else // FOR_SPU
# define TICK_TO_TIME64(tick) ((int64) (tick))
#endif // FOR_SPU


//#define NZ_PATH_TIMING   1

#if FOR_SPU && defined(NZ_PATH_TIMING)

# define NZSPU_PATH_TIMING_BEGIN() \
    uint64 const _nzp_start_ts = GET_TICKS64()

# define NZSPU_PATH_TIMING_LOG_IF_OVER(_usecs, _str, _data1, _data2) do { \
    uint64 const elapsed = GET_TICK64_DELTA_US(_nzp_start_ts); \
    if (NZ_UNLIKELY(elapsed > (_usecs))) \
        nzSpuPathTimingLog(_nzp_start_ts, elapsed, (_str), (int)(_data1), (int)(_data2)); \
} while (0)

extern void nzSpuPathTimingLog(const uint64& start_ts, const uint64& elapsed, const char *str, int d1, int d2);

#else // NZ_PATH_TIMING

# define NZSPU_PATH_TIMING_BEGIN()
# define NZSPU_PATH_TIMING_LOG_IF_OVER(_usecs, _str, _data1, _data2)

#endif // NZ_PATH_TIMING

/////////////////////////////////
/// Return the amount of microseconds elapsed since startTime
/// @param startTime
/// @return current time minus startTime
/////////////////////////////////
inline static int64 durationSince(const time64_t& startTime)
{
    time64_t const now = time64();
    return now - startTime;
}

/////////////////////////////////
/// Return the amount of microseconds elapsed since startTime;
/// update startTime with current time.
/// @param[out] startTime read, then set to current time
/// @return current time minus original value of startTime
/////////////////////////////////
inline static int64 getDurationAndUpdate(time64_t *startTime)
{
    time64_t const now = time64();
    int64 const duration = now - (*startTime);
    (*startTime) = now;
    return duration;
}


#endif // NZTIMESTAMP_INCLUDED
