// nztypes.h
//
// definitions of fundamental types shared across many components
//
// This header file is included in many other include files system-wide, so
// any additions to it must be applicable to a very wide variety of
// situations.
//
// NOTE TO DEVELOPERS:
//
// Please be careful about what you add to this file. Changes to this file
// impact almost every other source file in the system.
//
// (C) Copyright IBM Corp. 2002, 2011  All Rights Reserved.

#ifndef NZTYPES_INCLUDED
#define NZTYPES_INCLUDED

#ifndef NZSTD_INCLUDED
# include "nzstd.h"
#endif
#include <limits.h>
#include "comtypes.h"

#ifndef WIN32
# ifndef PTHREAD_INCLUDED
#  include <pthread.h>
#  define PTHREAD_INCLUDED
# endif // PTHREAD_INCLUDED
typedef pthread_mutex_t nzmutex_t;
#endif // !WIN32

typedef char record_t;
typedef int32 table_id_t;
typedef int64 TRowId;
typedef uint64 TRecLoc;
typedef uint64 nzotuid_t;       // Netezza Owner's Time Unique ID
typedef uint16 TSpuId; // actually a data slice id
#if !defined(AIX)
typedef int64 time64_t;
#endif

//
// postgres standard types
//
#ifdef __cplusplus
extern "C" {
#endif

typedef int64 Timestamp;
typedef int32 AbsoluteTime;
typedef int32 AbsoluteTimeus;
typedef int32 DateADT;
typedef int64 TimeADT;
typedef uint32 TNumericDigit;

typedef int      ColumnId;      ///< User columns (positive) and specials (negative)
typedef unsigned ColumnPos;     ///< User column position (user columns only,
                                ///< no specials, strictly ordinal)

#ifdef __cplusplus
}
#endif

// "vlen_t" used for variable length data size word
// we want to support different size word sizes
// using vlen will make it easier to find the dependencies

// NOTE: If you change VLEN_T or the typedef of vlen_t, be sure to change both.

#define VLEN_T 16       // 16-bit length-word size
typedef uint16 vlen_t;

#if VLEN_T == 16
# define ALIGN_VARLENS ALIGN_SHORT
#else
# if VLEN_T == 32
#  define ALIGN_VARLENS ALIGN_INT32
# else
#  error "Bad definition of VLEN_T"
# endif
#endif

#endif
