/*
 *  nzudsfenced.h
 *  Netezza C++ wrapper for nzuds
 * (C) Copyright IBM Corp. 2009, 2011  All Rights Reserved.
 */

#ifndef __NZUDSFENCED_H__
#define __NZUDSFENCED_H__


#include "nzudscpp.h"



class NzUdsFenced
{
public:
    NzUdsFenced();
    NzUdsFenced(const NzudsCppBuffer & arg);
    ~NzUdsFenced();

    void setData(const NzudsCppBuffer & arg);
    void setDataNoCopy(const NzudsCppBuffer & arg);

    void writeNull(int type);

    void write(int8_t data, int type=nz::udx::UdxBase::UDX_INT8);
    void write(int16_t data);
    void write(int32_t data, int type=nz::udx::UdxBase::UDX_INT32);
    void write(int64_t data, int type=nz::udx::UdxBase::UDX_INT64);
    void write(float data);
    void write(double data);
    void write(bool data, int type=nz::udx::UdxBase::UDX_BOOL);
    void write(const nz::udx::StringArg &data, int type, bool null);
    void write(const TimeTzADT  &data);
    void write(const Interval &data);

    // The following should be called for both null and non null
    // Since there is a need to write metadata as well
    void write(const nz::udx::StringReturn &data, int type, bool null);
    void write(const nz::udx::Numeric32Val &data, bool null);
    void write(const nz::udx::Numeric64Val &data, bool null);
    void write(const nz::udx::Numeric128Val &data, bool null);    
    


    int8_t readInt8(bool &isnull);
    int16_t readInt16(bool &isnull);
    int32_t readInt32(bool &isnull);
    int64_t readInt64(bool &isnull);
    float readFloat(bool &isnull);
    double readDouble(bool &isnull);
    bool readBool(bool &isnull);
    int32_t readDate(bool &isnull);
    int64_t readTime(bool &isnull);
    int64_t readTimeStamp(bool &isnull);

    nz::udx::StringArg *readString(bool &isnull, nz::udx::StringArg& arg, int type);
    nz::udx::StringReturn *readString(bool &isnull, nz::udx::StringReturn& arg, int type);
    Interval* readInterval(bool &isnull, Interval& arg);
    TimeTzADT* readTimeTz(bool &isnull, TimeTzADT& arg);
    nz::udx::Numeric32Val* readNumeric(bool &isnull, nz::udx::Numeric32Val& arg);
    nz::udx::Numeric64Val* readNumeric(bool &isnull, nz::udx::Numeric64Val& arg);
    nz::udx::Numeric128Val* readNumeric(bool &isnull, nz::udx::Numeric128Val& arg);

    int getNextType();

    void getBuffer(NzudsCppBuffer &buffer);
    void getBufferCompressed(NzudsCppBuffer &buffer);
    void readNext(NzudsCppData &data);
    int numElements();
    void setReadPosition(int pos);
    void shrinkTo(int num);
    
private:
    Nzuds *m_handle;

};

#endif
