//------------------------------------------------------------------------------
// rcAtomic.h
//
// For now, a simple wrapper around the OS-specific atomic.h includes.
//
// (C) Copyright IBM Corp. 2002, 2011  All Rights Reserved.
//
// Changes:
//
// 09-18-02  dick       Initial version.

#ifndef RCATOMIC_INCLUDED
#define RCATOMIC_INCLUDED

#include "nzstd.h" // for int64

#ifdef __cplusplus
extern "C" {
#endif

#define ATOMIC_INIT(i)          { (i) }
#define atomic_read(v)          ((v)->counter)
#define atomic_set(v,i)         (((v)->counter) = (i))

#if FOR_CLI || WIN32 // assume irrelvant when neither potentially-SMP host nor SPU

  typedef struct { int counter; } atomic_t;
  typedef struct { int64 counter64; } atomic64_t;

#define ATOMIC64_INIT(i)        { (i) }
#define atomic64_read(v)        ((v)->counter64)
#define atomic64_set(v,i)       (((v)->counter64) = (i))

# define atomic_add(i,v)        (((v)->counter) += (i))
# define atomic64_add(i,v)      (((v)->counter64) += (i))
# define atomic_sub(i,v)        (((v)->counter) -= (i))
# define atomic64_sub(i,v)      (((v)->counter64) -= (i))
# define atomic_inc(v)          (++((v)->counter))
# define atomic64_inc(v)        (++((v)->counter64))
# define atomic_dec(v)          (--((v)->counter))
# define atomic64_dec(v)        (--((v)->counter64))
# define atomic_inc_return(v)   (++((v)->counter))
# define atomic64_inc_return(v) (++((v)->counter64))
# define atomic_dec_return(v)   (--((v)->counter))
# define atomic64_dec_return(v) (--((v)->counter64))
# define atomic_set_mask(m,v)   ((v)->counter |= (m))
# define atomic_clear_mask(m,v) ((v)->counter &= ~(m))
# define atomic_cmpxchg(v, o, n) ({ \
    int const t = atomic_read(v); \
    if (t == (o)) \
        atomic_set((v), (n)); \
    t; \
})
# define atomic_xchg(v, val) ({ \
    int const t = atomic_read(v); \
    atomic_set((v), (val)); \
    t; \
})
# define atomic64_xchg(v, val) ({ \
    int64 const t = atomic64_read(v); \
    atomic64_set((v), (val)); \
    t; \
})

#else  // provide non-trivial defintions

  typedef struct { volatile int counter; } atomic_t;

// pick up processor specific definitions
# if __PPC__
#  include "ppc_atomic.h"
# else
#  include "i386_atomic.h"
# endif

#endif

#ifdef __cplusplus
}
#endif

#endif /* RCATOMIC_INCLUDED */
