//------------------------------------------------------------------------------
// rcBool.h
//
// A boolean type with enumeration values for yes, no, as is, and bad.
//
// (C) Copyright IBM Corp. 2002, 2011  All Rights Reserved.
//
// Changes:
//
// 01-23-02  dick       Initial version.

#ifndef RCBOOL_INCLUDED
#define RCBOOL_INCLUDED

#ifndef NZSTD_INCLUDED
# include "nzstd.h"
#endif

//------------------------------------------------------------------------------
// RcBool is an enum of boolean values that allows 'as is' and 'bad' in
// addition to true (yes) and false (no). 
//
// It is a global enum (rather than class scoped) in order to shorten the symbol
// name for convenience - since it's a highly used enum.

enum RcBool {
    RC_B_BAD = -2, RC_B_AS_IS = -1, RC_B_NO = 0, RC_B_YES = 1,

    RC_B_FALSE = RC_B_NO,
    RC_B_TRUE  = RC_B_YES
};

namespace rc {  // rc == reComps

class Err;

//==============================================================================
// Class: Bool - a class that holds a boolean enumeration value and translates
// to and from strings.

class Bool {
private:
    RcBool  m_val;

public:
    // CREATORS
    Bool (int val = RC_B_NO);
    Bool (const char* symbol);
    Bool (const Bool& src);
    ~Bool ();

    // MANIPULATORS
    Bool&   operator= (const Bool& src);

    // ACCESSORS
                        operator void* () const;
    bool                operator! () const;

    RcBool              val () const;
    static RcBool       val (const char* symbol);

    const char*         symbol () const;
    static const char*  symbol (int val);

    const char*         label () const;
    static const char*  label (int val);

    static RcBool       parse (const char* symbol, rc::Err* err = NULL);
    static RcBool       needArg (const char* keyword, const char* value);
};

//------------------------------------------------------------------------------
// INLINE ACCESSORS

inline RcBool Bool::val () const
{
    return m_val;
}

inline Bool::operator void* () const
{
    return (void*) (m_val == RC_B_YES);
}

inline bool Bool::operator! () const
{
    return (m_val != RC_B_YES);
}

} // end namespace rc

#endif /* RCBOOL_INCLUDED */



