//------------------------------------------------------------------------------
// rcPgmClass.h
//
// An enumeration of program classes, mostly for rc::Env.
//
// (C) Copyright IBM Corp. 2002, 2011  All Rights Reserved.
//
// Changes:
//
// 01-22-02  dick       Initial version.

#ifndef RCPGMCLASS_INCLUDED
#define RCPGMCLASS_INCLUDED

#ifndef NZSTD_INCLUDED
# include <nzstd.h>
#endif

namespace rc {  // rc == reComps

//==============================================================================
// Class: PgmClass - a class that holds a program class value and translates to
// and from strings.

class PgmClass {
public:
    enum Val {
        UNKNOWN_PGM_CLASS, DAEMON, SINGLE_DAEMON, CLI_APP, GUI_APP,
        SUB_PROCESS
    };

private:
    Val  m_id;

public:
    PgmClass (int id = UNKNOWN_PGM_CLASS);
    PgmClass (const char* symbol);
    PgmClass (const PgmClass& src);
    ~PgmClass ();

    // MANIPULATORS
    PgmClass&   operator= (const PgmClass& src);

    // ACCESSORS
    Val                id () const;
    static Val         id (const char* symbol);

    const char*         symbol () const;
    static const char*  symbol (int id);

    const char*         label () const;
    static const char*  label (int id);

    static bool         isApp (int id);
    static bool         isDaemon (int id);
    static bool         isSingleDaemon (int id);
    static bool         isBackgroundProc (int id);
};

//------------------------------------------------------------------------------
// INLINE ACCESSORS

inline rc::PgmClass::Val PgmClass::id () const
{
    return m_id;
}

} // end namespace rc

#endif /* RCSEVERITY_INCLUDED */



