//------------------------------------------------------------------------------
// rcRefCnt.h
//
// An abstract base class that provides an interface for reference counting
// objects. It is not implemented as a concrete class so that derived classes
// can decide on a memory-management strategy.
//
// (C) Copyright IBM Corp. 2002, 2011  All Rights Reserved.
//
// Changes:
//
// 01-19-02  dick       Initial version.

#ifndef RCREFCNT_INCLUDED
#define RCREFCNT_INCLUDED

#ifndef NZSTD_INCLUDED
# include "nzstd.h"
#endif

namespace rc {  // rc == reComps

//==============================================================================
// Class: RefCnt - a reference count and interface methods for implementing
// reference-counted objects.

class RefCnt {
private:
    int m_references;

public:
    // CREATORS
    RefCnt ();
    virtual ~RefCnt ();

    // MANIPULATORS
    virtual void    addReference ();
    virtual int     removeReference (); // Obj deleted by owner when refs == 0

    // ACCESSORS
    int             references () const;

protected:
    RefCnt (int references);
    RefCnt (const RefCnt& src);

private:
    RefCnt& operator= (const RefCnt& src);                      // Unimplemented
};

//------------------------------------------------------------------------------
// INLINE MANIPULATORS

inline void RefCnt::addReference ()
{
    m_references++;
}

inline int RefCnt::removeReference ()
{
    m_references--;

    return m_references;
}

//------------------------------------------------------------------------------
// INLINE ACCESSORS

inline int RefCnt::references () const
{
    return m_references;
}

} // end namespace rc

#endif
