//------------------------------------------------------------------------------
// rcSeverity.h
//
// An enumeration of severity levels for output messages, errors, etc.
//
// (C) Copyright IBM Corp. 2002, 2011  All Rights Reserved.
//
// Changes:
//
// 01-19-02  dick       Initial version.

#ifndef RCSEVERITY_INCLUDED
#define RCSEVERITY_INCLUDED

#ifndef NZSTD_INCLUDED
# include <nzstd.h>
#endif

//------------------------------------------------------------------------------
// RcSeverity is an enum of message/error severity levels. 
//
// It is a global enum (rather than class scoped) in order to shorten the symbol
// name for convenience - since it's a highly used enum.

enum RcSeverity {
    RC_NO_SEVERITY = 0, RC_DEBUG, RC_INFO, RC_WARNING, RC_ERROR, RC_NON_FATAL,
    RC_FATAL, RC_BAD_SEVERITY
};

namespace rc {  // rc == reComps

//==============================================================================
// Class: Severity - a class that holds a severity value and translates to
// and from strings.

class Severity {
private:
    RcSeverity  m_id;

public:
    Severity (int id = RC_NO_SEVERITY);
    Severity (const char* symbol);
    Severity (const Severity& src);
    ~Severity ();

    // MANIPULATORS
    Severity&   operator= (const Severity& src);

    // ACCESSORS
    RcSeverity          id () const;
    static RcSeverity   id (const char* symbol);

    const char*         symbol () const;
    static const char*  symbol (int id);

    const char*         label () const;
    static const char*  label (int id);

    static bool         isErrorSeverity (int id);
};

//------------------------------------------------------------------------------
// INLINE ACCESSORS

inline RcSeverity  Severity::id () const
{
    return m_id;
}

//------------------------------------------------------------------------------
// Deprecated symbols - here for backwards compatibility

#define RcErrSeverity   RcSeverity
#define ES_NONE         RC_NO_SEVERITY
#define ES_INFO         RC_INFO
#define ES_WARNING      RC_WARNING
#define ES_ERROR        RC_ERROR
#define ES_FATAL        RC_FATAL
#define ES_NON_FATAL    RC_NON_FATAL
#define ES_BAD_SEVERITY RC_BAD_SEVERITY

} // end namespace rc

#endif /* RCSEVERITY_INCLUDED */



