#ifndef ROWIDGEN_INCLUDED
#define ROWIDGEN_INCLUDED

#ifndef NZTYPES_INCLUDED
# include "nztypes.h" // for TRowId
#endif

struct dbos_msg_header_t;


class RowIdGen
{
    // per-process block of row ids
    static TRowId s_row_counter;
    static TRowId s_row_counter_fence;

public:

    /**
     * Pre-fork initialization for SPU and DBOS.
     */
    static void rowIdGenInitPreFork();

    /**
     * Get the next unique row number.
     */
    static TRowId getRowNum();

    /**
     * Get another batch of row numbers
     */
    static void getMoreRowNums();

    /**
     * Supply SPU with a range of rowids
     */
    static void sendMoreRowIdsToSpu(int spuId);

    /**
     * update rowid cache with range of rowids in msg
     */
    static void handleRowidsMsg(const dbos_msg_header_t *rowids_msg);

    /**
     * Update rowid fence to passed value
     */
    static void setMinRowIdFence(const TRowId& minfence);

    /**
     * Get current fence
     */
    static TRowId getRowIdFence();

#if FOR_SPU
    /**
     * invalidate rowids on SPUs to force them to ask for more and
     * get in synch with new fence
     */
    static void InvalidateRowIds();
#endif
};

inline TRowId
RowIdGen::getRowNum()
{
    if (NZ_UNLIKELY(s_row_counter >= s_row_counter_fence))
        getMoreRowNums();

    return s_row_counter++;
}

#endif
