// Copyright 2008-2010 Netezza Corporation.  All rights reserved.

#ifndef SPUCBS_INCLUDED
#define SPUCBS_INCLUDED

#include "spudisksched.h"

#ifdef __cplusplus
extern "C" {
#endif

// SPU compression buffer scheduling and scan stream generator API

struct cbs_cfg_t
{
    unsigned int ccf_rmin;
    unsigned int ccf_rmax;
    unsigned int ccf_keep_thresh;
};

// cbs_create_stream option bits
enum
{
    CBS_OPT_DEFAULT = 0,
    CBS_OPT_UNORDERED = 1,
    CBS_OPT_START_STREAM = 2,
};

typedef unsigned long cbs_stream_t;

struct cbs_init_stream_t
{
    const struct scanlist *so_sl;  // blocks to be scanned
    int so_ds_fd;                  // disk handle for stream
    const void *so_regmap;         // FPGA setup for stream
    unsigned int so_regmap_id;     // FPGA setup unique id
    unsigned int so_min_np;        // min num pages needed for read of single raw (compressed) disk page
    unsigned int so_options;       // CBS_OPT_* bitmask
    int so_planid;                 // for logging
    int so_snippetnum;             // for logging
    unsigned int so_requesthandle; // pass-through to TDiskJob::djRequestHandle

    unsigned long so_reserved1;    // reserved
    unsigned long so_reserved2;    // reserved
    unsigned long so_reserved3;    // reserved
};

struct cbs_stream_stats_t
{
    unsigned int cst_rmin;
    unsigned int cst_rmax;
    unsigned int cst_curjobs;
};

#define CBS_PAGE_SHIFT 17   /* for NZ_TABLE_DEFAULT_BLOCK_SIZE */
#define CBS_PAGE_SIZE (1UL << CBS_PAGE_SHIFT)
#define CBS_PAGE_MASK (~((1 << CBS_PAGE_SHIFT) - 1))

extern void cbs_init(void);
extern int cbs_init_shared_pools(unsigned int num_shared, unsigned int num_reserve);
extern int cbs_get_shared_pool_size(unsigned int *num_shared, unsigned int *nr_resv);
extern int cbs_init_ibufs(unsigned int device, int host_ibenable, int fpga_ibenable, unsigned int nr_host_ibufs);
extern int cbs_get_nr_ibufs(unsigned int device, unsigned int *nr_ibufs);
extern void cbs_configure(unsigned int device, struct cbs_cfg_t *curcfg, const struct cbs_cfg_t *newcfg);
extern cbs_stream_t cbs_create_stream(unsigned int device);
extern int cbs_get_fd(cbs_stream_t);
extern int cbs_init_stream(cbs_stream_t stream, const struct cbs_init_stream_t *so);
extern void cbs_start_stream(cbs_stream_t stream);
extern void cbs_pause_stream(cbs_stream_t stream);
extern void cbs_close_stream(cbs_stream_t stream);
extern void cbs_stream_data_consumed(cbs_stream_t stream, struct DiskJob *dj);
extern void cbs_get_stream_stats(cbs_stream_t stream, struct cbs_stream_stats_t *cst);
extern size_t cbs_dump(unsigned int device, char *buf, size_t len);

inline static size_t
cbs_page_size(void)
{
    return CBS_PAGE_SIZE;
}

#ifdef __cplusplus
}
#endif

#endif
