/*
 * Copyright 2007-2010 Netezza Corporation.  All rights reserved.
 */
#ifndef SPUDISKSCHED_INCLUDED
#define SPUDISKSCHED_INCLUDED

#include "nzsector.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __KERNEL__
# if USERSPACE_IOPATH
#  define DS_IOALIGN 32
# else
#  if NZ_SPU_HW_REV == 7
    // using nzhw.ko, 32 byte alignment requirement
#   define DS_IOALIGN 32
#  elif NZ_SPU_HW_REV == 10
    // FPGA nsoc driver requires 512 byte alignment for raw IO
#   define DS_IOALIGN 512
#  else
//#  error "missing definition of DS_IOALIGN"
#  endif
# endif
#endif // __KERNEL__

#define STRICT_WO_TAG ((uint64_t) -1)

#define DSPRI_SQB 0x40000000

#define DS_PRIO(p, sqb) ((int)((int)(p) >= 0 && (sqb) ? ((int)(p) | DSPRI_SQB) : (int)(p)))
#define DS_GET_PRIO(pri) ((int)((int)(pri) < 0 ? (int)(pri) : ((int)(pri) & ~DSPRI_SQB)))
#define DS_GET_SQB(pri) ((int)(pri) > 0 && ((int)(pri) & DSPRI_SQB) != 0)

#define DS_RT_PRIORITY   -1

#define DS_ADVISE_DONTNEED 1
#define DS_ADVISE_INVALIDATE 2

#if !defined(__KERNEL__) && USERSPACE_IOPATH
    typedef unsigned long ds_handle_t; // opaque pointer
#else
    typedef int ds_handle_t; // biased file descriptor
#endif

struct DiskJob;

struct ds_cfg_t
{
    unsigned int dscfg_starvation_threshold;
    unsigned int dscfg_elevator_depth;
};

struct ds_busy_t
{
    uint64_t dsb_busy_accum;
    uint64_t dsb_last_db_job_tick;
};

extern void ds_init(void);
extern void ds_configure(unsigned int device, struct ds_cfg_t *curcfg, const struct ds_cfg_t *newcfg);
extern void ds_init_nzeng_params(unsigned int emulator_weight);

extern ds_handle_t ds_open_handle(unsigned int device, int priority);
extern int ds_get_fd(ds_handle_t handle);
extern ds_handle_t ds_handle_from_fd(int fd);
extern int ds_set_priority(ds_handle_t handle, int newprio);
extern int ds_get_priority(ds_handle_t handle, int *priority);
extern void ds_close_handle(ds_handle_t handle);
extern int ds_submit_job(ds_handle_t handle, struct DiskJob *job);
extern int ds_wait(ds_handle_t handle, struct DiskJob **dj);
extern int ds_wait_ms(ds_handle_t handle, unsigned int millis, struct DiskJob **dj);
extern int ds_lock_sectors(ds_handle_t handle, nzsector_t start_sector, unsigned int num_sectors);
extern int ds_unlock_sectors(ds_handle_t handle, nzsector_t start_sector, unsigned int num_sectors);
extern size_t ds_dump(unsigned int device, char *buf, size_t len);
extern int ds_busy_ticks(unsigned int device, struct ds_busy_t *dsb);
extern int ds_cache_advise(ds_handle_t handle, int advice, nzsector_t start_sector, unsigned int num_sectors);
extern int ds_cache_page_validated(ds_handle_t handle, nzsector_t start_sector, unsigned int num_sectors, unsigned int cacheid);
extern int ds_count_cached_blocks(ds_handle_t handle, nzsector_t start_sector, unsigned int num_sectors);
extern int ds_discard_sectors(ds_handle_t handle, nzsector_t start_sector, nzsector_t num_sectors);

#ifdef __cplusplus
}
#endif

#endif
