/**
 * terror.h
 * (C) Copyright IBM Corp. 2008, 2014  All Rights Reserved.
 */

#ifndef TERROR_INCLUDED
#define TERROR_INCLUDED


// In general, NDE is an error throwing C++ system.
// However Postgres is not.  Therefore at the C++/C juncture
// caught errors are converted to error structures to send back
// to postgres or any other client using the C-API


//!NOTE-sks: #include "nzlimits.h" fails for some compilations...
#define ASRCLEN 127
#define MAX_ERROR_OBJ_LEN 255
#define MAX_ERROR_WHO_LEN 100
#define MAX_ERROR_STR_LEN 300


// NOTE: ERROR CODE NUMBERS MUST BE UNIQUE
// IF ADDING NON-FATAL ERROR CODES BEFORE NZ_FATAL_ASSERTION, CHECK
// NZ_FATAL_ASSERTION AND HIGHER (THRU NZ_ERROR_LAST) AND RENUMBER IF NECESSARY
// ALSO REMEMBER TO ADD A CORRESPONDING STRING TO NzErrorStrs !!
typedef enum {
    NZ_ERROR_NONE=0,
    NZ_ERROR_GENERIC=1,               // Never user this!
    // UNUSED - Ask a txmgr person before using this:
    NZ_TRANS_ABORT_INTERACTION=2,     // Interaction aborted
    // UNUSED - Ask a txmgr person before using this:
    NZ_TRANS_ABORT_STATEMENT=3,       // Statement aborted
    NZ_TRANS_SERIALIZE=4,             // Could not serialize - transaction aborted
    NZ_ERROR_TRANS_INACTIVE=5,        // transaction dead / not found
    NZ_ERROR_OUT_OF_MEMORY=6,         // Out of memory
    NZ_ERROR_INCORRECT_DATA_TYPE=7,   // Incorrect data type
    NZ_ERROR_ILLVAL=8,                // Illegal value
    NZ_ERROR_SPU_CROSSTALK=9,         // Internal: SPUs crosstalking
    NZ_ERROR_PIPE=10,                  // Internal pipe I/O failed
    NZ_ERROR_NO_HOSTNAME=11,           // Cannot determine hostname
    NZ_ERROR_ROGUE_SPU=12,             // Rogue SPU trying to register
    NZ_ERROR_NO_TABLE=13,              // No such table exists
    NZ_ERROR_NO_FIELD=14,              // No such field in table exists
    NZ_ERROR_TABLE_EXISTS=15,          // Table already exists
    NZ_ERROR_PLAN_EXISTS=16,           // Plan already exists
    NZ_ERROR_IMPL=17,                  // Functionality not implemented
    NZ_ERROR_CAT_FULL=18,              // Cannot get new catalog entry
    NZ_ERROR_DISK_FULL=19,             // Cannot find free space
    NZ_ERROR_INDEX_FULL=20,            // Cannot find free index slot
    NZ_ERROR_BAD_INDEX=21,             // Index points to unallocated block
    NZ_ERROR_BAD_DIR=22,               // dir points to unallocated block
    NZ_ERROR_BAD_DISK=23,              // Bad disk
    NZ_ERROR_TABLE_NOT_IN_EXTENT=24,   // Cannot find table in disk extent
    NZ_ERROR_CORETOOSMALL=25,          // Too few extents in CoreData partition
    NZ_ERROR_NULL_FIELD=26,            // Field cannot contain null values
    NZ_ERROR_TOO_BIG=27,               // Material too big
    NZ_ERROR_TOO_MANY_INTERACTIONS=28, // Too many concurrent interactions
    NZ_ERROR_TOO_MANY_CLIENTS=29,      // Too many clients
    NZ_ERROR_PROCESS_DIMENSION=30,     // internal error processing dimension
    NZ_ERROR_LOAD_DIMENSION=31,        // internal error loading dimension
    NZ_ERROR_DISK_AGGR_GEN=32,         // error generating a disk aggregate
    NZ_ERROR_LOAD_DIM_PROPERTIES=33,   // error loading dimension properties
    NZ_ERROR_LOAD_OLAP_METADATA=34,    // error loading olap metadata
    NZ_ERROR_OLAP_CACHE_BLOCK_TOO_BIG=35,// too big cache block
    NZ_ERROR_READ_ERROR=36,            // Disk read error
    NZ_ERROR_NOT_READY=37,             // System not ready
    NZ_ERROR_OVERFLOW=38,              // Arithmetic overflow
    NZ_ERROR_DISTKEYS=39,              // MAX DISTKEYS
    NZ_ERROR_BAD_SYS_STATE=40,         // State of the system does not allow oper.
    NZ_ERROR_NO_SWAP_PAGES=41,         // Disk temporary work space is full
    NZ_ERROR_RECORD_TOO_BIG=42,        // Record won't fit in a block
    NZ_ERROR_COLUMN_TOO_BIG=43,        // Column size limit exceeded
    NZ_ERROR_TOO_MANY_COLUMNS=44,      // Column count limit exceeded
    NZ_ERROR_MATH=45,                  // math error, e.g. divide by 0
    NZ_ERROR_PG_ELOG=46,               // from pg via elog call (generally from pg expr evals)
    NZ_ERROR_NO_CODE_GEN=47,           // code generation failure
    NZ_TRANS_ROLLBACK_CLIENT=48,       // transaction rollback by client
    NZ_TRANS_ROLLBACK_ADMIN=49,        // transaction rollback by DBA
    NZ_ERROR_NOT_PERMITTED=50,         // operation not permitted
    NZ_ERROR_TOO_MANY_TRANS=51,        // more than 32 concurrent transactions
    NZ_TRANS_ROLLBACK_RESTART=52,      // transaction rollback by restart
    NZ_TRANS_ROLLBACK_DISCONNECT=53,   // transaction rollback due to disconnect
    NZ_ERROR_NO_SORT_BUFFER=54,        // unable to get sort distribution buffe
    NZ_ERROR_INCORRECT_DATA_SIZE=55,   // Incorrect data size
    NZ_ERROR_NO_SESSION=56,            // No such session
    NZ_ERROR_CHAR_WIDTH_EXCEEDED=57,   // char width exceeded
    NZ_ERROR_BAD_DICT=58,                 // bad dictionary
    NZ_ERROR_CLIENT_TIMEOUT=59,        // client not responding
    NZ_ERROR_INSTRUMENTED_FAILURE=60,  // error induced by test harness
    NZ_ERROR_RECLAIM_ACTIVE=61,        // table has active reclaim
    NZ_ERROR_CLIENT_NOT_WAITING=62,    // client not waiting for response
    NZ_ERROR_NO_CLIENT=63,             // no client process
    NZ_ERROR_MEMORY_DEADLOCK=64,       // SPU swapper deadlock avoidance
    NZ_ERROR_AMBIGUOUS_UPDATE=65,      // Ambiguous rows in UPDATE statement
    NZ_ERROR_TRANS_STATE=66,           // transaction in unexpected state
    NZ_ERROR_TEMPTABLEID=67,           // tempTableID unable to remap column
    NZ_ERROR_TRANSL_TABLE_NOT_FOUND=68,// specified table mapping nucls to AAs not found
    NZ_ERROR_BAD_READING_FRAME=69,     // reading frame must be +1,+2,+3,-1,-2 or -3
    NZ_ERROR_RC=70,                    // generic error, translated from rcErr
    NZ_ERROR_NO_PARTITION=71,          // no matching data partition
    NZ_ERROR_TEMP_TIMING=72,           // temporary timing error
    NZ_ERROR_BAD_PROT=73,              // illegal char in protein sequence
    NZ_ERROR_INFLATE=74,               // zlib inflate error
    NZ_TRANSLATE_STRING_LENGTHS=75,    // Translate() FROM string is shorter than TO string
    NZ_ERROR_BAD_RELOAD=76,            // incompatibility found during reload
    NZ_ERROR_STACK_USAGE=77,           // excessive stack usage
    NZ_ERROR_SEQUENCE_EXISTS=78,       // Sequence already exists
    NZ_ERROR_SEQUENCE_EXHAUSTED=79,    // Sequence exhausted
    NZ_ERROR_SEQUENCE_ERROR=80,        // General error for sequence problem
    NZ_ERROR_FCOMM_CHANNEL_BAD=81,     // Invalid FComm channel id
    NZ_ERROR_FCOMM_NODEID_BAD=82,      // Invalid FComm node id
    NZ_ERROR_FCOMM_PAYLOADSIZE_BAD=83, // Invalid FComm payload size
    NZ_ERROR_XML=84,                   // Malformed XML
    NZ_ERROR_FCOMM_API_ALREADY_INITED=85,  // FComm API already initialized
    NZ_ERROR_INVALID_CODEPOINT=86,     // Illegal codepoint value
    NZ_ERROR_BAD_SUBSTRING_LEN=87,     // Illegal value passed as substring length
    NZ_ERROR_TRANS_NOT_RESTARTABLE=88, // transaction was not restartable
    NZ_ERROR_TDJ_INVALID=89,           // transaction detail journal error
    NZ_ERROR_RELEASE_JOB_ABORT=90,     // Used to abort a release job, and not change job state
    NZ_TRANS_REQ_QUEUED=91,            // request was queued to be restarted
    NZ_ERROR_TDJ_MAXOPEN=92,           // max. number of tdj files open
    NZ_ERROR_UDX_EXCEPTION=93,         // Used by writers of udxs to throw exceptions
    NZ_ERROR_OUT_OF_RECLAIM_STATES=94, // All reclaim recovery states in-use (deprecated)
    NZ_TRANS_ROLLBACK_IN_PROGRESS=95,  // more meaningful than NZ_ERROR_NOT_PERMITTED
    NZ_ERROR_LARGE_TABLE_LIMIT=96,     // Large table limit per SPU
    NZ_ERROR_AHIST_STORAGELIMIT_EXCEEDED=97, // ahist staged data exceeds storage limit
    NZ_ERROR_AHIST_DFULLTHRESHOLD_BROKEN=98, // ahist staged data violates Disk Full Threshold
    NZ_ERROR_AHIST_CAPTURE_FAILED=99,  // ahist capture failed for unspecified reason
    NZ_ERROR_AHIST_LOAD_CONNECTION=100,// ahist load connection could not be est'd
    NZ_ERROR_AHIST_LOAD_INFO_LOST=101, // ahist load failed because we were unable to retrieve
                                       // the configuration info
    NZ_ERROR_AHIST_LOAD_FAILED=102,    // ahist load failed for an unspecified reason
    NZ_ERROR_NO_TRANSACTION=103,       // Transaction required, but no transaction started
    NZ_ERROR_SCSI_CDB=104,             // SCSI CDB error
    NZ_ERROR_SCSI_RESPONSE_PARSING=105,// SCSI Resonse Could not be fully parsed
    NZ_ERROR_LIBRARY_EXISTS=106,       // Library already exists
    NZ_ERROR_LIBRARY_ERROR=107,        // General error for library problem
    NZ_ERROR_FC_ATTACH_FAILED=108,     // FC_APIAttach could not connect
    NZ_ERROR_NO_DISKSN=109,            // No primary and mirror disksn for device
    NZ_ERROR_MDM_UNSUPPORTED=110,      // Functionality (temporarily?) not supported under MDM
    NZ_ERROR_MDM_BAD_HANDLE=111,       // Stale or corrupted handle in an MDM API
    NZ_ERROR_TABLE_IN_GROOM=112,       // Groom in progress on table
    NZ_ERROR_GROOM_OTHER_UPDATERS=113, // Table to groom is being modified by other transactions
    NZ_DPM_NO_FREE_STORAGE_TRANSACTION=114,// Storage transaction table is full
    NZ_DPM_NO_FREE_EXTENT=115,         // Data partition is full
    NZ_ERROR_CONCURRENT_UPDATE=116,    // Concurrent update or delete of same row
    NZ_ERROR_SEQUENCE_DROPPED= 117,    // sequence dropped
    NZ_ERROR_DATA_SLICE_UPGRADE_NEEDED=118, // Need to upgrade metadata to DPM 
    NZ_DPM_EXTENT_IN_USE=119,          // Unable to acquire extent
    NZ_ERROR_TRANSIENT_COMPRESS=120,   // Error in transient table handling
    NZ_ERROR_TABLE_IN_TRUNCATE=121,    // cannot access table that is being truncated
    NZ_ERROR_BAD_PAGE_CRC=122,         // Bad CRC reading disk page
    NZ_ERROR_TRUNCWMLOG_INVALID=123,   // truncate water-mark file error
    NZ_DPM_NOTE_DELETE_UNKNOWN_STORAGE_ID=124, // Storage is not part of current table
    NZ_DPM_STALE_TCHAND=125,           // Stale table connection handle
    NZ_ERROR_RULE_ABORTED=126,         // Job has been aborted by scheduler abort rule
    NZ_ERROR_FUNCTION_SEQUENCE=127,    // Function called when some affected
                                       // object is not in an appropriate state
    NZ_ERROR_SRC_CRC=128,              // bad CRC on SRC reuse attempt
    NZ_ERROR_REDR_SPACE_CHECK_FAILED=129, // REDR free disk space check failed
    NZ_ERROR_SRC_FAILURE=130,          // miscellaneous add-to-cache failure
    NZ_TRANS_PLAN_WAIT=131,            // Timed out waiting for plan completion
    NZ_ERROR_PARENT_TRANS_STATE=132,   // Parent transaction in unexpected state
    NZ_ERROR_DEVICE_IS_BUSY=133,       // Device or resource busy

// ALL NON-FATAL ERRORS GO BEFORE THIS!
// WHEN ADDING NEW NON-FATAL ERROR CODES HERE, REMEMBER TO CHECK
// AND UPDATE (IF NEEDED) THE
// NUMBERS FOR THE FATAL ERROR CODES THAT FOLLOW
// ALSO REMEMBER TO ADD A CORRESPONDING STRING TO NzErrorStrs[] (below) !!

    NZ_FATAL_ASSERTION=134,             // Internal inconsistency
    NZ_FATAL_TXMGR=135,                 // transaction manager
    NZ_FATAL_SWAPMGR=136,               // SPU swap manager
    NZ_FATAL_HEAP=137,                  // NzFree/AllocObject heap problem
    NZ_FATAL_FILE_ACCESS=138,           // unable to access system file
    NZ_FATAL_SPU_EXCEPTION=139,         // SPU micro-processor exception
    NZ_FATAL_AHIST=140,                 // A failure in the audithist mechanism

// IF ADDING A FATAL ERROR CODE, ADD IT HERE
// AND ADD IT TO NzErrorStrs
// AND UPDATE NZ_ERROR_LAST:
    NZ_ERROR_LAST=141                   // THIS MUST BE LAST IN TABLE
} NzErrorCode;

#ifdef __cplusplus
// "Expected" error codes for which 'cliqa -abort -1' should not dump core.
inline bool NzErrorIsExpected ( int code )
{
    return (code == NZ_TRANS_SERIALIZE)
        || (code == NZ_ERROR_NO_SESSION)
        || (code == NZ_ERROR_TRANS_INACTIVE)
        || (code == NZ_TRANS_ROLLBACK_CLIENT)
        || (code == NZ_TRANS_ROLLBACK_ADMIN)
        || (code == NZ_TRANS_ROLLBACK_RESTART)
        || (code == NZ_TRANS_ROLLBACK_DISCONNECT)
        || (code == NZ_TRANS_REQ_QUEUED)
        ;
}

// "fatal" error codes
// out of range error codes (NZ_ERROR_LAST or greater) also count as fatal ?
inline bool NzErrorIsFatal ( int code )
{
    return (code >= NZ_FATAL_ASSERTION || code < NZ_ERROR_NONE );
}
#endif // __cplusplus

#if MAIN_ERROR_FILE

// WHEN ADDING AN ERROR CODE TO NzErrorCode,
// ALSO REMEMBER TO ADD A CORRESPONDING STRING HERE !!
// The following table will be machine generated SOON:
// Yes, please - see Bug 2115.4
const char *NzErrorStrs[NZ_ERROR_LAST+1] = {
    "(no error code provided)",                 // NZ_ERROR_NONE
    "Operation failed",                         // NZ_ERROR_GENERIC
    "Interaction aborted",                      // NZ_TRANS_ABORT_INTERACTION
    "Statement aborted",                        // NZ_TRANS_ABORT_STATEMENT
    "Could not serialize - transaction aborted", // NZ_TRANS_SERIALIZE
    "Transaction no longer active",             // NZ_ERROR_TRANS_INACTIVE
    "Out of memory",                            // NZ_ERROR_OUT_OF_MEMORY
    "Incorrect data type",                      // NZ_ERROR_INCORRECT_DATA_TYPE
    "Illegal value",                            // NZ_ERROR_ILLVAL
    "Internal: SPUs crosstalking",              // NZ_ERROR_SPU_CROSSTALK
    "Internal pipe I/O failed",                 // NZ_ERROR_PIPE
    "Cannot determine hostname",                // NZ_ERROR_NO_HOSTNAME
    "Rogue SPU trying to register",             // NZ_ERROR_ROGUE_SPU
    "No such table exists",                     // NZ_ERROR_NO_TABLE
    "No such field exists in table",            // NZ_ERROR_NO_FIELD
    "Table already exists",                     // NZ_ERROR_TABLE_EXISTS
    "Plan already exists in plan dictionary",   // NZ_ERROR_PLAN_EXISTS
    "Functionality not implemented",            // NZ_ERROR_IMPL
    "Cannot get new catalog entry",             // NZ_ERROR_CAT_FULL
    "Disk is full",                             // NZ_ERROR_DISK_FULL
    "Cannot find free index slot",              // NZ_ERROR_INDEX_FULL
    "Index points to unallocated block",        // NZ_ERROR_BAD_INDEX
    "dir points to unallocated block",          // NZ_ERROR_BAD_DIR
    "Bad disk",                                 // NZ_ERROR_BAD_DISK
    "Cannot find table in disk extent",         // NZ_ERROR_TABLE_NOT_IN_EXTENT
    "Core Data size is too small",              // NZ_ERROR_CORETOOSMALL
    "Field cannot contain null values",         // NZ_ERROR_NULL_FIELD
    "Material too big",                         // NZ_ERROR_TOO_BIG
    "Too many concurrent interactions",         // NZ_ERROR_TOO_MANY_INTERACTIONS
    "The permitted number of client connections has been exceeded",    // NZ_ERROR_TOO_MANY_CLIENTS
    "Error processing dimension",               // NZ_ERROR_PROCESS_DIMENSION
    "Error loading dimension",                  // NZ_ERROR_LOAD_DIMENSION
    "Error generating a disk aggregate",        // NZ_ERROR_DISK_AGGR_GEN
    "Error loading dimension properties",       // NZ_ERROR_LOAD_DIM_PROPERTIES
    "Error loading olap metadata",              // NZ_ERROR_LOAD_OLAP_METADATA
    "requested memory cache block is too big",  // NZ_ERROR_OLAP_CACHE_BLOCK_TOO_BIG
    "Disk read error",                          // NZ_ERROR_READ_ERROR
    "System not ready",                         // NZ_ERROR_NOT_READY
    "Arithmetic overflow",                      // NZ_ERROR_OVERFLOW
    "Cannot exceed Maximum Distribution Key limit", // NZ_ERROR_DISTKEYS
    "System state invalid for request",         // NZ_ERROR_BAD_SYS_STATE,
    "Disk temporary work space is full",        // NZ_ERROR_NO_SWAP_PAGES
    "Record size limit exceeded",               // NZ_ERROR_RECORD_TOO_BIG
    "Column size limit exceeded",               // NZ_ERROR_COLUMN_TOO_BIG
    "Column count limit exceeded",              // NZ_ERROR_TOO_MANY_COLUMNS
    "Math error",                               // NZ_ERROR_MATH
    "Postgres math error?",                     // NZ_ERROR_PG_ELOG
    "Code generation failure",                  // NZ_ERROR_NO_CODE_GEN
    "Transaction rolled back by client",        // NZ_TRANS_ROLLBACK_CLIENT
    "Transaction rolled back by DBA",           // NZ_TRANS_ROLLBACK_ADMIN
    "Operation not permitted",                  // NZ_ERROR_NOT_PERMITTED
    "Too many concurrent transactions",         // NZ_ERROR_TOO_MANY_TRANS
    "Transaction rolled back due to restart or failover",// NZ_TRANS_ROLLBACK_RESTART
    "Transaction rolled back due to client disconnect",  // NZ_TRANS_ROLLBACK_DISCONNECT
    "Cant allocate sort buffer",        // NZ_ERROR_NO_SORT_BUFFER
    "Incorrect data size",                      // NZ_ERROR_INCORRECT_DATA_SIZE
    "No such session",                          // NZ_ERROR_NO_SESSION
    "Character width exceeded",                 // NZ_ERROR_CHAR_WIDTH_EXCEEDED
    "Dictionary lost",                          // NZ_ERROR_BAD_DICT
    "Client not responding",                    // NZ_ERROR_CLIENT_TIMEOUT
    "Induced error by test harness",            // NZ_ERROR_INSTRUMENTED_FAILURE
    "Reclaim active against this table",        // NZ_ERROR_RECLAIM_ACTIVE
    "Client not waiting for response",          // NZ_ERROR_CLIENT_NOT_WAITING=62
    "No client process",                        // NZ_ERROR_NO_CLIENT=63
    "SPU Oversubscribed memory",                // NZ_ERROR_MEMORY_DEADLOCK
    "Update canceled: attempt to update a target row with values from multiple join rows", // NZ_ERROR_AMBIGUOUS_UPDATE
    "Transaction in unexpected state",          // NZ_ERROR_TRANS_STATE
    "tempTableID unable to remap column",       // NZ_ERROR_TEMPTABLEID
    "Specified NUCL to AA translation table not found", // NZ_ERROR_TRANSL_TABLE_NOT_FOUND
    "Reading frame must be +1,+2,+3,-1,-2 or -3",// NZ_ERROR_BAD_READING_FRAME
    "RC error type",                            // NZ_ERROR_RC
    "no matching data partition",               // NZ_ERROR_NO_PARTITION
    "Temporary timing issue. Repeat the command",  // NZ_ERROR_TEMP_TIMING
    "Illegal char in protein sequence",         // NZ_ERROR_BAD_PROT
    "ZLib inflation error",                     // NZ_ERROR_INFLATE
    "Translate() FROM string is shorter than TO string", // NZ_TRANSLATE_STRING_LENGTHS
    "Reload incompatibility.",                  // NZ_ERROR_BAD_RELOAD
    "Excessive stack usage",                    // NZ_ERROR_STACK_USAGE
    "Sequence already exists",                  // NZ_ERROR_SEQUENCE_EXISTS
    "Sequence exhausted",                       // NZ_ERROR_SEQUENCE_EXHAUSTED
    "Sequence error",                           // NZ_ERROR_SEQUENCE_ERROR
    "Invalid FComm channel",                    // NZ_ERROR_FCOMM_CHANNEL_BAD
    "Invalid FComm node id",                    // NZ_ERROR_FCOMM_NODEID_BAD
    "Invalid FComm payload size",               // NZ_ERROR_FCOMM_PAYLOADSIZE_BAD
    "XML Error",                                // NZ_ERROR_XML
    "FComm API already initialized",            // NZ_ERROR_FCOMM_API_ALREADY_INITED
    "Illegal codepoint value",                  // NZ_ERROR_INVALID_CODEPOINT
    "Illegal value passed as substring length", // NZ_ERROR_BAD_SUBSTRING_LEN
    "Transaction was not in a restartable state",// NZ_ERROR_TRANS_NOT_RESTARTABLE
    "Transaction detail journal error",         // NZ_ERROR_TDJ_INVALID=89,
    "Aborted NZ_RELEASE_JOB",                   // NZ_ERROR_RELEASE_JOB_ABORT
    "Request was queued for restart",           // NZ_TRANS_REQ_QUEUED
    "Limit for max number of open tdj files reached", // NZ_ERROR_TDJ_MAXOPEN
    "UDX exception throw", // NZ_ERROR_UDX_EXCEPTION
    "Reclaim recovery state NVRAM limit reached",// NZ_ERROR_OUT_OF_RECLAIM_STATES (deprecated)
    "Rollback in progress",                     // NZ_TRANS_ROLLBACK_IN_PROGRESS
    "Table size per SPU limited",               // NZ_ERROR_LARGE_TABLE_LIMIT
    "History Storage Limit Exceeded",           // NZ_ERROR_AUDIT_STORAGELIMIT_EXCEEDED
    "History Disk Full Threshold Violated",     // NZ_ERROR_AUDIT_DFULLTHRESHOLD_BROKEN
    "History Capture Failure",                  // NZ_ERROR_AUDIT_CAPTURE_FAILED
    "History Load Connection Failure",          // NZ_ERROR_AUDIT_LOAD_CONNECTION
    "History Load Config Info Not Found",       // NZ_ERROR_CONFIG_INFO_LOST
    "History Load Failure",                     // NZ_ERROR_AUDIT_LOAD_FAILED
    "Transaction required, but no transaction started", // NZ_ERROR_NO_TRANSACTION
    "SCSI Disk CDB Error",                      // NZ_ERROR_SCSI_CDB
    "SCSI Response Parsing Error",              // NZ_ERROR_SCSI_RESPONSE_PARSING
    "Library already exists",                   // NZ_ERROR_LIBRARY_EXISTS
    "Library error",                            // NZ_ERROR_LIBRARY_ERROR
    "Process failed to attach to fcomm api",    // NZ_ERROR_FC_ATTACH_FAILED
    "No primary and mirror disk serial number for the device", // NZ_ERROR_NO_DISKSN
    "Functionality (temporarily?) not supported under MDM", // NZ_ERROR_MDM_UNSUPPORTED
    "Stale or corrupted handle in an MDM API",  // NZ_ERROR_MDM_BAD_HANDLE
    "Groom in progress on table",               // NZ_ERROR_TABLE_IN_GROOM
    "Table to be groomed is being modified",    // NZ_ERROR_GROOM_OTHER_UPDATERS
    "Storage transaction table is full",        // NZ_DPM_NO_FREE_STORAGE_TRANSACTION
    "Data partition is full",                   // NZ_DPM_NO_FREE_EXTENT
    "Concurrent update or delete of same row",  // NZ_ERROR_CONCURRENT_UPDATE
    "Sequence dropped",                         // NZ_ERROR_SEQUENCE_DROPPED
    "Data slice upgrade needed",                // NZ_ERROR_DATA_SLICE_UPGRADE_NEEDED
    "Temporarily unable to free extent", // NZ_DPM_EXTENT_IN_USE
    "Error in transient table handling",        // NZ_ERROR_TRANSIENT_COMPRESS
    "Cannot access table that is being truncated", // NZ_ERROR_TABLE_IN_TRUNCATE
    "Bad CRC reading disk page",               // NZ_ERROR_BAD_PAGE_CRC
    "Truncate water-mark file error",           // NZ_ERROR_TRUNCWMLOG_INVALID
    "Storage is not part of current table",     // NZ_DPM_NOTE_DELETE_UNKNOWN_STORAGE_ID
    "Stale table connection handle",           // NZ_DPM_STALE_TCHAND
    "Aborted by scheduler rule",                // NZ_ERROR_RULE_ABORTED
    "Function sequence error",                  // NZ_ERROR_FUNCTION_SEQUENCE
    "Bad CRC attempting snippet-result reuse",  // NZ_ERROR_SRC_CRC
    "REDR free disk space check failed",        // NZ_ERROR_REDR_SPACE_CHECK_FAILED
    "Snippet-result reuse failure",             // NZ_ERROR_SRC_FAILURE
    "Timed out waiting for plan completion",    // NZ_TRANS_PLAN_WAIT
    "Parent transaction in unexpected state",   // NZ_ERROR_PARENT_TRANS_STATE
    "Device or resource busy",                  // NZ_ERROR_DEVICE_IS_BUSY

// WHEN ADDING AN ERROR CODE TO NzErrorCode,
// ALSO REMEMBER TO ADD A CORRESPONDING STRING HERE !!

    // FATAL errors start at 134...
    "Internal inconsistency",                   // NZ_FATAL_ASSERTION
    "Fatal Error in Transaction Mgr",           // NZ_FATAL_TXMGR
    "Fatal Error in Swap Mgr",                  // NZ_FATAL_SWAPMGR
    "Fatal Defect detected in heap control data", // NZ_FATAL_HEAP
    "Fatal error accessing system file",        // NZ_FATAL_FILE_ACCESS
    "SPU Hardware Exception signalled",         // NZ_FATAL_SPU_EXCEPTION
    "Fatal History Error",                      // NZ_FATAL_AHIST

// IF ADDING A FATAL ERROR CODE, ADD THE STRING HERE

    "Unknown error code"  // NZ_ERROR_LAST
};
#else
extern char *NzErrorStrs[NZ_ERROR_LAST+1];
#endif

#endif
