// udxbasemc.h
// Base class for Multi Column User Defined Functions and Aggregates
//
// (C) Copyright IBM Corp. 2009, 2014  All Rights Reserved.

#ifndef UDX_BASEMC_INCLUDED
#define UDX_BASEMC_INCLUDED

#define CHECK_ROW_RANGE(n) if (n < 0 || n >= ((RowFuncCallInfo*)m_pInfo)->numReturnColumns) \
                  throwError(NZ_ERROR_ILLVAL, "Return Column number out of range", n)

namespace nz {
namespace udx_ver2 {

class UdxNonScalarBase : public UdxBase
{
public:
    UdxNonScalarBase(UdxInit *pInit) : UdxBase(pInit) {}
    
 
	
	// New interface for row functions
public:
    // Return methods
    int numReturnColumns() const;
    int returnTypeColumn(int n) const;
    void setReturnColumnNull(int n, bool val) const;
    bool isReturnColumnNull(int n) const;  
    StringReturn* stringReturnColumn(int n) const;
    Numeric32Val* numeric32ReturnColumn(int n) const;
    Numeric64Val* numeric64ReturnColumn(int n) const;
    Numeric128Val* numeric128ReturnColumn(int n) const;
    TimeTzADT* timetzReturnColumn(int n) const;    
    struct Interval* intervalReturnColumn(int n) const;
    float* floatReturnColumn(int n) const; 
    double* doubleReturnColumn(int n) const;
    int8* int8ReturnColumn(int n) const;
    int16* int16ReturnColumn(int n) const;
    int32* int32ReturnColumn(int n) const;
    int64* int64ReturnColumn(int n) const;
    bool* boolReturnColumn(int n) const;
    int32* dateReturnColumn(int n) const;
    int64* timeReturnColumn(int n) const;
    int64* timestampReturnColumn(int n) const;


};


inline int UdxNonScalarBase::numReturnColumns() const
{
    CHECK_INFO_AVAILABLE();
    return ((RowFuncCallInfo*)m_pInfo)->numReturnColumns;
}

inline int UdxNonScalarBase::returnTypeColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    return ((RowFuncCallInfo*)m_pInfo)->returnColumnTypes[n];
}

inline void UdxNonScalarBase::setReturnColumnNull(int n, bool val) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    ((RowFuncCallInfo*)m_pInfo)->returnColumnNulls[n] = val;

}
inline bool UdxNonScalarBase::isReturnColumnNull(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    return ((RowFuncCallInfo*)m_pInfo)->returnColumnNulls[n];

}
inline StringReturn* UdxNonScalarBase::stringReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_FIXED && nType != UDX_VARIABLE && nType != UDX_NATIONAL_FIXED &&
        nType != UDX_NATIONAL_VARIABLE && nType != UDX_GEOMETRY && nType != UDX_VARBINARY)
        throwError(NZ_ERROR_ILLVAL, "Expected string, st_geometry, or varbinary column value", nType);
    return (StringReturn*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline Numeric32Val* UdxNonScalarBase::numeric32ReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_NUMERIC32)
        throwError(NZ_ERROR_ILLVAL, "Expected numeric32 column value", nType);
    return (Numeric32Val*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline Numeric64Val* UdxNonScalarBase::numeric64ReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_NUMERIC64)
        throwError(NZ_ERROR_ILLVAL, "Expected numeric64 column value", nType);
    return (Numeric64Val*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline Numeric128Val* UdxNonScalarBase::numeric128ReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_NUMERIC128)
        throwError(NZ_ERROR_ILLVAL, "Expected numeric128 column value", nType);
    return (Numeric128Val*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline    TimeTzADT* UdxNonScalarBase::timetzReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_TIMETZ)
        throwError(NZ_ERROR_ILLVAL, "Expected timetz column value", nType);
    return (TimeTzADT*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline struct Interval* UdxNonScalarBase::intervalReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_INTERVAL)
        throwError(NZ_ERROR_ILLVAL, "Expected interval column value", nType);
    return (struct Interval*) ((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline float* UdxNonScalarBase::floatReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_FLOAT)
        throwError(NZ_ERROR_ILLVAL, "Expected float column value", nType);
    return (float*) ((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline double* UdxNonScalarBase::doubleReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_DOUBLE)
        throwError(NZ_ERROR_ILLVAL, "Expected double column value", nType);
    return (double*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline int8* UdxNonScalarBase::int8ReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_INT8)
        throwError(NZ_ERROR_ILLVAL, "Expected int8 column value", nType);
    return (int8*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline int16* UdxNonScalarBase::int16ReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_INT16)
        throwError(NZ_ERROR_ILLVAL, "Expected int16 column value", nType);
    return (int16*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline int32* UdxNonScalarBase::int32ReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_INT32)
        throwError(NZ_ERROR_ILLVAL, "Expected int32 column value", nType);
    return (int32*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline int64* UdxNonScalarBase::int64ReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_INT64)
        throwError(NZ_ERROR_ILLVAL, "Expected int64 column value", nType);
    return (int64*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline bool* UdxNonScalarBase::boolReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_BOOL)
        throwError(NZ_ERROR_ILLVAL, "Expected bool column value", nType);
    return (bool*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline int32* UdxNonScalarBase::dateReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_DATE)
        throwError(NZ_ERROR_ILLVAL, "Expected date column value", nType);
    return (int32*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline int64* UdxNonScalarBase::timeReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_TIME)
        throwError(NZ_ERROR_ILLVAL, "Expected time column value", nType);
    return (int64*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}
inline int64* UdxNonScalarBase::timestampReturnColumn(int n) const
{
    CHECK_INFO_AVAILABLE();
    CHECK_ROW_RANGE(n);
    int nType = returnTypeColumn(n);
    if (nType != UDX_TIMESTAMP)
        throwError(NZ_ERROR_ILLVAL, "Expected timestamp column value", nType);
    return (int64*)((RowFuncCallInfo*)m_pInfo)->returnColumns[n];

}



} // end namespace udx_ver2
} // end namespace nz

#endif //  #ifndef UDX_BASEMC_INCLUDED
