// udxparamter.h
// Parameter class for User Defined Functions and Aggregates
//
// (C) Copyright IBM Corp. 2009, 2011  All Rights Reserved.

#ifndef UDX_PARAMTER_INCLUDED
#define UDX_PARAMTER_INCLUDED


namespace nz {  
namespace udx_ver2 {


class UdxParameter
{
public:
    UdxParameter() {
        m_nSize = 0;
        m_nAllocated = 8;
        m_pEntries = new char*[m_nAllocated];
        if (m_pEntries == NULL)
            throwUdxException("Couldn't allocate parameters array");
        m_bReadOnly = false;
    }
    ~UdxParameter() {
        if (m_pEntries) 
        {
            for (int i=0; i < m_nSize; i++)
                delete m_pEntries[i];
            delete [] m_pEntries;
        }
    }
    void setReadOnly()  { m_bReadOnly = true; }
    int getNumEntries() const { return m_nSize; }

    const char* getEntry(int idx) const ;

    void addEntry(const char* strName);

private:
    char **m_pEntries;
    int m_nSize;
    int m_nAllocated;
    bool m_bReadOnly;
};

inline const char* UdxParameter::getEntry(int idx) const
{
    if (idx < 0 || idx >= m_nSize)
        throwUdxException("Index is out of bounds");
    return m_pEntries[idx];

}

inline void UdxParameter::addEntry(const char* strName)
{
    if (strName == NULL)
        throwUdxException("NULL name");
    if (m_bReadOnly)
        throwUdxException("Object is read only now");
    
    if (m_nSize == m_nAllocated-1)
    {
        m_nAllocated *= 2;
        char ** temp = new char*[m_nAllocated];
        if (temp == NULL)
            throwUdxException("Couldn't allocate new parameters array");
        for (int i=0; i < m_nSize; i++)
            temp[i] = m_pEntries[i];
        delete [] m_pEntries;
        m_pEntries = temp;
    }       
    int length = strlen(strName)+1;
    m_pEntries[m_nSize] = new char[length];
    if (m_pEntries[m_nSize] == NULL)
        throwUdxException("Couldn't allocate new parameter entry");
    strcpy(m_pEntries[m_nSize], strName);
    m_nSize++;
    
}
} // end namespace udx_ver2
} // end namespace nz

#endif //  #ifndef UDX_PARAMTER_INCLUDED
