// udxsupport.h
// Support helpers for User Defined Functions and Aggregates
//
// (C) Copyright IBM Corp. 2007, 2014  All Rights Reserved.

#ifndef UDX_SUPPORT_INCLUDED
#define UDX_SUPPORT_INCLUDED

#ifdef GENCODE
#ifndef NZ_ERROR_ILLVAL
#define NZ_ERROR_ILLVAL 8
#endif
#if __GNUC__
# define __THROW throw ()
#else // old gcc or diab
# define __THROW
#endif
#ifndef NZ_ERROR_UDX_EXCEPTION
#define NZ_ERROR_UDX_EXCEPTION 93
#endif
extern "C" int sprintf(char*, const char*, ...) __THROW;
#endif

extern "C" int throwError(int code, const char *buf, const char *obj);

class CNumeric128;
class CNumeric64;
class CNumeric32;


static int throwError(int code, const char* buf, int num)
{
  char str[1024];
  sprintf(str, "%d", num);
  throwError(code, buf, str);

}
static void throwUdxException(const char *msg)
{
    throwError(NZ_ERROR_UDX_EXCEPTION, msg, (const char*)0);
}
#ifdef GENCODE
#include <stdarg.h>
extern void logvprint(const char* fmt, va_list args);
#endif

#if FOR_SPU
extern "C" unsigned TCT_Check_Stack();
#ifdef GENCODE
#if __DCC__
#include <time.h>
extern "C" int strcasecmp(const char *s1, const char *s2);
extern "C" int strncasecmp(const char *s1, const char *s2, size_t n);
extern "C" void setzone(const char* tz);
extern "C" const char* getzone();
extern "C" long _timezone();
extern "C" int _daylight();
extern "C" char* _tzname(int num);
extern "C" int gettimeofday(struct timeval * tp, void * tzp);
#else
extern "C" void setzone(const char* tz);
#endif
#endif
#endif
extern "C" bool checkPlanInterrupt();
extern "C" bool isUserQuery();

// Define macros to tell which
// UDX versions are supported
#ifndef UDX_V1
#define UDX_V1
#endif

#ifndef UDX_V2
#define UDX_V2
#endif

extern "C" int getNumberLocalDataslices();
extern "C" int getCurrentLocalRankId();


extern "C" const char* getCurrentUsername();

// return sessionid
extern "C" int getCurrentSessionId();

// return datasliceid on spu, 0 elsewhere
extern "C" int getCurrentDatasliceId();

// return current NPS transaction id, if it exists
extern "C" int64 getCurrentTransaction();

// return current hardware id on spu, 0 otherwise
extern "C" int getCurrentHardwareId();

#define UDX_LOCUS_POSTGRES 0
#define UDX_LOCUS_DBOS 1
#define UDX_LOCUS_SPU 2

// return UDX_LOCUS_POSTGRES, UDX_LOCUS_DBOS or UDX_LOCUS_SPU
// depending on locus
extern "C" int getCurrentLocus();

// return number of dataslices (may be more than spus)
extern "C" int getNumberDataslices();

// return number of spus
extern "C" int getNumberSpus();


// Numeric helper routines
int32 convertNumeric32( int32 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
int64 convertNumeric64( int32 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
CNumeric128 convertNumeric128( int32 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
int32 convertNumeric32( int64 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
int64 convertNumeric64( int64 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
CNumeric128 convertNumeric128( int64 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
int32 convertNumeric32( CNumeric128 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
int64 convertNumeric64( CNumeric128 value, int curPrec, int curScale, int desiredPrec, int desiredScale);
CNumeric128 convertNumeric128( CNumeric128 value, int curPrec, int curScale, int desiredPrec, int desiredScale);

// Shared library helper function
const char* udxLibraryName(const char* name, bool caseSensitive);

#endif //  #ifndef UDX_SUPPORT_INCLUDED
