#ifndef UTILS_INCLUDED
#define UTILS_INCLUDED
/// @file utils.h
/// @brief Utility methods for nde
//////////////////////////////////////////////////////////////////////
/// (C) Copyright IBM Corp. 2001,2011 All rights reserved.
//////////////////////////////////////////////////////////////////////

#include "nzstd.h"

#ifndef BYTE_BITS
#define BYTE_BITS 8
#endif

namespace mem { class Pool; };

class CTable;

/*

	Random utility functions

*/

// Convenience function get value from environment
int WhackFlagFromEnv(const char *name,int defaultVal);

// from utils.cpp

/**********************************************************************bmz
function: NzSetBit - Set bit in passed array of bytes
*/
#define NzSetBit(bytes,bitpos) \
    do {  \
    ((uint8 *)(bytes))[(bitpos)/8] |= ( 1 << ((bitpos) & 7)); \
    } while (0)

/**********************************************************************bmz
function: NzClearBit - Clear bit in passed array of bytes
*/
#define NzClearBit(bytes,bitpos) \
    do {  \
    ((uint8 *)(bytes))[(bitpos)/8] &= ~( 1 << ((bitpos) & 7)); \
    } while (0)

/**********************************************************************bmz
function: NzBitIsSet - Return whether a bit is set in array of bytes
*/
#define NzBitIsSet(bytes,bitpos) \
    (((uint8 *)(bytes))[(bitpos)/8] & ( 1 << ((bitpos) & 7)))

/********************************************************************dlm
function: NzAnyBitInByte - Test for any bit set in specified byte of array
*/
#define NzAnyBitInByte(bytes,bytepos) \
    (((uint8 *)(bytes))[bytepos] != 0)

CEXTERN uint16 NzFindNextSetBit (const uint8 *bv, uint16 bitBvSize, uint16 bitOffset);

inline static uint16
NzFindFirstSetBit(const uint8 *bv, uint16 bitBvSize)
{
    return NzFindNextSetBit(bv, bitBvSize, 0);
}

#define NzForEachSetBit(bit, addr, size) \
    for ((bit) = NzFindFirstSetBit((addr), (size)); \
         (bit) < (size); \
         (bit) = NzFindNextSetBit((addr), (size), (bit) + 1))

// Random stuff

CEXTERN int DbosPgTypeVal(int pgType);  // convert postgres coltype to NDE coltype
CEXTERN int PgDbosTypeVal(int dbType);  // convert NDE coltype to postgres coltype

#define BITMASK_N_LOWBITS_SET(n) ((1<<(n))-1)
#define BITMASK_NTH_BIT_SET(n) (1<<(n))

CEXTERN void NzSpinFile(const char *filename); // For debug, wait while file exists

int zeropack( // Create a compressed buffer by masking zero-bytes
    char *outBuf,		// written output buffer
    int size,           // object size (in memory)
    char *inBuf);       // ptr to object

int zerounpack( // Uncompress a zero-packed buffer, rtn memory size
    char *outBuf,	// result buffer
    char *inP,		// input buffer
    int size);		// input size

// Functions useful for servers' working directory...
#ifndef WIN32
extern int mkdirhier(const char *path, mode_t mode);
#endif

CEXTERN uint32 NzRoundRobinSpuId(); // Sequentially return num from 1 to numSPUs
CEXTERN uint32 NzChunkRandomDsId (CTable *tbl, int _rdc_size);

#endif
