#ifndef INCLUDED_AGGREGATE
#define INCLUDED_AGGREGATE
/* Header for aggregation node */

#include "nztypes.h"
#include "xpsort.h"

class CTable;
class CHash;
class TProjectNode;
class TAggregateNode;
class TStreamInfo;
struct UdxLibraryInfo;
class UdxEnvInfo;
class CRecordStore;

struct THashInfo;
struct Node; // postgres

namespace zm { struct Accumulator; };
typedef int (*TCmpRecFn)(void *, record_t**, record_t**); //from recordcompare.h
typedef void (*TWidenRecFn)(record_t*, record_t*, CRecordStore *);

#include "nzpack_x86_on.h"

// NOTE: There is a dependency on these value assignments in fpgaprep.cpp
//       in AggrTranslationTable[].
// This enumeration must be synchronized to "OpStrings" in aggregate.cpp.
enum EAggrTypes {
    aggrAVG=1,
    aggrCOUNT,
    aggrMAX,
    aggrMIN,
    aggrSTDDEV,
    aggrSUM,
    aggrVARIANCE,
    aggrLCOUNT, // limit count (used in statistics)
    aggrCHECKSUM,
    aggrSIZE,
    aggrLEN,       // GENERATE STATISTICS on lengthy(?) char types
    aggrMAXLEN,    // GENERATE STATISTICS on lengthy(?) char types
    aggrDATASLICEID,
    aggrLEAD,
    aggrLAG,
    aggrFIRST_VALUE,
    aggrLAST_VALUE,
    aggrROW_NUMBER,
    aggrDENSE_RANK,
    aggrSPARSE_RANK,
    aggrSCAN_SAMPLES,    // number of extents sampled by jitstats
    aggrSCAN_EXTENTS,     // extents that will be read from table
    aggrSCAN_EXTENTS_BJ,     // extents that will be read from table
    aggrMAXLABEL,        // maximally restrictive label combination
    aggrSCAN_JCOUNT,
    aggrSCAN_JPAGES,
    aggrSCAN_SEGMENT_DISTKEY,
    aggrSCAN_SEGMENT_HEADER,
    aggrSCAN_CREATE_SEGMENT, // create a segment for JIT SPU-to-SPU distribution
    aggrSCAN_PARSE_SEGMENT, // parse a segment for JIT SPU-to-SPU dispersion
    aggrSCAN_CREATE_DISP_VECTOR, // create a dispersion vector to transfer to host
    aggrSCAN_PARSE_DISP_VECTOR, // parse a dispersion vector on host
    aggrSCAN_DISP_SLOPE,
    aggrSCAN_DISP_YINTERCEPT,
    aggrSCAN_SAMPLE_DISPERSION,
    aggrUDA,
    aggrNTILE,
    aggrPERCONT,
    aggrPERDISC,
    aggrMEDIAN
};

// This enumeration must be synchronized (but offset by 1) to "StyleStringsOffset" in aggregate.cpp.
enum EAggrStyles {
    AggrLoop=0,   // nested loop  (obsolete!)   Don't believe it!  These get created all the time.
    AggrHash=1    // hashed grouping, requires an upstream HashNode
};

enum EDispersionSampling {
    DispersionSampleOff,
    DispersionSampleAll,
    DispersionSampleSome
};


#ifndef POSTGRES_H
// !REWORK - can we get these from a common header rather than duplicate?
#define INDEX_MAX_KEYS      64
extern "C" {
typedef Oid oidvector[INDEX_MAX_KEYS];
}
#endif
struct TAggrField {
    EAggrTypes aggr;
    int        m_srcColDex; //column to aggregate in input stream
    bool       distinct;
    bool       colSpec; // if true, expr is a simple integer fieldnum (for dev work)
    int        dstType;      // PG field type (!NOTE-bmz should be enum)
    int32      dstTypmod;  // PG aggr field shape
    int        srcType; // type of src material (differs from fType when COUNT(src) )
    int        exprScale; // !FIX-used for scale of numeric source expression
    int        distinctTbl;        // only used for gathering stats
    int        limitDistinct;      // only used for gathering stats
    int        tableSize;  // used for limitDistinct, this is accumulator
    int64      numStreamRows;    // number of rows in the stream hint for distinct
    bool       m_forExpressStats; // whether this aggr is hashed/extrapolated
    bool       m_hashExpressStats; // true if this is hashing field (rather than size)
    bool       m_isDistKey;        // true if need to expressStats with Knuth hash
    
    int        m_dispersionCol; // index into m_tableHash
    
    EAggrTypes m_segmentAggr;
    int        m_segmentHdrStrLimit; // Byte limit for truncating strings (on char boundary).
    
    bool       forHaving;
    void *     m_windowSpec;  // PG window spec
    void *     m_withingroupspec;  // PG within group spec
    int        m_arg2Val;        // -1 if default, else lead/lag arg2 literal (TEMP - will be colDex one day)
    int        m_arg3Val;        // -1 if default, else coldex of "default" lead/lag arg3
    bool       m_windowIgnoreNulls;   //ignore nulls specified for window aggregate
    int        m_filterDex;     // 0 =>no FILTER expr, else 1+(offset of AggrNode->m_filterList entry)
    // runtime state:
    bool           m_bInitDistinct;  // final runtime init of distincter complete
    CTable *       table;  // temp for distinct - if grouping, this contains group numbers
    CRecordStore * recStore;   // of "table"
    CTable *       wideTable;  // temp with group numbers widened to group fields
    CTable *       finalTable;  // final with group numbers widened to group fields
    int            m_distinctHashColumnCount; // 1 if one row; 2 if group by
    int            m_distinctHashColumn[2];   // for Hash() interface (runtime)

    int             dstNzType;    // NzType for this aggr field (derived from dstType)

    char *          name;       // !REWORK - need comments on this and following UDA stuff
    oidvector       argTypmods;
    oidvector       intTypmods;
    oidvector       intTypes;
    oidvector       argCols;
    int             nargs;
    int             nints;
    char*           externalName;
    bool            fenced;
    int16           log;
    char*           memory;
    char*           hostcksum;
    char*           spucksum;
    char*           datadir;
    Oid             aggid;
    bool            stub;
    int             numdependencies;
    UdxLibraryInfo* dependencies;
    int             numenvironment;
    UdxEnvInfo *    environment;

    int             numparameter;
    char**          parameters;

    int             version;

    TAggregateNode * pParent;            // parent aggregate node
    TStreamInfo *    m_gencHelperStream; // aid to code gen for a unique rec ptr
    CSorter          m_sortInfo;         // main engine of the hybrid quick/merge sorter
    CComparison *    m_narrowCompare;    // compare for qsort of narrow distincting table
    CComparison *    m_wideCompare;      // compare for merging of widened table
    int              m_colDex;           //position of this aggregate in aggr result rec
    TWidenRecFn      m_widenRecFn;

    void setupSorter(CMemForSorting& memFor,
                     CPlan *plan); // init distincter sorter

    void setupNarrowCompare(void); // (internal) Setup the narrow qsort compare
    void setupWideCompare(CPlan *plan); // (internal) Setup the wide merge compare

    void InitializeDistincter(         // install generated code functions
        TCmpRecFn   wideCmpRecFn,      // widened with grouping record compare
        TCmpRecFn narrowCmpRecFn,      // narrow (indirect-to-grouping) record compare
        TWidenRecFn widenRecFn);       // narrow to wide record transformation

    // Notification of growing the distincter store
    void goDiskBased(bool lastCall);
    void distincterNewBlock(bool lastCall, void*** hashslot, CHash* hashobj, uint32 hashnum);
    void afSortSection(bool lastCall); // specialized for grouping distincts
    void addSection(); // specialized to expand group nums to group fields

    void SetupMerge();       // of disk based sort's merge phase
    void distincterDone();   // distincter store processing complete
    void getAggrType(char * buf);
};

class TAggregateNodeDef {
    // These members were selected as the 'definition' by virtue of their being
    //  supplied by the postgres->dbos-transition calls to AddAggregateNode().
protected:
    int           m_numGroups;          // grouping (not projecting) fields
    int         * m_pInputGroupingList; // grouping fields referring to input stream
    int           m_numAggrs;           // how many aggregating fields
    TAggrField *  m_aggrs;              // list of agrregating info
    EAggrStyles   m_aggrStyle;          // AggrHash...
    int           m_groupingSetID;      // handle to plan's m_groupingSetInfoP entry (or -1)
    int         * m_ordercolsList;      // to force ordering of aggr-result by grouping fields
    int         * m_ascendingList;      // asc/desc flags for ordercolsList
    int         * m_nullsLoList;        // null collation booleans for ordercolsList
    int           m_numFilters;         // number of entries in m_filterExpr array               
    Node       ** m_filterList;         // array of FILTER expressions for aggrs in this node.
    bool          m_forExpressStats;    // whether express stats enabled
    uint64        m_approxRows;         // known per-spu rowcount
    bool          m_bRollup;
};

class TAggregateNodeRuntimeInfo {
protected:
    record_t   * m_curRec;           // allows for merge aggr across calls
    record_t   * m_curBuf[2];        // stable buffers for agging into
    int          m_bufSelect;        // to choose which m_curBuf is aggr into
    uint32       m_groupRecId;
    int         * m_pGroupingList;   // grouping fields referring to aggr result record
    CComparison * m_groupComparer;   // compare function for grouping fields, sorted
    int          m_varyingFieldSize; // length of the portion of record containing
                                     // varchar aggregates
    int          m_limit;            // to limit result set for "limit" select
    bool         m_bInitialized;     // for one-time init from gened code
    int          m_pcodeStart;       // number of first pcode instr              
    int          m_pcodeCount;       // count of pcode instrs

    // For _vt_plan_aggregate statistics.
    uint64       m_numIncomingRecs;
    uint64       m_DisplacementCount;// Count of displacements in open-hashed tables.
    uint32       m_RebuildCount;     // Count of rebuilds of open-hashed tables.

    // For Dispersion
    EDispersionSampling m_dispersionSampling;
    int          m_dispersionDatapoints;      
    int          m_numDispersionCols;         
    int        * m_dispersionWorkMemSize;     
    int        * m_dispersionHashVal;         
    int        * m_dispersionCheckSaturation; 
    int        * m_dispersionIsDistKey;       
    void       * m_dispersionWorkspace;       
    int          m_dispersionFirstSpuMod;     

    // Support for express stats
    uint32 m_tblSlots;        // how many slots in the sampling table
    int    m_sampleValue;     // how often to sample rows
    uint32 m_sampleCounter;   // rolling sampler counter (cycles at m_sampleValue)
    int    m_jitterValue;     // to varying the skip value in 0<->2N range

    uint8 **m_hashTables;     // ptr to array of ptrs to hash tables
    nzotuid_t m_hashTables_Owner; // process that allocated m_hashTables array

    TStreamInfo *m_gencHelperStream; // aid to code gen for a temp var
    int m_numSourceFields;   // aid to codegen - number of fields in input flow table
    int *m_stateMap; // !REWORK need comment here
    int *m_srcMap; //list of fields in input stream for aggr materialzation
    int m_numFieldsToMat; //number of fields to materialize into aggr record
    int m_grpRecIdIdx;    // where to find the "groupRecId" info.
    int m_hashHintIdx;    // where to find the saved hash number.  If this is < 0, it isn't stored.

    bool m_haveSampleCount; // for jitstats sampling, used by aggrSCAN_SAMPLES
    bool m_haveExtentsCount; // for jitstats sampling, used by aggrSCAN_EXTENTS

    // for node stats gathering
    uint32         m_numHashChains;      // number of non-empty chains in hash table

    bool           m_bNoMerge;
    int            m_nStateFields;
    CMemForSorting memFor;
    CSorter        m_sortInfo;           // main engine of the hybrid quick/merge sorter
};

// group-by (always followed by a project)
class TAggregateNode : public SnippetNode,
    private TAggregateNodeDef, private TAggregateNodeRuntimeInfo {
public:

    TAggregateNode();
    virtual ~TAggregateNode();

    // SnippetNode interface
    TableShape const * Init(TableShape const * src, EPlanNodeLocus place);

    bool IsValidForSrc() { return true; }
    bool Serialize( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );
    // end SnippetNode interface

    static char const* OpStrings[];
    static char const* StyleStringsOffset[];

    // Address-of accessors
    // Used for things like virtual-table materialization.
    inline CMemForSorting*  AddrOfMemFor()           { return &memFor; }
    inline CSorter*         AddrOfSortInfo()         { return &m_sortInfo; }
    inline uint64*          AddrOfNumIncomingRecs()  { return &m_numIncomingRecs; }
    inline uint64*          AddrOfDisplacementCount(){ return &m_DisplacementCount; }
    inline uint32*          AddrOfRebuildCount()     { return &m_RebuildCount; }
    inline char const*      AddrOfAggrStyle()        { return StyleStringsOffset[m_aggrStyle+1]; }

    // Accessors
    inline TAggrField*  Aggrs()                const { return m_aggrs; }
    inline EAggrStyles  AggrStyle()            const { return m_aggrStyle; }
    inline uint64       ApproxRows()           const { return m_approxRows; }
    inline int*         AscendingList()        const { return m_ascendingList; }
    inline int          BufSelect()            const { return m_bufSelect; }
    inline record_t**   CurBuf()                     { return m_curBuf; }
    inline record_t*    CurRec()               const { return m_curRec; }

    inline int*         DispersionCheckSaturation() const { return m_dispersionCheckSaturation; }
    inline int          DispersionDatapoints()      const { return m_dispersionDatapoints; }
    inline int          DispersionFirstSpuMod()     const { return m_dispersionFirstSpuMod; }
    inline int*         DispersionHashVal()         const { return m_dispersionHashVal; }
    inline int*         DispersionIsDistKey()       const { return m_dispersionIsDistKey; }
    inline EDispersionSampling DispersionSampling() const { return m_dispersionSampling; }
    inline int*         DispersionWorkMemSize()     const { return m_dispersionWorkMemSize; }
    inline void*        DispersionWorkspace()             { return m_dispersionWorkspace; }

    inline Node**       FilterList()           const { return m_filterList; }
    inline bool         ForExpressStats()      const { return m_forExpressStats; }
    inline TStreamInfo* GencHelperStream()     const { return m_gencHelperStream; }
    inline CComparison* GroupComparer()        const { return m_groupComparer; }
    inline int*         GroupingList()         const { return m_pGroupingList; }
    inline uint32       GroupRecId()           const { return m_groupRecId; }
    inline int          GroupingSetID()        const { return m_groupingSetID; }
    inline uint8**      HashTables()           const { return m_hashTables; }
    inline nzotuid_t    HashTables_Owner()     const { return m_hashTables_Owner; }
    inline bool         HaveExtentsCount()     const { return m_haveExtentsCount; }
    inline bool         HaveSampleCount()      const { return m_haveSampleCount; }
    inline bool         Initialized()          const { return m_bInitialized; }
    inline int*         InputGroupingList()    const { return m_pInputGroupingList; }
    inline int          JitterValue()          const { return m_jitterValue; }
    virtual CMemForSorting      & MemFor()           { return memFor; }
    virtual CMemForSorting const& MemFor()     const { return memFor; }
    inline bool         NoMerge()              const { return m_bNoMerge; }
    inline int*         NullsLoList()          const { return m_nullsLoList; }
    inline int          NumAggrs()             const { return m_numAggrs; }
    inline int          NumDispersionCols()    const { return m_numDispersionCols; }
    inline int          NumFieldsToMat()       const { return m_numFieldsToMat; }
    inline int          NumFilters()           const { return m_numFilters; }
    inline int          NumGroups()            const { return m_numGroups; }
    inline int          GrpRecIdIdx()          const { return m_grpRecIdIdx; }
    inline int          HashHintIdx()          const { return m_hashHintIdx; }
    inline int64        NumIncomingRecs()      const { return m_numIncomingRecs; }
    inline int64        DisplacementCount()    const { return m_DisplacementCount; }
    inline int32        RebuildCount()         const { return m_RebuildCount; }

    inline int*         OrderColsList()        const { return m_ordercolsList; }
    inline int          PcodeCount()           const { return m_pcodeCount; }
    inline int          PcodeStart()           const { return m_pcodeStart; }
    inline int          NumSourceFields()      const { return m_numSourceFields; }
    inline int          NStateFields()         const { return m_nStateFields; }
    inline bool         Rollup()               const { return m_bRollup; }
    inline uint32       SampleCounter()        const { return m_sampleCounter; }
    inline int          SampleValue()          const { return m_sampleValue; }
    inline CSorter &    SortInfo()                   { return m_sortInfo; }
    inline int*         SrcMap()               const { return m_srcMap; }
    inline int*         StateMap()             const { return m_stateMap; }
    inline uint32       TblSlots()             const { return m_tblSlots; }
    inline int          VaryingFieldSize()     const { return m_varyingFieldSize; }

    // Special accessors
    inline bool         DiskBased()            const { return m_sortInfo.DiskBased(); }


    // Mutators
    inline int  PreIncBufSelect()                               { return ++m_bufSelect; }
    inline void AddToRebuildCount( uint32 val )                 { m_RebuildCount += val; }

    // Setters
    inline void SetAggrs( TAggrField* aggrs )                   { m_aggrs = aggrs; }
    void SetAggrStyle( EAggrStyles style );
    inline void SetApproxRows( uint64 rows )                    { m_approxRows = rows; }
    inline void SetAscendingList( int* list )                   { m_ascendingList = list; }
    inline void SetCurRec( record_t* val )                      { m_curRec = val; }
    inline void SetDispersionCheckSaturation( int* val )        { m_dispersionCheckSaturation = val; }
    inline void SetDispersionDatapoints( int pts )              { m_dispersionDatapoints = pts; }
    inline void SetDispersionFirstSpuMod( int val )             { m_dispersionFirstSpuMod = val; }
    inline void SetDispersionHashVal( int* val )                { m_dispersionHashVal = val; }
    inline void SetDispersionIsDistKey( int* val )              { m_dispersionIsDistKey = val; }
    inline void SetDispersionSampling( EDispersionSampling ds ) { m_dispersionSampling = ds; }
    inline void SetDispersionWorkMemSize( int* size )           { m_dispersionWorkMemSize = size; }
    inline void SetFilterList( Node** list )                    { m_filterList = list; }
    inline void SetForExpressStats( bool val )                  { m_forExpressStats = val; }
    inline void SetGencHelperStream( TStreamInfo* val )         { m_gencHelperStream = val; }
    inline void SetGroupRecId( uint32 val )                     { m_groupRecId = val; }
    inline void SetGroupingSetID( int val )                     { m_groupingSetID = val; }
    inline void SetHaveExtentsCount( bool val )                 { m_haveExtentsCount = val; }
    inline void SetHaveSampleCount( bool val )                  { m_haveSampleCount = val; }
    inline void SetInputGroupingList( int* list )               { m_pInputGroupingList = list; }
    inline void SetJitterValue( int val )                       { m_jitterValue = val; }
    inline void SetNoMerge( bool bb )                           { m_bNoMerge = bb; }
    inline void SetNullsLoList( int* list )                     { m_nullsLoList = list; }
    inline void SetNumAggrs( int num )                          { m_numAggrs = num; }
    inline void SetNumDispersionCols( int cols )                { m_numDispersionCols = cols; }
    inline void SetNumFilters( int num )                        { m_numFilters = num; }
    inline void SetNumIncomingRecs( uint64 val )                { m_numIncomingRecs = val; }
    void SetDisplacementCount( uint64 val );
    inline void SetNumGroups( int num )                         { m_numGroups = num; }
    inline void SetGrpRecIdIdx( int num )                       { m_grpRecIdIdx = num; }
    inline void SetOrderColsList( int* list )                   { m_ordercolsList = list; }
    inline void SetPcodeCount( int count )                      { m_pcodeCount = count; }
    inline void SetPcodeStart( int start )                      { m_pcodeStart = start; }
    inline void SetRollup( bool val )                           { m_bRollup = val; }
    inline void SetSampleCounter( uint32 val )                  { m_sampleCounter = val; }

    // Non-inlines
    void InitializeSorters();   // init called from gened code

    bool HasGrpRecId();         // returns TRUE if has grpRecId used for distinct aggrs

    void initDistincter(TAggrField *aggrP,
                        const TableShape *src,
                        CPlan *plan,
                        EPlanNodeLocus place,
                        int aggrListIndex);

    TableShape *DefineAggrFields( // Set up aggregator fields base don expression type
        EPlanNodeLocus place,     // DBOS or SPU running this call?
        const TableShape *src,    // the oncoming table
        TProjectNode *pProj,      // project node following aggr node
        int *pGroupingList);      // grouping field list referring to result rec

    bool HasDistinctAggrs(); //returns TRUE if node has distinct aggrs

    int getNumFieldsInAggrRec(TProjectNode *pProj,      // proj node following this aggr node
                              const TableShape *src );  // input stream to this aggregation

    void defineAggr(TAggrField *aggrP,       // the aggr to be defined
                    int fDex,                // position of this aggr in dst (aggr result rec)
                    TableShape *dst,         // the table describing aggr result rec
                    const TableShape *src);  // the input stream for this aggregation

    int  AccumulatorCount(void); // number of non-trivial output tables

    // Notification of growing the aggregate store
    void goDiskBased(bool lastCall);
    void aggregateNewBlock(bool lastCall, void*** hashslot, CHash* hashobj, uint32 hashnum);

    void lastAggregateNewBlock(); //flush grouping aggregation

    void SetLimitAttr(int limit) {
        m_limit = limit;
    }

//    bool i_diskBased() const {
//        return m_sortInfo.DiskBased();
//    }

    void initExpressStats(); // For hash based stats

    void initDispersion (); // For sampling dispersion through sampling

    unsigned int allocSampleBufs( // For stats, compute tblSlots & allocate bufs
        const TableShape *table, uint8 **fieldTables);

    uint64 countExpressDistinct( // Resolve the count
        int col);

    uint64 countExpressSize( // Return size of hashTable
        int col);

    uint64 extrapolateExpressStats( // Convert samples to numVals
        uint64 numRows, uint64 bitsSet, uint64 tblSize);

    void createSegmentHeader (int nHeaders, int* hdrSize, char* aggrRec, int segFldDex);

    bool transferSegmentToRecord (int* row, int32* col, int16* targetSpu, int32* segOffset, char* aggrRec, int segFldDex);

    void parseSegmentHeader (int nHeaders, int* hdrSize, char* aggrRec, const char* segmentField);
    
    void parseSegmentRecord (const char* const segmentField);

    int64 getNumSampledRows (int col);
    
    void sumUniqueCounts (int col, uint32* uniqueCount);

    void parseDispersionVector (int col, const char* dispVectorField);

    double calcDispersionSlope (int col);
    double calcDispersionYIntercept (int col);
    
    static int IdxInArray(int lookupVal, int arr[], int num);
    int DistinctSchedCost(uint32 scanBlocks);

    int calcAdditionalMemNeeds ();
};

struct DispersionColWork {
    uint64 rowsSampled;
    uint64 uniqueSampled;
    uint32 saturationCnt;
    uint8* spuSentData;
};

#include "nzpack_x86_off.h"

#define DispVectorDatapoints MIN (108, Nz->numSPUs)
#define DispVectorSpusPerDatapoint 1.0 * Nz->numSPUs / (DispVectorDatapoints)
#define DispVectorSpuOffset(spu) (int) ((spu)/(DispVectorSpusPerDatapoint)) %  DispVectorDatapoints
#define DispVectorVarcharSize (int) (sizeof(int16) + sizeof(int64) + (DispVectorDatapoints)*sizeof(uint32))
#define DispMaxSegmentField 8000

extern "C" void DistinctAggrHashDump( TAggregateNode *pNode ); //dump hash info

int SmallFieldCost(const TableShape* table, int col );

#endif // INCLUDED_AGGREGATE
