#ifndef XPBINDINDEX_INCLUDED
#define XPBINDINDEX_INCLUDED

struct blockheader_t;
class CTable;
class GatherFixed;
class GatherVarA4;
struct TIoStream;
struct TpSortContext;
class TStreamInfo;

typedef int (*TBindIndexCmpRecFn)(const TableShape *, int *, int *);

// Parameter for AddBindIndexNode() (from xpkeyscansort.h, RIP)
#define KEYSCANINVALID  -1
#define KEYSCANSORT1     1
#define KEYSCANSORT2     2
#define KEYSCANSORT3     3

class TBindIndexNodeDef {
protected:
    int   id;               // table to convert to index status
    int   indexType;        // NZ_INDEX_XXX
    int   parentId;         // base table
    int   numIndexFields;   // number of order-by fields
    int * indexFieldsList;  // order by
    int * indexAscendList;  // sort up?
    int * indexNullsLoList; // null collation
    int   numProjectFields; // thin-table column count
    int * indexProjectList; // thin-table column subset
    int   dataType;         // Data reference type                          
};

class TBindIndexNodeRuntimeInfo {
protected:
    int64              totalRecs;          // Total number of records in table  
    void             * recBuf;                                               
    char            ** recPtrs;                                              
    int                nrecs;              // number of records               
    int                recsLeft;                                             
    int                numBuckets;                                           
    int                bktCount;                                             
    char             * pNullMap;           // Pointer to a null map for keys    
    TBindIndexCmpRecFn compareLessFcn;     // key compare function
    TStreamInfo      * bindIndexStreamPtr; // for genc's benefit to find toInsert Info
    TStreamInfo      * bindKeyStreamPtr;   // for genc's benefit to find Key Info

    /* Private */
    CTable        * toInsert;      // clone of table                                   
    CTable        * table;         // (in spu) result of dictionary lookup on tblId    
    CTable        * keyTable;      // temp table with only SPM view's keys             
    void          * dataRef;       // Data reference                                 
    int             reclen;        // record length                                
    int             numrecs;       // number of records read                       
    int             numRecords;                                                        
    int             blockNum;                                                          
    int             lengthSoFar;                                                       
    int             recsWritten;                                                       
    int             recsGathered;                                                      
    int             blocksInBuffer;                                                    
    int             currentBlock;                                                      
    GatherFixed   * gf;
    GatherVarA4   * gv;
    int             rfm;
    int             offToLen;
    blockheader_t * hdr;
    bool            updateNewStuff;
    TIoStream     * oStream;
    TIoStream     * workStream;    // Work stream reference
};

class TBindIndexNode : public SnippetNode,
    private TBindIndexNodeDef, private TBindIndexNodeRuntimeInfo {
public:
    TableShape const * Init (TableShape const * src, EPlanNodeLocus place);
    virtual ~TBindIndexNode() { /* nothing to do here */ }

    // Accessors
    inline TStreamInfo* BindIndexStreamPtr() const { return bindIndexStreamPtr; }
    inline TStreamInfo* BindKeyStreamPtr()   const { return bindKeyStreamPtr; }
    inline int          DataType()           const { return dataType; }
    inline int          Id()                 const { return id; }
    inline int*         IndexAscendList()    const { return indexAscendList; }
    inline int*         IndexFieldsList()    const { return indexFieldsList; }
    inline int*         IndexNullsLoList()   const { return indexNullsLoList; }
    inline int*         IndexProjectList()   const { return indexProjectList; }
    inline int          IndexType()          const { return indexType; }
    inline CTable*      KeyTable()           const { return keyTable; }
    inline char*        NullMap()            const { return pNullMap; }
    inline int          NumIndexFields()     const { return numIndexFields; }
    inline int          NumProjectFields()   const { return numProjectFields; }
    inline int          ParentId()           const { return parentId; }
    inline void*        RecBuf()                   { return recBuf; }
    inline CTable*      Table()              const { return table; }
    inline CTable*      ToInsert()           const { return toInsert; }

    // Mutators
    inline void IncTotalRecs()                  { totalRecs++; }

    // Setters
    inline void SetBindIndexStreamPtr( TStreamInfo* val )    { bindIndexStreamPtr = val; }
    inline void SetBindKeyStreamPtr( TStreamInfo* val )      { bindKeyStreamPtr = val; }
    inline void SetCompareLessFcn( TBindIndexCmpRecFn& val ) { compareLessFcn = val; }

    inline void SetDataRef( void* val )         { dataRef = val; }
    inline void SetDataType( int val )          { dataType = val; }
    inline void SetId( int val )                { id = val; }
    inline void SetIndexAscendList( int* val )  { indexAscendList = val; }
    inline void SetIndexFieldsList( int* val )  { indexFieldsList = val; }
    inline void SetIndexNullsLoList( int* val ) { indexNullsLoList = val; }
    inline void SetIndexProjectList( int* val ) { indexProjectList = val; }
    inline void SetIndexType( int val )         { indexType = val; }
    inline void SetNumIndexFields( int val )    { numIndexFields = val; }
    inline void SetNumProjectFields( int val )  { numProjectFields = val; }
    inline void SetParentId( int val )          { parentId = val; }
    inline void SetTable( CTable* val )         { table = val; }
};

void ExecBindIndexDone  (TBindIndexNode *pNode);

#endif // !XPBINDINDEX_INCLUDED
