// External declarations for DataSlice nodes
// (C) Copyright IBM Corp. 2013  All Rights Reserved.
//
#ifndef XPDATASLICE_INCLUDED
#define XPDATASLICE_INCLUDED

#include "nodecommon.h"
#include "nzlimits.h"
#include "BitVector.h"
#include <list>

enum EDSliceFilterType
{
    eDSFTBase,
    eDSFTJoin,
    eDSFTNumTypes
};

class TDataSliceFilterNodeDef {
protected:
    // none at this time
};

class TDataSliceFilterNodeRuntimeInfo {
protected:
    bool m_dataQualified;
};

struct List;
struct DataSliceFilter;
struct DataSliceFilterValue;
struct DataSliceHash;

class TDataSliceFilterNode:
    public SnippetNode,
    private TDataSliceFilterNodeDef, TDataSliceFilterNodeRuntimeInfo {
public:
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);
    virtual ~TDataSliceFilterNode() { /* nothing to do here */ }

    static const char *FilterTypeStr(EDSliceFilterType filter);

    bool IsValidForSrc() { return true; }
    bool Serialize(CCommBuf *cbuf, ESerializeMode mode);
    int invalidFuncP;

private:
    EDSliceFilterType m_filterType;
    List *m_baseFilterValues;
    int m_numBaseFilterValues;
    int m_tblid;

public:
    // Accessors:
    EDSliceFilterType FilterType() const { return m_filterType; }
    int NumBaseFilterValues() const { return m_numBaseFilterValues; }
    List *BaseFilterValues() { return m_baseFilterValues; }
    DataSliceFilterValue *BaseFilterValue(int c) const;
    int Tblid() const { return m_tblid; }
    bool DataQualified() const { return m_dataQualified; } 

    // Setters:
    void SetFilterType(EDSliceFilterType filterType) { m_filterType = filterType; }
    void SetTblid(int tblid) { m_tblid = tblid; }
    void SetBaseFilterValues(List *baseFilterValues);
    void SetDataQualified(bool dataQualified) { m_dataQualified = dataQualified;} 

    void Dump(int detail, int indent);
};

class TDataSliceHashNodeDef {
protected:
    // none at this time
};

class TDataSliceHashNodeRuntimeInfo {
protected:
    // none at this time
};

class TDataSliceHashNode:
    public SnippetNode,
    private TDataSliceHashNodeDef, TDataSliceHashNodeRuntimeInfo {
public:
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);
    virtual ~TDataSliceHashNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize(CCommBuf * cbuf, ESerializeMode mode);
    
    CBitVector dataSlices; //Bitvector of dataslices hashed for this DSHN
    bool m_stoppedEarly; //We have stopped hashing due to performance loss check before using dataSlices to set m_DataSliceHasStreamingData
    int m_hashedValues; //The number of values we have hashed so far with this DSHN

private:
    int m_numFields;
    int m_fieldsList[MAX_DIST_KEY_FIELDS];
//    int m_pgvartype;
//    int m_paramid;
    int m_tblid;
    CTable* m_tbl;

public:
    // Accessors:
    int NumFields() const { return m_numFields; }
    const int *FieldsList() const { return m_fieldsList; }
    // OK sslice never used
    // int PgVarType() const { return m_pgvartype; }
    // int Paramid() const { return m_paramid; }
    int Tblid() const { return m_tblid; }
    CTable* Table();
    void Init();

    // Setters:
    void SetHashFields(int numDistKeys, int *fieldIds);
//    void SetPgVarType(int pgvartype) { m_pgvartype = pgvartype; }
//    void SetParamid(int paramid) { m_paramid = paramid; }
    void SetTblid(int tblid) { m_tblid = tblid; }
    void SetTbl(CTable* tbl) { m_tbl = tbl; }

//    void Dump(int detail, int indent);
};

#endif // !XPDATASLICE_INCLUDED
