#ifndef XPDONE_INCLUDED
#define XPDONE_INCLUDED

#include "nodecommon.h"
#include "fcommchannel.h"

class TDoneNodeDef {
protected:
    // Nothing but runtime info at this time.
};

class TDoneNodeRuntimeInfo {
protected:
    unsigned m_numPackets;      // number of send-side packet buffers
    size_t   m_sizeReassembly;  // maximum size of reassembly buffers
    unsigned m_numReassembly;   // number of reassembly buffers
    bool     m_channelOpened;   // true if channel already opened
    TChannel m_channel;         // return channel info
};

class TDoneNode : public SnippetNode,
    private TDoneNodeDef, private TDoneNodeRuntimeInfo {
public:
    TableShape const * Init( TableShape const * src, EPlanNodeLocus place ) { return src; }
    virtual ~TDoneNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Address-of accessors
    // Used for things like virtual-table materialization.
    inline unsigned* AddrOfNumPackets()     { return &m_numPackets; }
    inline unsigned* AddrOfNumReassembly()  { return &m_numReassembly; }
    inline size_t*   AddrOfSizeReassembly() { return &m_sizeReassembly; }

    // Accessors
    inline TChannel Channel()         const { return m_channel; }
    inline bool     ChannelOpened()   const { return m_channelOpened; }
    inline unsigned NumPackets()      const { return m_numPackets; }
    inline unsigned NumReassembly()   const { return m_numReassembly; }
    inline size_t   SizeReassembly()  const { return m_sizeReassembly; }

    // Setters
    inline void SetChannel( TChannel ch )        { m_channel = ch; }
    inline void SetNumPackets( unsigned num )    { m_numPackets = num; }
    inline void SetNumReassembly( unsigned num ) { m_numReassembly = num; }
    inline void SetSizeReassembly( size_t size ) { m_sizeReassembly = size; }

    // Non-inlines
    void InitFcomm ( // initialize FComm for node
        EPlanNodeLocus calledFrom,  // caller (dbos or spu)
        int numNodes);  // number of nodes in the snippet
    void verifyChannelOpen(); // open the fcomm channel
    void computeCommMemory (
        unsigned numNodes,    // total number of nodes in the snippet
        unsigned *numPackets, // OUT. numPackets needed for transmit
        size_t *sizeReassembly, // size of reassembly buffer
        unsigned *numReassembly ); // number of reassembly buffers if any
};

#endif // !XPDONE_INCLUDED
