#ifndef XPDOWNLOAD_INCLUDED
#define XPDOWNLOAD_INCLUDED

#include "nodecommon.h"
#include "fcommchannel.h"
#include "nztimestamp.h"

class COSInteraction;
class TCreateTempTableNode;
class ZLib;
class DSliceInteraction;
struct TExttabHeader;
struct dbos_open_close_channel_t;
class TStreamInfo;
struct CloseChnlCallbackInfo;
struct TMemblockInfo;
struct List;

namespace mem { class Pool; };

enum EDownloadType {
    downloadBroadcast = 1,
    downloadDistribute
};

enum ESegmentType {
    NoSegment,
    DispersionSegment
};

#define DlTfNone              (0)     // No DownloadTable flags
#define DlTfChooseRowNum      (1<<0)  // DownloadTable: generate rowNums for records
#define DlTfEnableRowNum      (1<<1)  // DownloadTable: do we need row num
#define DlTfTrackNoDeletes    (1<<2)  // mark extents that have no deleted records
#define DlTfCaptureWaterMark  (1<<3)  // for propagation to materialized views

struct TDownloadTableCnt {
    uint64          m_avail;
    uint64          m_numRecords;
    uint32          m_pages;
    uint64          m_bytes;
};

class TDownloadTableSpace {
public:
    void        initCnt (bool distributed, mem::Pool *pool);
    void        updateResultTable (CTable * pTable);
    void        applyBlockCnt (TMemblockInfo *pBlock);
    void        applyBlockCnt (int dsid, TMemblockInfo *pBlock);
    uint32      getPages ();
    uint32      getMaxPages ();
    uint32      getPages (int dsid);
    uint64      getRecords ();
    uint64      getMaxRecords ();
    uint64      getRecords (int dsid);
    uint64      getBytes ();
    uint64      getMaxBytes ();
    uint64      getBytes (int dsid);
    uint64      getTransientPayloadSize ();

private:
    void        initCnt (TDownloadTableCnt *cnt);
    void        applyBlockCnt (TMemblockInfo *pBlock, TDownloadTableCnt *cnt);
    bool                m_distributed;
    TDownloadTableCnt   m_all;
    //Extra slot allocated so dataslices can be indexed at 1
    TDownloadTableCnt  *m_slice;
};

class TDownloadTableNodeDef {
protected:
    EDownloadType m_distMode;     // broadcast/distribute
    int           linkId;         // temp table id to use in SPU
    int           m_numDistKeys;  // how many distKey fields (MAX_DIST_KEY_FIELDS)
    int         * distKeys;       // array of distKeys
    int           spuId;          // for non-broadcast target
    int           keepStore;      // if true keep store on dbos after download
    ESegmentType  segmentType;    // Where the logical row is split into multiple physical rows.

    bool          m_haveComputedSpuId;     // allows for earlier spuId setting
    bool          m_bUsedInMultipleJoins;  // true if downloaded table is used in multipe joins
    bool          m_isPrebroadcastTbl;     // true if this table was prebroadcasted
    bool          m_snipLastDownloadNode;  // true if the last download node in the snippet
    bool          m_bCompressed;           // true if we are loading compressed data
    uint32        m_downloadTableFlags;    // Download table flags provide hints to logic
    List        * m_nullTestQual; // PG expression for not exist join (NOT IN and relop ALL)
    bool          m_bDownloadReplFormat; // true if downloaded data in replication format
};

class TDownloadTableNodeRuntimeInfo {
protected:
    int64           insCount;     // how many records inserted     ckpt
    uint64          tableSize;    // how much of data downloaded  ckpt
    int64           insCountCkpt;
    uint64          tableSizeCkpt;
    TMemblockInfo **m_distBlocks; // use more direct access
    TDownloadTableSpace
                   *m_tableSpace;
    int m_distBlockSize;    // size of each block (includes header)
    int m_minDistBlockSize; // minimum size of distribute block needed
    int m_spuId;            // current spu's block being packed
    int m_partialBlock;     // non-zero size if raw block loading split a block
    int m_partialBlockSpu;  // distblock number of last partial block

    DSliceInteraction         *m_downloadDsItcn;       // dataslice-specific interaction state
    COSInteraction            *m_downloadInteraction;  // interaction state for broadcasts
    COSInteraction            *m_closeChnlInteraction; // SPU_CLOSE_CHANNEL interaction
    dbos_open_close_channel_t *m_closeChnlMsgP;        // content for close channel interaction
    CloseChnlCallbackInfo     *m_closeChnlCallBkInfoP; // content for close channel call back

    bool     m_haveSyncedChannel;   // true after SPU_SYNC_DIST_PROCEED recieved
    int      m_interactionId;       // spu2spu: download interaction id (of above)
    int      m_syncId;              // spu2spu: sync interaction id

    TChannel m_fcommChannel;    // spu2spu: channel as passed down from host
    unsigned m_numPackets;      // spu2spu: number of send-side packet buffers
    size_t   m_sizeReassembly;  // spu2spu: maximum size of reassembly buffers
    unsigned m_numReassembly;   // spu2spu: number of reassembly buffers

    bool     m_isDiskTable;     // true if actually writing to disk table

    TCreateTempTableNode *m_createNode;

    uint32   m_messagesSent;     // Messages originated
    uint64   m_bytesInMessages;  // Total bytes originated
    int      m_recPayloadSize;   // size of biggest record that can fit in packet

    TMemblockInfo * m_largeDistBlock;      // work area for large distros

    TStreamInfo   * m_toDownloadStreamPtr; // for genc's benefit to find download Info
    uint32          m_snSizePrediction;    // link in dist. size prediction feed-fwd
    bool            m_dsIdDist;            // true if dist key should value should be treated like a datasliceid

    bool            m_haveProcessedHeader; // only read header on compressed ext file once
    char          * m_reloadRecBuf;      // buffer for uncompressing blocks on host
    char          * m_partialBlockBuf;   // buffer for a partial block
    bool            m_bUncompressOnHost;   // uncompress on host when machine size differs
    bool            m_fromLittleEndianSpu;     // unload was from spu's that were little endian
    nzotuid_t       m_ZLibOwner;     // Since we are allocating the memory for zlib in Heap we need to save the owner for cleanup
    ZLib          * m_ZLib;          // compression

    time64_t        m_usLoadStartTime;
    time64_t        m_usChunkStartTime;
    int64           m_usTotalSleepCredits;
    int64           m_maxSleepCreditAccum; //maximum allowed "sleep credits."

    int             m_targetChecksPerSecond;

    uint64          m_downloadBytesSent;
    int             m_actualKBPerSec;
    uint64          m_chunkBytesSent;
    uint64          m_chunkUncompressedBytesSent;
    uint64          m_chunkCompressedBytesSent;

    // Periodic resource update counters
    int           * m_unreportedBytesSpu;
    int             m_unreportedBytesHost;

    int             m_pcodeStartNullTestQual; // number of first pcode instruction null test qual
    int             m_pcodeCountNullTestQual; // number of pcode instructions for null test qual

};

class TDownloadTableNode : public SnippetNode,
    private TDownloadTableNodeDef, private TDownloadTableNodeRuntimeInfo {
public:
    TDownloadTableNode();
    virtual ~TDownloadTableNode();
    
    TableShape const * Init(TableShape const * src, EPlanNodeLocus place);
// dlm-check    virtual ~TDownloadTableNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Address-of accessors
    // Used for things like virtual-table materialization.
    inline EDownloadType*  AddrOfDistMode()            { return &m_distMode; }
    inline int*            AddrOfLinkId()              { return &linkId; }
    inline uint64*         AddrOfTableSize()           { return &tableSize; }

    // Accessors
    inline bool            Compressed()          const { return m_bCompressed; }
    inline TMemblockInfo** DistBlocks()                { return m_distBlocks; }
    inline int*            DistKeys()            const { return distKeys; }
    inline EDownloadType   DistMode()            const { return m_distMode; }
    inline COSInteraction* DownloadInteraction() const { return m_downloadInteraction; }
    inline uint32          DownloadTableFlags()  const { return m_downloadTableFlags; }
    inline bool            DsIdDist()            const { return m_dsIdDist; }
    inline TChannel        FcommChannel()        const { return m_fcommChannel; }
    inline bool            HaveComputedSpuId()   const { return m_haveComputedSpuId; }
    inline int64           InsCount()            const { return insCount; }
    inline int             InteractionId()       const { return m_interactionId; }
    inline bool            IsBroadcast()         const { return m_distMode == downloadBroadcast; }
    inline bool            IsDistribute()        const { return m_distMode == downloadDistribute; }
    inline bool            IsDiskTable()         const { return m_isDiskTable; }
    inline bool            IsPrebroadcastTbl()   const { return m_isPrebroadcastTbl; }
    inline bool            IsSpu2SpuDownload()   const { return Place() == inSPU; }  // override SnippetNode
    inline int             KeepStore()           const { return keepStore; }
    inline TMemblockInfo*  LargeDistBlock()            { return m_largeDistBlock; }
    inline int             LinkId()              const { return linkId; }
    inline List*           NullTestQual()        const { return m_nullTestQual; }
    inline int             NumDistKeys()         const { return m_numDistKeys; }
    inline unsigned        NumPackets()          const { return m_numPackets; }
    inline unsigned        NumReassembly()       const { return m_numReassembly; }
    inline int             PcodeStartNullTestQual() const { return m_pcodeStartNullTestQual; }
    inline int             PcodeCountNullTestQual() const { return m_pcodeCountNullTestQual; }
    inline int             RecPayloadSize()      const { return m_recPayloadSize; }
    inline ESegmentType    SegmentType()         const { return segmentType; }
    inline size_t          SizeReassembly()      const { return m_sizeReassembly; }
    inline int             SpuId()               const { return spuId; }
    inline int             SyncId()              const { return m_syncId; }
    inline uint64          TableSize()           const { return tableSize; }
    inline TStreamInfo*    ToDownloadStreamPtr() const { return m_toDownloadStreamPtr; }
    inline bool            UsedInMultipleJoins() const { return m_bUsedInMultipleJoins; }
    inline bool            UncompressOnHost()    const { return m_bUncompressOnHost; }
    inline bool            DownloadReplFormat()  const { return m_bDownloadReplFormat; }

    // Mutators
    inline void IncInsCount()                               { insCount++; }
    inline void IncTableSize(uint64 val)                    { tableSize += val; }

    // Setters
    inline void SetCreateNode(TCreateTempTableNode* node)   { m_createNode = node; }
    inline void SetDistBlockSize(int size)                  { m_distBlockSize = size; }
    inline void SetCompressed(bool val)                     { m_bCompressed = val; }
    inline void SetDistKeys(int* keys)                      { distKeys = keys; }
    inline void SetDistMode(EDownloadType mode)             { m_distMode = mode; }
    inline void SetDownloadInteraction(COSInteraction* itp) { m_downloadInteraction = itp; }
    inline void SetDownloadTableFlags(uint32 flags)         { m_downloadTableFlags = flags; }
    inline void SetDsIdDist(bool val)                       { m_dsIdDist = val; }
    inline void SetFcommChannel(TChannel ch)                { m_fcommChannel = ch; }
    inline void SetHaveComputedSpuId(int val)               { m_haveComputedSpuId = val; }
    inline void SetInteractionId(int itn)                   { m_interactionId = itn; }
    inline void SetIsPrebroadcastTbl(bool val)              { m_isPrebroadcastTbl = val; }
    inline void SetKeepStore(int keep)                      { keepStore = keep; }
    inline void SetLinkId(int link)                         { linkId = link; }
    inline void SetNullTestQual(List* val)                  { m_nullTestQual = val; }
    inline void SetNumDistKeys(int num)                     { m_numDistKeys = num; }
    inline void SetNumPackets(unsigned num)                 { m_numPackets = num; }
    inline void SetNumReassembly(unsigned num)              { m_numReassembly = num; }
    inline void SetPcodeStartNullTestQual(int start)        { m_pcodeStartNullTestQual = start; }
    inline void SetPcodeCountNullTestQual(int count)        { m_pcodeCountNullTestQual = count; }
    inline void SetSegmentType(ESegmentType type)           { segmentType = type; }
    inline void SetSizeReassembly(size_t size)              { m_sizeReassembly = size; }
    inline void SetSnipLastDownloadNode(bool bb)            { m_snipLastDownloadNode = bb; }
    inline void SetSnSizePrediction(uint32 size)            { m_snSizePrediction = size; }
    inline void SetSpuId(int spu)                           { spuId = spu; }
    inline void SetSyncId(int sid)                          { m_syncId = sid; }
    inline void SetToDownloadStreamPtr(TStreamInfo* val)    { m_toDownloadStreamPtr = val; }
    inline void SetUsedInMultipleJoins(bool val)            { m_bUsedInMultipleJoins = val; }
    inline void SetDownloadReplFormat(bool val)             { m_bDownloadReplFormat = val; }

    // Non-inlines
    COSInteraction *createTheTempTable(CPlan *plan);
    TDownloadTableNode *NeedsPrep();

    uint pagesNeeded() const; // returns the number of pages that will need
    void getDistBuffer(const TChannel& ch, int dsId);
    void doneDistBuffer(int dsId);
    void freeDistBuffer(int dsId);

    DSliceInteraction *cleanupDistBlocks();
    void verifyBuffers(); // initialize m_distBlocks
                              // to be allocated for m_distBlocks
    void spuSendBlock(  // Spu2Spu recordstore block sender
        int spuId);      // dataslice id (1-based)

    void cleanupDownloadInteraction(bool notifySched);
    void cleanupDownloadZLib(void);
    void cleanupDownloadLargeDistBlock(void);

    void broadcastNewBlock( // wrapper to plan's xmit logic
        bool lastCall);

    int writeBlock(        // direct block reload
        TMemblockInfo *block,
        char *fence,       // end-of-buffer may require a block-splitting
        bool lastcall);

    void hostSendBlock( // Host recordstore block sender
        int spuId);     // dataslice id (1-based)

    void hostSendBlockInReplFormat (int spuId);

    void hostSendLargeBlock( // send the large block attached to download node
        int spuId);

    void flushBlocks(   // Send outstanding packet plus large buffer
        int spuId);     // dataslice id (1-based)

    void broadcastWrapup(); // Complete broadcast interaction
    void hostDistWrapup();  // Complete hostDist interaction
    void spuDistWrapup();   // Complete spuDist interaction

    void bcastThisBlock(    // Broadcast current block
            CTable *dst, uint8 msgFlags);   // Table is assumed to have just one block

    void computeCommMemory (  // compute comm memory for distribute
        size_t maxRecordSize,
        EChannelType chType,
        unsigned *numPkts,
        size_t *sizeReassembly,
        unsigned *numReassembly);

    void checkpoint();
    void rollback();
    void StopDownload ();
    void RestartDownload ();
    bool FlushPartialBlocks ();

    void AllocateResources();

 private:
    void hostSendHelper( // Lowest level distributer
        TMemblockInfo *m,       // memory block to send
        int dsId,              // dataslice to send it to (1-based)
        bool isSmall,           // if true, comm-owned packet
        uint8 msgFlags);         // if MSG_IS_LAST is set, spu can close interaction

    void updateRateInfo( time64_t chunkEndTime);
    void updateResourceUsage(int spuId, uint32 newBytesDownloaded, uint32 newBytesWrittenToSPU);



    void setupInteractionAndChannel ();

    int processExttabHeader(    // read & process exttab header (when exists)
        const TMemblockInfo * const block, int bytecount );
    void verifyExternalTable(   // verify reload state compatible with unload
                              const TExttabHeader* etheader );
    uint verifyExternalTableHeader( // read header from external table file
        TExttabHeader * const srcetheader, int bytecount );
    int sendCompressedToSPUs (  // send compressed rec blocs direct to SPUs
                                const TMemblockInfo * const block,
                                const char * const fence );
    int uncompressAndDistribute ( // uncompress record block on host & dist
                                TMemblockInfo * const block,
                                const char * const fence );
    void failReloadValidityCheck( bool checkdisabled, const char *failure,
                                  const char *tblname, int colno );
    void verifyBlockHeader(     // check record block header
                                const TMemblockInfo * const block ) const;
    // for redistributed compressed reload support
    void addRec(record_t *pRec, int size, int spuId);
    void redistributeRecords ( CTable *pTable, char *pBuf, int totalSize,
                     bool fromBigEndianSpu );

    void spuSendHelper( // Lowest level distributer
        TMemblockInfo *m,       // memory block to send
        int spuId,              // spu to send it to (1-based)
        bool isSmall,           // if true, comm-owned packet
        uint8 msgFlags);         // if MSG_IS_LAST is set, spu can close interaction

    void initPacketBuffers(); // Create the m_distBlocks info
};

#endif // !XPDOWNLOAD_INCLUDED

