#ifndef XPEXISTS_INCLUDED
#define XPEXISTS_INCLUDED

#include "xphashjoin.h"

//
//  exists node has only postJoinQuals, other quals will be null
//

class TExistsJoinNodeDef {
protected:
    int   m_notJoin;        // if TRUE, is a non-existance check
    int   m_alreadySorted;  // if TRUE, don't need to sort the temp table
    int   m_numFields;      // how many refs in tblField/joinField
    int * m_tblFields;      // field in temp table
    int * m_joinFields;     // field in pipeline table
    // true => nulls considered equal for this join field  (/=/ operator)
    // may be NULL => all false
    bool * m_nullsCompareFields;
    int   m_leftMode;       // if TRUE, return 'temp' recs rather than 'stream' recs
};

class TExistsJoinNodeRuntimeInfo  {
protected:
    TMemblockInfo * m_tempTrackingBlock;       // for sorted exists node (Host)
    uint32          m_tempTrackingBlockNum;    // for sorted exists node (SPU)
    record_t      * m_tempTrackingBlockRecord; // for sorted exists node: trackingRec
    int             m_joinDex;                 // for sorted exists node
    uint64          m_numRowsLeftInTemp;

//    // for node stats gathering
//    uint64          m_numRecsInInner;
//    uint64          m_numIncomingRecs;    // The usual "recs" field in node stats is the *outgoing* count.
};

class TExistsJoinNode : public THashJoinNode,
    private TExistsJoinNodeDef, private TExistsJoinNodeRuntimeInfo {
public:
    TExistsJoinNode();
    virtual ~TExistsJoinNode();
    
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);

    // Accessors
    inline int            AlreadySorted()           const { return m_alreadySorted; }
    inline int*           JoinFields()              const { return m_joinFields; }
    inline int            JoinDex()                 const { return m_joinDex; }
    inline int            LeftMode()                const { return m_leftMode; }
    inline int            NotJoin()                 const { return m_notJoin; }
    inline int            NumFields()               const { return m_numFields; }
    inline uint64         NumRowsLeftInTemp()       const { return m_numRowsLeftInTemp; }
    inline int*           TblFields()               const { return m_tblFields; }
    inline bool*          NullsCompareFields()      const { return m_nullsCompareFields; }

    inline TMemblockInfo* TempTrackingBlock()             { return m_tempTrackingBlock; }
    inline uint32         TempTrackingBlockNum()    const { return m_tempTrackingBlockNum; }
    inline record_t*      TempTrackingBlockRecord()       { return m_tempTrackingBlockRecord; }

    // Testers: Overrides of JoinNode defaults:
    bool IsExprStyle()             const { return (0 == m_numFields); }                    // ExistsJoin override JoinNode
    bool IsHashed()                const { return (m_numFields > 0) && !m_alreadySorted; } // ExistsJoin override JoinNode
    bool IsMergeStyle()            const { return m_alreadySorted && (m_numFields > 0); }  // ExistsJoin override JoinNode
    bool IsStreamOuterStyle()      const { return !m_leftMode; }                           // ExistsJoin override JoinNode
    bool NeedsRightOuterJoinPass() const { return m_leftMode && m_notJoin; }               // ExistsJoin override JoinNode

    // Non-void mutators
    inline int  PreIncJoinDex()                       { return ++m_joinDex; }

    // Mutators
    inline void DecNumRowsLeftInTemp()                { m_numRowsLeftInTemp--; }

    // Setters
    inline void SetNotJoin( int val )                       { m_notJoin = val; }
    inline void SetAlreadySorted( int val )                 { m_alreadySorted = val; }
    inline void SetNumFields( int val )                     { m_numFields = val; }
    inline void SetJoinFields( int* val )                   { m_joinFields = val; }
    inline void SetLeftMode( int val )                      { m_leftMode = val; }
    inline void SetNumRowsLeftInTemp( uint64 num )          { m_numRowsLeftInTemp = num; }
    inline void SetTblFields( int* val )                    { m_tblFields = val; }
    inline void SetNullsCompareFields( bool* val )          { m_nullsCompareFields = val; }
    inline void SetTempTrackingBlock( TMemblockInfo* val )  { m_tempTrackingBlock = val; }
    inline void SetTempTrackingBlockNum( uint32 val )       { m_tempTrackingBlockNum = val; }
    inline void SetTempTrackingBlockRecord( record_t* val ) { m_tempTrackingBlockRecord = val; }
};


#endif // !XPEXISTS_INCLUDED
