/*
 * Copyright 2014 IBM Corporation
 */

#ifndef XPFUNCJOIN_INCLUDED
#define XPFUNCJOIN_INCLUDED

#include "xpjoin.h"
#include "nodecommon.h"

/* Table function join */

struct TFuncField {
    uint16 type;
    uint16 size;
};

struct Func;

class TFuncJoinNodeDef {
protected:
    bool         m_correlated;
    int          m_numArgs;    // number of arguments table function takes
    int        * m_argCols;    // array of m_numArgs ints mapping input
                               // stream cols to func args
    int          m_numFields;  // tfunc field count
    TFuncField * m_fieldList;  // tfunc field descriptions

    Func       * m_pFuncInfo;  // The Function info (similiar to UDF)
    bool         m_bOver;      // Over clause present
    bool         m_bOrder;     // Order clause present
    bool         m_bPartition;       // Partition clause present
    int          m_numPartitionCols; // number of partition columns
    int        * m_partitionColList; // list of partition columns
    int          m_numNotNulls;
    bool       * m_notNulls;
    void       * m_partitionRow;    // Used for memory of stuct
    bool         m_endOfPartition;
    bool         m_firstRow;

};





class TFuncJoinNodeRuntimeInfo {
protected:
    CTable * m_funcTable;   // the CTable shape of the table function
    void   * m_funcRecord;  // the record produced by the table function
    void   * m_varBuf;      // buffer to hold varying data returned from UDTF
};

class TFuncJoinNode : public TJoinNode, 
    private TFuncJoinNodeDef, private TFuncJoinNodeRuntimeInfo {
public:                        // join (always followed by a project)
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);
    virtual ~TFuncJoinNode() { /* nothing to do here */ }

    // Accessors
    inline int*        ArgCols()          const { return m_argCols; }
    inline bool        Correlated()       const { return m_correlated; }
    inline TFuncField* FieldList()        const { return m_fieldList; }
    inline Func*       FuncInfo()         const { return m_pFuncInfo; }
    inline void*       FuncRecord()       const { return m_funcRecord; }
    inline CTable*     FuncTable()        const { return m_funcTable; }
    inline int         NumArgs()          const { return m_numArgs; }
    inline int         NumFields()        const { return m_numFields; }
    inline void*       VarBuf()           const { return m_varBuf; }
    inline void*       PartitionRow()           { return m_partitionRow; }
    inline int         NumPartitionCols() const { return m_numPartitionCols; }
    inline int*        PartitionColList()       { return m_partitionColList; }
    inline int         NumNotNulls()      const { return m_numNotNulls; }
    inline bool*       NotNulls()               { return m_notNulls; }
    inline bool        Over()             const { return m_bOver; }
    inline bool        Order()            const { return m_bOrder; }
    inline bool        Partition()        const { return m_bPartition; }
    inline bool        EndOfPartition()   const { return m_endOfPartition; }
    inline bool        FirstRow()         const { return m_firstRow; }

    // Setters
    inline void SetArgCols( int* val )          { m_argCols = val; }
    inline void SetCorrelated( bool val )       { m_correlated = val; }
    inline void SetFieldList( TFuncField* val ) { m_fieldList = val; }
    inline void SetFuncInfo( Func* val )        { m_pFuncInfo = val; }
    inline void SetNumArgs( int val )           { m_numArgs = val; }
    inline void SetNumFields( int val )         { m_numFields = val; }
    inline void SetPartitionRow( void* val )    { m_partitionRow = val; }
    inline void SetPartitionColList( int* val ) { m_partitionColList = val; }
    inline void SetNumPartitionCols( int val )  { m_numPartitionCols = val; }
    inline void SetNotNulls( bool* val )        { m_notNulls = val; }
    inline void SetNumNotNulls( int val )       { m_numNotNulls = val; }
    inline void SetOver(bool val)               { m_bOver = val; }
    inline void SetOrder(bool val)              { m_bOrder = val; }
    inline void SetPartition(bool val)          { m_bPartition = val; }
    inline void SetEndOfPartition(bool val)     { m_endOfPartition = val; }
    inline void SetFirstRow(bool val)           { m_firstRow = val; }
};

#endif // !XPFUNCJOIN_INCLUDED
