#ifndef INCLUDED_XPGROUPINGSET
#define INCLUDED_XPGROUPINGSET

#include "nodecommon.h"
#include "groupingsetinfo.h"

class TGroupingSetNodeDef {
protected:
    int             m_groupingSetID; // handle to plan's m_groupingSetInfoP entry 
};

class TGroupingSetNodeRuntimeInfo {
protected:
    int              m_vecSz;        // number of bytes to hold null state bits AND size of a grouping set list
    unsigned char   *m_nullFlags;    // that many bytes to hold them
    GroupingSetInfo *m_gsInfo;       // info packet for this node's grouping set ID
};

class TGroupingSetNode : public SnippetNode,
    private TGroupingSetNodeDef, private TGroupingSetNodeRuntimeInfo {
public:
    const TableShape *Init(const TableShape *src, EPlanNodeLocus calledFrom);
    virtual ~TGroupingSetNode() { /* nothing to do here */ }

    // Accessors
    inline int GroupingSetID() const { return m_groupingSetID; }

    // Testers
    inline bool GetNullFlag(int bitNum)           const { return 0 != (m_nullFlags[bitNum/8] & (1<<(bitNum%8))); }
    inline bool SuperAggregate(int gsn, int gnum) const { return 0 == (m_gsInfo->m_grpVectors[gsn][gnum/8] & (1<<(gnum%8))); }

    // Mutators
    inline void ClearNullFlags()            { memset(m_nullFlags, 0, m_vecSz); }
    inline void SetNullFlag(int bitNum)     { m_nullFlags[bitNum/8] |= (1<<(bitNum%8)); }

    // Setters
    inline void SetGroupingSetID( int val ) { m_groupingSetID = val; }
};

#endif // !INCLUDED_XPGROUPINGSET
