#ifndef XPHASH_INCLUDED
#define XPHASH_INCLUDED

#include "nodecommon.h"

class THashJoinNode;

/* Support for execution plan hashing */

#define HASH_FLAG_NONE     0
#define HASH_FLAG_FOR_JOIN (1<<0) // hash is for a hash join
#define HASH_FLAG_TYPE_MISMATCH (1<<1) // types don't match in hash for join

class THashNodeDef {
protected:
    int numFields;      // how many fields
    int *fieldsList;    // field nums
    int tblId;          // table to pick fields from.  0 means stream
    int hashFlags;      // extra hints to dbos/spu
};

class THashNodeRuntimeInfo {
protected:
    bool handledByFpga; // true if handled by fpga
    bool hashed;        // goes true once plan hashes non-zero tblId member

    // for node stats gathering
    // These are no longer maintained.  (In fact, they haven't been for quite some time.)
    // They are here only because the "_vt_plan_Hash" virtual table references them.
    // See nde/vt/vt_master.src.
    uint32 m_numChains; // number of non-empty chains in hash table
    uint32 m_longestChain;  // which collision chain has most members
};

class THashNode : public SnippetNode,
    private THashNodeDef, private THashNodeRuntimeInfo {
public:
    THashNode(); 
    virtual ~THashNode();
    
    TableShape const * Init( TableShape const * src, EPlanNodeLocus place);
// dlm-check    virtual ~THashNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize  ( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Address-of accessors
    // Used for things like virtual-table materialization.
    inline uint32* AddrOfLongestChain() { return &m_longestChain; }
    inline uint32* AddrOfNumChains()    { return &m_numChains; }
    inline int*    AddrOfTblId()        { return &tblId; }

    // Accessors
    inline int*   FieldsList()     const { return fieldsList; }
    inline bool   HandledByFpga()  const { return handledByFpga; }
    inline bool   Hashed()         const { return hashed; }
    inline int    HashFlags()      const { return hashFlags; }
    inline uint32 LongestChain()   const { return m_longestChain; }
    inline uint32 NumChains()      const { return m_numChains; }
    inline int    NumFields()      const { return numFields; }
    inline int    TblId()          const { return tblId; }

    // Mutators
    inline void AddNumChains( uint32 num ) { m_numChains += num; }

    // Setters
    inline void SetFieldsList( int* list )      { fieldsList = list; }
    inline void SetHandledByFpga( bool bb )     { handledByFpga = bb; }
    inline void SetHashed( bool bb )            { hashed = bb; }
    inline void SetHashFlags( int flags )       { hashFlags = flags; }
    inline void SetLongestChain( uint32 chain ) { m_longestChain = chain; }
    inline void SetNumChains( uint32 num )      { m_numChains = num; }
    inline void SetNumFields( uint32 num )      { numFields = num; }
    inline void SetTblId( int val )             { tblId = val; }

    // Non-inlines
    THashNode *Clone();

};

void HashNodesTable(THashNode *pNode, THashJoinNode *pHJ, ENodeTypes consumer);
void HashStatDump(THashNode *pNode , CRecordStore *pRS);
#endif // !XPHASH_INCLUDED
