#ifndef XPINSERT_INCLUDED
#define XPINSERT_INCLUDED

#include "nodecommon.h"
#include "storagedefs.h" // for EPartition

class TStreamInfo;
class CDistSet;

enum InsertNodeFlags {
    IfNone             =  0,      // No insert flags
    IfTarCopyRowNum    = (1<<0),  // Insert: copy rowNum from source to target
    IfCtas             = (1<<1),  // CTAS insert
    IfCMview           = (1<<2),  // MView creation
    IfCaptureWaterMark = (1<<3),  // for MView propagation
    IfStartNewPage     = (1<<4),  // for groom - start appending to a new page
    IfCBT              = (1<<5),  // for groom versions - inserting into a CBT
    IfTrackNoDeletes   = (1<<6),  // mark extents that have no deleted records
};

struct TInsertNodeDef {
protected:
    table_id_t    tblId;            // which table to insert records into
    uint32 m_insertFlags;    // hints about how to handle specials on insert
};

struct TInsertNodeRuntimeInfo {
protected:
    int64         insCount;          // how many records inserted
    CTable      * toInsert;          // (in spu) result of dictionary lookup on tblId
    TStreamInfo * toInsertStreamPtr; // for genc's benefit to find toInsert Info
    CDistSet    * m_pTableDist;      // distribution control
    int           m_dsid;            // derivable from m_plan, cached for convenience
};

// insert records
class TInsertNode : public SnippetNode,
    private TInsertNodeDef, private TInsertNodeRuntimeInfo {
public:
    TInsertNode();
    virtual ~TInsertNode();
    
    TableShape const * Init( TableShape const * src, EPlanNodeLocus place );

    // Accessors
    inline int64        InsCount()          const { return insCount; }
    inline uint32       InsertFlags()       const { return m_insertFlags; }
    inline CDistSet*    TableDist()         const { return m_pTableDist; }
    inline table_id_t   TblId()             const { return tblId; }
    inline CTable*      ToInsert()          const { return toInsert; }
    inline TStreamInfo* ToInsertStreamPtr() const { return toInsertStreamPtr; }

    // Testers
    inline bool IsStartNewPage()                   const { return m_insertFlags & IfStartNewPage; }

    // Mutators
    inline void IncrInsertCount()                        { insCount++; }

    // Setters
    inline void SetInsertFlags( uint32 flags )           { m_insertFlags = flags; }
    inline void SetToInsert( CTable* tbl )               { toInsert = tbl; }
    inline void SetTableDist( CDistSet* dist )           { m_pTableDist = dist; }
    inline void SetTblId( table_id_t val )               { tblId = val; }
    inline void SetToInsertStreamPtr( TStreamInfo* val ) { toInsertStreamPtr = val; }

    // Multi-line setters
    inline void SetStartNewPage(bool newValue) {
        if (newValue)
            m_insertFlags |= (uint32) IfStartNewPage;
        else
            m_insertFlags &= ~(uint32)IfStartNewPage;
    }

    // Non-inlines
    void insertNewBlock(bool lastCall); // Finish inserting

    void writeTableRecords();
};

#endif // !XPINSERT_INCLUDED
