#ifndef XPINSERTSTATS_INCLUDED
#define XPINSERTSTATS_INCLUDED

#include "nodecommon.h"
#include "table.h"

struct TStreamInfo;

class TInsertStatsNodeDef {
protected:
    bool m_consolidating;         // host consolidation for spu stats
    bool m_noMinMax;              // don't track min/max (e.g. for thintables or reclaim)
    bool m_forReclaim;            // inserting for reclaim
    // Processing the merge action table of a MERGE statement. In this case
    // the node does the following:
    // - count inserted, updated and deleted rows separately
    // - determine min/max values of inserted and updated rows
    // Thus, the resulting table contains three columns for the
    // insert/update/deleted counters, followed by the min/max values
    // for all rows of the target table.
    // The resulting table contains either one or two rows:
    // - If both the insert and update counters are 0, there is no point in
    //   collecting min/max values, such that only one row containing the 3
    //   counters is returned.
    // - Otherwise, one row holds the counters and the max values and another
    //   row holds the min values. The min value row has a negative insert
    //   counter.
    bool m_IsForMerge;
};

class TInsertStatsNodeRuntimeInfo {
protected:
    record_t    * m_minRec;        // min record pointer
    record_t    * m_maxRec;        // max record pointer
    TStreamInfo * m_gencMinStream; // for min values record
    TStreamInfo * m_gencMaxStream; // for max values record
};

class TInsertStatsNode : public SnippetNode,
    private TInsertStatsNodeDef, private TInsertStatsNodeRuntimeInfo {
public:
    TInsertStatsNode();
    virtual ~TInsertStatsNode();

    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);

    // Accessors
    inline bool         Consolidating()   const { return m_consolidating; }
    inline bool         ForReclaim()      const { return m_forReclaim; }
    inline TStreamInfo* GencMaxStream()   const { return m_gencMaxStream; }
    inline TStreamInfo* GencMinStream()   const { return m_gencMinStream; }
    inline record_t*    MaxRec()          const { return m_maxRec; }
    inline record_t*    MinRec()          const { return m_minRec; }
    inline bool         NoMinMax()        const { return m_noMinMax; }
    inline bool         IsForMerge()      const { return m_IsForMerge; }

    // Setters
    inline void SetConsolidating( bool val )         { m_consolidating = val; }
    inline void SetForReclaim( bool val )            { m_forReclaim = val; }
    inline void SetGencMaxStream( TStreamInfo* val ) { m_gencMaxStream = val; }
    inline void SetGencMinStream( TStreamInfo* val ) { m_gencMinStream = val; }
    inline void SetNoMinMax( bool val )              { m_noMinMax = val; }
    inline void SetIsForMerge( bool val )            { m_IsForMerge = val; }

    // Non-inlines
    void cleanUp();               // cleanUp entry point
};

#endif // !XPINSERTSTATS_INCLUDED

