#ifndef XPMERGE_INCLUDED
#define XPMERGE_INCLUDED

#include "nodecommon.h"

class TMergeNodeDef {
protected:
    int   m_tblId;         // already existant temp table
    int   m_nCols;         // number columns to compare for merging
    int * m_pCols;         // array of columns to compare for merging
    int * m_pAsc;          // ascending or descending list
    int * m_pNullsLo;      // nulls collation list
};

class TMergeNodeRuntimeInfo {
protected:
    CTable        * m_pTempTbl; // temp table itself 

    uint32          m_blockNum; // the blocknum of the block we are processing
    TMemblockInfo * m_mnBlock;  // the block we are processing
    record_t      * m_mnRecord; // the record (of the block) we are  processing 
    record_t      * m_pBuf;     //for materializing    
};

//
//  Merge records from stream and temp table.
//  The shape of the stream and temp table is identical and both the
//  tables are ordered 
//
class TMergeNode : public SnippetNode,
    private TMergeNodeDef, private TMergeNodeRuntimeInfo {
public:
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);
    virtual ~TMergeNode() { /* nothing to do here */ }

    // Accessors
    inline int*           Asc()      const { return m_pAsc; }
    inline record_t*      Buf()            { return m_pBuf; }
    inline uint32         BlockNum() const { return m_blockNum; }
    inline int*           Cols()     const { return m_pCols; }
    inline TMemblockInfo* MnBlock()        { return m_mnBlock; }
    inline record_t*      MnRecord()       { return m_mnRecord; }
    inline int            NCols()    const { return m_nCols; }
    inline int*           NullsLo()  const { return m_pNullsLo; }
    inline int            TblId()    const { return m_tblId; }
    inline CTable*        TempTbl()  const { return m_pTempTbl; }

    // Non-void mutators
    inline uint32         PostIncBlockNum() { return m_blockNum++; }

    // Setters
    inline void SetAsc( int* val )               { m_pAsc = val; }
    inline void SetBlockNum( uint32 val )        { m_blockNum = val; }
    inline void SetCols( int* val )              { m_pCols = val; }
    inline void SetMnBlock( TMemblockInfo* val ) { m_mnBlock = val; }
    inline void SetMnRecord( record_t* val )     { m_mnRecord = val; }
    inline void SetNCols( int val )              { m_nCols = val; }
    inline void SetNullsLo( int* val )           { m_pNullsLo = val; }
    inline void SetTblId( int val )              { m_tblId = val; }
};


#endif // !XPMERGE_INCLUDED
