#ifndef XPMERGEJOIN_INCLUDED
#define XPMERGEJOIN_INCLUDED

#include "xpjoin.h"

//  Quals for MergeJoinNode
//  MergeJoin nodes may contain joinQuals
//  In addition to 'joinQual' outer join nodes may contain 'otherQual'
//
class TMergeJoinNodeDef {
protected:
    int  m_joinDex;    // for merge joins, last spot in join table
    bool m_preSorted;  // true if temp table already sorted
};

class TMergeJoinNodeRuntimeInfo {
protected:
    TMemblockInfo * m_tempTrackingBlock;       // tracking block (Host)
    uint32          m_tempTrackingBlockNum;    // blockNum of tracking block (SPU)
    record_t      * m_tempTrackingBlockRecord; // trackingRec in block
};

class TMergeJoinNode : public TJoinNode,
    private TMergeJoinNodeDef, private TMergeJoinNodeRuntimeInfo {
public:
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);
    virtual ~TMergeJoinNode() { /* nothing to do here */ }

    // Accessors
    inline int            JoinDex()                 const { return m_joinDex; }
    inline bool           PreSorted()               const { return m_preSorted; }
    inline TMemblockInfo* TempTrackingBlock()             { return m_tempTrackingBlock; }
    inline uint32         TempTrackingBlockNum()    const { return m_tempTrackingBlockNum; }
    inline record_t*      TempTrackingBlockRecord()       { return m_tempTrackingBlockRecord; }

    // Testers: Override of JoinNode defaults
    inline bool           IsMergeStyle()            const { return true; } // MergeJoin override JoinNode

    // Non-void mutators
    inline int  PreIncJoinDex()                           { return ++m_joinDex; }

    // Setters
    inline void SetPreSorted( bool bb )                     { m_preSorted = bb; }
    inline void SetTempTrackingBlock( TMemblockInfo* val )  { m_tempTrackingBlock = val; }
    inline void SetTempTrackingBlockNum( uint32 val )       { m_tempTrackingBlockNum = val; }
    inline void SetTempTrackingBlockRecord( record_t* val ) { m_tempTrackingBlockRecord = val; }
};

#endif // !XPMERGEJOIN_INCLUDED
