#ifndef XPPARSER_INCLUDED
#define XPPARSER_INCLUDED

#if !FOR_SPU

#include <stdlib.h>
#include <time.h>
#include "nodecommon.h"
#include "nzlimits.h"

class CTable;

namespace rc {
  class File;
}

namespace ldr {
  class Parser;
}

namespace et {
  class ETOptions;
  class Log;
}

#include "xpunload_enums.h"

struct TLoadBlockHeader { // appears at beginning of (10M) load buffers
    char *slop;		  // ptr to end of last record parsed in the buffer
    char **parseLoc;      // used to update where (which byte) parsing stopped
                          // in the buffer area.
    char *endBuf;         // end of written area
    bool haltedFlag;      // set by gencode, indicator to rdr to stop reading.
};

#define MAX_DELIM_LEN 10 /* max delimiter length excluding trailing null char */

class TParserNodeDef {
protected:
    ldr::Parser            *m_parser;
    const et::ETOptions    *m_loadOpts;

    bool m_rawLoad;        // if true, just read/write raw blocks (compressed)
    bool m_2gParser;       // if true, new parser is used (temporary coding)

    int  m_tblId;          // table to use for field types info (external table)
    bool m_parseReplFormatOnHost; // when downloading into transient table this flag is set to true to prevent parsing repl format on the SPU
    int  m_numDistKeyFields;
    int  m_sortedDistKeyFields[MAX_DIST_KEY_FIELDS];
};

class TParserNodeRuntimeInfo {
protected:
    time_t m_startTime;    // keep track of parse time;
                           // instead of 'et::Log' live during the parsing, just store start-time
    TLoadBlockHeader *m_loadBlockHdr;
};

class TParserNode : public SnippetNode,
    private TParserNodeDef, private TParserNodeRuntimeInfo
{
public:
    const TableShape *Init(const TableShape *src, EPlanNodeLocus place);
    virtual ~TParserNode() { /* nothing to do here */ }

    void Serialize(CCommBuf *cbuf) const;
    void Deserialize(CCommBuf *cbuf, mem::Pool *pool);

    // Accessors
    bool              G2Parser()     const { return m_2gParser; }
    TLoadBlockHeader* LoadBlockHdr() const { return m_loadBlockHdr; }
    const et::ETOptions* LoadOpts()  const { return m_loadOpts; }
    ldr::Parser*      Parser()       const { return m_parser; }
    bool              RawLoad()      const { return m_rawLoad; }
    int               TblId()        const { return m_tblId; }
    bool              ParseReplFormatOnHost()  const { return m_parseReplFormatOnHost; }
    const int*        GetSortedDistKeyFields()  const { return m_sortedDistKeyFields; }
    int               GetNumSortedDistKeyFields()  const { return m_numDistKeyFields; }

    // Setters
    void Set2gParser( bool val )           { m_2gParser = val; }
    void SetLoadOpts( et::ETOptions* val ) { m_loadOpts = val; }
    void SetRawLoad( bool val )            { m_rawLoad = val; }
    void SetParseReplFormatOnHost( bool val ) { m_parseReplFormatOnHost = val; }
    void SetTblId( int val )               { m_tblId = val; }

    // Non-inlines
    void logParseResults();
    void logStats();
    void checkpoint();
    void rollback();
    void setSortedDistKeyFields(int numFields, const int *fieldIds);
};

#endif // !FOR_SPU

#endif // !XPPARSER_INCLUDED
