/*
 * Copyright 2014 IBM Corporation.
 */

#ifndef XPPROJECT_INCLUDED
#define XPPROJECT_INCLUDED

#include "nodecommon.h"

class TAggregateNode;
class TProjectNode;
struct Node; // postgres
class CCommBuf;

// For now, encode computed field info using simple masking macro
#define CompExprInfoType(x) (((intptr_t)(x)) & 0x007F)
#define CompExprInfoNullable(x) (((intptr_t)(x)) & 0x0080)
#define CompExprInfoSize(x) ((((intptr_t)(x)) & 0xFFFFFF00) >> 8)

#ifdef PfNone
# undef PfNone
#endif
#define PfNone         (0)    // no project flag
#define PfFpgaCreated  (1<<0) // project created for FPGA

class TProjectNodeDef {
protected:

    int     numFields;
    // src table field id or -1 (PROJECT_EXPR) for expression
    // or -15 (PROJECT_DEPENDENTMULTICOLEXPR) if second the n-th output col
    // or a multi-column expression. The first output col uses PROJECT_EXPR
    // and the MultiColExpr node is in the exprList.
    int   * fieldsList;
    Node ** exprList;           // used only if fieldsList[ix] = -1 (PROJECT_EXPR)
    int     m_reproject;        // for reprojections, how many project nodes to back up
    int     physIDCount;
    int   * physIDs;            // if fpga-generated, phys order of result-table fields
    int     m_projectFlags;
    bool    m_suppressMaxCheck; // This should only be done for table
    bool    m_useStruct;        // funcs for now. May be doable
                                // everywhere eventually.

    bool    m_forFuncJoin;

    void  * m_resultRow;        // Used for memory of stuct
};

class TProjectNodeRuntimeInfo {
protected:
    int     pcodeStart;         // number of first pcode instruction
    int     pcodeCount;         // number of pcode instructions
    int   * varStrSizeList;     // for varchar expr result sizes
    int     aggrBias;      // set by the AggregateNode (see comments in plan.cpp)
    bool    handledByFpga; // temp, real code prunes node out
    bool    fsmAdjusted;   // logical->physical converted by t->adjustFsmRefs
    char ** dbgColNames;   // projection column names for DBOS plan dumper
                           // only
    CCommBuf* savedProjectFields; // same as for a restrict, a kludgery
};

class  TProjectNode : public SnippetNode,
    private TProjectNodeDef, private TProjectNodeRuntimeInfo {
public:
    TProjectNode();
    virtual ~TProjectNode();

    TableShape const * Init( TableShape const * src, EPlanNodeLocus place );
// dlm-check    virtual ~TProjectNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize  ( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Accessors
    inline int    AggrBias()         const { return aggrBias; }
    inline char** DbgColNames()      const { return dbgColNames; }
    inline Node** ExprList()         const { return exprList; }
    inline int*   FieldsList()       const { return fieldsList; }
    inline bool   ForFuncJoin()      const { return m_forFuncJoin; }
    inline bool   HandledByFpga()    const { return handledByFpga; }
    inline int    NumFields()        const { return numFields; }
    inline int    PcodeCount()       const { return pcodeCount; }
    inline int    PcodeStart()       const { return pcodeStart; }
    inline int    PhysIDCount()      const { return physIDCount; }
    inline int *  PhysIDs()          const { return physIDs; }
    inline int    ProjectFlags()     const { return m_projectFlags; }
    inline int    Reproject()        const { return m_reproject; }
    inline bool   SuppressMaxCheck() const { return m_suppressMaxCheck; }
    inline bool   UseStruct()        const { return m_useStruct; }
    inline int *  VarStrSizeList()   const { return varStrSizeList; }
    inline void*  ResultRow()              { return m_resultRow; }
    inline CCommBuf* SavedProjectFields() const { return savedProjectFields; }

    // Mutators
    inline void   AddProjectFlag( int flag )    { m_projectFlags |= flag; }

    // Setters
    inline void SetAggrBias( int bias )         { aggrBias = bias; }
    inline void SetDbgColNames( char** val )    { dbgColNames = val; }
    inline void SetExprList( Node** val )       { exprList = val; }
    inline void SetFieldsList( int* val )       { fieldsList = val; }
    inline void SetForFuncJoin( bool val )      { m_forFuncJoin = val; }
    inline void SetHandledByFpga( bool h )      { handledByFpga = h; }
    inline void SetNumFields( int val )         { numFields = val; }
    inline void SetPcodeCount( int count )      { pcodeCount = count; }
    inline void SetPcodeStart( int start )      { pcodeStart = start; }
    inline void SetPhysIDCount( int count )     { physIDCount = count; }
    inline void SetPhysIDs( int* pids )         { physIDs = pids; }
    inline void SetProjectFlags( int val )      { m_projectFlags = val; }
    inline void SetReproject( int val )         { m_reproject = val; }
    inline void SetSuppressMaxCheck( bool val ) { m_suppressMaxCheck = val; }
    inline void SetUseStruct( bool val )        { m_useStruct = val; }
    inline void SetVarStrSizeList( int* val )   { varStrSizeList = val; }
    inline void SetResultRow(void *val)         { m_resultRow = val; }
    inline void SetSavedProjectFields(CCommBuf* val) { savedProjectFields = val; }


    // Non-inlines
#if !FOR_SPU
   TProjectNode *findPreviousProject();    // find base of reprojection
   int getOtherScanProject (TProjectNode **scanProjectP); // is there a previous project
#endif
};

void setAggrBiasInProjectProjectNode (
    TAggregateNode *pNode,
    int numSrcFields);

TProjectNode* FindProjector( TAggregateNode** ppAggrNode,
                             TProjectNode* projector);

#endif
