#ifndef XPREORGMERGE_INCLUDED
#define XPREORGMERGE_INCLUDED


/****************************************************************/
/* FOR CLIENT+DBOS+SPU */
/****************************************************************/

#include "groomcgen.h"

#include "error.h"  // for NZ_VERIFY

class CCommBuf;
class CPlan;


class TReorgMergeClntNode: public GroomNodeBase {
    const TableShape* Init( const TableShape* src, EPlanNodeLocus place ) { NZ_VERIFY( false ); } // dummy to make this class concrete
public:
    TReorgMergeClntNode();
    void Dump(int detail, int indent);

    virtual ~TReorgMergeClntNode();
    virtual void Serialize(CCommBuf* cbuf);
    virtual void Deserialize(CCommBuf* cbuf);

};



class TReorgMergeCommonNode: public TReorgMergeClntNode {
public:
//    TReorgMergeCommonNode(CCommBuf* buf, CPlan* plan) {};
    /** Node initialization before node excecution. */
    virtual const TableShape* Init(const TableShape *src, EPlanNodeLocus place) = 0;
    
};


/****************************************************************/
/* end FOR CLIENT+DBOS+SPU */
/****************************************************************/


/****************************************************************/
#if FOR_DBOS
/****************************************************************/

class TReorgPrepDbosNode;

class TReorgMergeDbosNode: public TReorgMergeCommonNode, public IGroomCGen {
public:

    //! Creates node from communication buffer
    TReorgMergeDbosNode(CCommBuf* buf, CPlan* plan);
    virtual ~TReorgMergeDbosNode();

    virtual const TableShape* Init(const TableShape *src, EPlanNodeLocus place);
    // Reuse destructor, serializer and deserializer from ClntNode

/* IGroomCGen interface ***********************************/
    virtual const TableShape* DeriveSnippetProjection(const TableShape* src);
    virtual bool IsNextLoopStartingNode();
    virtual void EmitNeedNode(CGenCode*  genc);
    virtual TStreamInfo* EmitStreamInfo(CGenCode* genc, int nodeIx, TStreamInfo *flowInfo);
    virtual void EmitLoopHdrStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);
    virtual void EmitLoopHdr(CGenCode* genc, TStreamInfo* sP);
    virtual void EmitLoopBodyStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);
    virtual void EmitLoopFtr(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);

    virtual void InitOnNewPlan();

    virtual void SpuSnippetSchedCost(
        unsigned& numChannels,  // incremental
        bool& sorter,  // default: false; optionally assigned true; never assign false
        bool& spuDist, // default: false; optionally assigned true; never assign false
        int& readPasses,     // incremental
        int& writePasses,    // incremental
        int& outBlocks,      // default: set in ScanNode; optionally assigned
        int& dest,           // default: outToFabric; optionally assigned
        int& scanBlocks,     // assigned in ScanNode, used by some other nodes
        int& newMem,         // default 0, reset per node; optionally assigned
        int& newInnerRead,   // default 0, reset per node; optionally assigned
        int& newInnerWrite,  // default 0, reset per node; optionally assigned
        bool& looper         // default false; optionally assinged true
        );
/* IGroomCGen interface end ***/

};


/****************************************************************/
#endif // FOR_DBOS
/****************************************************************/


/****************************************************************/
#if FOR_SPU
/****************************************************************/

#include "xpreorgprep.h"
#include "plan.h"
#include "spujob.h"

class TableShape;

class TReorgStatsSpuNode;

class TReorgMergeSpuNode: public TReorgMergeCommonNode {
    friend class TReorgStatsSpuNode;
public:

    //! Creates node from communication buffer
    TReorgMergeSpuNode(CCommBuf* buf, CPlan* plan);
    virtual ~TReorgMergeSpuNode();

    //! Initilalization
    virtual const TableShape* Init(const TableShape *src, EPlanNodeLocus place);


    /** Loads partitioning data prepared by ReorgPrepNode.
     *  Called by gened code before starting seconf loop over input data.
     *  Returns number of prepared records.
     */
    int LoadPrepData();

    //! Called by gened code to see if we need to LoadPartData
    bool PrepDataLoaded() { return m_prepDataLoaded; }

    /** Called by gened code to to get organized index for current record.
     *  Returns false when all organized indexes have been returned.
     */
    void NextOrgIx(int& rOrgIx) {
        NZ_VERIFY(m_scanRecIx < m_recCnt);
        rOrgIx = m_reorgRecIxs[m_scanRecIx++];
    }

    bool bufferFull() const {
        return m_scanRecIx == m_recCnt; 
    }

    void HandleExtentBoundary(bool lastCall);
    void EmptyExtent(const PageId& pageId);

private:
    GroomSpuState* groomState() const;

private:
    //! Number records to reorganize
    int m_recCnt;
    nzotuid_t m_owner;
    uint32* m_reorgRecIxs;   // Reorged record indexes (number)

    /** Transient table to store and pass data between DotPrep and DotReorg
     *  nodes across snippets. 
     *  Created and filled by ReorgPrepNode and read and destroyed by ReorgMergeNode.
     */
    CTable* m_tempTable;

    //! Indicates if data prepared by ReorgPrepNode has been loaded
    bool m_prepDataLoaded;

    //! Current scanned record number
    int m_scanRecIx;
    //! Current page id: to track extent boundaries
    PageId m_curPageId;
};

/****************************************************************/
#endif // FOR_SPU
/****************************************************************/

#endif // XPREORGMERGE_INCLUDED
