#ifndef XPREORGSTATS_INCLUDED
#define XPREORGSTATS_INCLUDED


/****************************************************************/
/* FOR CLIENT+DBOS+SPU */
/****************************************************************/

#include "groomcgen.h"
#include "xpgroomstats.h"

#if FOR_SPU || FOR_DBOS
#include "error.h"  // for NZ_VERIFY
#endif

class CCommBuf;
class CPlan;
class TableShape;

class TReorgStatsClntNode: public TGroomStatsClntNode {
    const TableShape* Init( const TableShape* src, EPlanNodeLocus place ) { 
#if FOR_SPU || FOR_DBOS
        NZ_VERIFY( false );
#endif
    } // dummy to make this class concrete
public:
    TReorgStatsClntNode(int64 groomId);

    void Dump(int detail, int indent);

    virtual ~TReorgStatsClntNode();
    virtual void Serialize(CCommBuf* cbuf);
    virtual void Deserialize(CCommBuf* cbuf);

    Oid InsertTblId() const {return m_insertTblId;}
    Oid InsertTblId(Oid insertTblid) { m_insertTblId = insertTblid; return m_insertTblId; }

    
protected:
    TReorgStatsClntNode(); // For derived classes constructors from CCommBuf
    Oid         m_insertTblId;
    
};


/****************************************************************/
/* end FOR CLIENT+DBOS+SPU */
/****************************************************************/

/****************************************************************/
#if FOR_DBOS || FOR_SPU
/****************************************************************/

class TReorgStatsCommonNode: public TReorgStatsClntNode {
public:
    /** Node initialization before node excecution. */
    virtual const TableShape* Init(const TableShape *src, EPlanNodeLocus place) = 0;

protected:
    /** Defines stats table shape.  Used by spu and dbos nodes. */
    void defineTableShape(TableShape* t);
};


/****************************************************************/
#endif // FOR_DBOS || FOR_SPU
/****************************************************************/


/****************************************************************/
#if FOR_DBOS
/****************************************************************/


class TReorgStatsDbosNode: public TReorgStatsCommonNode, public IGroomCGen {
public:

    //! Creates node from communication buffer
    TReorgStatsDbosNode(CCommBuf* buf, CPlan* plan);
    virtual ~TReorgStatsDbosNode();
    virtual const TableShape* Init(const TableShape *src, EPlanNodeLocus place);    

    // Reuse destructor, serializer, deserializer and Init from ClntNode

/* IGroomCGen interface ***********************************/

    virtual const TableShape* DeriveSnippetProjection(const TableShape* dst);
    virtual bool IsNextLoopStartingNode();
    virtual void EmitNeedNode(CGenCode*  genc);
    virtual TStreamInfo* EmitStreamInfo(CGenCode* genc, int nodeIx, TStreamInfo *flowInfo);
    virtual void EmitLoopHdrStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);
    virtual void EmitLoopHdr(CGenCode* genc, TStreamInfo* sP);
    virtual void EmitLoopBodyStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);
    virtual void EmitLoopFtr(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);

    virtual void InitOnNewPlan();

    virtual void SpuSnippetSchedCost(
        unsigned& numChannels,  // incremental
        bool& sorter,  // default: false; optionally assigned true; never assign false
        bool& spuDist, // default: false; optionally assigned true; never assign false
        int& readPasses,     // incremental
        int& writePasses,    // incremental
        int& outBlocks,      // default: set in ScanNode; optionally assigned
        int& dest,           // default: outToFabric; optionally assigned
        int& scanBlocks,     // assigned in ScanNode, used by some other nodes
        int& newMem,         // default 0, reset per node; optionally assigned
        int& newInnerRead,   // default 0, reset per node; optionally assigned
        int& newInnerWrite,  // default 0, reset per node; optionally assigned
        bool& looper         // default false; optionally assinged true
        );
/* IGroomCGen interface end ***/
};


/****************************************************************/
#endif // FOR_DBOS
/****************************************************************/


/****************************************************************/
#if FOR_SPU
/****************************************************************/

#include "plan.h"
#include "spujob.h"

class TableShape;

class TReorgStatsSpuNode: public TReorgStatsCommonNode {
public:

    //! Default constructor on SPU is usually followed by deserialization from CCommBuf
    TReorgStatsSpuNode(CCommBuf* buf, CPlan* plan);
    virtual ~TReorgStatsSpuNode();

    virtual const TableShape* Init(const TableShape *src, EPlanNodeLocus place);

    /*  Precommits changes to the groomed table and
        saves statistics in _vt_groom_step virtual table
        Called by gen'd code at the end of the loop on last call
    */
    void Precommit();

    GroomSpuState* groomState() const;
    const dpm::TCStats& tcStats() const { return m_tcstats; }



private:
    void insert_vt_groom_step_record();
    dpm::TCStats m_tcstats;

};

/****************************************************************/
#endif // FOR_SPU
/****************************************************************/


#endif // XPREORGSTATS_INCLUDED 
