#ifndef XPRESTRICT_INCLUDED
#define XPRESTRICT_INCLUDED

#include "nodecommon.h"

struct List;
struct ExprContext;
class CCommBuf;

enum EHandled {
    eHandledNo = 0,  // nothing fpga-able
    eHandledYes,     // completely handled by fpga
    eHandledPart     // partly handled by fpga
};

struct TRestrictNodeDef {
protected:
    List * qual;         // points to qualifier list
    nzbool bForHaving;   // iff for Having clause
    enum EHandled handledByFpgaExt;  // indicates to what extent handled by fpga
};

struct TRestrictNodeRuntimeInfo {
protected:
    CCommBuf    * savedQual;  // kludgery to handle some Param cases
    ExprContext * econtext;   // PG expr context
    int    pcodeStart;   // number of first pcode instruction
    int    pcodeCount;   // number of pcode instructions
};

// restrict (the where clause)
class TRestrictNode : public SnippetNode,
    private TRestrictNodeDef, private TRestrictNodeRuntimeInfo {
public:
    TableShape const * Init( TableShape const * src, EPlanNodeLocus place );
    virtual ~TRestrictNode() { /* nothing to do here */ }
    bool IsValidForSrc() { return true; }
    bool Serialize( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Accessors
    inline List**       AddrOfQual()              { return &qual; }
    inline ExprContext* EContext()          const { return econtext; }
    inline nzbool       ForHaving()         const { return bForHaving; }
    inline EHandled     HandledByFpgaExt()  const { return handledByFpgaExt; }
    inline int          PcodeCount()        const { return pcodeCount; }
    inline int          PcodeStart()        const { return pcodeStart; }
    inline List *       Qual()              const { return qual; }
    inline CCommBuf*    SavedQual()         const { return savedQual; }

    // Setters
    inline void SetEContext( ExprContext* ctx )    { econtext = ctx; }
    inline void SetForHaving( nzbool val )         { bForHaving = val; }
    inline void SetHandledByFpgaExt( EHandled hh ) { handledByFpgaExt = hh; }
    inline void SetPcodeCount( int count )         { pcodeCount = count; }
    inline void SetPcodeStart( int start )         { pcodeStart = start; }
    inline void SetQual( List* qq )                { qual = qq; }
    inline void SetSavedQual( CCommBuf* qual )     { savedQual = qual; }
};

#endif // !XPRESTRICT_INCLUDED
