/** \file xpreturn.cpp
 * \brief Defines TReturnNode class and ERetStrMode enumeration.
 */

#ifndef XPRETURN_INCLUDED
#define XPRETURN_INCLUDED

#include "nodecommon.h"
#include "fcommchannel.h"
#include "eretstrmode.h"

struct TMemblockInfo;

struct TReturnNodeDef {
protected:
    int m_forBackup;          // true if no host-side plan

    ERetStrMode m_retStrMode; // non-zero for formatted strings method
};

struct TReturnNodeRuntimeInfo {
protected:
    int      m_streamSize;     // if >0, size before returning
    bool     m_responseSent;   // true after first repsonse of multi-response stream

    unsigned m_numPackets;     // number of send-side packet buffers
    size_t   m_sizeReassembly; // maximum size of reassembly buffers
    unsigned m_numReassembly;  // number of reassembly buffers
    TChannel m_channel;        // return channel info
    bool     m_channelOpened;  // true if channel is open
};

class TReturnNode : public SnippetNode,
    private TReturnNodeDef, private TReturnNodeRuntimeInfo {
public:
    TReturnNode();
    virtual ~TReturnNode();

    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom); // initializer called on plan-init
// dlm-check    virtual ~TReturnNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Accessors
    inline TChannel &  Channel()              { return m_channel; }
    inline bool        ChannelOpened()  const { return m_channelOpened; }
    inline int         ForBackup()      const { return m_forBackup; }
    inline unsigned    NumPackets()     const { return m_numPackets; }
    inline unsigned    NumReassembly()  const { return m_numReassembly; }
    inline ERetStrMode RetStrMode()     const { return m_retStrMode; }
    inline size_t      SizeReassembly() const { return m_sizeReassembly; }
    inline int         StreamSize()     const { return m_streamSize; }

    // Setters
    inline void SetChannel( TChannel ch )         { m_channel = ch; }
    inline void SetForBackup( int bkup )          { m_forBackup = bkup; }
    inline void SetNumPackets( int num )          { m_numPackets = num; }
    inline void SetNumReassembly( int num )       { m_numReassembly = num; }
    inline void SetRetStrMode( ERetStrMode mode ) { m_retStrMode = mode; }
    inline void SetSizeReassembly( int size )     { m_sizeReassembly = size; }
    inline void SetStreamSize( int size )         { m_streamSize = size; }

    // Non-inlines
    void returnNewBlock( // wrapper to plan's return logic
        bool lastCall);

    void batchReturnStore( // Stream a store to the client, xoff the spus
            int cliInteractId);

    void compressRecBlock( // Apply ZLib compression to table's block
        TMemblockInfo *block);

    void computeCommMemory ( // compute comm memory requirements for node
        unsigned *numPackets,
        size_t *sizeReassembly,
        unsigned *numReassembly );

    void verifyChannelOpen();
};

#endif // !XPRETURN_INCLUDED
