#ifndef XPSAVE_INCLUDED
#define XPSAVE_INCLUDED

#include "nodecommon.h"
#include "storagerefs.h"

struct List;

class TSaveTempNodeDef {
protected:
    int     m_tblId;                // temp table id
    bool    m_append;               // whether to append to existing tblId
    bool    m_saveInFpgaStyle;      // if true save in fpga style
    bool    m_bUsedInMultipleJoins; // true if saved table is used in multipe joins
    bool    m_isPrebroadcastTbl;
    bool    m_isBlockList;          // true if table used for mview pivoting
    List  * m_nullTestQual;         // PG expression for not exist join (NOT IN and relOp ALL)
};

class TSaveTempNodeRuntimeInfo {
protected:
    int m_nPages;                 // for tablesize node
    int m_pcodeStartNullTestQual; // number of first pcode instruction null test qual
    int m_pcodeCountNullTestQual; // number of pcode instructions for null test qual
};

// save as temp table
class TSaveTempNode : public SnippetNode,
    private TSaveTempNodeDef, private TSaveTempNodeRuntimeInfo {
public:
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);
    virtual ~TSaveTempNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Address-of accessors
    // Used for things like virtual-table materialization.
    inline int* AddrOfTblId()               { return &m_tblId; }

    // Accessors:
    inline bool  Append()              const { return m_append; }
    inline bool  IsBlockList()         const { return m_isBlockList; }
    inline bool  IsPrebroadcastTbl()   const { return m_isPrebroadcastTbl; }
    inline List* NullTestQual()        const { return m_nullTestQual; }
    inline int   Pages()               const { return m_nPages; }
    inline int   PcodeStartNullTestQual() const { return m_pcodeStartNullTestQual; }
    inline int   PcodeCountNullTestQual() const { return m_pcodeCountNullTestQual; }
    inline bool  SaveInFpgaStyle()     const { return m_saveInFpgaStyle; }
    inline int   TblId()               const { return m_tblId; }
    inline bool  UsedInMultipleJoins() const { return m_bUsedInMultipleJoins; }

    // Setters
    inline void  SetAppend(bool val)                { m_append = val; }
    inline void  SetIsBlockList(bool val)           { m_isBlockList = val; }
    inline void  SetIsPrebroadcastTbl(bool val)     { m_isPrebroadcastTbl = val; }
    inline void  SetNullTestQual(List* val)         { m_nullTestQual = val; }
    inline void  SetPcodeStartNullTestQual(int start) { m_pcodeStartNullTestQual = start; }
    inline void  SetPcodeCountNullTestQual(int count) { m_pcodeCountNullTestQual = count; }
    inline void  SetSaveInFpgaStyle(bool val)       { m_saveInFpgaStyle = val; }
    inline void  SetTblId(int tbl)                  { m_tblId = tbl; }
    inline void  SetUsedInMultipleJoins(bool val)   { m_bUsedInMultipleJoins = val; }

    // Non-inlines
    void saveTempNewBlock(int unUsed); //called on new block on saved temp
    void addBlockListMember(PageId pageId); // adds page id to blocklist
};

#endif // !XPSAVE_INCLUDED

