#ifndef INCLUDED_SORTEDAGGRDBOS
#define INCLUDED_SORTEDAGGRDBOS

#include "xpsortedaggr.h"
#include <map>
#include <math.h>

class TSortedAggregateRunNode;
struct field_t;

class TSortedAggregateGenNode: public INodeCGen
{

public:
    TSortedAggregateGenNode( TSortedAggregateRunNode *in);
    virtual ~TSortedAggregateGenNode();

    bool IsNextLoopStartingNode() ;

    //Called from CGenCode::genStreamNodes
    // This method creates stream structure and emits it if needed
    virtual TStreamInfo* EmitStreamInfo(CGenCode* genc, int nodeIx) ;

    // Called from CGenCode::doEmitLoopHdrStuff
    // Base implementation emits nothing
    virtual void EmitLoopHdrStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) ;

    // Called from CGenCode::doEmitLoopHdr
    virtual void EmitLoopHdr(CGenCode* genc, TStreamInfo* sP) ;

    // Called from CGenCode::emitLoopBodyStuff
    // Base implementation emits nothing
    virtual void EmitLoopBodyStuff(
        CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) ;

    virtual void EmitLoopFtr(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) ;

    //virtual void InitOnNewPlan() = 0;    ?????????? is this required
                       //! called by void CPlan::DbosNewPlan ()
    virtual void SpuSnippetSchedCost(
        unsigned& numChannels,  // incremental
        bool& sorter,  // default: false; optionally assigned true; never assign false
        bool& spuDist, // default: false; optionally assigned true; never assign false
        int& readPasses,     // incremental
        int& writePasses,    // incremental
        int& outBlocks,      // default: set in ScanNode; optionally assigned
        int& dest,           // default: outToFabric; optionally assigned
        int& scanBlocks,     // assigned in ScanNode, used by some other nodes
  //      int& sortExtra,      // optionally assigned (in Aggregate and Sort Nodes)
        int& newMem,         // default 0, reset per node; optionally assigned
        int& newInnerRead,   // default 0, reset per node; optionally assigned
        int& newInnerWrite,  // default 0, reset per node; optionally assigned
        bool& looper         // default false; optionally assinged true
        ) { }

    void emitCompareFunc(CGenCode*, TStreamInfo*);
private:
   TSortedAggregateRunNode *m_runnode;
   TStreamInfo *m_tmpFlowP;
   TStreamInfo **m_argFlowP;
   TStreamInfo *m_orderP;
   TStreamInfo *m_orderFlowP;
   int m_hdrSize;
   int m_orderHdrSize;
   int m_numMaterialArgs;
   std::map< int, int> argRowMap;
//private methods
void doGroupInitProcessing( CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);
int  doGroupInitResult( CGenCode *genc, TStreamInfo *sP);
void emitGroupCompareBlock( CGenCode* genc, TStreamInfo* sP);
void emitProcessGroupRows( CGenCode* genc, TStreamInfo *sP, TStreamInfo *flowP);
void emitEndOfGroupProcessing( CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP);
void doGroupPostProcessing( CGenCode *genc, TStreamInfo *sP, TStreamInfo *flowP);
void setUpGenStreamInfo( CGenCode *genc, TStreamInfo *sP, TStreamInfo *flowP);
int  getArgRowMapValue( int fld );
void copyOrderData( CGenCode *genc, TStreamInfo *srcP, TStreamInfo *destP, int numCols, int *srcPcols, char *adstVarPtr);
void getNumericStrForPerCont( char *colStr, int colStrSize, char *argbuf, const field_t *fld);

};

class TSortedAggregateRunNode: public TSortedAggregateClntNode, INodeRunT
{

public:
    TSortedAggregateRunNode(
        EPlanNodeLocus locus, CPlan *plan,  CCommBuf *buf);
    ~TSortedAggregateRunNode();
    virtual const TableShape* DeriveSnippetProjection(const TableShape* src);
    virtual const TableShape* Init(const TableShape* src, EPlanNodeLocus place);

    int getNumGroups() const
        { return m_numGroups; }

    void * m_resultRow;
    void * m_tmpSortRow;
    record_t ** m_argRow;
    bool bGotNewGroup;
    bool bGotData;
    int64 *ceil;
    int64 *floor;
    int64 *currowcnt;
private:
    TableShape *m_result;
    TableShape *m_order;

    int needArgMaterialized( );
    int getNumDenseRank( );
    friend class TSortedAggregateGenNode;
};

#endif
