#ifndef XPSTREAMAPPENDCGEN_INCLUDED
#define XPSTREAMAPPENDCGEN_INCLUDED

#include "xpstreamappend.h"
#include "recordstore.h"

//
//  Appends records from stream and temp table.
//  The shape of the stream and temp table is identical
//
class TStreamAppendRunNode;

class TStreamAppendGenNode:public INodeCGen {
public:
    TStreamAppendGenNode(TStreamAppendRunNode *in);

    virtual ~TStreamAppendGenNode() ;

    virtual bool IsNextLoopStartingNode()   ;
    // This method creates stream structure that will be the resulting stream of this node and emits it if needed
    virtual TStreamInfo* EmitStreamInfo(CGenCode* genc, int nodeIx) ;

    // Called from CGenCode::doEmitLoopHdrStuff
    // Base implementation emits nothing
    // 1. emit declarations and assignments
    virtual void EmitLoopHdrStuff(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP)  ;

    // Called from CGenCode::doEmitLoopHdr
    // Base implementation emits nothing
    //2. called after EmitLoopHdrStuff
    virtual void EmitLoopHdr(CGenCode* genc, TStreamInfo* sP) ;

    // Called from CGenCode::emitLoopBodyStuff
    // Base implementation emits nothing
    // 3. called after EmitLoopHdr
    virtual void EmitLoopBodyStuff(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP)  ;

    virtual void EmitLoopFtr(CGenCode* genc, TStreamInfo* sP, TStreamInfo* flowP) ;

private:
    TStreamInfo *m_tempP; //reference to temp table
    TStreamAppendRunNode *m_runnode;
};


class TStreamAppendRunNode: public TStreamAppendClntNode, INodeRunT
{
public:
    TStreamAppendRunNode(EPlanNodeLocus, CPlan*, CCommBuf*);
    const TableShape *Init(const TableShape *src, EPlanNodeLocus calledFrom);
    virtual const TableShape* DeriveSnippetProjection(const TableShape* src);
    virtual ~TStreamAppendRunNode();

    // Accessors:
    inline record_t*      Buf()                        { return m_pBuf; }
    inline CTable*        TempTbl()                    { return m_pTempTbl; }
    inline TMemblockInfo* TempTrackingBlock()          { return m_tempTrackingBlock; }
    inline uint32         TempTrackingBlockNum() const { return m_tempTrackingBlockNum; }

    inline void SetTempTrackingBlock( TMemblockInfo* val ) { m_tempTrackingBlock = val; }
    inline void SetTempTrackingBlockNum( uint32 val )      { m_tempTrackingBlockNum = val; }
    
    friend class TStreamAppendGenNode;

private:
    CTable *m_pTempTbl;   // temp table itself 
    record_t *m_pBuf;     //for materializing    
    TMemblockInfo *m_tempTrackingBlock;  // tracking block (Host)
    uint32 m_tempTrackingBlockNum;    // blockNum of tracking block (SPU)
};

#endif // !XPSTREAMAPPEND_INCLUDED
