#ifndef XPTABLESIZE_INCLUDED
#define XPTABLESIZE_INCLUDED

#include "nodecommon.h"

struct List;

class TTableSizeNodeDef {
protected:
    int m_tblId;    //id of the table whose size is needed
    bool m_bGetSize; //if true retreive size, if false set size
    bool m_forMergeJoin;    //if true this is for merge-join
    int m_mergeCol;         //merge column if we are processing for a merge-join
};

class TTableSizeNodeRuntimeInfo {
protected:
    CTable *m_tempTbl;      //pointer to the temp table
    uint8 *m_pPrevRec;      //for MJ
    uint8 *m_pPrevRecBuf;
    int m_nMaxBlocks;
    bool m_bCheckDone;    //for MJ, true if we already performed comparision for this block
    int m_pcodeStart;       // first pcode instr for null test qual
    int m_pcodeCount;       // num pcode instr for null test qual
};

/* returns the size of the specfied table */
/* for MergeJoins, we return the block span for the largest duplicate key set*/
class TTableSizeNode : public SnippetNode,
    private TTableSizeNodeDef, private TTableSizeNodeRuntimeInfo {
public:
    TTableSizeNode();
    virtual ~TTableSizeNode();
    
    TableShape const * Init(TableShape const * src, EPlanNodeLocus calledFrom);
// dlm-check    virtual ~TTableSizeNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Accessors
    inline bool    CheckDone()    const { return m_bCheckDone; }
    inline bool    ForMergeJoin() const { return m_forMergeJoin; }
    inline bool    GetSize()      const { return m_bGetSize; }
    inline int     MaxBlocks()    const { return m_nMaxBlocks; }
    inline int     MergeCol()     const { return m_mergeCol; }
    inline int     PcodeCount()   const { return m_pcodeCount; }
    inline int     PcodeStart()   const { return m_pcodeStart; }
    inline uint8*  PrevRec()      const { return m_pPrevRec; }
    inline uint8*  PrevRecBuf()   const { return m_pPrevRecBuf; }
    inline int     TblId()        const { return m_tblId; }
    inline CTable* TempTbl()      const { return m_tempTbl; }

    // Setters
    inline void SetCheckDone( bool val )     { m_bCheckDone = val; }
    inline void SetForMergeJoin( bool val )  { m_forMergeJoin = val; }
    inline void SetGetSize( bool val )       { m_bGetSize = val; }
    inline void SetMaxBlocks( int val )      { m_nMaxBlocks = val; }
    inline void SetMergeCol( int val )       { m_mergeCol = val; }
    inline void SetPcodeCount( int count )   { m_pcodeCount = count; }
    inline void SetPcodeStart( int start )   { m_pcodeStart = start; }
    inline void SetPrevRec( uint8* val )     { m_pPrevRec = val; }
    inline void SetTblId( int val )          { m_tblId = val; }


};

#endif // !XPTABLESIZE_INCLUDED
