#ifndef XPTIMER_INCLUDED
#define XPTIMER_INCLUDED

#include "i386_tsc.h"

// This uses the Intel processor's internal CPU clock.  It counts hardware cycles.
// Unfortunately, that means it isn't calibrated in any universal way.  Getting
// a faster processor won't make these times go faster.
//
// On the Mercury platform, it was 5 million ticks/sec.  On Intel, it is much faster.

inline uint64 XpGetTimebase() {
    uint64 val;
    rdtscll(val);
    return(val);
}

class CTimer64 {
 public:

    // Constructor.
    CTimer64() : m_time(0), m_start(XpGetTimebase()) {}

    uint64 m_time;  // This is the *SUM* of the times of all start/stop pairs.
                    // Clients can read this, or reset it to zero, as they see fit.

    void Start()
    {
        m_start = XpGetTimebase();
    }

    // Close a timing interval. (It doesn't really stop; if you want to run
    // multiple non-cumulative timing runs on the same timer you must call "Start"
    // to reset the clock for the start of the next interval.)  Return the length
    // of the just-ended interval, and also add it to the public member "m_time",
    // which therefore keeps the cumulative sum of all intervals.
    uint64 Finish()
    {
        uint64 interval = XpGetTimebase() - m_start;
        m_time += interval;
        return interval;
    }

 private:

    uint64 m_start;   // Measures an interval.
};

#endif // XPTIMER_INCLUDED
