#ifndef XPUNIQUE_INCLUDED
#define XPUNIQUE_INCLUDED

#include "nodecommon.h"

class CComparison;

/* Support for execution plan joining */

class TUniqueNodeDef {
protected:
    CComparison *uniqueCompare;
};

class TUniqueNodeRuntimeInfo {
protected:
    record_t * m_prevRec;    // previous record for compares
    record_t * m_curBuf[2];  // stable buffers for remembering cur and prev recs
    int        m_bufSelect;  // to choose which m_curBuf is current or prev record
};

class TUniqueNode : public SnippetNode,
    private TUniqueNodeDef, private TUniqueNodeRuntimeInfo {

public:
    TableShape const * Init(TableShape const * src, EPlanNodeLocus place);
    virtual ~TUniqueNode() { /* nothing to do here */ }

    bool IsValidForSrc() { return true; }
    bool Serialize  ( CCommBuf * cbuf, ESerializeMode mode );
    void Deserialize( CCommBuf const * cbuf, ESerializeMode mode );

    // Accessors
    inline record_t**   CurBuf()              { return m_curBuf; }
    inline record_t*    PrevRec()             { return m_prevRec; }
    inline int BufSelect()              const { return m_bufSelect; }
    inline CComparison* UniqueCompare() const { return uniqueCompare; }

    // Mutators
    inline void InvertBufSelect()           { m_bufSelect ^= 0x1; }

    // Setters
    inline void SetPrevRec( record_t* val ) { m_prevRec = val; }
};

#endif // !XPUNIQUE_INCLUDED
