#ifndef ZMACCUMS_H
#define ZMACCUMS_H

//========================================================================
///\file: zmaccums.h Accumulator types to build accumulator sequences.
//
// Copyright (c) 2010 Netezza Corporation.  All rights reserved.
//========================================================================

#include "zmops.h"
#include "zmstrn.h"
#include "nztypes.h"
#include <vector>
#include "zmval8.h"

namespace zm {

struct Accumulator {
    /// Accumulator base class irrespective of datatype being collected.
    EAccumOp    m_op : 8;               ///< Specifies type and nullability
    uint16      m_offset;               ///< Fix byte offset or var skip count
    ColumnId    m_columnId : 16;        ///< ColumnId for zm::Rec constructor
} __attribute__(( packed ));

struct Nullable {
    /// Accumulator suffix if column is nullable.
    ColumnPos   m_nullVecPos : 16;      ///< Bit position within null vector
    bool        m_noNulls;              ///< Updatable state
} __attribute__(( packed ));


// Given a pointer to a nullable accumulator object and its size return
// a pointer to the embedded, trailing Nullable object.
#define NULLABLE(ACCUM,SIZE) \
    ((Nullable*)( ((char*)(ACCUM)) + (SIZE) - sizeof(Nullable) ))


// Hack! the min/max order must match Val8Pair (see the harvesting code)
#define ACCUM_STRUCTS(DTYPE) \
    struct Accumulator_##DTYPE : public Accumulator { \
        DTYPE   m_min;                  /**< Currently accumulated minimun */ \
        DTYPE   m_max;                  /**< Currently accumulated maximun */ \
    } __attribute__(( packed )); \
\
    struct Accumulator_##DTYPE##_nullable : public Accumulator_##DTYPE { \
        Nullable m_nullable;            /**< Currently accumulated noNulls */ \
    } __attribute__(( packed )); \


ACCUM_STRUCTS(int8);
ACCUM_STRUCTS(int16);
ACCUM_STRUCTS(int32);
ACCUM_STRUCTS(int64);
ACCUM_STRUCTS(float);
ACCUM_STRUCTS(double);
ACCUM_STRUCTS(str1);
ACCUM_STRUCTS(str2);
ACCUM_STRUCTS(str3);
ACCUM_STRUCTS(str4);
ACCUM_STRUCTS(str5);
ACCUM_STRUCTS(str6);
ACCUM_STRUCTS(str7);
ACCUM_STRUCTS(str8);


extern const uint8 accumulatorSize[];   /// Map Accum types to their sizes
extern const uint8 compare[];           /// Map Accum types to EZMCmp


struct oneColumnZmRecord_t{
  oneColumnZmRecord_t(uint32 opType, ColumnId col):
    m_opType(opType),
    m_colId(col),
    m_noNulls(true)
    {}

  uint32 m_opType;
  ColumnId m_colId;
  Val8Pair m_record;
  bool m_noNulls;
};

void initStatsAccumulators(Accumulator*);

void accumulate(Accumulator*, char const*, char const*, char const* =0, int64 const* =0, int64 const* =0, int64 const* =0);

void parseAccumulatorSequence(Accumulator* accumStart, std::vector<oneColumnZmRecord_t>& zmRecords);

}; // namespace zm
#endif // ZMACCUMS_H
