#ifndef ZMOPS_H
#define ZMOPS_H

//========================================================================
///\file: zmops.h
//
// Copyright (c) 2010 Netezza Corporation.  All rights reserved.
//========================================================================

#include "nzrecord.h"

namespace zm {

// WARNING: zm::getEAccumOp depends on various ordering properties:
// - the _nullable variant of an opcode being exactly 1 greater than the
//   corresponding non-nullable opcode
// - the [str1A, str2A, ..., str7A] sequence being ordered and dense
// - the [str1E, str2E, ..., str7E] sequence being ordered and dense
//
// Because there is no padding there is only a str8 (no str8A and str8E).

#define ACCUMULATOR_OPS \
_ROW_( ACCUM_rowid,             0, 8, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_createtxid,        0, 8, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_deletetxid,        0, 8, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_interval,          0,12, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_interval_nullable, 1,12, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_timetz,            0,12, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_timetz_nullable,   1,12, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_cnum64,            0, 8, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_cnum64_nullable,   1, 8, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_int64,             0, 8, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_int64_nullable,    1, 8, zmCmpI8, Accumulator_int64 ) _SEP_ \
_ROW_( ACCUM_double,            0, 8, zmCmpD8, Accumulator_double) _SEP_ \
_ROW_( ACCUM_double_nullable,   1, 8, zmCmpD8, Accumulator_double) _SEP_ \
_ROW_( ACCUM_int32,             0, 4, zmCmpI8, Accumulator_int32 ) _SEP_ \
_ROW_( ACCUM_int32_nullable,    1, 4, zmCmpI8, Accumulator_int32 ) _SEP_ \
_ROW_( ACCUM_float,             0, 4, zmCmpD8, Accumulator_float ) _SEP_ \
_ROW_( ACCUM_float_nullable,    1, 4, zmCmpD8, Accumulator_float ) _SEP_ \
_ROW_( ACCUM_int16,             0, 2, zmCmpI8, Accumulator_int16 ) _SEP_ \
_ROW_( ACCUM_int16_nullable,    1, 2, zmCmpI8, Accumulator_int16 ) _SEP_ \
_ROW_( ACCUM_int8,              0, 1, zmCmpI8, Accumulator_int8  ) _SEP_ \
_ROW_( ACCUM_int8_nullable,     1, 1, zmCmpI8, Accumulator_int8  ) _SEP_ \
\
_ROW_( ACCUM_str1A,             0, 1, zmCmpS8, Accumulator_str1  ) _SEP_ \
_ROW_( ACCUM_str1A_nullable,    1, 1, zmCmpS8, Accumulator_str1  ) _SEP_ \
_ROW_( ACCUM_str2A,             0, 2, zmCmpS8, Accumulator_str2  ) _SEP_ \
_ROW_( ACCUM_str2A_nullable,    1, 2, zmCmpS8, Accumulator_str2  ) _SEP_ \
_ROW_( ACCUM_str3A,             0, 3, zmCmpS8, Accumulator_str3  ) _SEP_ \
_ROW_( ACCUM_str3A_nullable,    1, 3, zmCmpS8, Accumulator_str3  ) _SEP_ \
_ROW_( ACCUM_str4A,             0, 4, zmCmpS8, Accumulator_str4  ) _SEP_ \
_ROW_( ACCUM_str4A_nullable,    1, 4, zmCmpS8, Accumulator_str4  ) _SEP_ \
_ROW_( ACCUM_str5A,             0, 5, zmCmpS8, Accumulator_str5  ) _SEP_ \
_ROW_( ACCUM_str5A_nullable,    1, 5, zmCmpS8, Accumulator_str5  ) _SEP_ \
_ROW_( ACCUM_str6A,             0, 6, zmCmpS8, Accumulator_str6  ) _SEP_ \
_ROW_( ACCUM_str6A_nullable,    1, 6, zmCmpS8, Accumulator_str6  ) _SEP_ \
_ROW_( ACCUM_str7A,             0, 7, zmCmpS8, Accumulator_str7  ) _SEP_ \
_ROW_( ACCUM_str7A_nullable,    1, 7, zmCmpS8, Accumulator_str7  ) _SEP_ \
_ROW_( ACCUM_str8,              0, 8, zmCmpS8, Accumulator_str8  ) _SEP_ \
_ROW_( ACCUM_str8_nullable,     1, 8, zmCmpS8, Accumulator_str8  ) _SEP_ \
\
_ROW_( ACCUM_varA,              0, 0, zmCmpS8, Accumulator_str8  ) _SEP_ \
_ROW_( ACCUM_varA_nullable,     1, 0, zmCmpS8, Accumulator_str8  ) _SEP_ \
\
/* Logic zmexec assumes that EBCDIC has high ordinal values than ASCII */ \
\
_ROW_( ACCUM_str1E,             0, 1, zmCmpS8, Accumulator_str1  ) _SEP_ \
_ROW_( ACCUM_str1E_nullable,    1, 1, zmCmpS8, Accumulator_str1  ) _SEP_ \
_ROW_( ACCUM_str2E,             0, 2, zmCmpS8, Accumulator_str2  ) _SEP_ \
_ROW_( ACCUM_str2E_nullable,    1, 2, zmCmpS8, Accumulator_str2  ) _SEP_ \
_ROW_( ACCUM_str3E,             0, 3, zmCmpS8, Accumulator_str3  ) _SEP_ \
_ROW_( ACCUM_str3E_nullable,    1, 3, zmCmpS8, Accumulator_str3  ) _SEP_ \
_ROW_( ACCUM_str4E,             0, 4, zmCmpS8, Accumulator_str4  ) _SEP_ \
_ROW_( ACCUM_str4E_nullable,    1, 4, zmCmpS8, Accumulator_str4  ) _SEP_ \
_ROW_( ACCUM_str5E,             0, 5, zmCmpS8, Accumulator_str5  ) _SEP_ \
_ROW_( ACCUM_str5E_nullable,    1, 5, zmCmpS8, Accumulator_str5  ) _SEP_ \
_ROW_( ACCUM_str6E,             0, 6, zmCmpS8, Accumulator_str6  ) _SEP_ \
_ROW_( ACCUM_str6E_nullable,    1, 6, zmCmpS8, Accumulator_str6  ) _SEP_ \
_ROW_( ACCUM_str7E,             0, 7, zmCmpS8, Accumulator_str7  ) _SEP_ \
_ROW_( ACCUM_str7E_nullable,    1, 7, zmCmpS8, Accumulator_str7  ) _SEP_ \
\
_ROW_( ACCUM_varE,              0, 0, zmCmpS8, Accumulator_str8  ) _SEP_ \
_ROW_( ACCUM_varE_nullable,     1, 0, zmCmpS8, Accumulator_str8  ) _SEP_ \
\
_ROW_( ACCUM_stop,              0,99, zmCmpI8, Accumulator_int8  ) _SEP_ \
_ROW_( ACCUM_invalid,           0, 0, zmCmpI8, Accumulator_int8  ) _SEP_ \
_ROW_( ACCUM_cnum128,           0,16, zmCmpI8, Accumulator_int8  ) _SEP_ \
_ROW_( ACCUM_cnum128_nullable,  1,16, zmCmpI8, Accumulator_int8  )

#define _ROW_( ENUMERATOR, NULLABLE, FIELD, CMP, TYPE ) ENUMERATOR
#define _SEP_ ,
    /// Define the EAccumOp enumeration
    enum EAccumOp { ACCUMULATOR_OPS
                  , ACCUM_ebcdic_min    = ACCUM_str1E
                  };
#undef _SEP_
#undef _ROW_

// Convenient maps indexed by EAccumOp
extern uint8 const accumOp[];
extern uint8 const fixedBytes[];
extern bool  const nullable[];

/////////////////////////////////////////////////////////////////////////

inline
EAccumOp getAccumOp
    ( field_t const* f
    )
{
    return (EAccumOp)(accumOp[fieldTypeWithSize(f)] + (f->nullAllowed != 0));
}


}; // namespace zm
#endif // ZMOPS_H
