#ifndef ZMVAL8_H
#define ZMVAL8_H

//========================================================================
///\file: zmval8.h
//
// (C) Copyright IBM Corp. 2010, 2011  All Rights Reserved.
//========================================================================

#include "zmstrn.h"

namespace zm {

// A uniform 8 byte container to represent integer, floating and string bounds.
union Val8 {
    uint64 u8;
    int64  i8;
    double d8;
    str8   s8;

    Val8() : u8(0) {}
    Val8(uint64 u8) : u8(u8) {}

} __attribute__(( packed, aligned(4) ));


// Val8s are often used within a min/max pair
struct Val8Pair {
    Val8 min;
    Val8 max;
    Val8Pair (int64 min = 0, int64 max = 0) : min(min), max(max) {};

    bool operator==(const Val8Pair& other){
        return (min.u8 == other.min.u8) && (max.u8 == other.max.u8);
    }

} __attribute__((packed));

// a 'trait' class
template<class ColumnType> class Val8Rep {};

// specialization mapping column type to zone map representation employed
template<> class Val8Rep<  int8> { public: typedef  int64 type; };
template<> class Val8Rep< int16> { public: typedef  int64 type; };
template<> class Val8Rep< int32> { public: typedef  int64 type; };
template<> class Val8Rep< int64> { public: typedef  int64 type; };
template<> class Val8Rep< float> { public: typedef double type; };
template<> class Val8Rep<double> { public: typedef double type; };
template<> class Val8Rep<  str1> { public: typedef   str8 type; };
template<> class Val8Rep<  str2> { public: typedef   str8 type; };
template<> class Val8Rep<  str3> { public: typedef   str8 type; };
template<> class Val8Rep<  str4> { public: typedef   str8 type; };
template<> class Val8Rep<  str5> { public: typedef   str8 type; };
template<> class Val8Rep<  str6> { public: typedef   str8 type; };
template<> class Val8Rep<  str7> { public: typedef   str8 type; };
template<> class Val8Rep<  str8> { public: typedef   str8 type; };

typedef Val8Pair zmrec_t;

#define CREATE_STACK_ZMRECORD(ncols) (::new( alloca(sizeof(zm::zmrec_t) * (ncols) ) ) zm::zmrec_t)

}; // namespace zm
#endif // ZMVAL8_H
